@@ -14,12 +14,11 @@
  * limitations under the License.
  */
 
-#include <string_view>
+#include <map>
+#include <string>
 
 namespace apache::thrift::detail {
 
-struct bundled_annotations {
-  static const std::string_view scope_file_content();
-};
+const std::map<std::string, std::string>& bundled_annotation_files();
 
 } // namespace apache::thrift::detail
