@@ -16,10 +16,28 @@
 
 #include "thrift/compiler/ast/t_const_value.h"
 
-#include <type_traits>
-
 namespace apache::thrift::compiler {
 
+std::string_view t_const_value::kind_to_string(t_const_value_kind kind) {
+  switch (kind) {
+    case CV_BOOL:
+      return "bool";
+    case CV_INTEGER:
+      return "integer";
+    case CV_DOUBLE:
+      return "double";
+    case CV_STRING:
+      return "string";
+    case CV_MAP:
+      return "map({})";
+    case CV_LIST:
+      return "list([])";
+    case CV_IDENTIFIER:
+      return "identifier";
+  }
+  abort();
+}
+
 const std::vector<t_const_value*>& t_const_value::get_list_or_empty_map()
     const {
   if (kind_ == CV_LIST) {
