@@ -14,17 +14,15 @@
  * limitations under the License.
  */
 
-#include <algorithm>
-#include <map>
 #include <string>
 #include <fmt/core.h>
+#include <range/v3/view.hpp>
 
 #include <thrift/compiler/ast/ast_visitor.h>
 #include <thrift/compiler/ast/t_program_bundle.h>
-#include <thrift/compiler/ast/t_struct.h>
-#include <thrift/compiler/ast/uri.h>
+#include <thrift/compiler/ast/t_structured.h>
+#include <thrift/compiler/codemod/codemod.h>
 #include <thrift/compiler/codemod/file_manager.h>
-#include <thrift/compiler/compiler.h>
 #include <thrift/compiler/generate/cpp/util.h>
 
 using namespace apache::thrift::compiler;
@@ -35,22 +33,43 @@ bool is_container(const t_type& type) {
   return true_type && true_type->is_container();
 }
 
+std::string quote(std::string_view str) {
+  if (str.find('"') == std::string::npos) {
+    return fmt::format("\"{}\"", str);
+  } else if (str.find('\'') == std::string::npos) {
+    return fmt::format("'{}'", str);
+  } else {
+    std::string out(str);
+    size_t start_pos = 0;
+    while ((start_pos = out.find('"', start_pos)) != std::string::npos) {
+      out.replace(start_pos, 1, "\\\"");
+      start_pos += 2;
+    }
+    return fmt::format("\"{}\"", out);
+  }
+}
+
 class structure_annotations {
  public:
   structure_annotations(source_manager& sm, t_program& program)
       : fm_(sm, program), sm_(sm), prog_(program) {}
 
-  std::set<std::string> visit_type(t_type_ref typeRef, const t_named& node) {
+  // if annotations_for_catch_all is non-null, type annotations will be removed
+  // and added to that map. (This only makes sense for typedefs).
+  std::set<std::string> visit_type(
+      t_type_ref type_ref,
+      const t_named& node,
+      std::map<std::string, std::string>* annotations_for_catch_all) {
     std::set<std::string> to_add;
-    if (!typeRef.resolve() || typeRef->is_primitive_type() ||
-        typeRef->is_container() ||
-        (typeRef->is_typedef() &&
-         static_cast<const t_typedef&>(*typeRef).typedef_kind() !=
+    if (!type_ref.resolve() || type_ref->is_primitive_type() ||
+        type_ref->is_container() ||
+        (type_ref->is_typedef() &&
+         static_cast<const t_typedef&>(*type_ref).typedef_kind() !=
              t_typedef::kind::defined)) {
-      auto type = typeRef.get_type();
+      auto type = type_ref.get_type();
       std::vector<t_annotation> to_remove;
-      bool has_cpp_type = node.find_structured_annotation_or_null(kCppTypeUri);
-      for (const auto& [name, data] : type->annotations()) {
+      bool has_cpp_type = node.has_structured_annotation(kCppTypeUri);
+      for (const auto& [name, data] : type->unstructured_annotations()) {
         // cpp type
         if (name == "cpp.template" || name == "cpp2.template") {
           to_remove.emplace_back(name, data);
@@ -59,23 +78,44 @@ class structure_annotations {
                 fmt::format("@cpp.Type{{template = \"{}\"}}", data.value));
             fm_.add_include("thrift/annotation/cpp.thrift");
           }
-        }
-        if (name == "cpp.type" || name == "cpp2.type") {
+        } else if (name == "cpp.type" || name == "cpp2.type") {
           to_remove.emplace_back(name, data);
           if (!std::exchange(has_cpp_type, true)) {
             to_add.insert(
                 fmt::format("@cpp.Type{{name = \"{}\"}}", data.value));
             fm_.add_include("thrift/annotation/cpp.thrift");
           }
-        }
-        if (name == "cpp.ref" || name == "cpp2.ref" || name == "cpp.ref_type" ||
+        } else if (
+            name == "cpp.ref" || name == "cpp2.ref" || name == "cpp.ref_type" ||
             name == "cpp2.ref_type") {
           to_remove.emplace_back(name, data);
         }
+
+        // rust type
+        else if (name == "rust.type") {
+          to_remove.emplace_back(name, data);
+          to_add.insert(fmt::format("@rust.Type{{name = \"{}\"}}", data.value));
+          fm_.add_include("thrift/annotation/rust.thrift");
+        }
+
+        // haskell annotations are ignored by the main compiler
+        else if (name.find("hs.") == 0) {
+        }
+
+        // THRIFTX doesn't work with structured annotations, but deprecation is
+        // planned for Q1 2025.
+        else if (name.find("thriftx.") == 0) {
+        }
+
+        // catch-all (if typedef)
+        else if (annotations_for_catch_all) {
+          to_remove.emplace_back(name, data);
+          annotations_for_catch_all->emplace(name, data.value);
+        }
       }
 
       if (!to_remove.empty() &&
-          to_remove.size() == type->annotations().size()) {
+          to_remove.size() == type->unstructured_annotations().size()) {
         fm_.remove_all_annotations(*type);
       } else {
         for (const auto& annot : to_remove) {
@@ -88,21 +128,23 @@ class structure_annotations {
   }
 
   void visit_def(const t_named& node) {
+    std::map<std::string, std::string> annotations_for_catch_all;
     std::set<std::string> to_add;
     if (auto typedf = dynamic_cast<const t_typedef*>(&node)) {
-      to_add = visit_type(typedf->type(), node);
+      to_add = visit_type(typedf->type(), node, &annotations_for_catch_all);
     } else if (auto field = dynamic_cast<const t_field*>(&node)) {
-      to_add = visit_type(field->type(), node);
+      to_add = visit_type(field->type(), node, nullptr);
     }
 
     std::vector<t_annotation> to_remove;
-    bool has_cpp_type = node.find_structured_annotation_or_null(kCppTypeUri);
-    bool has_cpp_ref = node.find_structured_annotation_or_null(kBoxUri) ||
-        node.find_structured_annotation_or_null(kInternBoxUri) ||
-        node.find_structured_annotation_or_null(kCppRefUri);
-    for (const auto& [name, data] : node.annotations()) {
+    bool has_cpp_type = node.has_structured_annotation(kCppTypeUri);
+    bool has_cpp_ref = node.has_structured_annotation(kBoxUri) ||
+        node.has_structured_annotation(kInternBoxUri) ||
+        node.has_structured_annotation(kCppRefUri);
+    for (const auto& [name, data] : node.unstructured_annotations()) {
       // cpp type
       bool is_typedef = dynamic_cast<const t_typedef*>(&node);
+      bool is_field = dynamic_cast<const t_field*>(&node);
       if (name == "cpp.template" || name == "cpp2.template") {
         to_remove.emplace_back(name, data);
         if (is_typedef &&
@@ -113,8 +155,7 @@ class structure_annotations {
               fmt::format("@cpp.Type{{template = \"{}\"}}", data.value));
           fm_.add_include("thrift/annotation/cpp.thrift");
         }
-      }
-      if (name == "cpp.type" || name == "cpp2.type") {
+      } else if (name == "cpp.type" || name == "cpp2.type") {
         to_remove.emplace_back(name, data);
         if (is_typedef && !std::exchange(has_cpp_type, true)) {
           to_add.insert(fmt::format("@cpp.Type{{name = \"{}\"}}", data.value));
@@ -123,25 +164,23 @@ class structure_annotations {
       }
 
       // cpp ref
-      bool is_field = dynamic_cast<const t_field*>(&node);
-      if (name == "thrift.box") {
+      else if (name == "thrift.box") {
         to_remove.emplace_back(name, data);
         if (is_field && !std::exchange(has_cpp_ref, true)) {
           to_add.insert("@thrift.Box");
           fm_.add_include("thrift/annotation/thrift.thrift");
         }
-      }
-      if (name == "cpp.name") {
+      } else if (name == "cpp.name") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kCppNameUri)) {
+        if (!node.has_structured_annotation(kCppNameUri)) {
           to_add.insert(fmt::format("@cpp.Name{{value = \"{}\"}}", data.value));
           fm_.add_include("thrift/annotation/cpp.thrift");
         }
-      }
-      if (name == "cpp.ref_type" || name == "cpp2.ref_type") {
+      } else if (name == "cpp.ref_type" || name == "cpp2.ref_type") {
         to_remove.emplace_back(name, data);
         if (is_field &&
-            !node.find_annotation_or_null({"cpp.box", "thrift.box"}) &&
+            !node.find_unstructured_annotation_or_null(
+                {"cpp.box", "thrift.box"}) &&
             !std::exchange(has_cpp_ref, true)) {
           if (data.value == "unique") {
             to_add.insert("@cpp.Ref{type = cpp.RefType.Unique}");
@@ -152,11 +191,10 @@ class structure_annotations {
           }
           fm_.add_include("thrift/annotation/cpp.thrift");
         }
-      }
-      if (name == "cpp.ref" || name == "cpp2.ref") {
+      } else if (name == "cpp.ref" || name == "cpp2.ref") {
         to_remove.emplace_back(name, data);
         if (is_field &&
-            !node.find_annotation_or_null(
+            !node.find_unstructured_annotation_or_null(
                 {"cpp.box", "thrift.box", "cpp.ref_type", "cpp2.ref_type"}) &&
             !std::exchange(has_cpp_ref, true)) {
           to_add.insert("@cpp.Ref{type = cpp.RefType.Unique}");
@@ -165,13 +203,13 @@ class structure_annotations {
       }
 
       // thrift
-      if (name == "priority") {
+      else if (name == "priority") {
         if (!dynamic_cast<const t_function*>(&node)) {
           // pydeprecated consumes this annotation on services as well
           continue;
         }
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kPriorityUri) &&
+        if (!node.has_structured_annotation(kPriorityUri) &&
             std::set<std::string>({"HIGH_IMPORTANT",
                                    "HIGH",
                                    "IMPORTANT",
@@ -182,29 +220,33 @@ class structure_annotations {
               "@thrift.Priority{{level = thrift.RpcPriority.{}}}", data.value));
           fm_.add_include("thrift/annotation/thrift.thrift");
         }
-      }
-      if (name == "serial") {
+      } else if (name == "serial") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kSerialUri)) {
+        if (!node.has_structured_annotation(kSerialUri)) {
           to_add.insert("@thrift.Serial");
           fm_.add_include("thrift/annotation/thrift.thrift");
         }
+      } else if (name == "thrift.uri") {
+        to_remove.emplace_back(name, data);
+        if (!node.has_structured_annotation(kUriUri)) {
+          to_add.insert(
+              fmt::format("@thrift.Uri{{value = \"{}\"}}", data.value));
+          fm_.add_include("thrift/annotation/thrift.thrift");
+        }
       }
 
       // cpp
-      if (name == "cpp.coroutine") {
+      else if (name == "cpp.coroutine") {
         to_remove.emplace_back(name, data);
-      }
-      if (name == "cpp.minimize_padding") {
+      } else if (name == "cpp.minimize_padding") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kCppMinimizePaddingUri)) {
+        if (!node.has_structured_annotation(kCppMinimizePaddingUri)) {
           to_add.insert("@cpp.MinimizePadding");
           fm_.add_include("thrift/annotation/cpp.thrift");
         }
-      }
-      if (name == "cpp.enum_type" || name == "cpp2.enum_type") {
+      } else if (name == "cpp.enum_type") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kCppEnumTypeUri)) {
+        if (!node.has_structured_annotation(kCppEnumTypeUri)) {
           std::string_view value = data.value;
           if (value.substr(0, 2) == "::") {
             value = value.substr(2);
@@ -232,35 +274,29 @@ class structure_annotations {
               "@cpp.EnumType{{type = cpp.EnumUnderlyingType.{}}}", type));
           fm_.add_include("thrift/annotation/cpp.thrift");
         }
-      }
-      if (name == "cpp.declare_bitwise_ops" ||
-          name == "cpp2.declare_bitwise_ops") {
+      } else if (
+          name == "cpp.declare_bitwise_ops" &&
+          (node.has_structured_annotation(kBitmaskEnumUri) ||
+           node.has_unstructured_annotation("bitmask"))) {
         // This annotation is a subset of @thrift.BitmaskEnum, which carries
         // additional restrictions. We can't turn this into that but we can use
         // it if already present to remove this.
-        if (node.find_structured_annotation_or_null(kBitmaskEnumUri) ||
-            node.has_annotation("bitmask")) {
-          to_remove.emplace_back(name, data);
-        }
-      }
-      if (name == "cpp.mixin") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kMixinUri)) {
+      } else if (name == "cpp.mixin") {
+        to_remove.emplace_back(name, data);
+        if (!node.has_structured_annotation(kMixinUri)) {
           to_add.insert("@thrift.Mixin");
           fm_.add_include("thrift/annotation/thrift.thrift");
         }
-      }
-      if (name == "cpp.experimental.lazy") {
+      } else if (name == "cpp.experimental.lazy") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kCppLazyUri)) {
+        if (!node.has_structured_annotation(kCppLazyUri)) {
           to_add.insert("@cpp.Lazy");
           fm_.add_include("thrift/annotation/cpp.thrift");
         }
-      }
-      if (name == "thread" || name == "process_in_event_base") {
+      } else if (name == "thread" || name == "process_in_event_base") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(
-                kCppProcessInEbThreadUri) &&
+        if (!node.has_structured_annotation(kCppProcessInEbThreadUri) &&
             (name != "thread" || data.value == "eb")) {
           to_add.insert("@cpp.ProcessInEbThreadUnsafe");
           fm_.add_include("thrift/annotation/cpp.thrift");
@@ -268,7 +304,7 @@ class structure_annotations {
       }
 
       // hack
-      if (name == "hack.attributes") {
+      else if (name == "hack.attributes") {
         const char* pos = sm_.get_text(data.src_range.begin);
         const char* end = sm_.get_text(data.src_range.end);
         for (; pos != end && *pos != '"' && *pos != '\''; ++pos) {
@@ -305,69 +341,62 @@ class structure_annotations {
           }
         }
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kHackAttributeUri)) {
+        if (!node.has_structured_annotation(kHackAttributeUri)) {
           to_add.insert(fmt::format(
               "@hack.Attributes{{attributes = [{}]}}", fmt::join(attrs, ", ")));
           fm_.add_include("thrift/annotation/hack.thrift");
         }
-      }
-      if (name == "hack.name") {
+      } else if (name == "hack.name") {
         to_remove.emplace_back(name, data);
         to_add.insert(fmt::format("@hack.Name{{name = \"{}\"}}", data.value));
         fm_.add_include("thrift/annotation/hack.thrift");
-      }
-      if (name == "code") {
+      } else if (name == "code") {
         to_remove.emplace_back(name, data);
-      }
-      if (name == "message") {
+      } else if (name == "message") {
         to_remove.emplace_back(name, data);
         if (auto field =
                 dynamic_cast<const t_structured&>(node).get_field_by_name(
                     data.value);
-            field &&
-            !node.find_structured_annotation_or_null(kExceptionMessageUri)) {
+            field && !node.has_structured_annotation(kExceptionMessageUri)) {
           fm_.add(
               {field->src_range().begin.offset(),
                field->src_range().begin.offset(),
                "@thrift.ExceptionMessage\n"});
           fm_.add_include("thrift/annotation/thrift.thrift");
         }
-      }
-      if (name == "bitmask") {
+      } else if (name == "bitmask") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kBitmaskEnumUri)) {
+        if (!node.has_structured_annotation(kBitmaskEnumUri)) {
           to_add.insert("@thrift.BitmaskEnum");
           fm_.add_include("thrift/annotation/thrift.thrift");
         }
       }
 
       // python
-      if (name == "py3.hidden") {
+      else if (name == "py3.hidden") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kPythonPy3HiddenUri)) {
+        if (!node.has_structured_annotation(kPythonPy3HiddenUri)) {
           to_add.insert("@python.Py3Hidden");
           fm_.add_include("thrift/annotation/python.thrift");
         }
-      }
-      if (name == "py3.name") {
+      } else if (name == "py3.name") {
         to_remove.emplace_back(name, data);
-        if (!node.find_structured_annotation_or_null(kPythonNameUri)) {
+        if (!node.has_structured_annotation(kPythonNameUri)) {
           to_add.insert(
               fmt::format("@python.Name{{name = \"{}\"}}", data.value));
           fm_.add_include("thrift/annotation/python.thrift");
         }
-      }
-      if (name == "py3.flags") {
+      } else if (name == "py3.flags") {
         to_remove.emplace_back(name, data);
         if (dynamic_cast<const t_enum*>(&node) &&
-            !node.find_structured_annotation_or_null(kPythonFlagsUri)) {
+            !node.has_structured_annotation(kPythonFlagsUri)) {
           to_add.insert("@python.Flags");
           fm_.add_include("thrift/annotation/python.thrift");
         }
       }
 
       // java
-      if (name == "java.swift.mutable") {
+      else if (name == "java.swift.mutable") {
         to_remove.emplace_back(name, data);
         if (data.value == "true" &&
             (dynamic_cast<const t_struct*>(&node) &&
@@ -375,8 +404,7 @@ class structure_annotations {
           to_add.insert("@java.Mutable");
           fm_.add_include("thrift/annotation/java.thrift");
         }
-      }
-      if (name == "java.swift.annotations") {
+      } else if (name == "java.swift.annotations") {
         to_remove.emplace_back(name, data);
         to_add.insert(fmt::format(
             "@java.Annotation{{java_annotation = \"{}\"}}", data.value));
@@ -384,51 +412,137 @@ class structure_annotations {
       }
 
       // go
-      if (name == "go.name") {
+      else if (name == "go.name") {
         to_remove.emplace_back(name, data);
         to_add.insert(fmt::format("@go.Name{{name = \"{}\"}}", data.value));
         fm_.add_include("thrift/annotation/go.thrift");
-      }
-      if (name == "go.tag") {
+      } else if (name == "go.tag") {
         to_remove.emplace_back(name, data);
         to_add.insert(fmt::format("@go.Tag{{tag = '{}'}}", data.value));
         fm_.add_include("thrift/annotation/go.thrift");
       }
 
       // erlang
-      if (name == "erl.name") {
+      else if (name == "erl.name") {
+        to_remove.emplace_back(name, data);
+        to_add.insert(
+            fmt::format("@erlang.NameOverride{{name = \"{}\"}}", data.value));
+        fm_.add_include("thrift/annotation/erlang.thrift");
+      } else if (name == "erl.struct_repr") {
         to_remove.emplace_back(name, data);
         to_add.insert(fmt::format(
-            "@annotation.NameOverride{{name = \"{}\"}}", data.value));
-        fm_.add_include("thrift/facebook/erlang/annotation.thrift");
+            "@erlang.StructRepr{{repr = {}}}",
+            data.value == "record" ? "erlang.StructReprType.RECORD"
+                                   : "erlang.StructReprType.MAP"));
+        fm_.add_include("thrift/annotation/erlang.thrift");
+      } else if (name == "erl.default_value") {
+        to_remove.emplace_back(name, data);
+        to_add.insert(
+            fmt::format("@erlang.DefaultValue{{value = \"{}\"}}", data.value));
+        fm_.add_include("thrift/annotation/erlang.thrift");
+      } else if (name == "iq.node_type") {
+        to_remove.emplace_back(name, data);
+        to_add.insert(fmt::format(
+            "@erlang.Iq{{node_type = {}}}",
+            data.value == "xmlcdata"      ? "erlang.IqNodeType.XMLCDATA"
+                : data.value == "xmlnode" ? "erlang.IqNodeType.XMLNODE"
+                                          : "erlang.IqNodeType.XMLATTRIBUTE"));
+        fm_.add_include("thrift/annotation/erlang.thrift");
       }
-      if (name == "erl.struct_repr") {
+
+      // rust
+      else if (name == "rust.name") {
         to_remove.emplace_back(name, data);
-        to_add.insert(fmt::format(
-            "@annotation.StructRepr{{repr = {}}}",
-            data.value == "record" ? "annotation.StructReprType.RECORD"
-                                   : "annotation.StructReprType.MAP"));
-        fm_.add_include("thrift/facebook/erlang/annotation.thrift");
+        to_add.insert(fmt::format("@rust.Name{{name = \"{}\"}}", data.value));
+        fm_.add_include("thrift/annotation/rust.thrift");
+      } else if (name == "rust.copy") {
+        to_remove.emplace_back(name, data);
+        if (!dynamic_cast<const t_enum*>(&node)) {
+          // enums automatically implement Copy
+          to_add.insert("@rust.Copy");
+          fm_.add_include("thrift/annotation/rust.thrift");
+        }
+      } else if (name == "rust.arc") {
+        to_remove.emplace_back(name, data);
+        to_add.insert("@rust.Arc");
+        fm_.add_include("thrift/annotation/rust.thrift");
+      } else if (name == "rust.box") {
+        to_remove.emplace_back(name, data);
+        to_add.insert("@rust.Box");
+        fm_.add_include("thrift/annotation/rust.thrift");
+      } else if (name == "rust.exhaustive") {
+        to_remove.emplace_back(name, data);
+        if (!dynamic_cast<const t_union*>(&node) &&
+            !dynamic_cast<const t_enum*>(&node)) {
+          // annotation is no-op on unions and enums
+          to_add.insert("@rust.Exhaustive");
+          fm_.add_include("thrift/annotation/rust.thrift");
+        }
+      } else if (name == "rust.ord") {
+        to_remove.emplace_back(name, data);
+        if (!dynamic_cast<const t_enum*>(&node)) {
+          // enums automatically implement Ord
+          to_add.insert("@rust.Ord");
+          fm_.add_include("thrift/annotation/rust.thrift");
+        }
+      } else if (name == "rust.newtype") {
+        to_remove.emplace_back(name, data);
+        to_add.insert("@rust.NewType");
+        fm_.add_include("thrift/annotation/rust.thrift");
+      } else if (name == "rust.type") {
+        to_remove.emplace_back(name, data);
+        if (dynamic_cast<const t_typedef*>(&node)) {
+          to_add.insert(fmt::format("@rust.Type{{name = \"{}\"}}", data.value));
+          fm_.add_include("thrift/annotation/rust.thrift");
+        }
+      } else if (name == "rust.serde") {
+        to_remove.emplace_back(name, data);
+        if (data.value == "false") {
+          to_add.insert("@rust.Serde{enabled = false}");
+        } else {
+          to_add.insert("@rust.Serde");
+        }
+        fm_.add_include("thrift/annotation/rust.thrift");
+      } else if (name == "rust.mod") {
+        to_remove.emplace_back(name, data);
+        to_add.insert(fmt::format("@rust.Mod{{name = \"{}\"}}", data.value));
+        fm_.add_include("thrift/annotation/rust.thrift");
+      } else if (name == "rust.derive") {
+        to_remove.emplace_back(name, data);
+        auto derives =
+            data.value | ranges::views::split(',') |
+            ranges::views::transform([](auto s) {
+              auto str = s | ranges::to<std::string>;
+              str.erase(remove_if(str.begin(), str.end(), isspace), str.end());
+              return fmt::format("\"{}\"", str);
+            }) |
+            ranges::views::join(',') | ranges::to<std::string>;
+        to_add.insert(fmt::format("@rust.Derive{{derives = [{}]}}", derives));
+        fm_.add_include("thrift/annotation/rust.thrift");
+      } else if (name == "rust.request_context") {
+        to_remove.emplace_back(name, data);
+        to_add.insert("@rust.RequestContext");
+        fm_.add_include("thrift/annotation/rust.thrift");
       }
-      if (name == "erl.default_value") {
-        to_remove.emplace_back(name, data);
-        to_add.insert(fmt::format(
-            "@annotation.DefaultValue{{value = \"{}\"}}", data.value));
-        fm_.add_include("thrift/facebook/erlang/annotation.thrift");
+
+      // haskell annotations are ignored by the main compiler
+      else if (name.find("hs.") == 0) {
       }
-      if (name == "iq.node_type") {
+
+      // THRIFTX doesn't work with structured annotations, but deprecation is
+      // planned for Q1 2025.
+      else if (name.find("thriftx.") == 0) {
+      }
+
+      // catch-all
+      else {
         to_remove.emplace_back(name, data);
-        to_add.insert(fmt::format(
-            "@annotation.Iq{{node_type = {}}}",
-            data.value == "xmlcdata" ? "annotation.IqNodeType.XMLCDATA"
-                : data.value == "xmlnode"
-                ? "annotation.IqNodeType.XMLNODE"
-                : "annotation.IqNodeType.XMLATTRIBUTE"));
-        fm_.add_include("thrift/facebook/erlang/annotation.thrift");
+        annotations_for_catch_all.emplace(name, data.value);
       }
     }
 
-    if (!to_remove.empty() && to_remove.size() == node.annotations().size()) {
+    if (!to_remove.empty() &&
+        to_remove.size() == node.unstructured_annotations().size()) {
       fm_.remove_all_annotations(node);
     } else {
       for (const auto& annot : to_remove) {
@@ -436,6 +550,18 @@ class structure_annotations {
       }
     }
 
+    if (!annotations_for_catch_all.empty()) {
+      std::vector<std::string> annotations_for_catch_all_strs;
+      for (const auto& [name, value] : annotations_for_catch_all) {
+        annotations_for_catch_all_strs.push_back(
+            fmt::format(R"("{}": {})", name, quote(value)));
+      }
+      to_add.insert(fmt::format(
+          "@thrift.DeprecatedUnvalidatedAnnotations{{items = {{{}}}}}",
+          fmt::join(annotations_for_catch_all_strs, ", ")));
+      fm_.add_include("thrift/annotation/thrift.thrift");
+    }
+
     if (!to_add.empty()) {
       fm_.add(
           {node.src_range().begin.offset(),
@@ -447,6 +573,11 @@ class structure_annotations {
   void run() {
     const_ast_visitor visitor;
     visitor.add_named_visitor([=](const auto& node) { visit_def(node); });
+    visitor.add_function_visitor([=](const t_function& node) {
+      for (const t_field& param : node.params().fields()) {
+        visit_def(param);
+      }
+    });
     visitor(prog_);
 
     fm_.apply_replacements();
@@ -460,14 +591,8 @@ class structure_annotations {
 } // namespace
 
 int main(int argc, char** argv) {
-  auto source_mgr = source_manager();
-  auto program_bundle = parse_and_get_program(
-      source_mgr, std::vector<std::string>(argv, argv + argc));
-  if (!program_bundle) {
-    return 1;
-  }
-  auto program = program_bundle->root_program();
-  structure_annotations(source_mgr, *program).run();
-
-  return 0;
+  return apache::thrift::compiler::run_codemod(
+      argc, argv, [](source_manager& sm, t_program_bundle& pb) {
+        structure_annotations(sm, *pb.get_root_program()).run();
+      });
 }
