@@ -31,6 +31,8 @@
 #else
 #include <unistd.h>
 #endif
+#include <stdio.h>
+#include <algorithm>
 #include <ctime>
 #include <filesystem>
 #include <fstream>
@@ -38,7 +40,6 @@
 
 #include <boost/algorithm/string/split.hpp>
 
-#include <thrift/annotation/bundled_annotations.h>
 #include <thrift/compiler/ast/t_program_bundle.h>
 #include <thrift/compiler/detail/system.h>
 #include <thrift/compiler/diagnostic.h>
@@ -51,111 +52,109 @@
 
 namespace apache::thrift::compiler {
 namespace {
+
 /**
- * Flags to control code generation
+ * Display the usage message to the given C stream.
+ *
+ * @param stream See https://en.cppreference.com/w/c/io/FILE
  */
-struct gen_params {
-  bool gen_recurse = false;
-  std::string genfile;
-  std::vector<std::string> targets;
-  std::string out_path;
-  bool add_gen_dir = true;
+void printUsageTo(FILE* stream) {
+  fprintf(stream, R"(Usage: thrift [options] file
 
-  // If true, code generation will be skipped (regardless of other parameters).
-  // This is useful, for example, to parse and validate source Thrift IDL
-  // without a particular target language in mind.
-  bool skip_gen = false;
+Options:
+  --help      Print this message to stdout and exit (successfully).
+  -o dir      Set the output directory for gen-* packages (default: current
+              directory).
+              At most one of this option (-o) or -out (see below) can be
+              specified.
+  -out dir    Set the output location for generated files (no gen-* folder will
+              be created).
+              At most one of this option (-out) or -o (see above) can be
+              specified.
+  -I dir      Add a directory to the list of directories searched for include
+              directives
 
-  bool inject_schema_const = false;
-};
+  -nowarn     Suppress all compiler warnings (BAD!)
+  -legacy-strict
+              Strict compiler warnings on (DEPRECATED)
+  -v[erbose]  Verbose mode
+  -r[ecurse]  Also generate included files
+  -debug      Parse debug trace to stdout
+  --allow-neg-keys
+              Allow negative field keys (IGNORED: always true).
+  --allow-neg-enum-vals
+              Allow negative enum vals (DEPRECATED)
+  --allow-64bit-consts
+              Do not print warnings about using 64-bit constants
 
-constexpr bool bundle_annotations() {
-#ifdef THRIFT_OSS
-  return false;
-#else
-  // Bundling is temporarily disabled because of issues with configerator.
-  return false;
-#endif
+  --gen STR   Generate code with a dynamically-registered generator.
+              STR has the form language[:key1=val1[,key2,[key3=val3]]].
+              Keys and values are options passed to the generator.
+              Many options will not require values.
+              If --skip-gen (see below) is specified, then --gen
+              cannot be used. Otherwise (i.e., --skip-gen is NOT
+              specified), then --gen must be specified at least once,
+              and may be specified multiple times (eg. to generate
+              multiple languages).
+  --skip-gen  Skip code generation. This is useful, for example, to
+              parse and validate Thrift IDL without generating code
+              for any particular language.
+              If --skip-gen is specified, no --gen argument may be
+              given (see above).
+  --record-genfiles FILE
+              Save the list of generated files to FILE
+  --inject-schema-const
+              Inject generated schema constant (must use thrift2ast)
+  --extra-validation
+              Comma-separated list of opt-in validators to run. Recognized
+              values include:
+                unstructured_annotations
+                  Forbids all unstructured annotations
+
+                warn_on_redundant_custom_default_values
+                  Issues warnings for struct fields that have custom default
+                  values equal to the intrinsic default for that type (eg. 0
+                  for integers, false for boolean, etc.)
+
+                forbid_non_optional_cpp_ref_fields (IGNORED: set by default).
+                  Struct (and exception) fields with a @cpp.Ref (or
+                  cpp[2].ref[_type]) annotation must be optional, unless
+                  annotated with @cpp.AllowLegacyNonOptionalRef.
+
+                implicit_field_ids (IGNORED: always present, i.e. implicit field
+                  IDs are always forbidden).
+
+Available generators (and options):
+)");
+  for (const auto& gen : generator_registry::get_generators()) {
+    const generator_factory& generator_factory = *gen.second;
+    fmt::print(
+        stream,
+        "  {} ({}):\n{}",
+        generator_factory.name(),
+        generator_factory.long_name(),
+        generator_factory.documentation());
+  }
 }
 
 /**
- * Display the usage message.
+ * Display the usage message to the standard error stream.
  */
-void usage() {
-  fprintf(stderr, "Usage: thrift [options] file\n");
-  fprintf(stderr, "Options:\n");
-  fprintf(
-      stderr,
-      "  -o dir      Set the output directory for gen-* packages\n"
-      "              (default: current directory).\n"
-      "              At most one of this option (-o) or -out (see below) can\n"
-      "              be specified.\n");
-  fprintf(
-      stderr,
-      "  -out dir    Set the output location for generated files\n"
-      "              (no gen-* folder will be created).\n"
-      "              At most one of this option (-out) or -o (see above) can\n"
-      "              be specified.\n");
-  fprintf(
-      stderr,
-      "  -I dir      Add a directory to the list of directories\n"
-      "              searched for include directives\n");
-  fprintf(stderr, "  -nowarn     Suppress all compiler warnings (BAD!)\n");
-  fprintf(
-      stderr,
-      "  -legacy-strict     Strict compiler warnings on (DEPRECATED)\n");
-  fprintf(stderr, "  -v[erbose]  Verbose mode\n");
-  fprintf(stderr, "  -r[ecurse]  Also generate included files\n");
-  fprintf(stderr, "  -debug      Parse debug trace to stdout\n");
-  fprintf(
-      stderr,
-      "  --allow-neg-keys  Allow negative field keys (Used to preserve protocol\n"
-      "                    compatibility with older .thrift files)\n");
-  fprintf(
-      stderr,
-      "  --allow-neg-enum-vals Allow negative enum vals (DEPRECATED)\n");
-  fprintf(
-      stderr,
-      "  --allow-64bit-consts  Do not print warnings about using 64-bit constants\n");
-  fprintf(
-      stderr,
-      "  --gen STR   Generate code with a dynamically-registered generator.\n"
-      "              STR has the form language[:key1=val1[,key2,[key3=val3]]].\n"
-      "              Keys and values are options passed to the generator.\n"
-      "              Many options will not require values.\n"
-      "              If --skip-gen (see below) is specified, then --gen\n"
-      "              cannot be used. Otherwise (i.e., --skip-gen is NOT\n"
-      "              specified), then --gen must be specified at least once,\n"
-      "              and may be specified multiple times (eg. to generate\n"
-      "              multiple languages).");
-  fprintf(
-      stderr,
-      "  --skip-gen  Skip code generation. This is useful, for example, to\n"
-      "              parse and validate Thrift IDL without generating code\n"
-      "              for any particular language.\n"
-      "              If --skip-gen is specified, no --gen argument may be\n"
-      "              given (see above).\n");
-  fprintf(
-      stderr,
-      "  --record-genfiles FILE\n"
-      "              Save the list of generated files to FILE,\n");
-  fprintf(
-      stderr,
-      "  --inject-schema-const  Inject generated schema constant (must use thrift2ast)\n");
-  fprintf(
-      stderr,
-      "  --extra-validation  Comma-separated list of opt-in validators to run\n");
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Available generators (and options):\n");
+void printUsageError() {
+  printUsageTo(stderr);
+}
 
-  for (const auto& gen : generator_registry::get_generators()) {
-    fmt::print(
-        stderr,
-        "  {} ({}):\n{}",
-        gen.second->name(),
-        gen.second->long_name(),
-        gen.second->documentation());
+/**
+ * If `arguments` includes "--help" (in any position), prints usage message
+ * to STDOUT and returns true. Otherwise returns false.
+ */
+bool maybeHandleHelpInvocation(const std::vector<std::string>& arguments) {
+  if (std::find(arguments.begin(), arguments.end(), "--help") ==
+      arguments.end()) {
+    return false;
   }
+  printUsageTo(stdout);
+  return true;
 }
 
 bool isPathSeparator(const char& c) {
@@ -191,7 +190,7 @@ const std::string* consume_next_arg(
         arg_name,
         arguments[arg_i].c_str(),
         arguments[arg_i + 1].c_str());
-    usage();
+    printUsageError();
     return nullptr;
   }
   return &arguments[++arg_i];
@@ -212,7 +211,7 @@ const std::string* consume_next_arg(
 std::string parse_flag(const std::string& argument) {
   if (argument.size() < 2 || argument[0] != '-' || (argument == "--")) {
     fprintf(stderr, "!!! Expected flag, got: %s\n\n", argument.c_str());
-    usage();
+    printUsageError();
     return {};
   }
 
@@ -239,14 +238,14 @@ bool validate_params(const gen_params& gparams) {
   const bool skip_gen = gparams.skip_gen;
   if (has_targets && skip_gen) {
     fprintf(stderr, "!!! Cannot specify both --skip-gen and --gen.\n\n");
-    usage();
+    printUsageError();
     return false;
   }
   if (!has_targets && !skip_gen) {
     fprintf(
         stderr,
         "!!! No output language(s) specified: need --gen or --skip-gen.\n\n");
-    usage();
+    printUsageError();
     return false;
   }
   // Exactly one of skip_gen and has_targets is true => valid.
@@ -281,168 +280,6 @@ bool validate_params(const gen_params& gparams) {
   return true;
 }
 
-// Returns the input file name if successful, otherwise returns an empty
-// string.
-std::string parse_args(
-    const std::vector<std::string>& arguments,
-    parsing_params& pparams,
-    gen_params& gparams,
-    diagnostic_params& dparams,
-    sema_params& sparams) {
-  // Check for necessary arguments, you gotta have at least a filename and
-  // an output language flag.
-  if (arguments.size() < 3 && gparams.targets.empty()) {
-    usage();
-    return {};
-  }
-
-  size_t arg_i = 1; // Skip the binary name.
-
-  // Convenient closure to call consume_next_arg() without repeating local
-  // variables.
-  auto consume_arg = [&](const char* arg_name) -> const std::string* {
-    return consume_next_arg(arg_name, arguments, arg_i);
-  };
-
-  // Hacky parameter handling... I didn't feel like using a library sorry!
-
-  // Guard so --nowarn and --legacy-strict are order agnostic.
-  bool nowarn = false;
-
-  for (; arg_i < arguments.size() - 1;
-       ++arg_i) { // Last argument is the src file.
-    // Parse flag.
-    const std::string flag = parse_flag(arguments[arg_i]);
-    if (flag.empty()) {
-      return {};
-    }
-
-    // Interpret flag.
-    if (flag == "debug") {
-      dparams.debug = true;
-    } else if (flag == "nowarn") {
-      dparams.warn_level = 0;
-      nowarn = true;
-    } else if (flag == "legacy-strict") {
-      pparams.strict = 255;
-      if (!nowarn) { // Don't override nowarn.
-        dparams.warn_level = 2;
-      }
-    } else if (flag == "v" || flag == "verbose") {
-      dparams.info = true;
-    } else if (flag == "r" || flag == "recurse") {
-      gparams.gen_recurse = true;
-    } else if (flag == "allow-neg-keys") {
-      pparams.allow_neg_field_keys = true;
-    } else if (flag == "allow-neg-enum-vals") {
-      // no-op
-    } else if (flag == "allow-64bit-consts") {
-      pparams.allow_64bit_consts = true;
-    } else if (flag == "record-genfiles") {
-      const std::string* arg = consume_arg("genfile file specification");
-      if (arg == nullptr) {
-        return {};
-      }
-      gparams.genfile = *arg;
-    } else if (flag == "gen") {
-      const std::string* arg = consume_arg("generator specification");
-      if (arg == nullptr) {
-        return {};
-      }
-      gparams.targets.push_back(*arg);
-    } else if (flag == "skip-gen") {
-      gparams.skip_gen = true;
-    } else if (flag == "I") {
-      const std::string* arg = consume_arg("include directory");
-      if (arg == nullptr) {
-        return {};
-      }
-      // An argument of "-I\ asdf" is invalid and has unknown results
-      pparams.incl_searchpath.push_back(*arg);
-    } else if (flag == "o" || flag == "out") {
-      const std::string* arg = consume_arg("output directory");
-      if (arg == nullptr) {
-        return {};
-      }
-
-      if (!gparams.out_path.empty()) {
-        fprintf(stderr, "!!! Cannot specify both -o and --out.\n\n");
-        usage();
-        return {};
-      }
-
-      std::string out_path = *arg;
-
-      // Strip out trailing \ on a Windows path
-      if (detail::platform_is_windows()) {
-        int last = out_path.length() - 1;
-        if (out_path[last] == '\\') {
-          out_path.erase(last);
-        }
-      }
-
-      gparams.out_path = std::move(out_path);
-      gparams.add_gen_dir = (flag == "o");
-    } else if (flag == "inject-schema-const") {
-      gparams.inject_schema_const = true;
-    } else if (flag == "extra-validation") {
-      const std::string* arg = consume_arg("extra validation");
-      if (arg == nullptr) {
-        return {};
-      }
-      std::vector<std::string> validators;
-      boost::algorithm::split(
-          validators, *arg, [](char c) { return c == ','; });
-      for (const auto& validator : validators) {
-        if (validator == "unstructured_annotations_on_field_type") {
-          sparams.forbid_unstructured_annotations_on_field_types = true;
-        } else if (validator == "implicit_field_ids") {
-          sparams.forbid_implicit_field_ids = true;
-        } else {
-          fprintf(
-              stderr, "!!! Unrecognized validator: %s\n\n", validator.c_str());
-          usage();
-          return {};
-        }
-      }
-    } else {
-      fprintf(
-          stderr, "!!! Unrecognized option: %s\n\n", arguments[arg_i].c_str());
-      usage();
-      return {};
-    }
-  }
-
-  if (const char* env_p = std::getenv("THRIFT_INCLUDE_PATH")) {
-    std::vector<std::string> components;
-    boost::algorithm::split(components, env_p, isPathSeparator);
-    pparams.incl_searchpath.insert(
-        pparams.incl_searchpath.end(), components.begin(), components.end());
-  }
-
-  if (!validate_params(gparams)) {
-    return {};
-  }
-
-  // Return the input file name.
-  assert(arg_i == arguments.size() - 1);
-  if (detail::platform_is_windows()) {
-    // The path here is used in schema.thrift
-    //
-    // * In Linux, buck uses relative path.
-    // * In Windows, buck uses absolute path.
-    //
-    // To make the generated schema.thrift consistent with Unix, we need to
-    // convert it to relative path when possible.
-    //
-    // Not that the path might not exist since when thrift compiler is used as
-    // library, user's code can use `source_manager::add_virtual_file(...)`
-    // method to create a virtual file.
-    return std::filesystem::proximate(arguments[arg_i]).generic_string();
-  }
-  return arguments[arg_i];
-}
-
 enum class parse_control : bool { stop, more };
 
 // `callback(key, value)` will be called for each key=value generator's option.
@@ -520,7 +357,7 @@ std::unique_ptr<t_generator> create_generator(
       generator_name, program, program_bundle, diags);
   if (generator == nullptr) {
     fmt::print(stderr, "Error: Invalid generator name: {}\n", generator_name);
-    usage();
+    printUsageError();
     return nullptr;
   }
   generator->process_options(
@@ -734,20 +571,6 @@ std::unique_ptr<t_program_bundle> parse_and_mutate(
     const std::string& input_filename,
     const parsing_params& pparams,
     const gen_params& gparams) {
-  auto found_or_error = source_manager::path_or_error();
-  if (bundle_annotations()) {
-    const std::string scope_path = "thrift/annotation/scope.thrift";
-    found_or_error =
-        source_mgr.find_include_file(scope_path, "", pparams.incl_searchpath);
-    if (found_or_error.index() != 0) {
-      // Fall back to the bundled annotation file.
-      std::string_view content =
-          apache::thrift::detail::bundled_annotations::scope_file_content();
-      source_mgr.add_virtual_file(
-          scope_path, std::string(content.data(), content.size()));
-    }
-  }
-
   // Parse it!
   std::unique_ptr<t_program_bundle> program_bundle = parse_ast(
       source_mgr, ctx, input_filename, pparams, &ctx.sema_parameters());
@@ -757,29 +580,31 @@ std::unique_ptr<t_program_bundle> parse_and_mutate(
 
   // Load standard library if available.
   const std::string schema_path = "thrift/lib/thrift/schema.thrift";
-  found_or_error =
+  auto found_or_error =
       source_mgr.find_include_file(schema_path, "", pparams.incl_searchpath);
-  if (found_or_error.index() == 0 &&
-      !program_bundle->find_program(schema_path)) {
-    sema_context stdlib_ctx(
-        source_mgr,
-        [&](diagnostic&& d) {
-          ctx.report(
-              source_location{},
-              diagnostic_level::debug,
-              "Could not load Thrift standard libraries: {}",
-              d);
-        },
-        diagnostic_params::only_errors());
-    std::unique_ptr<t_program_bundle> inc = parse_ast(
-        source_mgr,
-        stdlib_ctx,
-        schema_path,
-        pparams,
-        nullptr,
-        program_bundle.get());
-    if (inc && !stdlib_ctx.has_errors()) {
-      program_bundle->add_implicit_includes(std::move(inc));
+  if (found_or_error.index() == 0) {
+    if (!program_bundle->find_program_by_full_path(
+            std::get<0>(found_or_error))) {
+      sema_context stdlib_ctx(
+          source_mgr,
+          [&](diagnostic&& d) {
+            ctx.report(
+                source_location{},
+                diagnostic_level::debug,
+                "Could not load Thrift standard libraries: {}",
+                d);
+          },
+          diagnostic_params::only_errors());
+      std::unique_ptr<t_program_bundle> inc = parse_ast(
+          source_mgr,
+          stdlib_ctx,
+          schema_path,
+          pparams,
+          nullptr,
+          program_bundle.get());
+      if (inc && !stdlib_ctx.has_errors()) {
+        program_bundle->add_implicit_includes(std::move(inc));
+      }
     }
   }
 
@@ -803,6 +628,185 @@ std::unique_ptr<t_program_bundle> parse_and_mutate(
 
 } // namespace
 
+// Returns the input file name if successful, otherwise returns an empty
+// string.
+std::string parse_args(
+    const std::vector<std::string>& arguments,
+    parsing_params& pparams,
+    gen_params& gparams,
+    diagnostic_params& dparams,
+    sema_params& sparams) {
+  // Check for necessary arguments, you gotta have at least a filename and
+  // an output language flag.
+  if (arguments.size() < 3 && gparams.targets.empty()) {
+    printUsageError();
+    return {};
+  }
+
+  size_t arg_i = 1; // Skip the binary name.
+
+  // Convenient closure to call consume_next_arg() without repeating local
+  // variables.
+  auto consume_arg = [&](const char* arg_name) -> const std::string* {
+    return consume_next_arg(arg_name, arguments, arg_i);
+  };
+
+  // Hacky parameter handling... I didn't feel like using a library sorry!
+
+  // Guard so --nowarn and --legacy-strict are order agnostic.
+  bool nowarn = false;
+
+  for (; arg_i < arguments.size() - 1;
+       ++arg_i) { // Last argument is the src file.
+    // Parse flag.
+    const std::string flag = parse_flag(arguments[arg_i]);
+    if (flag.empty()) {
+      return {};
+    }
+
+    // Interpret flag.
+    if (flag == "debug") {
+      dparams.debug = true;
+    } else if (flag == "nowarn") {
+      dparams.warn_level = 0;
+      nowarn = true;
+    } else if (flag == "legacy-strict") {
+      pparams.strict = 255;
+      if (!nowarn) { // Don't override nowarn.
+        dparams.warn_level = 2;
+      }
+    } else if (flag == "v" || flag == "verbose") {
+      dparams.info = true;
+    } else if (flag == "r" || flag == "recurse") {
+      gparams.gen_recurse = true;
+    } else if (flag == "allow-neg-keys") {
+      // no-op
+    } else if (flag == "allow-neg-enum-vals") {
+      // no-op
+    } else if (flag == "allow-64bit-consts") {
+      pparams.allow_64bit_consts = true;
+    } else if (flag == "record-genfiles") {
+      const std::string* arg = consume_arg("genfile file specification");
+      if (arg == nullptr) {
+        return {};
+      }
+      gparams.genfile = *arg;
+    } else if (flag == "gen") {
+      const std::string* arg = consume_arg("generator specification");
+      if (arg == nullptr) {
+        return {};
+      }
+      gparams.targets.push_back(*arg);
+    } else if (flag == "skip-gen") {
+      gparams.skip_gen = true;
+    } else if (flag == "I") {
+      const std::string* arg = consume_arg("include directory");
+      if (arg == nullptr) {
+        return {};
+      }
+      // An argument of "-I\ asdf" is invalid and has unknown results
+      pparams.incl_searchpath.push_back(*arg);
+    } else if (flag == "o" || flag == "out") {
+      const std::string* arg = consume_arg("output directory");
+      if (arg == nullptr) {
+        return {};
+      }
+
+      if (!gparams.out_path.empty()) {
+        fprintf(stderr, "!!! Cannot specify both -o and --out.\n\n");
+        printUsageError();
+        return {};
+      }
+
+      std::string out_path = *arg;
+
+      // Strip out trailing \ on a Windows path
+      if (detail::platform_is_windows()) {
+        int last = out_path.length() - 1;
+        if (out_path[last] == '\\') {
+          out_path.erase(last);
+        }
+      }
+
+      gparams.out_path = std::move(out_path);
+      gparams.add_gen_dir = (flag == "o");
+    } else if (flag == "inject-schema-const") {
+      gparams.inject_schema_const = true;
+    } else if (flag == "extra-validation") {
+      const std::string* arg = consume_arg("extra validation");
+      if (arg == nullptr) {
+        return {};
+      }
+      std::vector<std::string> validators;
+      boost::algorithm::split(
+          validators, *arg, [](char c) { return c == ','; });
+      for (const auto& validator : validators) {
+        if (validator == "unstructured_annotations") {
+          sparams.forbid_unstructured_annotations = true;
+        } else if (validator == "warn_on_redundant_custom_default_values") {
+          sparams.warn_on_redundant_custom_default_values = true;
+        } else if (validator == "forbid_non_optional_cpp_ref_fields") {
+          // no-op
+        } else if (validator == "implicit_field_ids") {
+          // no-op
+        } else {
+          fprintf(
+              stderr, "!!! Unrecognized validator: %s\n\n", validator.c_str());
+          printUsageError();
+          return {};
+        }
+      }
+    } else {
+      fprintf(
+          stderr, "!!! Unrecognized option: %s\n\n", arguments[arg_i].c_str());
+      printUsageError();
+      return {};
+    }
+  }
+
+  if (const char* env_p = std::getenv("THRIFT_INCLUDE_PATH")) {
+    std::vector<std::string> components;
+    boost::algorithm::split(components, env_p, isPathSeparator);
+    pparams.incl_searchpath.insert(
+        pparams.incl_searchpath.end(), components.begin(), components.end());
+  }
+
+  if (!validate_params(gparams)) {
+    return {};
+  }
+
+  // Return the input file name.
+  assert(arg_i == arguments.size() - 1);
+  if (detail::platform_is_windows()) {
+    // The path here is used in schema.thrift
+    //
+    // * In Linux, buck uses relative path.
+    // * In Windows, buck uses absolute path.
+    //
+    // To make the generated schema.thrift consistent with Unix, we need to
+    // convert it to relative path when possible.
+    //
+    // Not that the path might not exist since when thrift compiler is used as
+    // library, user's code can use `source_manager::add_virtual_file(...)`
+    // method to create a virtual file.
+    return std::filesystem::proximate(arguments[arg_i]).generic_string();
+  }
+  return arguments[arg_i];
+}
+
+std::optional<std::string> detail::parse_command_line_args(
+    const std::vector<std::string>& args,
+    parsing_params& parsing_params,
+    sema_params& sema_params) {
+  gen_params gen_params = {};
+  gen_params.targets.emplace_back(); // Avoid the need to pass --gen.
+  diagnostic_params diag_params = {};
+  auto filename =
+      parse_args(args, parsing_params, gen_params, diag_params, sema_params);
+  return filename.empty() ? std::nullopt
+                          : std::make_optional(std::move(filename));
+}
+
 std::pair<std::unique_ptr<t_program_bundle>, diagnostic_results>
 parse_and_mutate_program(
     source_manager& sm,
@@ -828,40 +832,14 @@ std::unique_ptr<t_program_bundle> parse_and_dump_diagnostics(
   return programs;
 }
 
-std::unique_ptr<t_program_bundle> parse_and_get_program(
-    source_manager& sm, const std::vector<std::string>& arguments) {
-  // Parse arguments.
-  parsing_params pparams;
-  pparams.allow_missing_includes = true;
-  gen_params gparams;
-  diagnostic_params dparams;
-  sema_params sparams;
-  gparams.targets.push_back(""); // Avoid needing to pass --gen
-  std::string filename =
-      parse_args(arguments, pparams, gparams, dparams, sparams);
-  if (filename.empty()) {
-    return {};
-  }
-
-  diagnostics_engine diags(
-      sm,
-      [](const diagnostic& d) { fmt::print(stderr, "{}\n", d); },
-      diagnostic_params::only_errors());
-  sparams.skip_lowering_type_annotations = true;
-  return parse_ast(sm, diags, filename, std::move(pparams), &sparams);
-}
-
-compile_result compile(
-    const std::vector<std::string>& arguments, source_manager& source_mgr) {
+compile_result compile_with_options(
+    parsing_params pparams,
+    gen_params gparams,
+    diagnostic_params dparams,
+    sema_params sparams,
+    const std::string& input_filename,
+    source_manager& source_mgr) {
   compile_result result;
-
-  // Parse arguments.
-  parsing_params pparams;
-  gen_params gparams;
-  diagnostic_params dparams;
-  sema_params sparams;
-  std::string input_filename =
-      parse_args(arguments, pparams, gparams, dparams, sparams);
   if (input_filename.empty()) {
     return result;
   }
@@ -884,4 +862,27 @@ compile_result compile(
   return result;
 }
 
+compile_result compile(
+    const std::vector<std::string>& arguments, source_manager& source_mgr) {
+  compile_result result;
+
+  // If --help is explicitly passed, print (non-error) usage and return
+  // successfully.
+  if (maybeHandleHelpInvocation(arguments)) {
+    result.retcode = compile_retcode::success;
+    return result;
+  }
+
+  // Parse arguments.
+  parsing_params pparams;
+  gen_params gparams;
+  diagnostic_params dparams;
+  sema_params sparams;
+  std::string input_filename =
+      parse_args(arguments, pparams, gparams, dparams, sparams);
+
+  return compile_with_options(
+      pparams, gparams, dparams, sparams, input_filename, source_mgr);
+}
+
 } // namespace apache::thrift::compiler
