@@ -137,16 +137,16 @@ const std::string& cpp_name_resolver::get_return_type(const t_function& fun) {
   }
 
   const t_stream* stream = fun.stream();
-  return detail::get_or_gen(type_cache_, stream, [&]() {
+  return detail::get_or_gen(stream_cache_, stream, [&]() {
     if (fun.has_return_type()) {
       return detail::gen_template_type(
           "::apache::thrift::ResponseAndServerStream",
           {resolve(resolve_fn, *fun.return_type().get_type()),
-           resolve(resolve_fn, *stream->get_elem_type())});
+           resolve(resolve_fn, *stream->elem_type().get_type())});
     }
     return detail::gen_template_type(
         "::apache::thrift::ServerStream",
-        {resolve(resolve_fn, *stream->get_elem_type())});
+        {resolve(resolve_fn, *stream->elem_type().get_type())});
   });
 }
 
@@ -246,7 +246,7 @@ const std::string& cpp_name_resolver::get_storage_type(
     // The storage type is just the type name.
     return native_type;
   }
-  return detail::get_or_gen(storage_type_cache_, {&field, ref_type}, [&]() {
+  return detail::get_or_gen(storage_type_cache_, &field, [&]() {
     return gen_storage_type(native_type, ref_type, field);
   });
 }
@@ -416,7 +416,7 @@ std::string cpp_name_resolver::gen_storage_type(
     cpp_reference_type& ref_type,
     const t_field& field) {
   const std::string* cpp_template =
-      field.find_annotation_or_null("cpp.template");
+      field.find_unstructured_annotation_or_null("cpp.template");
   switch (ref_type) {
     case cpp_reference_type::unique:
       if (cpp_template != nullptr) {
@@ -526,7 +526,7 @@ std::string cpp_name_resolver::gen_type_tag(const t_type& type) {
       ? gen_type_tag(*static_cast<const t_typedef&>(type).get_type())
       : gen_thrift_type_tag(type);
 
-  if (type.find_annotation_or_null("cpp.indirection")) {
+  if (type.find_unstructured_annotation_or_null("cpp.indirection")) {
     return fmt::format(
         "::apache::thrift::type::indirected<{}, {}>",
         get_native_type(type),
@@ -604,7 +604,7 @@ std::string cpp_name_resolver::gen_thrift_type_tag(
     return ns + "map<" + key_tag + ", " + val_tag + ">";
   } else if (type.is_union()) {
     return ns + "union_t<" + get_standard_type(type) + ">";
-  } else if (type.is_struct()) {
+  } else if (type.is_struct_or_union()) {
     return ns + "struct_t<" + get_standard_type(type) + ">";
   } else if (type.is_exception()) {
     return ns + "exception_t<" + get_standard_type(type) + ">";
