@@ -61,10 +61,12 @@ std::string mstch_base::get_option(const std::string& option) const {
 }
 
 void mstch_base::register_has_option(std::string key, std::string option) {
-  register_cached_method(
-      std::move(key), [this, option = std::move(option)]() -> mstch::node {
-        return has_option(option);
-      });
+  register_method(
+      std::move(key),
+      std::function<mstch::node()>(
+          [this, option = std::move(option)]() -> mstch::node {
+            return has_option(option);
+          }));
 }
 
 mstch::node mstch_base::is_struct() {
@@ -90,12 +92,15 @@ mstch::node mstch_enum::values() {
   return make_mstch_enum_values(enum_->get_enum_values());
 }
 
+// TODO(T219861020): Rename to get_structured
 mstch::node mstch_type::get_struct() {
-  if (type_->is_struct() || type_->is_exception()) {
+  if (type_->is_struct_or_union() || type_->is_exception() ||
+      type_->is_union()) {
     std::string id =
         type_->program()->name() + get_type_namespace(type_->program());
     return make_mstch_array_cached(
-        std::vector<const t_struct*>{dynamic_cast<const t_struct*>(type_)},
+        std::vector<const t_structured*>{
+            dynamic_cast<const t_structured*>(type_)},
         *context_.struct_factory,
         context_.struct_cache,
         id);
@@ -285,7 +290,7 @@ mstch::node mstch_const_value::is_const_struct() {
     return false;
   }
   const auto* type = const_value_->ttype()->get_true_type();
-  return type->is_struct() || type->is_exception();
+  return type->is_struct_or_union() || type->is_exception();
 }
 
 mstch::node mstch_const_value::const_struct_type() {
@@ -294,7 +299,7 @@ mstch::node mstch_const_value::const_struct_type() {
   }
 
   const auto* type = const_value_->ttype()->get_true_type();
-  if (type->is_struct() || type->is_exception()) {
+  if (type->is_struct_or_union() || type->is_exception()) {
     return context_.type_factory->make_mstch_object(type, context_);
   }
 
@@ -307,8 +312,8 @@ mstch::node mstch_const_value::const_struct() {
   mstch::array a;
 
   const auto* type = const_value_->ttype()->get_true_type();
-  if (type->is_struct() || type->is_exception()) {
-    const auto* strct = dynamic_cast<const t_struct*>(type);
+  if (type->is_struct_or_union() || type->is_exception()) {
+    const auto* strct = dynamic_cast<const t_structured*>(type);
     for (auto member : const_value_->get_map()) {
       const auto* field = strct->get_field_by_name(member.first->get_string());
       assert(field != nullptr);
@@ -416,8 +421,6 @@ mstch::node mstch_field::idl_type() {
       case t_type::type::t_structured:
         return BaseType::Struct;
       case t_type::type::t_service:
-      case t_type::type::t_stream:
-      case t_type::type::t_program:
         return std::nullopt;
     }
     // unneccessary, but prevents a GCC warning
@@ -515,9 +518,7 @@ mstch::node mstch_function::return_type() {
   if (function_->is_interaction_constructor()) {
     // The old syntax (performs) treats an interaction as a response.
     type = function_->interaction().get_type();
-  } else if (const t_stream* stream = function_->stream()) {
-    type = stream;
-  } else if (function_->sink()) {
+  } else if (function_->sink_or_stream()) {
     type = &t_primitive_type::t_void();
   }
   return context_.type_factory->make_mstch_object(type, context_, pos_);
@@ -569,7 +570,7 @@ mstch::node mstch_function::sink_final_response_exceptions() {
 mstch::node mstch_function::stream_elem_type() {
   const t_stream* stream = function_->stream();
   return stream ? context_.type_factory->make_mstch_object(
-                      stream->get_elem_type(), context_, pos_)
+                      stream->elem_type().get_type(), context_, pos_)
                 : mstch::node();
 }
 
