@@ -20,16 +20,15 @@
 #include <memory>
 #include <set>
 #include <string>
-#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
 #include <thrift/compiler/ast/t_program.h>
 #include <thrift/compiler/ast/uri.h>
-#include <thrift/compiler/detail/mustache/mstch.h>
 #include <thrift/compiler/generate/cpp/util.h>
 #include <thrift/compiler/sema/sema_context.h>
+#include <thrift/compiler/whisker/mstch_compat.h>
 
 namespace apache::thrift::compiler {
 
@@ -116,6 +115,7 @@ using mstch_const_map_element_factory = mstch_factory<
     const std::pair<const t_type*, const t_type*>&>;
 using mstch_structured_annotation_factory = mstch_factory<t_const>;
 using mstch_deprecated_annotation_factory = mstch_factory<t_annotation>;
+using mstch_stream_factory = mstch_factory<t_stream>;
 
 namespace detail {
 // Structured annotations don't have a separate AST node type yet so use a
@@ -227,7 +227,8 @@ class mstch_factories {
 
 // Mstch object construction context.
 struct mstch_context : mstch_factories {
-  std::map<std::string, std::string> options;
+  using compiler_options_map = std::map<std::string, std::string, std::less<>>;
+  compiler_options_map options;
 
   std::unordered_map<std::string, std::shared_ptr<mstch_base>> enum_cache;
   std::unordered_map<std::string, std::shared_ptr<mstch_base>> struct_cache;
@@ -236,6 +237,8 @@ struct mstch_context : mstch_factories {
 
   node_metadata_cache metadata_cache;
 
+  const whisker::prototype_database* prototypes = nullptr;
+
   /**
    * Sets or erases the option with the given `key` depending on the
    * `condition`.
@@ -289,7 +292,7 @@ class mstch_base : public mstch::object {
  public:
   mstch_base(mstch_context& ctx, mstch_element_position pos)
       : context_(ctx), pos_(pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"first?", &mstch_base::first},
@@ -304,7 +307,7 @@ class mstch_base : public mstch::object {
   mstch::node is_struct();
 
   mstch::node annotations(const t_named* annotated) {
-    return make_mstch_annotations(annotated->annotations());
+    return make_mstch_annotations(annotated->unstructured_annotations());
   }
 
   mstch::node structured_annotations(const t_named* annotated) {
@@ -313,6 +316,12 @@ class mstch_base : public mstch::object {
         *context_.structured_annotation_factory);
   }
 
+  template <typename T>
+  whisker::object make_self(const T& node) {
+    return whisker::object(whisker::native_handle<T>(
+        whisker::manage_as_static(node), context_.prototypes->of<T>()));
+  }
+
   template <typename Container, typename Factory, typename... Args>
   mstch::array make_mstch_array(
       const Container& container, const Factory& factory, const Args&... args) {
@@ -431,9 +440,10 @@ class mstch_program : public mstch_base {
   mstch_program(
       const t_program* p, mstch_context& ctx, mstch_element_position pos)
       : mstch_base(ctx, pos), program_(p) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"program:self", &mstch_program::self},
             {"program:name", &mstch_program::name},
             {"program:autogen_path", &mstch_program::autogen_path},
             {"program:includePrefix", &mstch_program::include_prefix},
@@ -459,6 +469,7 @@ class mstch_program : public mstch_base {
 
   virtual std::string get_program_namespace(const t_program*) { return {}; }
 
+  whisker::object self() { return make_self(*program_); }
   mstch::node name() { return program_->name(); }
   mstch::node autogen_path() {
     std::string path = program_->path();
@@ -480,7 +491,7 @@ class mstch_program : public mstch_base {
   mstch::node has_unions() {
     auto& structs = program_->structs_and_unions();
     return std::any_of(
-        structs.cbegin(), structs.cend(), std::mem_fn(&t_struct::is_union));
+        structs.cbegin(), structs.cend(), std::mem_fn(&t_structured::is_union));
   }
 
   mstch::node has_thrift_uris();
@@ -509,9 +520,10 @@ class mstch_service : public mstch_base {
         containing_service_(containing_service) {
     assert(containing_service_ == nullptr || service_->is_interaction());
 
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"service:self", &mstch_service::self},
             {"service:name", &mstch_service::name},
             {"service:functions", &mstch_service::functions},
             {"service:functions?", &mstch_service::has_functions},
@@ -533,7 +545,7 @@ class mstch_service : public mstch_base {
         });
 
     if (service_->is_interaction()) {
-      register_cached_methods(
+      register_methods(
           this,
           {
               {"interaction:serial?", &mstch_service::is_serial_interaction},
@@ -556,6 +568,7 @@ class mstch_service : public mstch_base {
 
   virtual std::string get_service_namespace(const t_program*) { return {}; }
 
+  whisker::object self() { return make_self(*service_); }
   mstch::node name() { return service_->get_name(); }
   mstch::node has_functions() { return !get_functions().empty(); }
   mstch::node has_extends() { return service_->get_extends() != nullptr; }
@@ -595,8 +608,8 @@ class mstch_service : public mstch_base {
     return service_->is_serial_interaction();
   }
   mstch::node is_event_base_interaction() {
-    return service_->has_annotation("process_in_event_base") ||
-        service_->find_structured_annotation_or_null(kCppProcessInEbThreadUri);
+    return service_->has_unstructured_annotation("process_in_event_base") ||
+        service_->has_structured_annotation(kCppProcessInEbThreadUri);
   }
   mstch::node definition_key();
 
@@ -629,9 +642,10 @@ class mstch_function : public mstch_base {
       mstch_element_position pos,
       const t_interface* iface)
       : mstch_base(ctx, pos), function_(f), interface_(iface) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"function:self", &mstch_function::self},
             {"function:name", &mstch_function::name},
             {"function:oneway?", &mstch_function::oneway},
             {"function:return_type", &mstch_function::return_type},
@@ -681,9 +695,13 @@ class mstch_function : public mstch_base {
             {"function:stream_exceptions?",
              &mstch_function::has_stream_exceptions},
             {"function:stream_exceptions", &mstch_function::stream_exceptions},
+
+            // Shared Sink/Stream methods:
+            {"function:sink_or_stream?", &mstch_function::has_sink_or_stream},
         });
   }
 
+  whisker::object self() { return make_self(*function_); }
   mstch::node name() { return function_->get_name(); }
   mstch::node oneway() {
     return function_->qualifier() == t_function_qualifier::oneway;
@@ -702,7 +720,7 @@ class mstch_function : public mstch_base {
           .get_enum_value()
           ->name();
     }
-    return function_->get_annotation("priority", "NORMAL");
+    return function_->get_unstructured_annotation("priority", "NORMAL");
   }
   mstch::node annotations() { return mstch_base::annotations(function_); }
 
@@ -779,6 +797,10 @@ class mstch_function : public mstch_base {
   }
   mstch::node stream_exceptions();
 
+  mstch::node has_sink_or_stream() {
+    return function_->sink_or_stream() != nullptr;
+  }
+
  protected:
   const t_function* function_;
   const t_interface* interface_;
@@ -801,9 +823,10 @@ class mstch_type : public mstch_base {
 
   mstch_type(const t_type* t, mstch_context& ctx, mstch_element_position pos)
       : mstch_base(ctx, pos), type_(t), resolved_type_(t->get_true_type()) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"type:self", &mstch_type::self},
             {"type:name", &mstch_type::name},
             {"type:void?", &mstch_type::is_void},
             {"type:string?", &mstch_type::is_string},
@@ -820,7 +843,6 @@ class mstch_type : public mstch_base {
             {"type:struct?", &mstch_type::is_struct},
             {"type:union?", &mstch_type::is_union},
             {"type:enum?", &mstch_type::is_enum},
-            {"type:service?", &mstch_type::is_service},
             {"type:base?", &mstch_type::is_base},
             {"type:container?", &mstch_type::is_container},
             {"type:list?", &mstch_type::is_list},
@@ -835,9 +857,10 @@ class mstch_type : public mstch_base {
             {"type:value_type", &mstch_type::get_value_type},
             {"type:typedef_type", &mstch_type::get_typedef_type},
             {"type:typedef", &mstch_type::get_typedef},
-            {"type:interaction?", &mstch_type::is_interaction},
         });
   }
+
+  whisker::object self() { return make_self(*type_); }
   mstch::node name() { return type_->get_name(); }
   mstch::node is_void() { return resolved_type_->is_void(); }
   mstch::node is_string() { return resolved_type_->is_string(); }
@@ -859,12 +882,14 @@ class mstch_type : public mstch_base {
   mstch::node is_floating_point() {
     return resolved_type_->is_floating_point();
   }
+  // TODO(T219861020): Evaluate if unions should be here and rename method as
+  // neccessary.
   mstch::node is_struct() {
-    return resolved_type_->is_struct() || resolved_type_->is_exception();
+    return resolved_type_->is_struct_or_union() ||
+        resolved_type_->is_exception();
   }
   mstch::node is_union() { return resolved_type_->is_union(); }
   mstch::node is_enum() { return resolved_type_->is_enum(); }
-  mstch::node is_service() { return resolved_type_->is_service(); }
   mstch::node is_base() { return resolved_type_->is_primitive_type(); }
   mstch::node is_container() { return resolved_type_->is_container(); }
   mstch::node is_list() { return resolved_type_->is_list(); }
@@ -880,7 +905,6 @@ class mstch_type : public mstch_base {
   mstch::node get_value_type();
   mstch::node get_typedef_type();
   mstch::node get_typedef();
-  mstch::node is_interaction() { return type_->is_service(); }
 
  protected:
   const t_type* type_;
@@ -894,11 +918,11 @@ class mstch_typedef : public mstch_base {
   mstch_typedef(
       const t_typedef* t, mstch_context& ctx, mstch_element_position pos)
       : mstch_base(ctx, pos), typedef_(t) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"typedef:self", &mstch_typedef::self},
             {"typedef:type", &mstch_typedef::type},
-            {"typedef:is_same_type", &mstch_typedef::is_same_type},
             {"typedef:name", &mstch_typedef::name},
             {"typedef:structured_annotations?",
              &mstch_typedef::has_structured_annotations},
@@ -906,11 +930,10 @@ class mstch_typedef : public mstch_base {
              &mstch_typedef::structured_annotations},
         });
   }
+
+  whisker::object self() { return make_self(*typedef_); }
   mstch::node type();
   mstch::node name() { return typedef_->name(); }
-  mstch::node is_same_type() {
-    return typedef_->get_name() == typedef_->get_type()->get_name();
-  }
   mstch::node has_structured_annotations() {
     return !typedef_->structured_annotations().empty();
   }
@@ -929,9 +952,10 @@ class mstch_struct : public mstch_base {
   mstch_struct(
       const t_structured* s, mstch_context& ctx, mstch_element_position pos)
       : mstch_base(ctx, pos), struct_(s) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"struct:self", &mstch_struct::self},
             {"struct:name", &mstch_struct::name},
             {"struct:fields?", &mstch_struct::has_fields},
             {"struct:fields", &mstch_struct::fields},
@@ -970,6 +994,8 @@ class mstch_struct : public mstch_base {
       prev = curr;
     }
   }
+
+  whisker::object self() { return make_self(*struct_); }
   mstch::node name() { return struct_->get_name(); }
   mstch::node has_fields() { return struct_->has_fields(); }
   mstch::node fields();
@@ -1007,8 +1033,7 @@ class mstch_struct : public mstch_base {
   const std::vector<const t_field*>& get_members_in_key_order();
 
   field_range get_members_in_serialization_order() {
-    if (struct_->find_structured_annotation_or_null(
-            kSerializeInFieldIdOrderUri)) {
+    if (struct_->has_structured_annotation(kSerializeInFieldIdOrderUri)) {
       return get_members_in_key_order();
     }
 
@@ -1043,9 +1068,10 @@ class mstch_field : public mstch_base {
       mstch_element_position pos,
       const field_generator_context* field_context)
       : mstch_base(ctx, pos), field_(f), field_context_(field_context) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"field:self", &mstch_field::self},
             {"field:name", &mstch_field::name},
             {"field:key", &mstch_field::key},
             {"field:value", &mstch_field::value},
@@ -1068,6 +1094,8 @@ class mstch_field : public mstch_base {
              &mstch_field::is_coding_error_action_report},
         });
   }
+
+  whisker::object self() { return make_self(*field_); }
   mstch::node name() { return field_->get_name(); }
   mstch::node key() { return field_->get_key(); }
   mstch::node value();
@@ -1115,7 +1143,7 @@ class mstch_field : public mstch_base {
         CodingErrorAction::Report);
   }
   bool has_compat_annotation(const char* uri) {
-    if (field_->find_structured_annotation_or_null(uri) != nullptr) {
+    if (field_->has_structured_annotation(uri)) {
       return true;
     }
     auto type = field_->get_type();
@@ -1126,12 +1154,10 @@ class mstch_field : public mstch_base {
       }
     }
     if (field_context_ != nullptr && field_context_->strct != nullptr) {
-      if (field_context_->strct->find_structured_annotation_or_null(uri) !=
-          nullptr) {
+      if (field_context_->strct->has_structured_annotation(uri)) {
         return true;
       }
-      if (field_context_->strct->program()->find_structured_annotation_or_null(
-              uri) != nullptr) {
+      if (field_context_->strct->program()->has_structured_annotation(uri)) {
         return true;
       }
     }
@@ -1175,11 +1201,7 @@ class mstch_field : public mstch_base {
         return item.second->get_integer() == action;
       }
     }
-    if (action == def && annotation->value()->get_map().size() == 0) {
-      return true;
-    }
-
-    return false;
+    return action == def && annotation->value()->get_map().empty();
   }
 
  protected:
@@ -1197,9 +1219,10 @@ class mstch_enum : public mstch_base {
 
   mstch_enum(const t_enum* e, mstch_context& ctx, mstch_element_position pos)
       : mstch_base(ctx, pos), enum_(e) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"enum:self", &mstch_enum::self},
             {"enum:name", &mstch_enum::name},
             {"enum:values", &mstch_enum::values},
             {"enum:structured_annotations?",
@@ -1214,6 +1237,7 @@ class mstch_enum : public mstch_base {
         });
   }
 
+  whisker::object self() { return make_self(*enum_); }
   mstch::node name() { return enum_->get_name(); }
   mstch::node values();
   mstch::node unused_value() { return enum_->unused(); }
@@ -1225,10 +1249,10 @@ class mstch_enum : public mstch_base {
   }
   mstch::node is_enums_compat() { return has_compat_annotation(kEnumsUri); }
   bool has_compat_annotation(const char* uri) {
-    if (enum_->find_structured_annotation_or_null(uri) != nullptr) {
+    if (enum_->has_structured_annotation(uri)) {
       return true;
     }
-    if (enum_->program()->find_structured_annotation_or_null(uri) != nullptr) {
+    if (enum_->program()->has_structured_annotation(uri)) {
       return true;
     }
 
@@ -1263,11 +1287,7 @@ class mstch_enum : public mstch_base {
         return item.second->get_integer() == action;
       }
     }
-    if (action == def && annotation->value()->get_map().size() == 0) {
-      return true;
-    }
-
-    return false;
+    return action == def && annotation->value()->get_map().empty();
   }
   mstch::node thrift_uri() { return enum_->uri(); }
 
@@ -1282,13 +1302,15 @@ class mstch_enum_value : public mstch_base {
   mstch_enum_value(
       const t_enum_value* ev, mstch_context& ctx, mstch_element_position pos)
       : mstch_base(ctx, pos), enum_value_(ev) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"enum_value:self", &mstch_enum_value::self},
             {"enum_value:name", &mstch_enum_value::name},
             {"enum_value:value", &mstch_enum_value::value},
         });
   }
+  whisker::object self() { return make_self(*enum_value_); }
   mstch::node name() { return enum_value_->get_name(); }
   mstch::node value() { return enum_value_->get_value(); }
 
@@ -1312,9 +1334,10 @@ class mstch_const : public mstch_base {
         current_const_(current_const),
         expected_type_(expected_type),
         field_(field) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"constant:self", &mstch_const::self},
             {"constant:name", &mstch_const::name},
             {"constant:index", &mstch_const::index},
             {"constant:type", &mstch_const::type},
@@ -1323,6 +1346,8 @@ class mstch_const : public mstch_base {
             {"constant:field", &mstch_const::field},
         });
   }
+
+  whisker::object self() { return make_self(*const_); }
   mstch::node name() { return const_->get_name(); }
   mstch::node index() { return pos_.index; }
   mstch::node type();
@@ -1353,9 +1378,10 @@ class mstch_const_value : public mstch_base {
         current_const_(current_const),
         expected_type_(expected_type),
         type_(cv->kind()) {
-    register_cached_methods(
+    register_methods(
         this,
         {
+            {"value:self", &mstch_const_value::self},
             {"value:bool?", &mstch_const_value::is_bool},
             {"value:double?", &mstch_const_value::is_double},
             {"value:integer?", &mstch_const_value::is_integer},
@@ -1387,6 +1413,7 @@ class mstch_const_value : public mstch_base {
         });
   }
 
+  whisker::object self() { return make_self(*const_value_); }
   mstch::node is_bool() { return type_ == cv::CV_BOOL; }
   mstch::node is_double() { return type_ == cv::CV_DOUBLE; }
   mstch::node is_integer() {
@@ -1457,7 +1484,7 @@ class mstch_const_map_element : public mstch_base {
         element_(*e),
         current_const_(current_const),
         expected_types_(expected_types) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"element:key", &mstch_const_map_element::element_key},
@@ -1480,7 +1507,7 @@ class mstch_structured_annotation : public mstch_base {
   mstch_structured_annotation(
       const t_const* c, mstch_context& ctx, mstch_element_position pos)
       : mstch_base(ctx, pos), const_(*c) {
-    register_cached_methods(
+    register_methods(
         this,
         {{"structured_annotation:const",
           &mstch_structured_annotation::constant},
@@ -1498,7 +1525,7 @@ class mstch_structured_annotation : public mstch_base {
   }
 
   mstch::node is_const_struct() {
-    return const_.type()->get_true_type()->is_struct();
+    return const_.type()->get_true_type()->is_struct_or_union();
   }
 
  protected:
@@ -1512,7 +1539,7 @@ class mstch_deprecated_annotation : public mstch_base {
   mstch_deprecated_annotation(
       const t_annotation* a, mstch_context& ctx, mstch_element_position pos)
       : mstch_base(ctx, pos), key_(a->first), val_(a->second) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"annotation:key", &mstch_deprecated_annotation::key},
