@@ -232,13 +232,15 @@ type::Schema t_ast_generator::gen_schema(
     }
   };
 
-  if (!root_program.scope()->find_by_uri("facebook.com/thrift/type/TypeUri")) {
+  if (!root_program.global_scope()->find_by_uri(
+          "facebook.com/thrift/type/TypeUri")) {
     throw std::runtime_error(
         "thrift/lib/thrift/schema.thrift must be present in one of the include paths.");
   }
 
   schema_opts.intern_value = intern_value;
-  schematizer schema_source(*root_program.scope(), source_mgr, schema_opts);
+  schematizer schema_source(
+      *root_program.global_scope(), source_mgr, schema_opts);
   const_ast_visitor visitor;
   bool is_root_program = true;
   visitor.add_program_visitor([&](const t_program& program) {
@@ -391,7 +393,7 @@ type::Schema t_ast_generator::gen_schema(
               ident.uri()->uri_ref() = uri;
             } else {
               ident.uri()->scopedName_ref() =
-                  enum_owner->program()->scope_name(*enum_owner);
+                  enum_owner->program()->scoped_name(*enum_owner);
             }
             ident.enumValue() = val->get_owner()->get_name();
             ast.identifierSourceRanges()->push_back(std::move(ident));
@@ -400,7 +402,7 @@ type::Schema t_ast_generator::gen_schema(
               ident.uri()->uri_ref() = uri;
             } else {
               ident.uri()->scopedName_ref() =
-                  owner->program()->scope_name(*owner);
+                  owner->program()->scoped_name(*owner);
             }
             ast.identifierSourceRanges()->push_back(std::move(ident));
           } else {
@@ -522,6 +524,7 @@ THRIFT_REGISTER_GENERATOR(
     "    source_ranges:     Enables population of the identifier source range map.\n"
     "    no_backcompat:     Disables double writes (breaking changes possible!).\n"
     "    use_hash:          Uses definitionKey in typeUri and instead of extern ids.\n"
+    "                       (Required for use with the SyntaxGraph API)\n"
     "    root_program_only: Only schematize the root program.\n"
     "");
 
@@ -534,7 +537,7 @@ std::string gen_schema(
       t_ast_generator::gen_schema(schema_opts, source_mgr, root_program);
   auto serialized = serialize<CompactProtocolWriter>(schema);
   auto compressed = folly::compression::getCodec(
-                        folly::io::CodecType::ZSTD,
+                        folly::compression::CodecType::ZSTD,
                         folly::compression::COMPRESSION_LEVEL_BEST)
                         ->compress(serialized);
   return compressed;
