@@ -355,7 +355,7 @@ void t_js_generator::generate_enum(const t_enum* tenum) {
  */
 void t_js_generator::generate_const(const t_const* tconst) {
   const t_type* type = tconst->type();
-  string name = tconst->get_name();
+  const string& name = tconst->get_name();
   const t_const_value* value = tconst->value();
 
   f_types_ << js_type_namespace(program_) << name << " = ";
@@ -404,7 +404,7 @@ string t_js_generator::render_const_value(
     }
   } else if (type->is_enum()) {
     out << value->get_integer();
-  } else if (type->is_struct() || type->is_exception()) {
+  } else if (type->is_struct_or_union() || type->is_exception()) {
     out << "new " << js_type_namespace(type->program()) << type->get_name()
         << "({" << endl;
     indent_up();
@@ -537,7 +537,7 @@ void t_js_generator::generate_js_struct_definition(
     string dval = declare_field(*m_iter, false, true);
     const t_type* t = (*m_iter)->get_type()->get_true_type();
     if ((*m_iter)->get_value() != nullptr &&
-        !(t->is_struct() || t->is_exception())) {
+        !(t->is_struct_or_union() || t->is_exception())) {
       dval = render_const_value((*m_iter)->get_type(), (*m_iter)->get_value());
       out << indent() << "this." << (*m_iter)->get_name() << " = " << dval
           << ";" << endl;
@@ -551,7 +551,7 @@ void t_js_generator::generate_js_struct_definition(
     for (m_iter = members.begin(); m_iter != members.end(); ++m_iter) {
       const t_type* t = (*m_iter)->get_type()->get_true_type();
       if ((*m_iter)->get_value() != nullptr &&
-          (t->is_struct() || t->is_exception())) {
+          (t->is_struct_or_union() || t->is_exception())) {
         indent(out) << "this." << (*m_iter)->get_name() << " = "
                     << render_const_value(t, (*m_iter)->get_value()) << ";"
                     << endl;
@@ -1236,7 +1236,7 @@ void t_js_generator::generate_deserialize_field(
 
   string name = prefix + tfield->get_name();
 
-  if (type->is_struct() || type->is_exception()) {
+  if (type->is_struct_or_union() || type->is_exception()) {
     generate_deserialize_struct(out, (t_struct*)type, name);
   } else if (type->is_container()) {
     generate_deserialize_container(out, type, name);
@@ -1449,7 +1449,7 @@ void t_js_generator::generate_serialize_field(
         tfield->get_name());
   }
 
-  if (type->is_struct() || type->is_exception()) {
+  if (type->is_struct_or_union() || type->is_exception()) {
     generate_serialize_struct(
         out, (t_struct*)type, prefix + tfield->get_name());
   } else if (type->is_container()) {
@@ -1666,7 +1666,7 @@ string t_js_generator::declare_field(
       result += " = null";
     } else if (type->is_container()) {
       result += " = null";
-    } else if (type->is_struct() || type->is_exception()) {
+    } else if (type->is_struct_or_union() || type->is_exception()) {
       if (obj) {
         result += " = new " + js_type_namespace(type->program()) +
             type->get_name() + "()";
@@ -1759,7 +1759,7 @@ string t_js_generator ::type_to_enum(const t_type* type) {
     }
   } else if (type->is_enum()) {
     return "Thrift.Type.I32";
-  } else if (type->is_struct() || type->is_exception()) {
+  } else if (type->is_struct_or_union() || type->is_exception()) {
     return "Thrift.Type.STRUCT";
   } else if (type->is_map()) {
     return "Thrift.Type.MAP";
