@@ -16,10 +16,6 @@
 
 #include <thrift/compiler/generate/t_mstch_generator.h>
 
-#include <algorithm>
-#include <cassert>
-#include <cstdio>
-#include <fstream>
 #include <memory>
 #include <optional>
 #include <sstream>
@@ -29,17 +25,11 @@
 
 #include <fmt/format.h>
 
-#include <thrift/compiler/detail/mustache/mstch.h>
 #include <thrift/compiler/whisker/ast.h>
 #include <thrift/compiler/whisker/mstch_compat.h>
 #include <thrift/compiler/whisker/parser.h>
 #include <thrift/compiler/whisker/source_location.h>
-
-#include <thrift/compiler/detail/system.h>
-#include <thrift/compiler/generate/t_generator.h>
-#include <thrift/compiler/generate/templates.h>
-
-#include <boost/algorithm/string/split.hpp>
+#include <thrift/compiler/whisker/standard_library.h>
 
 using namespace std;
 
@@ -47,28 +37,7 @@ namespace fs = std::filesystem;
 
 namespace apache::thrift::compiler {
 
-namespace {
-
-fs::path from_components(
-    fs::path::const_iterator begin, fs::path::const_iterator end) {
-  fs::path tmp;
-  while (begin != end) {
-    tmp /= *begin++;
-  }
-  return tmp;
-}
-
-bool is_last_char(const string& data, char c) {
-  return !data.empty() && data.back() == c;
-}
-
-void chomp_last_char(string* data, char c) {
-  if (is_last_char(*data, c)) {
-    data->pop_back();
-  }
-}
-
-} // namespace
+namespace {} // namespace
 
 mstch::map t_mstch_generator::dump(const t_program& program) {
   mstch::map result{
@@ -96,7 +65,7 @@ mstch::map t_mstch_generator::dump(const t_structured& strct, bool shallow) {
       {"exception?", strct.is_exception()},
       {"union?", strct.is_union()},
       {"plain?", !strct.is_exception() && !strct.is_union()},
-      {"annotations", dump_elems(strct.annotations())},
+      {"annotations", dump_elems(strct.unstructured_annotations())},
   };
 
   mstch::map extension = extend_struct(strct);
@@ -116,7 +85,7 @@ mstch::map t_mstch_generator::dump(const t_field& field, int32_t index) {
       {"optional?", req == t_field::e_req::optional},
       {"opt_in_req_out?", req == t_field::e_req::opt_in_req_out},
       {"terse?", req == t_field::e_req::terse},
-      {"annotations", dump_elems(field.annotations())},
+      {"annotations", dump_elems(field.unstructured_annotations())},
   };
 
   if (field.get_value() != nullptr) {
@@ -134,7 +103,7 @@ mstch::map t_mstch_generator::dump(const t_type& orig_type) {
 
   mstch::map result{
       {"name", type.get_name()},
-      {"annotations", dump_elems(type.annotations())},
+      {"annotations", dump_elems(type.unstructured_annotations())},
 
       {"void?", type.is_void()},
       {"string?", type.is_string()},
@@ -147,10 +116,11 @@ mstch::map t_mstch_generator::dump(const t_type& orig_type) {
       {"double?", type.is_double()},
       {"float?", type.is_float()},
       {"floating_point?", type.is_floating_point()},
-      {"struct?", type.is_struct() || type.is_exception()},
+      // TODO(T219861020): Evaluate if unions should be included and rename as
+      // needed.
+      {"struct?", type.is_struct_or_union() || type.is_exception()},
       {"union?", type.is_union()},
       {"enum?", type.is_enum()},
-      {"service?", type.is_service()},
       {"base?", type.is_primitive_type()},
       {"container?", type.is_container()},
       {"list?", type.is_list()},
@@ -159,13 +129,11 @@ mstch::map t_mstch_generator::dump(const t_type& orig_type) {
       {"typedef?", type.is_typedef()},
   };
 
-  if (type.is_struct() || type.is_exception()) {
+  if (type.is_struct_or_union() || type.is_exception()) {
     // Shallow dump the struct
     result.emplace("struct", dump(dynamic_cast<const t_struct&>(type), true));
   } else if (type.is_enum()) {
     result.emplace("enum", dump(dynamic_cast<const t_enum&>(type)));
-  } else if (type.is_service()) {
-    result.emplace("service", dump(dynamic_cast<const t_service&>(type)));
   } else if (type.is_list()) {
     result.emplace(
         "list_elem_type",
@@ -193,7 +161,7 @@ mstch::map t_mstch_generator::dump(const t_enum& enm) {
   mstch::map result{
       {"name", enm.get_name()},
       {"values", dump_elems(enm.get_enum_values())},
-      {"annotations", dump_elems(enm.annotations())},
+      {"annotations", dump_elems(enm.unstructured_annotations())},
   };
 
   mstch::map extension = extend_enum(enm);
@@ -216,7 +184,7 @@ mstch::map t_mstch_generator::dump(const t_service& service) {
   const t_service* extends = service.get_extends();
   mstch::map result{
       {"name", service.get_name()},
-      {"annotations", dump_elems(service.annotations())},
+      {"annotations", dump_elems(service.unstructured_annotations())},
       {"functions", dump_elems(service.get_functions())},
       {"functions?", !service.get_functions().empty()},
       {"extends?", extends != nullptr},
@@ -236,7 +204,7 @@ mstch::map t_mstch_generator::dump(const t_function& function) {
       {"return_type", dump(*function.return_type())},
       {"exceptions", dump_elems(exceptions)},
       {"exceptions?", !exceptions.empty()},
-      {"annotations", dump_elems(function.annotations())},
+      {"annotations", dump_elems(function.unstructured_annotations())},
       {"args", dump_elems(function.params().fields())},
   };
 
@@ -354,7 +322,7 @@ mstch::map t_mstch_generator::dump(const string& value) {
 
 mstch::map t_mstch_generator::dump_options() {
   mstch::map result;
-  for (auto& elem : options_) {
+  for (auto& elem : compiler_options()) {
     result.emplace(elem.first, elem.second);
   }
   return prepend_prefix("option", std::move(result));
@@ -415,65 +383,26 @@ mstch::map t_mstch_generator::extend_annotation(const annotation&) {
   return {};
 }
 
-void t_mstch_generator::gen_template_map(const std::filesystem::path& root) {
-  for (size_t i = 0; i < templates_size; ++i) {
-    auto name = std::filesystem::path(
-        templates_name_datas[i],
-        templates_name_datas[i] + templates_name_sizes[i]);
-    auto mm = std::mismatch(name.begin(), name.end(), root.begin(), root.end());
-    if (mm.second == root.end()) {
-      name = from_components(mm.first, name.end());
-      name = name.parent_path() / name.stem();
-
-      auto tpl = std::string(
-          templates_content_datas[i],
-          templates_content_datas[i] + templates_content_sizes[i]);
-      // Remove a single '\n' or '\r\n' or '\r' at end, if present.
-      chomp_last_char(&tpl, '\n');
-      chomp_last_char(&tpl, '\r');
-      template_map_.emplace(name.generic_string(), std::move(tpl));
-    }
-  }
-}
-
 const std::string& t_mstch_generator::get_template(
     const std::string& template_name) {
-  auto itr = template_map_.find(template_name);
-  if (itr == template_map_.end()) {
-    std::ostringstream err;
-    err << "Could not find template \"" << template_name << "\"";
-    throw std::runtime_error{err.str()};
+  const templates_map& templates = templates_by_path();
+  if (auto t = templates.find(template_name); t != templates.end()) {
+    return t->second;
   }
-  return itr->second;
-}
-
-void t_mstch_generator::write_output(
-    const std::filesystem::path& path, const std::string& data) {
-  auto base_path = std::filesystem::path{get_out_dir()};
-  auto abs_path = detail::make_abs_path(base_path, path);
-  std::filesystem::create_directories(abs_path.parent_path());
-  std::ofstream ofs{abs_path.string()};
-  if (!ofs) {
-    std::ostringstream err;
-    err << "Couldn't open \"" << abs_path.string() << "\" for writing.";
-    throw std::runtime_error{err.str()};
-  }
-  ofs << data;
-  if (!is_last_char(data, '\n')) {
-    // Terminate with newline.
-    ofs << '\n';
-  }
-  record_genfile(abs_path.string());
+  throw std::runtime_error(
+      fmt::format("Could not find template '{}'", template_name));
 }
 
 bool t_mstch_generator::has_option(const std::string& option) const {
-  return options_.find(option) != options_.end();
+  return has_compiler_option(option);
 }
 
 std::optional<std::string> t_mstch_generator::get_option(
     const std::string& option) const {
-  auto itr = options_.find(option);
-  return itr != options_.end() ? itr->second : std::optional<std::string>();
+  if (std::optional<std::string_view> found = get_compiler_option(option)) {
+    return std::string{*found};
+  }
+  return std::nullopt;
 }
 
 mstch::map t_mstch_generator::prepend_prefix(
@@ -485,126 +414,36 @@ mstch::map t_mstch_generator::prepend_prefix(
   return res;
 }
 
-t_mstch_generator::whisker_render_state
-t_mstch_generator::gen_whisker_render_state(whisker_options whisker_opts) {
-  /**
-   * This implementation of whisker's template resolver builds on top of the
-   * template_map that already used by mstch. This allows template names to
-   * remain the same (albeit exact file names are not recoverable).
-   *
-   * When a new partial is requested, we lazily parse the corresponding entry in
-   * the template_map and cache the parsed AST.
-   */
-  class whisker_template_parser : public whisker::template_resolver {
-   public:
-    explicit whisker_template_parser(
-        const std::map<std::string, std::string>& template_map)
-        : template_map_(template_map) {}
-
-    std::optional<whisker::ast::root> resolve(
-        const std::vector<std::string>& partial_path,
-        diagnostics_engine& diags) override {
-      // Whisker always breaks down the path into components. However, the
-      // template_map stores them as one concatenated string.
-      std::string template_name =
-          fmt::format("{}", fmt::join(partial_path, "/"));
-
-      if (auto cached = cached_asts_.find(template_name);
-          cached != cached_asts_.end()) {
-        return cached->second;
-      }
-
-      auto source_code = template_map_.find(template_name);
-      if (source_code == template_map_.end()) {
-        return std::nullopt;
-      }
-      auto src =
-          src_manager_.add_virtual_file(template_name, source_code->second);
-      auto ast = whisker::parse(src, diags);
-      auto [result, inserted] =
-          cached_asts_.insert({std::move(template_name), std::move(ast)});
-      assert(inserted);
-      return result->second;
-    }
-
-    whisker::source_manager& source_manager() { return src_manager_; }
-
-   private:
-    const std::map<std::string, std::string>& template_map_;
-    whisker::source_manager src_manager_;
-    std::unordered_map<std::string, std::optional<whisker::ast::root>>
-        cached_asts_;
-  };
-
-  auto template_parser =
-      std::make_shared<whisker_template_parser>(get_template_map());
-  whisker::render_options render_options;
-  render_options.partial_resolver = template_parser;
-  render_options.strict_boolean_conditional = whisker::diagnostic_level::debug;
-  render_options.strict_printable_types = whisker::diagnostic_level::debug;
-  render_options.strict_undefined_variables = whisker::diagnostic_level::error;
-
-  for (const auto& undefined_name : whisker_opts.allowed_undefined_variables) {
-    render_options.globals.insert({undefined_name, whisker::make::null});
+whisker::map::raw t_mstch_generator::globals() const {
+  auto options = render_options();
+  whisker::map::raw result = t_whisker_generator::globals();
+  for (const auto& undefined_name : options.allowed_undefined_variables) {
+    result.insert({undefined_name, whisker::make::null});
   }
+  return result;
+}
 
-  return whisker_render_state{
-      whisker::diagnostics_engine(
-          template_parser->source_manager(),
-          [](const diagnostic& d) { fmt::print(stderr, "{}\n", d); },
-          diagnostic_params::only_errors()),
-      template_parser,
-      std::move(render_options),
-  };
+t_mstch_generator::strictness_options t_mstch_generator::strictness() const {
+  strictness_options strict;
+  // Our legacy code has a ton of non-boolean conditionals
+  strict.boolean_conditional = false;
+  // Our legacy code relies on printing null as empty string
+  strict.printable_types = false;
+  // Undefined variables are covered by globals(...)
+  strict.undefined_variables = true;
+  return strict;
 }
 
 std::string t_mstch_generator::render(
     const std::string& template_name, const mstch::node& context) {
-  if (std::holds_alternative<std::monostate>(whisker_render_state_)) {
-    auto whisker_requested = use_whisker();
-    whisker_render_state_ = whisker_requested
-        ? gen_whisker_render_state(*whisker_requested)
-        : std::optional<whisker_render_state>();
-  }
-  auto& render_state =
-      std::get<std::optional<whisker_render_state>>(whisker_render_state_);
-
-  if (!render_state.has_value()) {
-    // The implementation chose not to use Whisker.
-    return mstch::render(
-        get_template(template_name), context, get_template_map());
-  }
-
-  std::vector<std::string> partial_path;
-  boost::algorithm::split(
-      partial_path, template_name, [](char c) { return c == '/'; });
-
-  std::optional<whisker::ast::root> ast =
-      render_state->template_resolver->resolve(
-          partial_path, render_state->diagnostic_engine);
-  if (!ast.has_value()) {
-    throw std::runtime_error{
-        fmt::format("Could not find template \"{}\"", template_name)};
-  }
-
-  std::ostringstream out;
-  if (!whisker::render(
-          out,
-          *ast,
-          whisker::from_mstch(context),
-          render_state->diagnostic_engine,
-          render_state->render_options)) {
-    throw std::runtime_error{
-        fmt::format("Failed to render template \"{}\"", template_name)};
-  }
-  return out.str();
+  return render(template_name, whisker::from_mstch(context));
 }
 
 void t_mstch_generator::render_to_file(
     const mstch::map& context,
     const std::string& template_name,
     const std::filesystem::path& path) {
-  write_output(path, render(template_name, context));
+  write_to_file(path, render(template_name, context));
 }
 
 const std::shared_ptr<mstch_base>& t_mstch_generator::cached_program(
