@@ -45,7 +45,7 @@ class t_mstch_go_generator : public t_mstch_generator {
  public:
   using t_mstch_generator::t_mstch_generator;
 
-  std::optional<whisker_options> use_whisker() const override {
+  whisker_options render_options() const override {
     whisker_options opts;
     opts.allowed_undefined_variables = {
         "service:autogen_path",
@@ -71,7 +71,7 @@ class mstch_go_program : public mstch_program {
       mstch_element_position pos,
       go::codegen_data* data)
       : mstch_program(p, ctx, pos), data_(*data) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"program:go_pkg_name", &mstch_go_program::go_pkg_name},
@@ -84,8 +84,6 @@ class mstch_go_program : public mstch_program {
             {"program:compat_setters?",
              &mstch_go_program::go_gen_compat_setters},
             {"program:thrift_imports", &mstch_go_program::thrift_imports},
-            {"program:has_thrift_imports",
-             &mstch_go_program::has_thrift_imports},
             {"program:thrift_lib_import", &mstch_go_program::thrift_lib_import},
             {"program:thrift_metadata_import",
              &mstch_go_program::thrift_metadata_import},
@@ -93,8 +91,6 @@ class mstch_go_program : public mstch_program {
             {"program:gen_metadata?", &mstch_go_program::should_gen_metadata},
             {"program:gen_default_get?",
              &mstch_go_program::should_gen_default_get},
-            {"program:use_reflect_codec?",
-             &mstch_go_program::should_use_reflect_codec},
             {"program:import_metadata_package?",
              &mstch_go_program::should_import_metadata_package},
             {"program:metadata_qualifier",
@@ -124,9 +120,6 @@ class mstch_go_program : public mstch_program {
     }
     return a;
   }
-  mstch::node has_thrift_imports() {
-    return !program_->get_includes_for_codegen().empty();
-  }
   mstch::node go_import_path() { return get_go_import_path_(); }
   mstch::node thrift_lib_import() { return data_.thrift_lib_import; }
   mstch::node thrift_metadata_import() { return data_.thrift_metadata_import; }
@@ -135,7 +128,6 @@ class mstch_go_program : public mstch_program {
   }
   mstch::node should_gen_metadata() { return data_.gen_metadata; }
   mstch::node should_gen_default_get() { return data_.gen_default_get; }
-  mstch::node should_use_reflect_codec() { return data_.use_reflect_codec; }
   mstch::node should_import_metadata_package() {
     // We don't need to import the metadata package if we are
     // generating metadata inside the metadata package itself. Duh.
@@ -174,7 +166,7 @@ class mstch_go_enum : public mstch_enum {
       mstch_element_position pos,
       go::codegen_data* data)
       : mstch_enum(e, ctx, pos), data_(*data) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"enum:go_name", &mstch_go_enum::go_name},
@@ -222,7 +214,7 @@ class mstch_go_const : public mstch_const {
       go::codegen_data* data)
       : mstch_const(c, ctx, pos, current_const, expected_type, field),
         data_(*data) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"constant:go_name", &mstch_go_const::go_name},
@@ -265,7 +257,7 @@ class mstch_go_const_value : public mstch_const_value {
       : mstch_const_value(cv, ctx, pos, current_const, expected_type),
         data_(*data) {
     (void)data_;
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"value:go_quoted_value", &mstch_go_const_value::go_quoted_value},
@@ -291,7 +283,7 @@ class mstch_go_field : public mstch_field {
       go::codegen_data* data)
       : mstch_field(f, ctx, pos, field_context), data_(*data) {
     (void)data_;
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"field:go_name", &mstch_go_field::go_name},
@@ -300,10 +292,6 @@ class mstch_go_field : public mstch_field {
             {"field:pointer?", &mstch_go_field::is_pointer},
             {"field:non_struct_pointer?",
              &mstch_go_field::is_non_struct_pointer},
-            {"field:compat_setter_pointer?",
-             &mstch_go_field::is_compat_setter_pointer},
-            {"field:compat_setter_value_op",
-             &mstch_go_field::compat_setter_value_op},
             {"field:nilable?", &mstch_go_field::is_nilable},
             {"field:must_be_set_to_serialize?",
              &mstch_go_field::must_be_set_to_serialize},
@@ -368,18 +356,6 @@ class mstch_go_field : public mstch_field {
     auto real_type = field_->type()->get_true_type();
     return is_pointer_() && !go::is_type_go_struct(real_type);
   }
-  mstch::node is_compat_setter_pointer() { return is_compat_setter_pointer_(); }
-  mstch::node compat_setter_value_op() {
-    if (is_pointer_() && is_compat_setter_pointer_()) {
-      return std::string("");
-    } else if (is_pointer_() && !is_compat_setter_pointer_()) {
-      return std::string("&");
-    } else if (!is_pointer_() && !is_compat_setter_pointer_()) {
-      return std::string("");
-    } else { // if (!is_pointer_() && is_compat_setter_pointer_())
-      return std::string("*");
-    }
-  }
   mstch::node key_str() {
     // Legacy schemas may have negative tags - replace minus with an underscore.
     if (field_->get_key() < 0) {
@@ -424,23 +400,6 @@ class mstch_go_field : public mstch_field {
          !go::is_type_go_nilable(real_type));
   }
 
-  bool is_compat_setter_pointer_() {
-    // Whether this field's value should be a pointer in compat-setter.
-    // This is needed for a seamless migration from the legacy generator.
-    //
-    // Legacy generator treats optional fields with default values differently
-    // compared to the new generator (pointer vs non-pointer).
-    // This method helps with backwards compatibility.
-    bool has_default_value = (field_->default_value() != nullptr);
-    if (is_optional_() && has_default_value) {
-      auto real_type = field_->type()->get_true_type();
-      bool is_container =
-          (real_type->is_list() || real_type->is_map() || real_type->is_set());
-      return is_container;
-    }
-    return is_pointer_();
-  }
-
   bool is_inside_union_() {
     // Whether field is part of a union
     return field_context_ != nullptr && field_context_->strct != nullptr &&
@@ -456,7 +415,7 @@ class mstch_go_struct : public mstch_struct {
       mstch_element_position pos,
       go::codegen_data* data)
       : mstch_struct(s, ctx, pos), data_(*data) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"struct:go_name", &mstch_go_struct::go_name},
@@ -474,6 +433,8 @@ class mstch_go_struct : public mstch_struct {
             {"struct:req?", &mstch_go_struct::is_req_struct},
             {"struct:fields_sorted", &mstch_go_struct::fields_sorted},
             {"struct:scoped_name", &mstch_go_struct::scoped_name},
+            {"struct:use_reflect_codec?",
+             &mstch_go_struct::should_use_reflect_codec},
         });
   }
 
@@ -515,9 +476,27 @@ class mstch_go_struct : public mstch_struct {
     return "premadeStructSpec_" + struct_->name();
   }
   mstch::node fields_sorted() {
-    return make_mstch_fields(struct_->get_sorted_members());
+    auto fields_in_id_order = struct_->get_sorted_members();
+    // Fields (optionally) in the most optimal (memory-saving) layout order.
+    auto minimizePadding =
+        struct_->has_structured_annotation(kGoMinimizePaddingUri);
+    if (minimizePadding) {
+      std::vector<t_field*> fields_in_layout_order;
+      std::copy(
+          fields_in_id_order.begin(),
+          fields_in_id_order.end(),
+          std::back_inserter(fields_in_layout_order));
+      go::optimize_fields_layout(fields_in_layout_order, struct_->is_union());
+      return make_mstch_fields(fields_in_layout_order);
+    }
+    return make_mstch_fields(fields_in_id_order);
   }
   mstch::node scoped_name() { return struct_->get_scoped_name(); }
+  mstch::node should_use_reflect_codec() {
+    auto use_reflect_codec_annotation =
+        struct_->find_structured_annotation_or_null(kGoUseReflectCodecUri);
+    return data_.use_reflect_codec || use_reflect_codec_annotation != nullptr;
+  }
 
  private:
   go::codegen_data& data_;
@@ -558,7 +537,7 @@ class mstch_go_service : public mstch_service {
       mstch_element_position pos,
       go::codegen_data* data)
       : mstch_service(s, ctx, pos), data_(*data) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"service:go_name", &mstch_go_service::go_name},
@@ -606,15 +585,15 @@ class mstch_go_function : public mstch_function {
       go::codegen_data* data)
       : mstch_function(f, ctx, pos, iface), data_(*data) {
     (void)data_;
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"function:go_name", &mstch_go_function::go_name},
-            {"function:go_supported?", &mstch_go_function::is_go_supported},
+            {"function:go_client_supported?",
+             &mstch_go_function::is_go_client_supported},
+            {"function:go_server_supported?",
+             &mstch_go_function::is_go_server_supported},
             {"function:ctx_arg_name", &mstch_go_function::ctx_arg_name},
-            {"function:out_var_name", &mstch_go_function::out_var_name},
-            {"function:in_var_name", &mstch_go_function::in_var_name},
-            {"function:err_var_name", &mstch_go_function::err_var_name},
             {"function:retval_field_name",
              &mstch_go_function::retval_field_name},
             {"function:retval_nilable?", &mstch_go_function::is_retval_nilable},
@@ -622,7 +601,12 @@ class mstch_go_function : public mstch_function {
   }
   mstch::node go_name() { return go::get_go_func_name(function_); }
 
-  mstch::node is_go_supported() { return go::is_func_go_supported(function_); }
+  mstch::node is_go_client_supported() {
+    return go::is_func_go_client_supported(function_);
+  }
+  mstch::node is_go_server_supported() {
+    return go::is_func_go_server_supported(function_);
+  }
 
   mstch::node ctx_arg_name() {
     // This helper returns the Context object name to be used in the function
@@ -631,27 +615,6 @@ class mstch_go_function : public mstch_function {
     return get_unique_name("ctx");
   }
 
-  mstch::node out_var_name() {
-    // This helper function returns a unique "out" variable name,
-    // that doesn't conflict with any parameter names.
-    // The name is guaranteed not to collide.
-    return get_unique_name("out");
-  }
-
-  mstch::node in_var_name() {
-    // This helper function returns a unique "in" variable name,
-    // that doesn't conflict with any parameter names.
-    // The name is guaranteed not to collide.
-    return get_unique_name("in");
-  }
-
-  mstch::node err_var_name() {
-    // This helper function returns a unique "err" variable name,
-    // that doesn't conflict with any parameter names.
-    // The name is guaranteed not to collide.
-    return get_unique_name("err");
-  }
-
   std::string get_unique_name(std::string const& name) {
     auto& members = function_->params().get_members();
 
@@ -695,7 +658,7 @@ class mstch_go_type : public mstch_type {
       go::codegen_data* data)
       : mstch_type(t, ctx, pos), data_(*data) {
     (void)data_;
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"type:go_comparable?", &mstch_go_type::is_go_comparable},
@@ -767,7 +730,7 @@ class mstch_go_type : public mstch_type {
   std::string metadata_name_() {
     return "premadeThriftType_" + sanitized_full_name_();
   }
-  mstch::node codec_type_spec_name_() {
+  std::string codec_type_spec_name_() {
     return "premadeCodecTypeSpec_" + sanitized_full_name_();
   }
   std::string sanitized_full_name_() {
@@ -789,7 +752,7 @@ class mstch_go_typedef : public mstch_typedef {
       mstch_element_position pos,
       go::codegen_data* data)
       : mstch_typedef(t, ctx, pos), data_(*data) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"typedef:go_name", &mstch_go_typedef::go_name},
