@@ -27,10 +27,10 @@
 #include <openssl/evp.h>
 #include <thrift/compiler/ast/t_typedef.h>
 #include <thrift/compiler/ast/uri.h>
-#include <thrift/compiler/detail/mustache/mstch.h>
 #include <thrift/compiler/generate/cpp/name_resolver.h>
 #include <thrift/compiler/generate/java/util.h>
 #include <thrift/compiler/generate/t_mstch_generator.h>
+#include <thrift/compiler/whisker/mstch_compat.h>
 
 using namespace std;
 
@@ -91,7 +91,7 @@ mstch::node get_structed_annotation_attribute(
 
 template <typename Node>
 std::string get_java_swift_name(const Node* node) {
-  return node->get_annotation(
+  return node->get_unstructured_annotation(
       "java.swift.name", java::mangle_java_name(node->get_name(), false));
 }
 
@@ -159,6 +159,17 @@ class t_mstch_java_generator : public t_mstch_generator {
  public:
   using t_mstch_generator::t_mstch_generator;
 
+  whisker_options render_options() const override {
+    whisker_options opts;
+    opts.allowed_undefined_variables = {
+        "type:typedef_type", // in UnionWrite.mustache
+        "struct:isUnion?", // in WriteResponseType.mustache
+        "struct:asBean?", // in DefaultValue.mustache
+        "field:hasAdapter?", // in BoxedType.mustache
+    };
+    return opts;
+  }
+
   std::string template_prefix() const override { return "java"; }
 
   void generate_program() override;
@@ -442,7 +453,7 @@ class mstch_java_program : public mstch_program {
   mstch_java_program(
       const t_program* p, mstch_context& ctx, mstch_element_position pos)
       : mstch_program(p, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"program:javaPackage", &mstch_java_program::java_package},
@@ -491,7 +502,7 @@ class mstch_java_struct : public mstch_struct {
   mstch_java_struct(
       const t_structured* s, mstch_context& ctx, mstch_element_position pos)
       : mstch_struct(s, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"struct:javaPackage", &mstch_java_struct::java_package},
@@ -543,9 +554,7 @@ class mstch_java_struct : public mstch_struct {
   }
   mstch::node has_wrapper() {
     for (const auto& field : struct_->fields()) {
-      auto has_annotation =
-          field.find_structured_annotation_or_null(kJavaWrapperUri);
-      if (has_annotation) {
+      if (field.has_structured_annotation(kJavaWrapperUri)) {
         return true;
       }
     }
@@ -553,8 +562,9 @@ class mstch_java_struct : public mstch_struct {
   }
   mstch::node is_as_bean() {
     if (!struct_->is_exception() && !struct_->is_union()) {
-      return struct_->get_annotation("java.swift.mutable") == "true" ||
-          struct_->find_structured_annotation_or_null(kJavaMutableUri);
+      return struct_->get_unstructured_annotation("java.swift.mutable") ==
+          "true" ||
+          struct_->has_structured_annotation(kJavaMutableUri);
     } else {
       return false;
     }
@@ -562,20 +572,20 @@ class mstch_java_struct : public mstch_struct {
 
   mstch::node is_BigStruct() {
     return (
-        struct_->is_struct() && struct_->fields().size() > bigStructThreshold);
+        struct_->is_struct_or_union() &&
+        struct_->fields().size() > bigStructThreshold);
   }
 
   mstch::node java_capital_name() {
     return java::mangle_java_name(struct_->get_name(), true);
   }
   mstch::node has_java_annotations() {
-    return struct_->has_annotation("java.swift.annotations") ||
-        struct_->find_structured_annotation_or_null(kJavaAnnotationUri) !=
-        nullptr;
+    return struct_->has_unstructured_annotation("java.swift.annotations") ||
+        struct_->has_structured_annotation(kJavaAnnotationUri);
   }
   mstch::node java_annotations() {
-    if (struct_->has_annotation("java.swift.annotations")) {
-      return struct_->get_annotation("java.swift.annotations");
+    if (struct_->has_unstructured_annotation("java.swift.annotations")) {
+      return struct_->get_unstructured_annotation("java.swift.annotations");
     }
 
     return get_structed_annotation_attribute(
@@ -607,7 +617,7 @@ class mstch_java_service : public mstch_service {
       mstch_element_position pos,
       const t_service* containing_service = nullptr)
       : mstch_service(s, ctx, pos, containing_service) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"service:javaPackage", &mstch_java_service::java_package},
@@ -689,7 +699,7 @@ class mstch_java_interaction : public mstch_java_service {
       mstch_element_position pos,
       const t_service* containing_service)
       : mstch_java_service(interaction, ctx, pos, containing_service) {
-    register_cached_methods(
+    register_methods(
         this,
         {{"interaction:javaParentCapitalName",
           &mstch_java_interaction::java_parent_capital_name}});
@@ -708,29 +718,30 @@ class mstch_java_function : public mstch_function {
       mstch_element_position pos,
       const t_interface* iface)
       : mstch_function(f, ctx, pos, iface) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"function:javaName", &mstch_java_function::java_name},
             {"function:voidType", &mstch_java_function::is_void_type},
-        });
-    register_volatile_methods(
-        this,
-        {
-            {"function:nestedDepth", &mstch_java_function::get_nested_depth},
+            {"function:nestedDepth",
+             {with_no_caching, &mstch_java_function::get_nested_depth}},
             {"function:nestedDepth++",
-             &mstch_java_function::increment_nested_depth},
+             {with_no_caching, &mstch_java_function::increment_nested_depth}},
             {"function:nestedDepth--",
-             &mstch_java_function::decrement_nested_depth},
-            {"function:isFirstDepth?", &mstch_java_function::is_first_depth},
+             {with_no_caching, &mstch_java_function::decrement_nested_depth}},
+            {"function:isFirstDepth?",
+             {with_no_caching, &mstch_java_function::is_first_depth}},
             {"function:prevNestedDepth",
-             &mstch_java_function::preceding_nested_depth},
+             {with_no_caching, &mstch_java_function::preceding_nested_depth}},
             {"function:isNested?",
-             &mstch_java_function::get_nested_container_flag},
+             {with_no_caching,
+              &mstch_java_function::get_nested_container_flag}},
             {"function:setIsNested",
-             &mstch_java_function::set_nested_container_flag},
+             {with_no_caching,
+              &mstch_java_function::set_nested_container_flag}},
             {"function:unsetIsNested",
-             &mstch_java_function::unset_nested_container_flag},
+             {with_no_caching,
+              &mstch_java_function::unset_nested_container_flag}},
         });
   }
 
@@ -775,7 +786,7 @@ class mstch_java_field : public mstch_field {
       mstch_element_position pos,
       const field_generator_context* field_context)
       : mstch_field(f, ctx, pos, field_context) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"field:javaName", &mstch_java_field::java_name},
@@ -818,18 +829,21 @@ class mstch_java_field : public mstch_field {
              &mstch_java_field::get_field_adapter_class_name},
             {"field:FieldNameUnmangled?",
              &mstch_java_field::is_field_name_unmangled},
-        });
-    register_volatile_methods(
-        this,
-        {
-            {"field:nestedDepth", &mstch_java_field::get_nested_depth},
-            {"field:nestedDepth++", &mstch_java_field::increment_nested_depth},
-            {"field:nestedDepth--", &mstch_java_field::decrement_nested_depth},
-            {"field:isFirstDepth?", &mstch_java_field::is_first_depth},
+
+            {"field:nestedDepth",
+             {with_no_caching, &mstch_java_field::get_nested_depth}},
+            {"field:nestedDepth++",
+             {with_no_caching, &mstch_java_field::increment_nested_depth}},
+            {"field:nestedDepth--",
+             {with_no_caching, &mstch_java_field::decrement_nested_depth}},
+            {"field:isFirstDepth?",
+             {with_no_caching, &mstch_java_field::is_first_depth}},
             {"field:prevNestedDepth",
-             &mstch_java_field::preceding_nested_depth},
-            {"field:isNested?", &mstch_java_field::get_nested_container_flag},
-            {"field:setIsNested", &mstch_java_field::set_nested_container_flag},
+             {with_no_caching, &mstch_java_field::preceding_nested_depth}},
+            {"field:isNested?",
+             {with_no_caching, &mstch_java_field::get_nested_container_flag}},
+            {"field:setIsNested",
+             {with_no_caching, &mstch_java_field::set_nested_container_flag}},
         });
   }
 
@@ -837,8 +851,7 @@ class mstch_java_field : public mstch_field {
   bool isNestedContainerFlag = false;
 
   bool _has_wrapper() {
-    return field_->find_structured_annotation_or_null(kJavaWrapperUri) !=
-        nullptr;
+    return field_->has_structured_annotation(kJavaWrapperUri);
   }
 
   mstch::node has_wrapper() { return _has_wrapper(); }
@@ -902,8 +915,7 @@ class mstch_java_field : public mstch_field {
   }
 
   bool _has_field_adapter() {
-    return field_->find_structured_annotation_or_null(kJavaAdapterUri) !=
-        nullptr;
+    return field_->has_structured_annotation(kJavaAdapterUri);
   }
 
   mstch::node has_field_adapter() { return _has_field_adapter(); }
@@ -967,7 +979,7 @@ class mstch_java_field : public mstch_field {
 
   mstch::node is_object() {
     const t_type* field_type = field_->get_type()->get_true_type();
-    return field_type->is_struct() || field_type->is_exception() ||
+    return field_type->is_struct_or_union() || field_type->is_exception() ||
         field_type->is_union();
   }
 
@@ -987,15 +999,13 @@ class mstch_java_field : public mstch_field {
   }
 
   mstch::node java_tfield_name() {
-    return constant_name(field_->get_name()) + "_FIELD_DESC";
+    return constant_name(get_java_swift_name(field_)) + "_FIELD_DESC";
   }
   mstch::node java_capital_name() {
-    return java::mangle_java_name(
-        field_->get_annotation("java.swift.name", &field_->get_name()), true);
+    return java::mangle_java_name(get_java_swift_name(field_), true);
   }
   mstch::node java_constant_name() {
-    return constant_name(
-        field_->get_annotation("java.swift.name", &field_->get_name()));
+    return constant_name(get_java_swift_name(field_));
   }
   mstch::node java_all_caps_name() {
     auto field_name = field_->get_name();
@@ -1004,8 +1014,9 @@ class mstch_java_field : public mstch_field {
   }
   mstch::node java_default_value() { return default_value_for_field(field_); }
   mstch::node is_recursive_reference() {
-    return field_->get_annotation("swift.recursive_reference") == "true" ||
-        field_->find_structured_annotation_or_null(kJavaRecursiveUri);
+    return field_->get_unstructured_annotation("swift.recursive_reference") ==
+        "true" ||
+        field_->has_structured_annotation(kJavaRecursiveUri);
   }
   mstch::node is_negative_id() { return field_->get_key() < 0; }
   std::string default_value_for_field(const t_field* field) {
@@ -1041,7 +1052,7 @@ class mstch_java_field : public mstch_field {
   }
 
   mstch::node is_sensitive() {
-    return field_->has_annotation("java.sensitive");
+    return field_->has_unstructured_annotation("java.sensitive");
   }
   std::string constant_name(string name) {
     string constant_str;
@@ -1061,13 +1072,12 @@ class mstch_java_field : public mstch_field {
     return constant_str;
   }
   mstch::node has_java_annotations() {
-    return field_->has_annotation("java.swift.annotations") ||
-        field_->find_structured_annotation_or_null(kJavaAnnotationUri) !=
-        nullptr;
+    return field_->has_unstructured_annotation("java.swift.annotations") ||
+        field_->has_structured_annotation(kJavaAnnotationUri);
   }
   mstch::node java_annotations() {
-    if (field_->has_annotation("java.swift.annotations")) {
-      return field_->get_annotation("java.swift.annotations");
+    if (field_->has_unstructured_annotation("java.swift.annotations")) {
+      return field_->get_unstructured_annotation("java.swift.annotations");
     }
 
     return get_structed_annotation_attribute(
@@ -1075,8 +1085,7 @@ class mstch_java_field : public mstch_field {
   }
 
   mstch::node is_field_name_unmangled() {
-    return !!field_->find_structured_annotation_or_null(
-        kJavaFieldUseUnmangledNameUri);
+    return field_->has_structured_annotation(kJavaFieldUseUnmangledNameUri);
   }
 };
 
@@ -1085,7 +1094,7 @@ class mstch_java_enum : public mstch_enum {
   mstch_java_enum(
       const t_enum* e, mstch_context& ctx, mstch_element_position pos)
       : mstch_enum(e, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"enum:javaPackage", &mstch_java_enum::java_package},
@@ -1105,12 +1114,11 @@ class mstch_java_enum : public mstch_enum {
     return java::mangle_java_name(enum_->get_name(), true);
   }
   mstch::node java_skip_enum_name_map() {
-    return enum_->has_annotation("java.swift.skip_enum_name_map");
+    return enum_->has_unstructured_annotation("java.swift.skip_enum_name_map");
   }
   mstch::node num_values() { return enum_->get_enum_values().size(); }
   mstch::node use_intrinsic_default() {
-    if (enum_->find_structured_annotation_or_null(
-            kJavaUseIntrinsicDefaultUri) != nullptr) {
+    if (enum_->has_structured_annotation(kJavaUseIntrinsicDefaultUri)) {
       if (enum_->find_value(0) == nullptr) {
         throw std::runtime_error(
             "Enum " + enum_->get_name() +
@@ -1121,8 +1129,7 @@ class mstch_java_enum : public mstch_enum {
     return false;
   }
   mstch::node find_value_zero() {
-    if (enum_->find_structured_annotation_or_null(
-            kJavaUseIntrinsicDefaultUri) != nullptr) {
+    if (enum_->has_structured_annotation(kJavaUseIntrinsicDefaultUri)) {
       return java::mangle_java_constant_name(enum_->find_value(0)->get_name());
     }
     return mstch::node();
@@ -1134,7 +1141,7 @@ class mstch_java_enum_value : public mstch_enum_value {
   mstch_java_enum_value(
       const t_enum_value* ev, mstch_context& ctx, mstch_element_position pos)
       : mstch_enum_value(ev, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"enum_value:javaConstantName",
@@ -1158,7 +1165,7 @@ class mstch_java_const : public mstch_const {
       const t_type* expected_type,
       const t_field* field)
       : mstch_const(c, ctx, pos, current_const, expected_type, field) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"constant:javaCapitalName", &mstch_java_const::java_capital_name},
@@ -1179,21 +1186,21 @@ class mstch_java_const : public mstch_const {
     if (const_->type()->is_map()) {
       t_map* map = (t_map*)const_->type();
       if (map->get_key_type()->is_enum()) {
-        return map->get_key_type()->has_annotation(
+        return map->get_key_type()->has_unstructured_annotation(
             "java.swift.skip_enum_name_map");
       }
     }
     if (const_->type()->is_list()) {
       t_list* list = (t_list*)const_->type();
       if (list->get_elem_type()->is_enum()) {
-        return list->get_elem_type()->has_annotation(
+        return list->get_elem_type()->has_unstructured_annotation(
             "java.swift.skip_enum_name_map");
       }
     }
     if (const_->type()->is_set()) {
       t_set* set = (t_set*)const_->type();
       if (set->get_elem_type()->is_enum()) {
-        return set->get_elem_type()->has_annotation(
+        return set->get_elem_type()->has_unstructured_annotation(
             "java.swift.skip_enum_name_map");
       }
     }
@@ -1214,7 +1221,7 @@ class mstch_java_const_value : public mstch_const_value {
       const t_const* current_const,
       const t_type* expected_type)
       : mstch_const_value(cv, ctx, pos, current_const, expected_type) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"value:quotedString", &mstch_java_const_value::quote_java_string},
@@ -1243,7 +1250,7 @@ class mstch_java_type : public mstch_type {
   mstch_java_type(
       const t_type* t, mstch_context& ctx, mstch_element_position pos)
       : mstch_type(t, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"type:isContainer?", &mstch_java_type::is_container_type},
@@ -1254,23 +1261,33 @@ class mstch_java_type : public mstch_type {
              &mstch_java_type::get_structured_adapter_class_name},
             {"type:typeClassName",
              &mstch_java_type::get_structured_type_class_name},
-        });
-    register_volatile_methods(
-        this,
-        {
-            {"type:setIsMapKey", &mstch_java_type::set_is_map_key},
-            {"type:isMapKey?", &mstch_java_type::get_map_key_flag},
-            {"type:setIsMapValue", &mstch_java_type::set_is_map_value},
-            {"type:isMapValue?", &mstch_java_type::get_map_value_flag},
-            {"type:setIsNotMap", &mstch_java_type::set_is_not_map},
-            {"type:setAdapter", &mstch_java_type::set_adapter},
-            {"type:unsetAdapter", &mstch_java_type::unset_adapter},
-            {"type:isAdapterSet?", &mstch_java_type::is_adapter_set},
-            {"type:addAdapter", &mstch_java_type::add_type_adapter},
-            {"type:adapterDefined?", &mstch_java_type::is_type_adapter_defined},
-            {"type:lastAdapter?", &mstch_java_type::is_last_type_adapter},
-            {"type:setTypeName", &mstch_java_type::set_type_name},
-            {"type:getTypeName", &mstch_java_type::get_type_name},
+
+            {"type:setIsMapKey",
+             {with_no_caching, &mstch_java_type::set_is_map_key}},
+            {"type:isMapKey?",
+             {with_no_caching, &mstch_java_type::get_map_key_flag}},
+            {"type:setIsMapValue",
+             {with_no_caching, &mstch_java_type::set_is_map_value}},
+            {"type:isMapValue?",
+             {with_no_caching, &mstch_java_type::get_map_value_flag}},
+            {"type:setIsNotMap",
+             {with_no_caching, &mstch_java_type::set_is_not_map}},
+            {"type:setAdapter",
+             {with_no_caching, &mstch_java_type::set_adapter}},
+            {"type:unsetAdapter",
+             {with_no_caching, &mstch_java_type::unset_adapter}},
+            {"type:isAdapterSet?",
+             {with_no_caching, &mstch_java_type::is_adapter_set}},
+            {"type:addAdapter",
+             {with_no_caching, &mstch_java_type::add_type_adapter}},
+            {"type:adapterDefined?",
+             {with_no_caching, &mstch_java_type::is_type_adapter_defined}},
+            {"type:lastAdapter?",
+             {with_no_caching, &mstch_java_type::is_last_type_adapter}},
+            {"type:setTypeName",
+             {with_no_caching, &mstch_java_type::set_type_name}},
+            {"type:getTypeName",
+             {with_no_caching, &mstch_java_type::get_type_name}},
         });
   }
   bool isMapValueFlag = false;
@@ -1298,10 +1315,12 @@ class mstch_java_type : public mstch_type {
   }
 
   mstch::node java_type() {
-    return type_->get_true_type()->get_annotation("java.swift.type");
+    return type_->get_true_type()->get_unstructured_annotation(
+        "java.swift.type");
   }
   mstch::node is_binary_string() {
-    return type_->get_true_type()->get_annotation("java.swift.binary_string");
+    return type_->get_true_type()->get_unstructured_annotation(
+        "java.swift.binary_string");
   }
 
   mstch::node has_type_adapter() {
@@ -1369,8 +1388,7 @@ class mstch_java_type : public mstch_type {
     auto type = type_;
     while (type) {
       if (type_->is_typedef() &&
-          type->find_structured_annotation_or_null(kJavaAdapterUri) !=
-              nullptr) {
+          type->has_structured_annotation(kJavaAdapterUri)) {
         count++;
         if (const auto* as_typedef = dynamic_cast<const t_typedef*>(type)) {
           type = as_typedef->get_type();
