@@ -137,11 +137,12 @@ std::string to_flat_type_name(const t_type* type) {
 const std::string* get_py_adapter(const t_type* type) {
   assert(type != nullptr);
 
-  if (!type->get_true_type()->is_struct()) {
+  if (!type->get_true_type()->is_struct_or_union()) {
     return nullptr;
   }
 
-  return t_typedef::get_first_annotation_or_null(type, {"py.adapter"});
+  return t_typedef::get_first_unstructured_annotation_or_null(
+      type, {"py.adapter"});
 }
 
 std::set<std::string> get_distinct_adapters(const t_program* program) {
@@ -204,7 +205,7 @@ class pyi_mstch_program : public mstch_program {
       mstch_context& context,
       mstch_element_position position)
       : mstch_program(program, context, position) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"program:returnTypes", &pyi_mstch_program::get_return_types},
@@ -361,7 +362,7 @@ class pyi_mstch_field : public mstch_field {
       mstch_element_position position,
       const field_generator_context* field_context)
       : mstch_field(field, context, position, field_context) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"field:requireValue?", &pyi_mstch_field::get_require_value},
@@ -420,7 +421,7 @@ class pyi_mstch_type : public mstch_type {
       mstch_element_position position,
       const t_program* program)
       : mstch_type(type, context, position), program_(program) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"type:modulePath", &pyi_mstch_type::get_module_path},
@@ -469,7 +470,7 @@ class pyi_mstch_service : public mstch_service {
       mstch_element_position position,
       const t_program* program)
       : mstch_service(service, context, position), program_(program) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"service:externalProgram?",
@@ -533,7 +534,7 @@ class pyi_mstch_function : public mstch_function {
       mstch_element_position position,
       const t_interface* iface)
       : mstch_function(function, context, position, iface) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"function:isSupported?", &pyi_mstch_function::is_supported},
@@ -554,6 +555,15 @@ class t_mstch_pyi_generator : public t_mstch_generator {
  public:
   using t_mstch_generator::t_mstch_generator;
 
+  whisker_options render_options() const override {
+    whisker_options opts;
+    opts.allowed_undefined_variables = {
+        "service:autogen_path",
+        "program:autogen_path",
+    };
+    return opts;
+  }
+
   std::string template_prefix() const override { return "pyi"; }
 
   void generate_program() override;
