@@ -27,14 +27,13 @@
 
 #include <thrift/compiler/ast/t_field.h>
 #include <thrift/compiler/ast/uri.h>
-#include <thrift/compiler/detail/mustache/mstch.h>
 #include <thrift/compiler/generate/common.h>
 #include <thrift/compiler/generate/cpp/name_resolver.h>
 #include <thrift/compiler/generate/cpp/util.h>
 #include <thrift/compiler/generate/mstch_objects.h>
 #include <thrift/compiler/generate/python/util.h>
 #include <thrift/compiler/generate/t_mstch_generator.h>
-#include <thrift/compiler/sema/explicit_include_validator.h>
+#include <thrift/compiler/whisker/mstch_compat.h>
 
 namespace apache::thrift::compiler {
 
@@ -119,8 +118,9 @@ std::string_view get_cpp_template(const t_node& node) {
   }
   if (auto type = type_of_node(node)) {
     // finds unstructured annotation template, if present
-    if (const auto* _template = type->get_true_type()->find_annotation_or_null(
-            {"cpp.template", "cpp2.template"})) {
+    if (const auto* _template =
+            type->get_true_type()->find_unstructured_annotation_or_null(
+                {"cpp.template", "cpp2.template"})) {
       return *_template;
     }
   }
@@ -148,6 +148,17 @@ inline std::string get_capi_include(
       prog->name());
 }
 
+const std::string& gen_capi_module_prefix(const t_program* program) {
+  static std::unordered_map<const t_program*, std::string> cache;
+  auto it = cache.find(program);
+  if (it != cache.end()) {
+    return it->second;
+  }
+  auto inserted =
+      cache.emplace(program, python::gen_capi_module_prefix_impl(program));
+  return inserted.first->second;
+}
+
 // Formats a field's type, or a sub-type
 // of a compound type (e.g., a container)
 // Hence, assumes t_node is a t_field or t_type
@@ -271,11 +282,13 @@ std::string format_marshal_type_unadapted(
         "::apache::thrift::python::capi::ComposedEnum<{}::{}>",
         cpp2::get_gen_namespace(*true_type->program()),
         cpp2::get_name(true_type));
-  } else if (true_type->is_struct() || true_type->is_exception()) {
+  } else if (true_type->is_struct_or_union() || true_type->is_exception()) {
     return fmt::format(
-        "::apache::thrift::python::capi::ComposedStruct<{}::{}>",
+        "::apache::thrift::python::capi::ComposedStruct<{}::{}, ::{}::NamespaceTag>",
         cpp2::get_gen_namespace(*true_type->program()),
-        cpp2::get_name(true_type));
+        cpp2::get_name(true_type),
+        gen_capi_module_prefix(true_type->program()));
+
   } else if (true_type->is_list()) {
     const auto* elem_type =
         dynamic_cast<const t_list*>(true_type)->get_elem_type();
@@ -298,7 +311,7 @@ class python_capi_mstch_program : public mstch_program {
   python_capi_mstch_program(
       const t_program* p, mstch_context& ctx, mstch_element_position pos)
       : mstch_program(p, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"program:capi_includes",
@@ -337,13 +350,7 @@ class python_capi_mstch_program : public mstch_program {
     return a;
   }
 
-  mstch::node capi_module_prefix() {
-    std::string prefix = get_py3_namespace_with_name_and_prefix(
-        program_, get_option("root_module_prefix"), "__");
-    // kebab is not kosher in cpp fn names
-    std::replace(prefix.begin(), prefix.end(), '-', '_');
-    return prefix;
-  }
+  mstch::node capi_module_prefix() { return gen_capi_module_prefix(program_); }
 
   mstch::node module_path() {
     return get_py3_namespace_with_name_and_prefix(
@@ -452,7 +459,7 @@ class python_capi_mstch_struct : public mstch_struct {
   python_capi_mstch_struct(
       const t_structured* s, mstch_context& ctx, mstch_element_position pos)
       : mstch_struct(s, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"struct:py_name", &python_capi_mstch_struct::py_name},
@@ -510,7 +517,7 @@ class python_capi_mstch_struct : public mstch_struct {
   }
 
   bool capi_eligible_type(const t_type* type) {
-    if (type->find_structured_annotation_or_null(kCppAdapterUri)) {
+    if (type->has_structured_annotation(kCppAdapterUri)) {
       return false;
     }
 
@@ -525,13 +532,15 @@ class python_capi_mstch_struct : public mstch_struct {
     // used on type or field
     // TODO: delete these if structured annotation migration completed
     if (const std::string* template_anno =
-            type->find_annotation_or_null({"cpp.template", "cpp2.template"})) {
+            type->find_unstructured_annotation_or_null(
+                {"cpp.template", "cpp2.template"})) {
       if (!is_supported_template(*template_anno)) {
         return false;
       }
     }
     if (const std::string* type_anno =
-            type->find_annotation_or_null({"cpp.type", "cpp2.type"})) {
+            type->find_unstructured_annotation_or_null(
+                {"cpp.type", "cpp2.type"})) {
       return is_type_iobuf(*type_anno);
     }
     if (type->is_list() &&
@@ -559,7 +568,7 @@ class python_capi_mstch_struct : public mstch_struct {
   }
 
   bool capi_eligible_field(const t_field& field) {
-    if (field.find_structured_annotation_or_null(kCppAdapterUri)) {
+    if (field.has_structured_annotation(kCppAdapterUri)) {
       return false;
     }
     if (const auto* cpp_type_anno =
@@ -571,7 +580,7 @@ class python_capi_mstch_struct : public mstch_struct {
 
   mstch::node marshal_capi() {
     const auto marshal_override =
-        struct_->find_structured_annotation_or_null(kUseCAPIUri);
+        struct_->find_structured_annotation_or_null(kPythonUseCAPIUri);
     auto force_serialize = [marshal_override]() {
       const auto serialize_field = marshal_override
           ? marshal_override->get_value_from_structured_annotation_or_null(
@@ -625,11 +634,12 @@ class python_capi_mstch_field : public mstch_field {
       const field_generator_context* field_context)
       : mstch_field(field, ctx, pos, field_context),
         py_name_(python::get_py3_name(*field)) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"field:cpp_name", &python_capi_mstch_field::cpp_name},
             {"field:marshal_type", &python_capi_mstch_field::marshal_type},
+            {"field:iobuf?", &python_capi_mstch_field::iobuf},
         });
   }
 
@@ -637,6 +647,11 @@ class python_capi_mstch_field : public mstch_field {
 
   mstch::node marshal_type() { return format_marshal_type(*field_); }
 
+  mstch::node iobuf() {
+    const auto* ttype = field_->get_type()->get_true_type();
+    return ttype->is_binary() && is_type_iobuf(ttype);
+  }
+
  private:
   const std::string py_name_;
 };
@@ -646,7 +661,7 @@ class python_capi_mstch_enum : public mstch_enum {
   python_capi_mstch_enum(
       const t_enum* e, mstch_context& ctx, mstch_element_position pos)
       : mstch_enum(e, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"enum:cpp_name", &python_capi_mstch_enum::cpp_name},
@@ -673,19 +688,6 @@ class t_mstch_python_capi_generator : public t_mstch_generator {
     generate_types();
   }
 
-  void fill_validator_visitors(ast_validator& validator) const override {
-    validator.add_field_visitor([](sema_context& ctx, const t_field& f) {
-      if (f.is_injected()) {
-        return;
-      }
-      validate_explicit_include(ctx, f, *f.get_type(), diagnostic_level::error);
-    });
-    validator.add_typedef_visitor([](sema_context& ctx, const t_typedef& td) {
-      validate_explicit_include(
-          ctx, td, *td.get_type(), diagnostic_level::error);
-    });
-  }
-
  protected:
   void set_mstch_factories();
   void generate_file(
