@@ -16,7 +16,6 @@
 
 #include <algorithm>
 #include <filesystem>
-#include <iterator>
 #include <optional>
 #include <string>
 #include <unordered_map>
@@ -31,12 +30,12 @@
 #include <thrift/compiler/ast/t_field.h>
 #include <thrift/compiler/ast/t_service.h>
 #include <thrift/compiler/ast/uri.h>
-#include <thrift/compiler/detail/mustache/mstch.h>
 #include <thrift/compiler/generate/common.h>
 #include <thrift/compiler/generate/mstch_objects.h>
 #include <thrift/compiler/generate/python/util.h>
 #include <thrift/compiler/generate/t_mstch_generator.h>
 #include <thrift/compiler/sema/ast_validator.h>
+#include <thrift/compiler/whisker/mstch_compat.h>
 
 namespace apache::thrift::compiler {
 
@@ -70,8 +69,7 @@ const t_const* get_transitive_annotation_of_adapter_or_null(
   for (const auto& annotation : node.structured_annotations()) {
     const t_type& annotation_type = *annotation.type();
     if (is_transitive_annotation(annotation_type)) {
-      if (annotation_type.find_structured_annotation_or_null(
-              kPythonAdapterUri)) {
+      if (annotation_type.has_structured_annotation(kPythonAdapterUri)) {
         return &annotation;
       }
     }
@@ -79,12 +77,12 @@ const t_const* get_transitive_annotation_of_adapter_or_null(
   return nullptr;
 }
 
-std::string_view extract_module_path(std::string_view fully_qualified_name) {
-  size_t last_dot = fully_qualified_name.rfind('.');
-  if (last_dot == std::string_view::npos) {
-    return "";
-  }
-  return fully_qualified_name.substr(0, last_dot);
+std::string mangle_program_path(
+    const t_program* program, const std::string& root_module_prefix) {
+  std::string prefix = root_module_prefix.empty() ? std::string("_fbthrift")
+                                                  : root_module_prefix;
+  boost::algorithm::replace_all(prefix, ".", "__");
+  return get_py3_namespace_with_name_and_prefix(program, prefix, "__");
 }
 
 class python_mstch_const_value;
@@ -121,15 +119,71 @@ mstch::node adapter_node(
   return node;
 }
 
+bool is_invariant_container_type(const t_type* type) {
+  // Mapping is invariant in its key type
+  // For example, if `Derived` extends `Base`,
+  // then Mapping[Derived, Any] is not compatible with Mapping[Base, Any].
+  // We first check if the map has an `Base` key type (for abstract types).
+  // Then, we recursively verify whether the map's value type, or the element
+  // type of a list or set, contains any such mapping incompatibility.
+  const t_type* true_type = type->get_true_type();
+  if (true_type->is_map()) {
+    const t_map* map_type = dynamic_cast<const t_map*>(true_type);
+    const t_type* key_type = map_type->get_key_type()->get_true_type();
+    return key_type->is_struct_or_union() || key_type->is_exception() ||
+        key_type->is_container() ||
+        is_invariant_container_type(map_type->get_val_type());
+  } else if (true_type->is_list()) {
+    return is_invariant_container_type(
+        dynamic_cast<const t_list*>(true_type)->get_elem_type());
+  } else if (true_type->is_set()) {
+    return is_invariant_container_type(
+        dynamic_cast<const t_set*>(true_type)->get_elem_type());
+  }
+
+  return false;
+}
+
+bool is_invariant_adapter(
+    const t_const* adapter_annotation, const t_type* true_type) {
+  if (true_type->is_primitive_type() || !adapter_annotation) {
+    return false;
+  }
+
+  auto type_hint = get_annotation_property(adapter_annotation, "typeHint");
+  return boost::algorithm::ends_with(type_hint, "[]");
+}
+
+bool field_has_invariant_type(const t_field* field) {
+  if (is_invariant_adapter(
+          find_structured_adapter_annotation(*field),
+          field->get_type()->get_true_type())) {
+    return true;
+  }
+
+  if (is_invariant_adapter(
+          find_structured_adapter_annotation(*field->type()->get_true_type()),
+          field->get_type()->get_true_type())) {
+    return true;
+  }
+
+  return ::apache::thrift::compiler::is_invariant_container_type(
+      field->get_type());
+}
+
 class python_mstch_program : public mstch_program {
  public:
   python_mstch_program(
       const t_program* p, mstch_context& ctx, mstch_element_position pos)
       : mstch_program(p, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"program:module_path", &python_mstch_program::module_path},
+            {"program:safe_patch?", &python_mstch_program::safe_patch},
+            {"program:safe_patch_module_path",
+             &python_mstch_program::safe_patch_module_path},
+            {"program:module_mangle", &python_mstch_program::module_mangle},
             {"program:py_deprecated_module_path",
              &python_mstch_program::py_deprecated_module_path},
             {"program:py_asyncio_module_path",
@@ -145,22 +199,22 @@ class python_mstch_program : public mstch_program {
              &python_mstch_program::adapter_type_hint_modules},
             {"program:py3_auto_migrate?",
              &python_mstch_program::py3_auto_migrate},
-        });
-    register_volatile_methods(
-        this,
-        {
-            {"program:is_types_file?", &python_mstch_program::is_types_file},
+
+            {"program:is_types_file?",
+             {with_no_caching, &python_mstch_program::is_types_file}},
+            {"program:is_source_file?",
+             {with_no_caching, &python_mstch_program::is_source_file}},
+            {"program:is_type_stub?",
+             {with_no_caching, &python_mstch_program::is_type_stub}},
             {"program:generate_abstract_types",
-             &python_mstch_program::generate_abstract_types},
+             {with_no_caching, &python_mstch_program::generate_abstract_types}},
             {"program:generate_mutable_types",
-             &python_mstch_program::generate_mutable_types},
+             {with_no_caching, &python_mstch_program::generate_mutable_types}},
             {"program:generate_immutable_types",
-             &python_mstch_program::generate_immutable_types},
-            {"program:generate_to_mutable_python_conversion_methods?",
-             &python_mstch_program::
-                 generate_to_mutable_python_conversion_methods},
-            {"program:generate_unified_thrift_python_type_hints?",
-             &python_mstch_program::generate_unified_thrift_python_type_hints},
+             {with_no_caching,
+              &python_mstch_program::generate_immutable_types}},
+            {"program:enable_abstract_types?",
+             {with_no_caching, &python_mstch_program::enable_abstract_types}},
         });
     register_has_option("program:import_static?", "import_static");
     gather_included_program_namespaces();
@@ -174,9 +228,12 @@ class python_mstch_program : public mstch_program {
   mstch::node py3_auto_migrate() { return has_option("auto_migrate"); }
 
   mstch::node is_types_file() { return has_option("is_types_file"); }
+  mstch::node is_source_file() { return has_option("is_source_file"); }
+  mstch::node is_type_stub() { return has_option("is_type_stub"); }
 
   mstch::node include_namespaces() {
     std::vector<const Namespace*> namespaces;
+    namespaces.reserve(include_namespaces_.size());
     for (const auto& it : include_namespaces_) {
       namespaces.push_back(&it.second);
     }
@@ -186,10 +243,12 @@ class python_mstch_program : public mstch_program {
         });
     mstch::array a;
     for (const auto& it : namespaces) {
-      a.push_back(mstch::map{
+      a.emplace_back(mstch::map{
           {"included_module_path", it->ns},
+          {"included_module_mangle", it->ns_mangle},
           {"has_services?", it->has_services},
-          {"has_types?", it->has_types}});
+          {"has_types?", it->has_types},
+          {"is_patch?", it->is_patch}});
     }
     return a;
   }
@@ -199,6 +258,23 @@ class python_mstch_program : public mstch_program {
         program_, get_option("root_module_prefix"));
   }
 
+  mstch::node safe_patch() {
+    constexpr std::string_view prefix = "gen_safe_patch_";
+    return program_->name().substr(0, prefix.size()) == prefix;
+  }
+
+  mstch::node safe_patch_module_path() {
+    auto ns = std::get<std::string>(module_path());
+
+    // Change the namespace from "path.to.file" to "path.to.gen_safe_patch_file"
+    auto pos = ns.rfind('.');
+    return ns.substr(0, pos + 1) + "gen_safe_patch_" + ns.substr(pos + 1);
+  }
+
+  mstch::node module_mangle() {
+    return mangle_program_path(program_, get_option("root_module_prefix"));
+  }
+
   mstch::node py_deprecated_module_path() {
     std::string module_path = program_->get_namespace("py");
     if (module_path.empty()) {
@@ -243,19 +319,17 @@ class python_mstch_program : public mstch_program {
     return !get_option("generate_immutable_types").empty();
   }
 
-  mstch::node generate_to_mutable_python_conversion_methods() {
-    return !get_option("generate_to_mutable_python_conversion_methods").empty();
-  }
-
-  mstch::node generate_unified_thrift_python_type_hints() {
-    return !get_option("generate_unified_thrift_python_type_hints").empty();
+  mstch::node enable_abstract_types() {
+    return !get_option("enable_abstract_types").empty();
   }
 
  protected:
   struct Namespace {
     std::string ns;
+    std::string ns_mangle;
     bool has_services;
     bool has_types;
+    bool is_patch;
   };
 
   void gather_included_program_namespaces() {
@@ -269,9 +343,11 @@ class python_mstch_program : public mstch_program {
       include_namespaces_[included_program->path()] = Namespace{
           get_py3_namespace_with_name_and_prefix(
               included_program, get_option("root_module_prefix")),
+          mangle_program_path(
+              included_program, get_option("root_module_prefix")),
           !included_program->services().empty(),
           has_types,
-      };
+          is_patch_program(included_program)};
     }
   }
 
@@ -285,8 +361,11 @@ class python_mstch_program : public mstch_program {
       auto ns = Namespace();
       ns.ns = get_py3_namespace_with_name_and_prefix(
           prog, get_option("root_module_prefix"));
+      ns.ns_mangle =
+          mangle_program_path(prog, get_option("root_module_prefix"));
       ns.has_services = false;
       ns.has_types = true;
+      ns.is_patch = is_patch_program(prog);
       include_namespaces_[path] = std::move(ns);
     }
   }
@@ -308,7 +387,7 @@ class python_mstch_program : public mstch_program {
         if (function.has_return_type()) {
           visit_type(function.return_type().get_type());
         }
-        visit_type(stream->get_elem_type());
+        visit_type(stream->elem_type().get_type());
       } else {
         visit_type(function.return_type().get_type());
       }
@@ -347,19 +426,19 @@ class python_mstch_program : public mstch_program {
   void visit_types_for_adapters() {
     for (const t_structured* strct : program_->structs_and_unions()) {
       if (auto annotation = find_structured_adapter_annotation(*strct)) {
-        extract_module_and_insert_to(
+        python::extract_modules_and_insert_into(
             get_annotation_property(annotation, "name"), adapter_modules_);
-        extract_module_and_insert_to(
+        python::extract_modules_and_insert_into(
             get_annotation_property(annotation, "typeHint"), adapter_modules_);
-        extract_module_and_insert_to(
+        python::extract_modules_and_insert_into(
             get_annotation_property(annotation, "typeHint"),
             adapter_type_hint_modules_);
       }
       for (const auto& field : strct->fields()) {
         if (auto annotation = find_structured_adapter_annotation(field)) {
-          extract_module_and_insert_to(
+          python::extract_modules_and_insert_into(
               get_annotation_property(annotation, "name"), adapter_modules_);
-          extract_module_and_insert_to(
+          python::extract_modules_and_insert_into(
               get_annotation_property(annotation, "typeHint"),
               adapter_type_hint_modules_);
         }
@@ -367,25 +446,17 @@ class python_mstch_program : public mstch_program {
     }
     for (const auto typedef_def : program_->typedefs()) {
       if (auto annotation = find_structured_adapter_annotation(*typedef_def)) {
-        extract_module_and_insert_to(
+        python::extract_modules_and_insert_into(
             get_annotation_property(annotation, "name"), adapter_modules_);
-        extract_module_and_insert_to(
+        python::extract_modules_and_insert_into(
             get_annotation_property(annotation, "typeHint"), adapter_modules_);
-        extract_module_and_insert_to(
+        python::extract_modules_and_insert_into(
             get_annotation_property(annotation, "typeHint"),
             adapter_type_hint_modules_);
       }
     }
   }
 
-  void extract_module_and_insert_to(
-      std::string_view name, std::unordered_set<std::string_view>& modules) {
-    std::string_view module_path = extract_module_path(name);
-    if (module_path != "") {
-      modules.insert(module_path);
-    }
-  }
-
   enum TypeDef { NoTypedef, HasTypedef };
 
   void visit_type(const t_type* orig_type) {
@@ -397,9 +468,9 @@ class python_mstch_program : public mstch_program {
       return;
     }
     if (auto annotation = find_structured_adapter_annotation(*orig_type)) {
-      extract_module_and_insert_to(
+      python::extract_modules_and_insert_into(
           get_annotation_property(annotation, "name"), adapter_modules_);
-      extract_module_and_insert_to(
+      python::extract_modules_and_insert_into(
           get_annotation_property(annotation, "typeHint"),
           adapter_type_hint_modules_);
     }
@@ -428,7 +499,7 @@ class python_mstch_program : public mstch_program {
       const std::unordered_set<std::string_view>& modules) {
     mstch::array a;
     for (const auto& m : modules) {
-      a.push_back(mstch::map{{"module_path", m}});
+      a.emplace_back(mstch::map{{"module_path", m}});
     }
     return a;
   }
@@ -448,7 +519,7 @@ class python_mstch_service : public mstch_service {
       const t_program* prog,
       const t_service* containing_service = nullptr)
       : mstch_service(s, ctx, pos, containing_service), prog_(prog) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"service:module_path", &python_mstch_service::module_path},
@@ -470,7 +541,7 @@ class python_mstch_service : public mstch_service {
   mstch::node program_name() { return service_->program()->name(); }
 
   std::vector<t_function*> get_supported_functions(
-      std::function<bool(const t_function*)> func_filter) {
+      const std::function<bool(const t_function*)>& func_filter) {
     std::vector<t_function*> funcs;
     for (auto func : service_->get_functions()) {
       if (func_filter(func)) {
@@ -558,7 +629,7 @@ class python_mstch_function : public mstch_function {
       mstch_element_position pos,
       const t_interface* iface)
       : mstch_function(f, ctx, pos, iface) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"function:created_interaction",
@@ -621,7 +692,7 @@ class python_mstch_type : public mstch_type {
         adapter_annotation_(find_structured_adapter_annotation(*type)),
         transitive_adapter_annotation_(
             get_transitive_annotation_of_adapter_or_null(*type)) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"type:program_name", &python_mstch_type::program_name},
@@ -630,20 +701,23 @@ class python_mstch_type : public mstch_type {
             {"type:external_program?", &python_mstch_type::is_external_program},
             {"type:integer?", &python_mstch_type::is_integer},
             {"type:iobuf?", &python_mstch_type::is_iobuf},
+            {"type:contains_patch?", &python_mstch_type::contains_patch},
             {"type:has_adapter?", &python_mstch_type::adapter},
-        });
-    register_volatile_methods(
-        this,
-        {
-            {"type:module_path", &python_mstch_type::module_path},
-            {"type:patch_module_path", &python_mstch_type::patch_module_path},
-            {"type:need_module_path?", &python_mstch_type::need_module_path},
+
+            {"type:module_name",
+             {with_no_caching, &python_mstch_type::module_name}},
+            {"type:module_mangle",
+             {with_no_caching, &python_mstch_type::module_mangle}},
+            {"type:patch_module_path",
+             {with_no_caching, &python_mstch_type::patch_module_path}},
+            {"type:need_module_path?",
+             {with_no_caching, &python_mstch_type::need_module_path}},
             {"type:need_patch_module_path?",
-             &python_mstch_type::need_patch_module_path},
+             {with_no_caching, &python_mstch_type::need_patch_module_path}},
         });
   }
 
-  mstch::node module_path() {
+  mstch::node module_name() {
     std::string_view types_import_path = [this]() {
       if (!get_option("generate_abstract_types").empty()) {
         return ".thrift_abstract_types";
@@ -662,6 +736,27 @@ class python_mstch_type : public mstch_type {
                get_type_program(), get_option("root_module_prefix"))
         .append(types_import_path);
   }
+
+  mstch::node module_mangle() {
+    std::string_view types_import_path = [this]() {
+      if (!get_option("generate_abstract_types").empty()) {
+        return "__thrift_abstract_types";
+      }
+      if (!get_option("generate_mutable_types").empty()) {
+        return "__thrift_mutable_types";
+      }
+      if (!get_option("generate_immutable_types").empty()) {
+        return "__thrift_types";
+      }
+      throw std::runtime_error(
+          "Expected one option out of generate_abstract_types, generate_immutable_types, or generate_mutable_types to be set, and none are set.");
+    }();
+
+    return mangle_program_path(
+               get_type_program(), get_option("root_module_prefix"))
+        .append(types_import_path);
+  }
+
   mstch::node patch_module_path() {
     return get_py3_namespace_with_name_and_prefix(
                get_type_program(), get_option("root_module_prefix"))
@@ -671,6 +766,11 @@ class python_mstch_type : public mstch_type {
   mstch::node program_name() { return get_type_program()->name(); }
 
   mstch::node metadata_path() {
+    if (type_->is_enum()) {
+      return get_py3_namespace_with_name_and_prefix(
+                 get_type_program(), get_option("root_module_prefix")) +
+          ".thrift_enums";
+    }
     return get_py3_namespace_with_name_and_prefix(
                get_type_program(), get_option("root_module_prefix")) +
         ".thrift_metadata";
@@ -707,6 +807,8 @@ class python_mstch_type : public mstch_type {
 
   mstch::node is_iobuf() { return is_type_iobuf(type_); }
 
+  mstch::node contains_patch() { return type_contains_patch(type_); }
+
   mstch::node adapter() {
     return adapter_node(
         adapter_annotation_, transitive_adapter_annotation_, context_, pos_);
@@ -740,7 +842,7 @@ class python_mstch_typedef : public mstch_typedef {
       const t_typedef* t, mstch_context& ctx, mstch_element_position pos)
       : mstch_typedef(t, ctx, pos),
         adapter_annotation_(find_structured_adapter_annotation(*t)) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"typedef:has_adapter?", &python_mstch_typedef::adapter},
@@ -761,7 +863,7 @@ class python_mstch_struct : public mstch_struct {
       const t_structured* s, mstch_context& ctx, mstch_element_position pos)
       : mstch_struct(s, ctx, pos),
         adapter_annotation_(find_structured_adapter_annotation(*s)) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"struct:py_name", &python_mstch_struct::py_name},
@@ -772,8 +874,12 @@ class python_mstch_struct : public mstch_struct {
             {"struct:exception_message",
              &python_mstch_struct::exception_message},
             {"struct:has_adapter?", &python_mstch_struct::adapter},
+            {"struct:has_invariant_field?",
+             &python_mstch_struct::has_invariant_field},
             {"struct:legacy_api?", &python_mstch_struct::legacy_api},
             {"struct:fields_size", &python_mstch_struct::fields_size},
+            {"struct:allow_inheritance?",
+             &python_mstch_struct::allow_inheritance},
         });
   }
 
@@ -787,6 +893,13 @@ class python_mstch_struct : public mstch_struct {
     return make_mstch_fields(fields);
   }
 
+  mstch::node has_invariant_field() {
+    return std::any_of(
+        struct_->fields().begin(),
+        struct_->fields().end(),
+        [](const auto& field) { return field_has_invariant_type(&field); });
+  }
+
   mstch::node has_exception_message() {
     return !!dynamic_cast<const t_exception&>(*struct_).get_message_field();
   }
@@ -807,6 +920,13 @@ class python_mstch_struct : public mstch_struct {
 
   mstch::node fields_size() { return struct_->fields().size(); }
 
+  // While inheritance is discouraged, there is limited support for py3
+  // auto-migraters
+  mstch::node allow_inheritance() {
+    return struct_->has_structured_annotation(
+        kPythonMigrationBlockingAllowInheritanceUri);
+  }
+
  private:
   const t_const* adapter_annotation_;
 };
@@ -823,7 +943,7 @@ class python_mstch_field : public mstch_field {
         adapter_annotation_(find_structured_adapter_annotation(*field)),
         transitive_adapter_annotation_(
             get_transitive_annotation_of_adapter_or_null(*field)) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"field:py_name", &python_mstch_field::py_name},
@@ -833,6 +953,8 @@ class python_mstch_field : public mstch_field {
              &python_mstch_field::user_default_value},
             {"field:has_adapter?", &python_mstch_field::adapter},
             {"field:is_container_type", &python_mstch_field::is_container_type},
+            {"field:is_invariant_type?",
+             &python_mstch_field::is_invariant_type},
         });
   }
 
@@ -891,6 +1013,8 @@ class python_mstch_field : public mstch_field {
         type->get_true_type()->is_map() || type->get_true_type()->is_set();
   }
 
+  mstch::node is_invariant_type() { return field_has_invariant_type(field_); }
+
  private:
   const std::string py_name_;
   const t_const* adapter_annotation_;
@@ -902,21 +1026,16 @@ class python_mstch_enum : public mstch_enum {
   python_mstch_enum(
       const t_enum* e, mstch_context& ctx, mstch_element_position pos)
       : mstch_enum(e, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"enum:flags?", &python_mstch_enum::has_flags},
-            {"enum:legacy_api?", &python_mstch_enum::legacy_api},
         });
   }
 
   mstch::node has_flags() {
-    return enum_->has_annotation("py3.flags") ||
-        enum_->find_structured_annotation_or_null(kPythonFlagsUri);
-  }
-
-  mstch::node legacy_api() {
-    return ::apache::thrift::compiler::generate_legacy_api(*enum_);
+    return enum_->has_unstructured_annotation("py3.flags") ||
+        enum_->has_structured_annotation(kPythonFlagsUri);
   }
 };
 
@@ -925,7 +1044,7 @@ class python_mstch_enum_value : public mstch_enum_value {
   python_mstch_enum_value(
       const t_enum_value* ev, mstch_context& ctx, mstch_element_position pos)
       : mstch_enum_value(ev, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"enum_value:py_name", &python_mstch_enum_value::py_name},
@@ -938,8 +1057,13 @@ class python_mstch_enum_value : public mstch_enum_value {
 // Generator-specific validator that enforces "name" and "value" are not used
 // as enum member or union field names (thrift-py3).
 namespace enum_member_union_field_names_validator {
-void validate(const t_named* node, const std::string& name, sema_context& ctx) {
-  auto pyname = node->get_annotation("py3.name", &name);
+template <typename Pred>
+void validate(
+    const t_named* node,
+    const std::string& name,
+    sema_context& ctx,
+    Pred&& field_name_predicate) {
+  auto pyname = node->get_unstructured_annotation("py3.name", &name);
   if (const t_const* annot =
           node->find_structured_annotation_or_null(kPythonNameUri)) {
     if (auto annotation_name =
@@ -947,37 +1071,77 @@ void validate(const t_named* node, const std::string& name, sema_context& ctx) {
       pyname = annotation_name->get_string();
     }
   }
-  if (pyname == "name" || pyname == "value") {
+  if (field_name_predicate(pyname)) {
     ctx.report(
         *node,
         "enum-member-union-field-names-rule",
         diagnostic_level::error,
         "'{}' should not be used as an enum/union field name in thrift-py3. "
         "Use a different name or annotate the field with "
-        "`(py3.name=\"<new_py_name>\")`",
+        "`@python.Name{{name=\"<new_py_name>\"}}`",
         pyname);
   }
 }
 bool validate_enum(sema_context& ctx, const t_enum& enm) {
+  auto predicate = [](const auto& pyname) {
+    return pyname == "name" || pyname == "value";
+  };
   for (const t_enum_value* ev : enm.get_enum_values()) {
-    validate(ev, ev->get_name(), ctx);
+    validate(ev, ev->get_name(), ctx, predicate);
   }
   return true;
 }
 
-// TODO(T176314881): this never does anything?
-bool validate_structured(sema_context& ctx, const t_struct& s) {
-  if (!s.is_union()) {
-    return false;
-  }
+bool validate_union(sema_context& ctx, const t_union& s) {
+  auto predicate = [](const auto& pyname) {
+    return pyname == "type" || pyname == "value" || pyname == "Type";
+  };
   for (const t_field& f : s.fields()) {
-    validate(&f, f.name(), ctx);
+    validate(&f, f.name(), ctx, predicate);
   }
   return true;
 }
 
 } // namespace enum_member_union_field_names_validator
 
+namespace module_name_collision_validator {
+
+void validate_module_name_collision(
+    const t_named& node,
+    const std::string& name,
+    sema_context& ctx,
+    diagnostic_level level) {
+  // the structured annotation @python.Name overrides unstructured py3.name
+  std::reference_wrapper<const std::string> pyname =
+      node.get_unstructured_annotation("py3.name", &name);
+  if (const t_const* annot =
+          node.find_structured_annotation_or_null(kPythonNameUri)) {
+    if (auto annotation_name =
+            annot->get_value_from_structured_annotation_or_null("name")) {
+      pyname = annotation_name->get_string();
+    }
+  }
+  if (pyname.get() == node.program()->name()) {
+    ctx.report(
+        node,
+        "python-empty-namespace-symbol-collides-with-module",
+        level,
+        "'{}' is declared in module of the same name. "
+        "To fix, add a non-empty py3 namespace or change the name to "
+        " no longer collide with the module name. ",
+        pyname.get());
+  }
+}
+
+void validate_named(sema_context& ctx, const t_named& d) {
+  validate_module_name_collision(d, d.name(), ctx, diagnostic_level::error);
+}
+void warn_named(sema_context& ctx, const t_named& d) {
+  validate_module_name_collision(d, d.name(), ctx, diagnostic_level::warning);
+}
+
+} // namespace module_name_collision_validator
+
 std::filesystem::path program_to_path(const t_program& prog) {
   auto package = get_py3_namespace(&prog);
   return fmt::format("{}", fmt::join(package, "/"));
@@ -987,7 +1151,7 @@ class t_mstch_python_generator : public t_mstch_generator {
  public:
   using t_mstch_generator::t_mstch_generator;
 
-  std::optional<whisker_options> use_whisker() const override {
+  whisker_options render_options() const override {
     whisker_options opts;
     opts.allowed_undefined_variables = {
         "field:has_adapter?",
@@ -1016,11 +1180,23 @@ class t_mstch_python_generator : public t_mstch_generator {
     validator.add_program_visitor(validate_no_reserved_key_in_namespace);
     validator.add_enum_visitor(
         enum_member_union_field_names_validator::validate_enum);
-    validator.add_struct_visitor(
-        enum_member_union_field_names_validator::validate_structured);
+    validator.add_union_visitor(
+        enum_member_union_field_names_validator::validate_union);
+    if (get_py3_namespace(program_).empty()) {
+      validator.add_structured_definition_visitor(
+          module_name_collision_validator::validate_named);
+      validator.add_enum_visitor(
+          module_name_collision_validator::validate_named);
+      validator.add_const_visitor(
+          module_name_collision_validator::validate_named);
+      validator.add_typedef_visitor(
+          module_name_collision_validator::validate_named);
+      validator.add_interface_visitor(
+          module_name_collision_validator::warn_named);
+    }
   }
 
-  enum class IsTypesFile { Yes, No };
+  enum class TypesFileKind { NotATypesFile, SourceFile, TypeStub };
   enum class TypeKind { Abstract, Immutable, Mutable };
 
  protected:
@@ -1028,7 +1204,7 @@ class t_mstch_python_generator : public t_mstch_generator {
   void set_mstch_factories();
   void generate_file(
       const std::string& template_name,
-      IsTypesFile is_types_file,
+      TypesFileKind types_file_kind,
       TypeKind type_kind,
       const std::filesystem::path& base);
   void set_types_file(bool val);
@@ -1053,7 +1229,7 @@ class python_mstch_const : public mstch_const {
         adapter_annotation_(find_structured_adapter_annotation(*c)),
         transitive_adapter_annotation_(
             get_transitive_annotation_of_adapter_or_null(*c)) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"constant:has_adapter?", &python_mstch_const::has_adapter},
@@ -1107,12 +1283,13 @@ class python_mstch_const_value : public mstch_const_value {
       const t_const* current_const,
       const t_type* expected_type)
       : mstch_const_value(cv, ctx, pos, current_const, expected_type) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"value:py3_enum_value_name",
              &python_mstch_const_value::py3_enum_value_name},
             {"value:py3_binary?", &python_mstch_const_value::is_binary},
+            {"value:unicode_value", &python_mstch_const_value::unicode_value},
             {"value:const_enum_type",
              &python_mstch_const_value::const_enum_type},
             {"value:value_for_bool?",
@@ -1126,6 +1303,14 @@ class python_mstch_const_value : public mstch_const_value {
         });
   }
 
+  mstch::node unicode_value() {
+    if (type_ != cv::CV_STRING) {
+      return {};
+    }
+    return get_escaped_string<nonascii_handling::no_escape>(
+        const_value_->get_string());
+  }
+
   mstch::node is_binary() {
     auto& ttype = const_value_->ttype();
     return type_ == cv::CV_STRING && ttype &&
@@ -1159,10 +1344,12 @@ class python_mstch_const_value : public mstch_const_value {
   }
 
   mstch::node py3_enum_value_name() {
-    if (const_value_->is_enum() && const_value_->get_enum_value() != nullptr) {
-      return python::get_py3_name(*const_value_->get_enum_value());
+    if (!const_value_->is_enum() || const_value_->get_enum_value() == nullptr) {
+      return mstch::node();
     }
-    return mstch::node();
+    const auto& enum_name = const_value_->get_enum()->get_name();
+    return python::get_py3_name_class_scope(
+        *const_value_->get_enum_value(), enum_name);
   }
 
   mstch::node list_elem_type() {
@@ -1217,7 +1404,7 @@ class python_mstch_deprecated_annotation : public mstch_deprecated_annotation {
   python_mstch_deprecated_annotation(
       const t_annotation* a, mstch_context& ctx, mstch_element_position pos)
       : mstch_deprecated_annotation(a, ctx, pos) {
-    register_cached_methods(
+    register_methods(
         this,
         {
             {"annotation:value?",
@@ -1260,14 +1447,18 @@ void t_mstch_python_generator::set_mstch_factories() {
 
 void t_mstch_python_generator::generate_file(
     const std::string& template_name,
-    IsTypesFile is_types_file,
+    TypesFileKind types_file_kind,
     TypeKind type_kind,
     const std::filesystem::path& base = {}) {
   t_program* program = get_program();
   const std::string& program_name = program->name();
   mstch_context_
       .set_or_erase_option(
-          is_types_file == IsTypesFile::Yes, "is_types_file", "")
+          types_file_kind != TypesFileKind::NotATypesFile, "is_types_file", "")
+      .set_or_erase_option(
+          types_file_kind == TypesFileKind::SourceFile, "is_source_file", "")
+      .set_or_erase_option(
+          types_file_kind == TypesFileKind::TypeStub, "is_type_stub", "")
       .set_or_erase_option(
           type_kind == TypeKind::Abstract, "generate_abstract_types", "yes")
       .set_or_erase_option(
@@ -1285,58 +1476,53 @@ void t_mstch_python_generator::generate_file(
 }
 
 void t_mstch_python_generator::generate_types() {
-  const bool experimental_generate_mutable_types =
-      has_option("experimental_generate_mutable_types");
-  const bool experimental_unify_thrift_python_type_hints =
-      has_option("experimental_unify_thrift_python_type_hints");
+  // DO_BEFORE(satishvk, 20250130): Remove flags related to abstract types after
+  // launch.
+  const bool enable_abstract_types = !has_option("disable_abstract_types");
 
   mstch_context_.set_or_erase_option(
-      experimental_generate_mutable_types,
-      "generate_to_mutable_python_conversion_methods",
-      "true");
-  mstch_context_.set_or_erase_option(
-      experimental_unify_thrift_python_type_hints,
-      "generate_unified_thrift_python_type_hints",
-      "true");
+      enable_abstract_types, "enable_abstract_types", "true");
   generate_file(
       "thrift_types.py",
-      IsTypesFile::Yes,
+      TypesFileKind::SourceFile,
       TypeKind::Immutable,
       generate_root_path_);
   generate_file(
       "thrift_types.pyi",
-      IsTypesFile::Yes,
+      TypesFileKind::TypeStub,
+      TypeKind::Immutable,
+      generate_root_path_);
+  generate_file(
+      "thrift_enums.py",
+      TypesFileKind::SourceFile,
       TypeKind::Immutable,
       generate_root_path_);
 
-  if (experimental_unify_thrift_python_type_hints) {
-    generate_file(
-        "thrift_abstract_types.py",
-        IsTypesFile::Yes,
-        TypeKind::Abstract,
-        generate_root_path_);
-  }
-  mstch_context_.options.erase("generate_to_mutable_python_conversion_methods");
+  generate_file(
+      "thrift_abstract_types.py",
+      TypesFileKind::SourceFile,
+      TypeKind::Abstract,
+      generate_root_path_);
 
-  if (experimental_generate_mutable_types) {
-    generate_file(
-        "thrift_mutable_types.py",
-        IsTypesFile::Yes,
-        TypeKind::Mutable,
-        generate_root_path_);
-    generate_file(
-        "thrift_mutable_types.pyi",
-        IsTypesFile::Yes,
-        TypeKind::Mutable,
-        generate_root_path_);
-  }
-  mstch_context_.options.erase("generate_unified_thrift_python_type_hints");
+  generate_file(
+      "thrift_mutable_types.py",
+      TypesFileKind::SourceFile,
+      TypeKind::Mutable,
+      generate_root_path_);
+
+  generate_file(
+      "thrift_mutable_types.pyi",
+      TypesFileKind::TypeStub,
+      TypeKind::Mutable,
+      generate_root_path_);
+
+  mstch_context_.options["enable_abstract_types"] = "true";
 }
 
 void t_mstch_python_generator::generate_metadata() {
   generate_file(
       "thrift_metadata.py",
-      IsTypesFile::Yes,
+      TypesFileKind::SourceFile,
       TypeKind::Immutable,
       generate_root_path_);
 }
@@ -1350,16 +1536,15 @@ void t_mstch_python_generator::generate_clients() {
 
   generate_file(
       "thrift_clients.py",
-      IsTypesFile::No,
+      TypesFileKind::NotATypesFile,
       TypeKind::Immutable,
       generate_root_path_);
-  if (has_option("experimental_generate_mutable_types")) {
-    generate_file(
-        "thrift_mutable_clients.py",
-        IsTypesFile::No,
-        TypeKind::Mutable,
-        generate_root_path_);
-  }
+
+  generate_file(
+      "thrift_mutable_clients.py",
+      TypesFileKind::NotATypesFile,
+      TypeKind::Mutable,
+      generate_root_path_);
 }
 
 void t_mstch_python_generator::generate_services() {
@@ -1370,23 +1555,31 @@ void t_mstch_python_generator::generate_services() {
   }
   generate_file(
       "thrift_services.py",
-      IsTypesFile::No,
+      TypesFileKind::NotATypesFile,
       TypeKind::Immutable,
       generate_root_path_);
-  if (has_option("experimental_generate_mutable_types")) {
-    generate_file(
-        "thrift_mutable_services.py",
-        IsTypesFile::No,
-        TypeKind::Mutable,
-        generate_root_path_);
-  }
+
+  generate_file(
+      "thrift_mutable_services.py",
+      TypesFileKind::NotATypesFile,
+      TypeKind::Mutable,
+      generate_root_path_);
 }
 
 class t_python_patch_generator : public t_mstch_generator {
  public:
   using t_mstch_generator::t_mstch_generator;
 
-  std::string template_prefix() const override { return "python"; }
+  whisker_options render_options() const override {
+    whisker_options opts;
+    opts.allowed_undefined_variables = {
+        "field:has_adapter?",
+        "field:type",
+    };
+    return opts;
+  }
+
+  std::string template_prefix() const override { return "patch"; }
 
   void generate_program() override {
     out_dir_base_ = "gen-python-patch";
@@ -1419,26 +1612,9 @@ THRIFT_REGISTER_GENERATOR(
     mstch_python,
     "Python",
     "    include_prefix:  Use full include paths in generated files.\n"
-    "    experimental_generate_mutable_types:\n"
-    "      DO NOT USE. Enables the experimental generation of mutable\n"
-    "      thrift-python types (i.e., structs and unions). This is for local\n"
-    "      experimentation, development and testing ONLY. When this option is\n"
-    "      enabled, NO GUARANTEE is provided on any output (including any\n"
-    "      seemingly unrelated logic, such as previously existing generated\n"
-    "      code). Any \"new\" behavior (including the existence of this\n"
-    "      option) should be considered UNSTABLE and is subject to arbitrary\n"
-    "      (backwards incompatible) changes and undefined behavior.\n"
-    "      NEVER ENABLE THIS OPTION in production environments.\n"
-    "    experimental_unify_thrift_python_type_hints:\n"
-    "      DO NOT USE. Enables the experimental generation of symbols\n"
-    "      that provide type-hints for thrift-python types. This is for local\n"
-    "      experimentation, development and testing ONLY. When this option is\n"
-    "      enabled, NO GUARANTEE is provided on any output (including any\n"
-    "      seemingly unrelated logic, such as previously existing generated\n"
-    "      code). Any \"new\" behavior (including the existence of this\n"
-    "      option) should be considered UNSTABLE and is subject to arbitrary\n"
-    "      (backwards incompatible) changes and undefined behavior.\n"
-    "      NEVER ENABLE THIS OPTION in production environments.\n");
+    "    disable_abstract_types:\n"
+    "      Disable the use of abstract types with thrift-python"
+    "      immutable and mutable types.\n");
 
 namespace patch {
 THRIFT_REGISTER_GENERATOR(
