@@ -174,15 +174,19 @@ class parser {
     auto str_range = token_.range;
     consume_token();
     switch (kind) {
-      case tok::kw_include:
-        actions_.on_include(range, str, str_range);
+      case tok::kw_include: {
+        auto alias = try_parse_include_alias();
+        actions_.on_include(range, str, alias, str_range);
         break;
-      case tok::kw_cpp_include:
+      }
+      case tok::kw_cpp_include: {
         actions_.on_cpp_include(range, str);
         break;
-      case tok::kw_hs_include:
+      }
+      case tok::kw_hs_include: {
         actions_.on_hs_include(range, str);
         break;
+      }
       default:
         assert(false);
     }
@@ -194,12 +198,22 @@ class parser {
     assert(token_.kind == tok::kw_package);
     auto range = range_tracker(loc, end_);
     consume_token();
-    if (token_.kind != tok::string_literal) {
-      report_expected("string literal");
+    std::string name;
+    switch (token_.kind) {
+      case tok::string_literal:
+        name = lex_string_literal(token_);
+        consume_token();
+        if (name.empty()) {
+          report_error("package name cannot be empty");
+        }
+        try_consume_token(';');
+        break;
+      case tok::semi:
+        consume_token();
+        break;
+      default:
+        report_expected("string literal");
     }
-    auto name = lex_string_literal(token_);
-    consume_token();
-    try_consume_token(';');
     actions_.on_package(range, std::move(attrs), name);
   }
 
@@ -329,7 +343,10 @@ class parser {
           report_expected("integer, bool or string");
         }
       }
-      annotations->strings[fmt::to_string(key.str)] = {range, std::move(value)};
+      annotations->strings[fmt::to_string(key.str)] = {
+          range,
+          std::move(value),
+          deprecated_annotation_value::origin::unstructured};
       if (!try_parse_comma_or_semicolon()) {
         break;
       }
@@ -359,7 +376,7 @@ class parser {
     auto range = range_tracker(loc, end_);
     expect_and_consume(tok::kw_service);
     auto name = parse_identifier();
-    auto base = identifier();
+    identifier base;
     if (try_consume_token(tok::kw_extends)) {
       base = parse_identifier();
     }
@@ -690,11 +707,11 @@ class parser {
       qual = t_field_qualifier::none;
     }
 
-    auto type = parse_type();
-    auto name = parse_identifier();
+    t_type_ref type = parse_type();
+    const identifier name = parse_identifier();
 
     // Parse the default value.
-    auto value = std::unique_ptr<t_const_value>();
+    std::unique_ptr<t_const_value> value;
     if (try_consume_token('=')) {
       value = parse_initializer();
     }
@@ -877,6 +894,14 @@ class parser {
   //   integer | float | string_literal | bool_literal | identifier |
   //   list_initializer | map_initializer | struct_initializer
   std::unique_ptr<t_const_value> parse_initializer() {
+    range_tracker range = track_range();
+    std::unique_ptr<t_const_value> initializer_const_value =
+        parse_initializer_const_value();
+    initializer_const_value->set_src_range(range);
+    return initializer_const_value;
+  }
+
+  std::unique_ptr<t_const_value> parse_initializer_const_value() {
     auto s = sign::plus;
     switch (token_.kind) {
       case tok::bool_literal:
@@ -1036,7 +1061,7 @@ class parser {
   //   | "server"
   //   | "client"
   std::optional<identifier> try_parse_identifier() {
-    auto range = track_range();
+    range_tracker range = track_range();
     switch (token_.kind) {
       case tok::identifier:
         return identifier{consume_token().string_value(), range};
@@ -1052,6 +1077,7 @@ class parser {
       case tok::kw_permanent:
       case tok::kw_server:
       case tok::kw_client:
+      case tok::kw_as:
         return identifier{to_string(consume_token().kind), range};
       default:
         return {};
@@ -1059,7 +1085,7 @@ class parser {
   }
 
   identifier parse_identifier() {
-    if (auto id = try_parse_identifier()) {
+    if (std::optional<identifier> id = try_parse_identifier()) {
       return *id;
     }
     report_expected("identifier");
@@ -1070,6 +1096,26 @@ class parser {
     return try_consume_token(',') || try_consume_token(';');
   }
 
+  std::optional<std::string_view> try_parse_include_alias() {
+    if (!try_consume_token(tok::kw_as)) {
+      return std::nullopt;
+    }
+
+    auto alias = try_parse_identifier();
+    if (!alias.has_value()) {
+      if (token_.kind == ';') {
+        report_error("Include alias cannot be empty");
+      } else {
+        const auto invalid_alias = consume_token();
+        report_error(
+            "Invalid include alias '{}'", to_string(invalid_alias.kind.value));
+      }
+      return std::nullopt;
+    }
+
+    return alias.value().str;
+  }
+
  public:
   parser(lexer& lex, parser_actions& actions, diagnostics_engine& diags)
       : lexer_(lex), actions_(actions), diags_(diags) {}
