@@ -28,13 +28,14 @@
 #include <thrift/compiler/ast/t_const.h>
 #include <thrift/compiler/ast/t_enum.h>
 #include <thrift/compiler/ast/t_exception.h>
+#include <thrift/compiler/ast/t_global_scope.h>
 #include <thrift/compiler/ast/t_program.h>
 #include <thrift/compiler/ast/t_program_bundle.h>
-#include <thrift/compiler/ast/t_scope.h>
 #include <thrift/compiler/ast/t_service.h>
 #include <thrift/compiler/ast/t_structured.h>
 #include <thrift/compiler/ast/t_typedef.h>
 #include <thrift/compiler/ast/t_union.h>
+#include <thrift/compiler/ast/type_visitor.h>
 
 namespace apache::thrift::compiler::detail {
 namespace {
@@ -51,31 +52,54 @@ std::unique_ptr<t_const_value> val(Enm val) {
 std::unique_ptr<t_const_value> val(std::string_view s) {
   return val(std::string{s});
 }
-std::string uri_or_name(const t_named& node) {
-  if (!node.uri().empty()) {
-    return node.uri();
+
+// Returns the ProtocolObject ValueType of a t_const_value
+t_type::value_type from_const_value_type(
+    t_const_value::t_const_value_kind kind) {
+  switch (kind) {
+    case t_const_value::t_const_value_kind::CV_BOOL:
+      return t_type::value_type::BOOL;
+    case t_const_value::t_const_value_kind::CV_INTEGER:
+      return t_type::value_type::I64;
+    case t_const_value::t_const_value_kind::CV_DOUBLE:
+      return t_type::value_type::DOUBLE;
+    case t_const_value::t_const_value_kind::CV_STRING:
+      return t_type::value_type::STRING;
+    case t_const_value::t_const_value_kind::CV_MAP:
+      return t_type::value_type::MAP;
+    case t_const_value::t_const_value_kind::CV_LIST:
+      return t_type::value_type::LIST;
+    case t_const_value::t_const_value_kind::CV_IDENTIFIER:
+      return t_type::value_type::STRING;
   }
-  if (node.program()) {
-    return node.program()->scope_name(node);
-  }
-  return node.name();
+  abort();
 }
 } // namespace
 
 t_type_ref schematizer::std_type(std::string_view uri) {
   return t_type_ref::from_req_ptr(
-      static_cast<const t_type*>(scope_.find_by_uri(uri)));
+      static_cast<const t_type*>(global_scope_.find_by_uri(uri)));
+}
+
+// Returns the `TypeUri` type & the corresponding Uri value for the given node
+schematizer::resolved_uri schematizer::calculate_uri(
+    const t_named& node, const bool use_hash) {
+  if (use_hash) {
+    return {"definitionKey", identify_definition(node)};
+  }
+  if (!node.uri().empty()) {
+    return {"uri", node.uri()};
+  }
+  if (node.program()) {
+    return {"scopedName", node.program()->scoped_name(node)};
+  }
+  return {"scopedName", node.name()};
 }
 
 std::unique_ptr<t_const_value> schematizer::type_uri(const t_type& type) {
   auto ret = t_const_value::make_map();
-  if (opts_.use_hash) {
-    ret->add_map(val("definitionKey"), val(identify_definition(type)));
-  } else if (!type.uri().empty()) {
-    ret->add_map(val("uri"), val(type.uri()));
-  } else {
-    ret->add_map(val("scopedName"), val(type.get_scoped_name()));
-  }
+  auto uri = calculate_uri(type, opts_.use_hash);
+  ret->add_map(val(uri.uri_type), val(std::move(uri.value)));
   ret->set_ttype(std_type("facebook.com/thrift/type/TypeUri"));
   return ret;
 }
@@ -100,17 +124,21 @@ void schematizer::add_definition(
       !structured.empty() && opts_.include_annotations) {
     auto annots = t_const_value::make_map();
     auto structured_annots = t_const_value::make_list();
+    auto annots_by_key = t_const_value::make_map();
 
     for (const auto& item : structured) {
       auto annot = t_const_value::make_map();
+
+      const auto ty_wrapper = protocol_value_builder{*item.type()};
+
       if (!item.value()->is_empty()) {
         auto protocol_value_ttype =
             std_type("facebook.com/thrift/protocol/Value");
         auto fields = t_const_value::make_map();
-        for (const auto& pair : item.value()->get_map()) {
+        for (const auto& [key, value] : item.value()->get_map()) {
           fields->add_map(
-              pair.first->clone(),
-              wrap_with_protocol_value(*pair.second, protocol_value_ttype));
+              key->clone(),
+              ty_wrapper.property(*key).wrap(*value, protocol_value_ttype));
         }
         annot->add_map(val("fields"), std::move(fields));
       }
@@ -128,7 +156,14 @@ void schematizer::add_definition(
       }
 
       annot->set_ttype(std_type("facebook.com/thrift/type/Annotation"));
-      annots->add_map(val(uri_or_name(*item.type())), std::move(annot));
+      auto unhashed_uri = calculate_uri(*item.type(), false /*use_hash*/);
+      // We're not hashing & ignoring the UriType here, as annotations are
+      // stored as map<string, Annotation>.
+      annots->add_map(val(std::move(unhashed_uri.value)), annot->clone());
+
+      auto hashed_uri = calculate_uri(*item.type(), true /*use_hash*/);
+      annots_by_key->add_map(
+          val(std::move(hashed_uri.value)), std::move(annot));
     }
 
     // Double write to deprecated externed path (T161963504).
@@ -136,10 +171,12 @@ void schematizer::add_definition(
       definition->add_map(
           val("structuredAnnotations"), std::move(structured_annots));
     }
+
     definition->add_map(val("annotations"), std::move(annots));
+    definition->add_map(val("annotationsByKey"), std::move(annots_by_key));
   }
 
-  if (auto unstructured = node.annotations();
+  if (auto unstructured = node.unstructured_annotations();
       !unstructured.empty() && opts_.include_annotations) {
     auto annots = t_const_value::make_map();
 
@@ -291,7 +328,7 @@ std::unique_ptr<t_const_value> schematizer::gen_type(
           type_uri(*resolved_type));
       break;
     }
-    default:
+    case t_type::type::t_service:
       assert(false);
   }
   schema->add_map(val("name"), std::move(type_name));
@@ -379,9 +416,9 @@ void schematize_recursively(
 
 const t_enum* find_enum(const t_program* program, const std::string& enum_uri) {
   // May be null in unit tests.
-  return program
-      ? dynamic_cast<const t_enum*>(program->scope()->find_by_uri(enum_uri))
-      : nullptr;
+  return program ? dynamic_cast<const t_enum*>(
+                       program->global_scope()->find_by_uri(enum_uri))
+                 : nullptr;
 }
 
 void add_qualifier(const t_enum* t_enum, t_const_value& schema, int enum_val) {
@@ -476,7 +513,7 @@ std::unique_ptr<t_const_value> schematizer::gen_full_schema(
   for (const auto& func : node.functions()) {
     const t_type_ref& ret = func.return_type();
     // TODO: Handle sink, stream, interactions
-    if (!func.sink_or_stream() && !ret->is_service()) {
+    if (!func.sink_or_stream()) {
       schematize_recursively(
           this, node.program(), dfns_schema.get(), *ret->get_true_type());
     }
@@ -715,39 +752,215 @@ std::unique_ptr<t_const_value> schematizer::gen_schema(const t_typedef& node) {
   return schema;
 }
 
-std::unique_ptr<t_const_value> wrap_with_protocol_value(
-    const t_const_value& value, t_type_ref ttype) {
-  auto ret = t_const_value::make_map();
-  ret->set_ttype(ttype);
-  switch (value.kind()) {
-    case t_const_value::CV_BOOL:
-      ret->add_map(val("boolValue"), value.clone());
-      break;
+const char* protocol_value_type_name(t_type::value_type ty) {
+  switch (ty) {
+    case t_type::value_type::BOOL:
+      return "boolValue";
+    case t_type::value_type::BYTE:
+      return "byteValue";
+    case t_type::value_type::I16:
+      return "i16Value";
+    case t_type::value_type::I32:
+      return "i32Value";
+    case t_type::value_type::I64:
+      return "i64Value";
+    case t_type::value_type::FLOAT:
+      return "floatValue";
+    case t_type::value_type::DOUBLE:
+      return "doubleValue";
+    case t_type::value_type::STRING:
+      return "stringValue";
+    case t_type::value_type::BINARY:
+      return "binaryValue";
+    case t_type::value_type::OBJECT:
+      return "objectValue";
+    case t_type::value_type::LIST:
+      return "listValue";
+    case t_type::value_type::SET:
+      return "setValue";
+    case t_type::value_type::MAP:
+      return "mapValue";
+  }
+  abort();
+}
+
+protocol_value_builder::protocol_value_builder(const t_type& struct_ty)
+    : ty_{struct_ty.get_true_type()} {}
+
+protocol_value_builder::protocol_value_builder() : ty_{nullptr} {}
+
+[[nodiscard]] protocol_value_builder protocol_value_builder::as_value_type() {
+  return protocol_value_builder{};
+}
+
+[[nodiscard]] protocol_value_builder protocol_value_builder::property(
+    const t_const_value& key) const {
+  if (ty_ == nullptr) {
+    return as_value_type();
+  }
+
+  return ty_->visit(
+      [&](const t_map& map) {
+        // Maps are not restricted to string-based field keys, so we should
+        // extend the look-up to any sealed key type.
+        return protocol_value_builder{*map.get_val_type()};
+      },
+      [&](const t_structured& strct) {
+        assert(
+            key.kind() == t_const_value::CV_STRING &&
+            "A struct only has named fields");
+        const auto* field = strct.get_field_by_name(key.get_string());
+        return protocol_value_builder{*field->get_type()};
+      },
+      [&](auto&&) -> protocol_value_builder {
+        throw std::logic_error(fmt::format(
+            "Invalid name={} type={} for property look-up",
+            ty_->get_full_name(),
+            t_type::type_name(ty_->get_type_value())));
+      });
+}
+
+[[nodiscard]] protocol_value_builder protocol_value_builder::key(
+    [[maybe_unused]] const t_const_value& key) const {
+  if (ty_ == nullptr) {
+    return as_value_type();
+  }
+
+  return ty_->visit(
+      [&](const t_structured&) {
+        assert(
+            key.kind() == t_const_value::CV_STRING &&
+            "A struct only has named fields");
+        return as_value_type();
+      },
+      [&](const t_map& map) {
+        return protocol_value_builder{*map.get_key_type()};
+      },
+      [&](auto&&) -> protocol_value_builder {
+        throw std::logic_error(fmt::format(
+            "Invalid name={} type={} for key look-up",
+            ty_->get_full_name(),
+            t_type::type_name(ty_->get_type_value())));
+      });
+}
+
+[[nodiscard]] protocol_value_builder protocol_value_builder::container_element(
+    const t_const_value& container) const {
+  assert(container.kind() == t_const_value::CV_LIST);
+  if (container.get_list().empty()) {
+    // If the list is empty, we don't care what the type resolves to
+    return as_value_type();
+  }
+
+  return ty_->visit(
+      [&](const t_list& list) {
+        return protocol_value_builder{*list.elem_type()};
+      },
+      [&](const t_set& set) {
+        return protocol_value_builder{*set.elem_type()};
+      },
+      [&](const t_map&) -> protocol_value_builder {
+        throw std::logic_error(
+            "Maps should not deduce a single container element");
+      },
+      [&](auto&&) -> protocol_value_builder {
+        throw std::logic_error(fmt::format(
+            "Invalid name={} type={} for container element look-up",
+            ty_->get_full_name(),
+            t_type::type_name(ty_->get_type_value())));
+      });
+}
+
+std::pair<std::unique_ptr<t_const_value>, std::unique_ptr<t_const_value>>
+protocol_value_builder::to_labeled_value(
+    const t_const_value& protocol_value) const {
+  if (ty_ == nullptr) {
+    auto label = val(
+        protocol_value_type_name(from_const_value_type(protocol_value.kind())));
+    return std::make_pair(std::move(label), protocol_value.clone());
+  }
+
+  std::unique_ptr<t_const_value> value;
+  auto raise_exception = [&] {
+    throw std::runtime_error(fmt::format(
+        "Could not match value of kind {} to type {}",
+        t_const_value::kind_to_string(protocol_value.kind()),
+        t_type::type_name(ty_->get_type_value())));
+  };
+
+  // Verify that the field is a valid type for a protocol value key
+  switch (protocol_value.kind()) {
     case t_const_value::CV_INTEGER:
-      ret->add_map(val("i64Value"), value.clone());
+      if (!(ty_->is_any_int() || ty_->is_enum() || ty_->is_floating_point())) {
+        raise_exception();
+      }
+      if (ty_->is_floating_point()) {
+        // Coalesce to double
+        value = std::make_unique<t_const_value>();
+        value->set_double(protocol_value.get_integer());
+      } else {
+        value = protocol_value.clone();
+      }
       break;
     case t_const_value::CV_DOUBLE:
-      ret->add_map(val("doubleValue"), value.clone());
+      if (!ty_->is_floating_point()) {
+        raise_exception();
+      }
+      value = protocol_value.clone();
       break;
     case t_const_value::CV_STRING:
-      ret->add_map(val("stringValue"), value.clone());
+      if (!ty_->is_string_or_binary()) {
+        raise_exception();
+      }
+      value = protocol_value.clone();
       break;
+    case t_const_value::CV_BOOL:
+    case t_const_value::CV_MAP:
+    case t_const_value::CV_LIST:
+    case t_const_value::CV_IDENTIFIER:
+      // These all have unequivocal keys
+      value = protocol_value.clone();
+      break;
+  }
+
+  // Generate a protocol value label for a given `t_const_value`, based on the
+  // type of the field it represents. e.g. `i64Value` for `i64`.
+  const auto obj_val_ty = ty_->as_value_type().value();
+  auto label = val(protocol_value_type_name(obj_val_ty));
+
+  return std::make_pair(std::move(label), std::move(value));
+}
+
+std::unique_ptr<t_const_value> protocol_value_builder::wrap(
+    const t_const_value& protocol_value, t_type_ref ttype) const {
+  auto ret = t_const_value::make_map();
+  ret->set_ttype(ttype);
+  switch (protocol_value.kind()) {
+    case t_const_value::CV_BOOL:
+    case t_const_value::CV_INTEGER:
+    case t_const_value::CV_DOUBLE:
+    case t_const_value::CV_STRING: {
+      auto [value_label, value] = to_labeled_value(protocol_value);
+      ret->add_map(std::move(value_label), std::move(value));
+      break;
+    }
     case t_const_value::CV_MAP: {
       auto map = t_const_value::make_map();
-      for (const auto& map_elem : value.get_map()) {
-        map->add_map(
-            wrap_with_protocol_value(*map_elem.first, ttype),
-            wrap_with_protocol_value(*map_elem.second, ttype));
+      for (const auto& [k, v] : protocol_value.get_map()) {
+        map->add_map(key(*k).wrap(*k, ttype), property(*k).wrap(*v, ttype));
       }
       ret->add_map(val("mapValue"), std::move(map));
       break;
     }
     case t_const_value::CV_LIST: {
       auto list = t_const_value::make_list();
-      for (const auto& list_elem : value.get_list()) {
-        list->add_list(wrap_with_protocol_value(*list_elem, ttype));
+      auto list_ty_resolver = container_element(protocol_value);
+      for (const auto& list_elem : protocol_value.get_list()) {
+        list->add_list(list_ty_resolver.wrap(*list_elem, ttype));
       }
-      ret->add_map(val("listValue"), std::move(list));
+      auto container_value_type = ty_->as_value_type().value();
+      ret->add_map(
+          val(protocol_value_type_name(container_value_type)), std::move(list));
       break;
     }
     case t_const_value::CV_IDENTIFIER:
@@ -770,6 +983,10 @@ std::string_view schematizer::program_checksum(const t_program& program) {
           std::string(reinterpret_cast<const char*>(hash), sizeof(hash)));
 }
 
+size_t schematizer::definition_identifier_length() {
+  return 16;
+}
+
 std::string schematizer::identify_definition(const t_named& node) {
   // @lint-ignore CLANGTIDY facebook-hte-CArray
   unsigned char hash[SHA256_DIGEST_LENGTH];
@@ -779,8 +996,8 @@ std::string schematizer::identify_definition(const t_named& node) {
       node.program()->path(),
       node.name());
   SHA256(reinterpret_cast<const unsigned char*>(val.c_str()), val.size(), hash);
-  constexpr size_t num_bytes = 16;
-  return std::string(reinterpret_cast<const char*>(hash), num_bytes);
+  return std::string(
+      reinterpret_cast<const char*>(hash), definition_identifier_length());
 }
 
 int64_t schematizer::identify_program(const t_program& node) {
@@ -798,7 +1015,7 @@ int64_t schematizer::identify_program(const t_program& node) {
 
 std::string schematizer::name_schema(
     source_manager& sm, const t_program& node) {
-  schematizer s(*node.scope(), sm, {});
+  schematizer s(*node.global_scope(), sm, {});
   return fmt::format(
       "_fbthrift_schema_{:x}", static_cast<uint64_t>(s.identify_program(node)));
 }
