@@ -18,7 +18,6 @@
 
 #include <algorithm>
 #include <functional>
-#include <type_traits>
 #include <unordered_set>
 
 #include <thrift/compiler/ast/t_program_bundle.h>
@@ -32,6 +31,10 @@
 namespace apache::thrift::compiler {
 namespace {
 
+void report_missing_type(sema_context& ctx, const t_named& named) {
+  ctx.error(named, "Type `{}` not defined.", named.name());
+}
+
 // Mutators have mutable access to the AST.
 struct mutator_context : visitor_context {
   t_program_bundle* bundle;
@@ -59,11 +62,33 @@ class ast_mutator
 /// An AST mutator that replaces placeholder_typedefs with resolved types.
 class type_ref_resolver {
  private:
+  sema_context& ctx_;
+  t_program_bundle& bundle_;
   bool unresolved_ = false;
 
  public:
+  explicit type_ref_resolver(sema_context& ctx, t_program_bundle& bundle)
+      : ctx_{ctx}, bundle_{bundle} {}
+
+  const t_type* resolve_implicit_includes(const t_placeholder_typedef& td) {
+    const scope::identifier id{td.name()};
+    return dynamic_cast<const t_type*>(bundle_.root_program()->find(id));
+  }
+
   void resolve_in_place(t_type_ref& ref) {
-    unresolved_ = !ref.resolve() || unresolved_;
+    if (ref.resolve()) {
+      return;
+    }
+
+    if (const auto* node =
+            resolve_implicit_includes(*ref.get_unresolved_type())) {
+      ref = t_type_ref{*node};
+      ref.resolve();
+      assert(ref.resolved());
+      return;
+    }
+
+    unresolved_ = true;
   }
 
   [[nodiscard]] t_type_ref resolve(t_type_ref ref) {
@@ -71,9 +96,8 @@ class type_ref_resolver {
     return ref;
   }
 
-  bool run(sema_context& ctx, t_program_bundle& bundle) {
+  bool run() {
     ast_mutator mutator;
-
     auto resolve_const_value = [&](t_const_value& node, auto& recurse) -> void {
       node.set_ttype(resolve(node.ttype()));
 
@@ -104,18 +128,19 @@ class type_ref_resolver {
         });
 
     mutator.add_function_visitor(
-        [&](sema_context& ctx, mutator_context& mctx, t_function& node) {
+        [&](sema_context&, mutator_context&, t_function& node) {
           resolve_in_place(node.return_type());
           resolve_in_place(node.interaction());
-          for (auto& field : node.params().fields()) {
-            mutator(ctx, mctx, field);
-          }
         });
-    mutator.add_throws_visitor(
-        [&](sema_context& ctx, mutator_context& mctx, t_throws& node) {
-          for (auto& field : node.fields()) {
-            mutator(ctx, mctx, field);
-          }
+    mutator.add_function_param_visitor(
+        [&](sema_context& ctx, mutator_context& mctx, t_field& param) {
+          // Delegates to field visitor.
+          mutator(ctx, mctx, param);
+        });
+    mutator.add_thrown_exception_visitor(
+        [&](sema_context& ctx, mutator_context& mctx, t_field& field) {
+          // Delegates to field visitor.
+          mutator(ctx, mctx, field);
         });
     mutator.add_stream_visitor(
         [&](sema_context&, mutator_context&, t_stream& node) {
@@ -145,14 +170,71 @@ class type_ref_resolver {
           resolve_const_value(*node.value(), resolve_const_value);
         });
 
-    mutator.mutate(ctx, bundle);
+    mutator.mutate(ctx_, bundle_);
     return !unresolved_;
   }
 };
 
+const t_const* try_resolve_enum_by_id(
+    const scope::identifier id, const t_const_value& value) {
+  const auto resolve_enum_alias =
+      [&](const scope::identifier enum_or_alias) -> const t_enum* {
+    const t_named* enum_ty = value.program().find(enum_or_alias);
+    if (const auto* enum_typedef = ast_detail::as<t_typedef>(enum_ty)) {
+      return ast_detail::as<t_enum>(enum_typedef->get_true_type());
+    }
+    return ast_detail::as<t_enum>(enum_ty);
+  };
+
+  const auto resolve_value =
+      [&](const scope::identifier value_id) -> const t_const* {
+    return value.program().find<t_const>(value_id);
+  };
+
+  const auto resolve_maybe_aliased_enum_with_value =
+      [&](const scope::identifier enum_alias,
+          const std::string_view value_name) -> const t_const* {
+    if (const t_enum* enum_node = resolve_enum_alias(enum_alias)) {
+      return enum_node->find_const_by_name(value_name);
+    }
+    return nullptr;
+  };
+
+  return id.visit(
+      [&](const scope::unscoped_id& id) {
+        // `id` is just the enum value name, e.g. MyEnum a = SOMETHING;
+        // So try to find `SOMETHING`.
+        return resolve_value(id);
+      },
+      [&](const scope::scoped_id& id) -> const t_const* {
+        // `id` is either the program scope or the enum name followed by
+        // the value e.g.
+        // 1. MyEnum a = my_prog.MY_VALUE;
+        // 2. MyEnum a = MY_ALIAS.VALUE;
+
+        // (1) should be resolvable as-is.
+        if (const auto* node = value.program().find<t_const>(id)) {
+          return node;
+        }
+
+        // (2) requires resolving the enum alias, then finding its value.
+        return resolve_maybe_aliased_enum_with_value(id.scope, id.name);
+      },
+      [&](const scope::enum_id& id) {
+        // `id` is the fully qualified name, e.g.
+        // MyEnum a = my_prog.MY_NAME.SOMETHING;
+        // `MY_NAME` is either:
+        // 1. An enum type
+        // 2. A typedef to an enum type
+        // So we'll try to resolve the enum type, then find the value.
+        return resolve_maybe_aliased_enum_with_value(
+            scope::scoped_id{id.scope, id.enum_name}, id.value_name);
+      });
+}
+
 void match_type_with_const_value(
     sema_context& ctx,
-    const t_program& program,
+    mutator_context& mctx,
     const t_type* long_type,
     t_const_value* value) {
   const t_type* type = long_type->get_true_type();
@@ -161,12 +243,35 @@ void match_type_with_const_value(
     return;
   }
 
+  // Verify that the const value is correctly resolved at this point.
+  if (value && value->kind() == t_const_value::CV_IDENTIFIER) {
+    const std::string& value_id = value->get_identifier();
+    const scope::identifier id{value_id};
+    const t_const* constant;
+    if (type->get_type_value() == t_type::type::t_enum) {
+      // Try to resolve enum values from typedefs
+      // or enums defined after use.
+      constant = try_resolve_enum_by_id(id, *value);
+    } else {
+      constant = mctx.program().find<t_const>(id);
+    }
+
+    if (!constant) {
+      ctx.error(
+          value->ref_range().begin,
+          "use of undeclared identifier '{}'",
+          value_id);
+      return;
+    }
+    value->assign(t_const_value(*constant->value()));
+  }
+
   switch (type->get_type_value()) {
     case t_type::type::t_list: {
       auto* elem_type = dynamic_cast<const t_list*>(type)->get_elem_type();
       if (value->kind() == t_const_value::CV_LIST) {
         for (auto list_val : value->get_list()) {
-          match_type_with_const_value(ctx, program, elem_type, list_val);
+          match_type_with_const_value(ctx, mctx, elem_type, list_val);
         }
       }
       break;
@@ -175,7 +280,7 @@ void match_type_with_const_value(
       auto* elem_type = dynamic_cast<const t_set*>(type)->get_elem_type();
       if (value->kind() == t_const_value::CV_LIST) {
         for (auto set_val : value->get_list()) {
-          match_type_with_const_value(ctx, program, elem_type, set_val);
+          match_type_with_const_value(ctx, mctx, elem_type, set_val);
         }
       }
       break;
@@ -185,33 +290,23 @@ void match_type_with_const_value(
       auto* val_type = dynamic_cast<const t_map*>(type)->get_val_type();
       if (value->kind() == t_const_value::CV_MAP) {
         for (auto map_val : value->get_map()) {
-          match_type_with_const_value(ctx, program, key_type, map_val.first);
-          match_type_with_const_value(ctx, program, val_type, map_val.second);
+          match_type_with_const_value(ctx, mctx, key_type, map_val.first);
+          match_type_with_const_value(ctx, mctx, val_type, map_val.second);
         }
       }
       break;
     }
     case t_type::type::t_structured: {
       const auto* structured = dynamic_cast<const t_structured*>(type);
-      if (auto ttype = value->ttype();
-          ttype && ttype->get_true_type() != type) {
-        ctx.error(
-            value->ref_range().begin,
-            "type mismatch: expected {}, got {}",
-            type->get_full_name(),
-            ttype->get_full_name());
-      }
-      if (value->kind() == t_const_value::CV_IDENTIFIER) {
-        const std::string& id = value->get_identifier();
-        const t_const* constant = program.scope()->find_constant(id);
-        if (!constant) {
+      if (auto ttype = value->ttype()) {
+        assert(ttype.resolved());
+        if (ttype->get_true_type() != type) {
           ctx.error(
               value->ref_range().begin,
-              "use of undeclared identifier '{}'",
-              id);
-          return;
+              "type mismatch: expected {}, got {}",
+              type->get_full_name(),
+              ttype->get_full_name());
         }
-        value->assign(t_const_value(*constant->value()));
       }
       if (value->kind() == t_const_value::CV_MAP) {
         for (const auto& [map_key, map_val] : value->get_map()) {
@@ -224,9 +319,10 @@ void match_type_with_const_value(
             return;
           }
           if (!resolved) {
+            // TODO(sadroeck) - Deprecate this behavior
             map_key->convert_identifier_to_string();
           }
-          match_type_with_const_value(ctx, program, field->get_type(), map_val);
+          match_type_with_const_value(ctx, mctx, field->get_type(), map_val);
         }
       }
       break;
@@ -242,38 +338,6 @@ void match_type_with_const_value(
           if (const auto* enum_value = enm->find_value(value->get_integer())) {
             value->set_enum_value(enum_value);
           }
-        } else if (value->kind() == t_const_value::CV_IDENTIFIER) {
-          // Resolve enum values defined after use.
-          const std::string& id = value->get_identifier();
-          const t_const* constant = program.scope()->find_constant(id);
-          if (!constant) {
-            constant =
-                program.scope()->find_constant(value->program().scope_name(id));
-          }
-          if (!constant) {
-            // Try to resolve enum values from typedefs.
-            auto last_dot_pos = id.find_last_of('.');
-            std::string enum_name = id.substr(0, last_dot_pos);
-            if (std::count(enum_name.begin(), enum_name.end(), '.') == 0) {
-              enum_name = value->program().scope_name(enum_name);
-            }
-            if (auto* def = dynamic_cast<const t_type*>(
-                    program.scope()->find(enum_name))) {
-              if (auto* enum_def =
-                      dynamic_cast<const t_enum*>(def->get_true_type())) {
-                constant =
-                    enum_def->find_const_by_name(id.substr(last_dot_pos + 1));
-              }
-            }
-          }
-          if (!constant) {
-            ctx.error(
-                value->ref_range().begin,
-                "use of undeclared identifier '{}'",
-                id);
-            return;
-          }
-          value->assign(t_const_value(*constant->value()));
         }
       }
       break;
@@ -293,8 +357,6 @@ void match_type_with_const_value(
       break;
     case t_type::type::t_void:
     case t_type::type::t_service:
-    case t_type::type::t_stream:
-    case t_type::type::t_program:
       assert(false);
   }
 
@@ -310,7 +372,7 @@ void maybe_match_type_with_const_value(
     return;
   }
 
-  match_type_with_const_value(ctx, mctx.program(), type, value);
+  match_type_with_const_value(ctx, mctx, type, value);
 }
 
 void match_const_type_with_value(
@@ -339,7 +401,7 @@ void mutate_terse_write_annotation_structured(
       ctx.program().inherit_annotation_or_null(node, kTerseWriteUri);
   for (auto& field : node.fields()) {
     bool field_has_terse_write =
-        field.find_structured_annotation_or_null(kTerseWriteUri);
+        field.has_structured_annotation(kTerseWriteUri);
     if (!field_has_terse_write && !program_has_terse_write) {
       continue;
     }
@@ -374,13 +436,16 @@ void mutate_inject_metadata_fields(
     ctx.error("{}", e.what());
     return;
   }
-  // If the specified type and annotation are from the same program, append
-  // the current program name.
-  if (type_string.find(".") == std::string::npos) {
-    type_string = annotation->program()->name() + "." + type_string;
-  }
+  const scope::identifier id{type_string};
 
-  const auto* ttype = node.program()->scope()->find_type(type_string);
+  const t_type* ttype = node.program()->find<t_type>(id);
+  if (!ttype && annotation->program()) {
+    // [TEMPORARY] Allow injected metadata fields to use the scope name
+    // of the program they're defined in. e.g.
+    // @internal.InjectMetadataFields{type = "Bar"} in foo.thrift should be
+    // resolved as foo.Bar, not just Bar.
+    ttype = annotation->program()->find<t_type>(id);
+  }
   if (!ttype) {
     ctx.error(
         "Can not find expected type `{}` specified in "
@@ -393,7 +458,7 @@ void mutate_inject_metadata_fields(
   const auto* structured = dynamic_cast<const t_structured*>(ttype);
   // We only allow injecting fields from a struct type.
   if (structured == nullptr || ttype->is_union() || ttype->is_exception() ||
-      ttype->is_paramlist()) {
+      ttype->is<t_paramlist>()) {
     ctx.error(
         "`{}` is not a struct type. `@internal.InjectMetadataFields` can be "
         "only used with a struct type.",
@@ -419,12 +484,83 @@ void mutate_inject_metadata_fields(
   }
 }
 
+// Strips haskell annotations and optionally inserts new annotations.
+void update_annotations(
+    t_named& node,
+    std::map<std::string, std::string> new_annotations = {},
+    deprecated_annotation_value::origin new_annotation_origin = {}) {
+  auto annotations = node.unstructured_annotations();
+  // First strip any haskell annotations
+  for (auto it = annotations.begin(); it != annotations.end();) {
+    if (it->first.find("hs.") == 0) {
+      it = annotations.erase(it);
+    } else {
+      ++it;
+    }
+  }
+  for (auto& [k, v] : new_annotations) {
+    annotations[k] = {source_range{}, v, new_annotation_origin};
+  }
+  node.reset_annotations(std::move(annotations));
+}
+
+// Updates field or typedef's type field to hold annotations
+template <typename Node>
+void add_annotations_to_node_type(
+    Node& node,
+    std::map<std::string, std::string> annotations,
+    t_program& program,
+    deprecated_annotation_value::origin origin) {
+  const t_type* node_type = node.get_type();
+
+  if (annotations.empty()) {
+    if (!node_type->unstructured_annotations().empty()) {
+      update_annotations(const_cast<t_type&>(*node_type));
+    }
+    return;
+  }
+
+  if (node_type->is_container() ||
+      (node_type->is_typedef() &&
+       static_cast<const t_typedef*>(node_type)->typedef_kind() !=
+           t_typedef::kind::defined) ||
+      (node_type->is_primitive_type() &&
+       !node_type->unstructured_annotations().empty())) {
+    // This is a new type we can modify in place
+    update_annotations(
+        const_cast<t_type&>(*node_type), std::move(annotations), origin);
+  } else if (node_type->is_primitive_type()) {
+    // Copy type as we don't handle unnamed typedefs to base types :(
+    auto unnamed = std::make_unique<t_primitive_type>(
+        *static_cast<const t_primitive_type*>(node_type));
+    for (auto& pair : annotations) {
+      unnamed->set_unstructured_annotation(pair.first, pair.second, {}, origin);
+    }
+    node.set_type(t_type_ref::from_ptr(unnamed.get()));
+    program.add_unnamed_type(std::move(unnamed));
+  } else {
+    // Wrap in an unnamed typedef :(
+    auto unnamed = t_typedef::make_unnamed(
+        const_cast<t_program*>(node_type->get_program()),
+        node_type->get_name(),
+        t_type_ref::from_ptr(node_type));
+    for (auto& pair : annotations) {
+      unnamed->set_unstructured_annotation(pair.first, pair.second, {}, origin);
+    }
+    node.set_type(t_type_ref::from_ptr(unnamed.get()));
+    program.add_unnamed_typedef(std::move(unnamed));
+  }
+}
+
 void lower_deprecated_annotations(
-    sema_context& ctx, mutator_context&, t_named& node) {
+    sema_context& ctx, mutator_context& mCtx, t_named& node) {
   if (auto cnst = node.find_structured_annotation_or_null(
           kDeprecatedUnvalidatedAnnotationsUri)) {
     ctx.check(
-        node.annotations().empty(),
+        std::all_of(
+            node.unstructured_annotations().begin(),
+            node.unstructured_annotations().end(),
+            [](const auto& pair) { return pair.first.find("hs.") == 0; }),
         "Cannot combine @thrift.DeprecatedUnvalidatedAnnotations with legacy annotation syntax.");
     auto val = cnst->get_value_from_structured_annotation_or_null("items");
     if (!val || val->get_map().empty()) {
@@ -432,11 +568,75 @@ void lower_deprecated_annotations(
           "Must specify at least one item in @thrift.DeprecatedUnvalidatedAnnotations.");
       return;
     }
-    deprecated_annotation_map map;
-    for (auto& [k, v] : val->get_map()) {
-      map[k->get_string()] = {{}, v->get_string()};
+    if (!ctx.sema_parameters().skip_lowering_annotations) {
+      deprecated_annotation_map map;
+      for (auto& [k, v] : val->get_map()) {
+        map[k->get_string()] = {
+            {},
+            v->get_string(),
+            deprecated_annotation_value::origin::lowered_unstructured};
+      }
+
+      // The java generator has some interesting logic due to limitations in
+      // mustache that are being fixed in whisker. Until that lands, we
+      // propagate the affected annotations to the type of a typedef in order to
+      // unblock deprecating unstructured annotations.
+      if (map.count("java.swift.type") ||
+          map.count("java.swift.binary_string")) {
+        auto annot = map.count("java.swift.type") ? "java.swift.type"
+                                                  : "java.swift.binary_string";
+        auto* type = dynamic_cast<t_typedef*>(&node);
+        if (!type) {
+          ctx.error("Annotation {} is only supported on typedefs.", annot);
+          return;
+        }
+
+        auto* inner_type = const_cast<t_type*>(type->get_type());
+        if (inner_type->is_typedef()) {
+          ctx.error("Cannot use {} on typedefs of typedefs", annot);
+        } else if (
+            !inner_type->is_container() && !inner_type->is_primitive_type()) {
+          ctx.error(
+              "Annotation {} is only supported on typedefs of primitive or container types.",
+              annot);
+        } else {
+          // Ensure annotations can be added to inner type
+          if (inner_type->is_primitive_type() &&
+              inner_type->unstructured_annotations().empty()) {
+            auto new_type = std::make_unique<t_primitive_type>(
+                static_cast<const t_primitive_type&>(*inner_type));
+            inner_type = new_type.get();
+            type->set_type(t_type_ref::from_ptr(inner_type));
+            mCtx.program().add_unnamed_type(std::move(new_type));
+          }
+
+          inner_type->set_unstructured_annotation(
+              annot,
+              map[annot].value,
+              {},
+              deprecated_annotation_value::origin::lowered_unstructured);
+        }
+
+        map.erase(annot);
+      }
+
+      // cpp.indirection does not handle typedefs correctly
+      if (auto* typedf = dynamic_cast<t_typedef*>(&node);
+          typedf && map.count("cpp.indirection")) {
+        add_annotations_to_node_type(
+            *typedf,
+            {{"cpp.indirection", map.at("cpp.indirection").value}},
+            mCtx.program(),
+            deprecated_annotation_value::origin::lowered_unstructured);
+        map.erase("cpp.indirection");
+      }
+
+      node.reset_annotations(std::move(map));
+    } else {
+      update_annotations(node);
     }
-    node.reset_annotations(std::move(map));
+  } else {
+    update_annotations(node);
   }
 }
 
@@ -463,7 +663,7 @@ void lower_type_annotations(
     sema_context& ctx, mutator_context& mctx, Node& node) {
   std::map<std::string, std::string> unstructured;
 
-  if (!ctx.sema_parameters().skip_lowering_type_annotations) {
+  if (!ctx.sema_parameters().skip_lowering_annotations) {
     if (const t_const* annot =
             node.find_structured_annotation_or_null(kCppTypeUri)) {
       if (auto type =
@@ -477,46 +677,18 @@ void lower_type_annotations(
     }
   }
 
-  if (unstructured.empty()) {
-    return;
-  }
-
-  const t_type* node_type = node.get_type();
-  if (node_type->is_container() ||
-      (node_type->is_typedef() &&
-       static_cast<const t_typedef*>(node_type)->typedef_kind() !=
-           t_typedef::kind::defined) ||
-      (node_type->is_primitive_type() && !node_type->annotations().empty())) {
-    // This is a new type we can modify in place
-    for (auto& pair : unstructured) {
-      const_cast<t_type*>(node_type)->set_annotation(pair.first, pair.second);
-    }
-  } else if (node_type->is_primitive_type()) {
-    // Copy type as we don't handle unnamed typedefs to base types :(
-    auto& program = mctx.program();
-    auto unnamed = std::make_unique<t_primitive_type>(
-        *static_cast<const t_primitive_type*>(node_type));
-    for (auto& pair : unstructured) {
-      unnamed->set_annotation(pair.first, pair.second);
-    }
-    node.set_type(t_type_ref::from_ptr(unnamed.get()));
-    program.add_unnamed_type(std::move(unnamed));
-  } else {
-    // Wrap in an unnamed typedef :(
-    auto& program = mctx.program();
-    auto unnamed = t_typedef::make_unnamed(
-        const_cast<t_program*>(node_type->get_program()),
-        node_type->get_name(),
-        t_type_ref::from_ptr(node_type));
-    for (auto& pair : unstructured) {
-      unnamed->set_annotation(pair.first, pair.second);
-    }
-    node.set_type(t_type_ref::from_ptr(unnamed.get()));
-    program.add_unnamed_typedef(std::move(unnamed));
-  }
+  add_annotations_to_node_type(
+      node,
+      std::move(unstructured),
+      mctx.program(),
+      deprecated_annotation_value::origin::lowered_cpp_type);
 }
 
 void inject_schema_const(sema_context& ctx, mutator_context&, t_program& prog) {
+  if (prog.has_structured_annotation(kDisableSchemaConstUri)) {
+    return;
+  }
+
   detail::schematizer::options opts;
   opts.only_root_program_ = true;
   opts.use_hash = true;
@@ -565,6 +737,21 @@ void deduplicate_thrift_includes(
   includes.erase(it, includes.end());
 }
 
+// scope.thrift can't use structured annotations for circular dependency reasons
+// so inject annotations here.
+void add_magic_annotations(sema_context&, mutator_context&, t_struct& node) {
+  if (node.uri() == "facebook.com/thrift/annotation/Function") {
+    node.set_unstructured_annotation("hack.name", "TFunction");
+    node.set_unstructured_annotation("js.name", "TFunction");
+  } else if (node.uri() == "facebook.com/thrift/annotation/Const") {
+    node.set_unstructured_annotation("hack.name", "TConst");
+  } else if (node.uri() == "facebook.com/thrift/annotation/Enum") {
+    node.set_unstructured_annotation("py3.hidden", "1");
+  } else if (node.uri() == "facebook.com/thrift/annotation/Interface") {
+    node.set_unstructured_annotation("hack.name", "TInterface");
+  }
+}
+
 std::vector<ast_mutator> standard_mutators() {
   std::vector<ast_mutator> mutators;
 
@@ -583,6 +770,7 @@ std::vector<ast_mutator> standard_mutators() {
   main.add_field_visitor(&match_field_type_with_default_value);
   main.add_named_visitor(&match_annotation_types_with_const_values);
   main.add_program_visitor(&deduplicate_thrift_includes);
+  main.add_struct_visitor(&add_magic_annotations);
   mutators.push_back(std::move(main));
 
   return mutators;
@@ -592,29 +780,67 @@ std::vector<ast_mutator> standard_mutators() {
 
 bool sema::resolve_all_types(sema_context& diags, t_program_bundle& bundle) {
   bool success = true;
+  type_ref_resolver resolver(diags, bundle);
+
   if (!use_legacy_type_ref_resolution_) {
-    success = type_ref_resolver().run(diags, bundle);
+    success = resolver.run();
   }
-  for (auto& td : bundle.root_program()->scope()->placeholder_typedefs()) {
+
+  for (auto& td :
+       bundle.root_program()->global_scope()->placeholder_typedefs()) {
     if (td.type()) {
       continue;
     }
 
+    if (const auto* ttype = resolver.resolve_implicit_includes(td)) {
+      td.set_type(t_type_ref::from_ptr(ttype));
+      assert(td.type().resolved());
+      continue;
+    }
+
     if (use_legacy_type_ref_resolution_ && !td.resolve()) {
       success = false;
     }
 
-    diags.error(td, "Type `{}` not defined.", td.name());
+    report_missing_type(diags, td);
+
     assert(!td.resolve());
     success = false;
   }
-  return success;
+  return success && check_circular_typedef(diags, bundle);
+}
+
+bool sema::check_circular_typedef(
+    sema_context& diags, t_program_bundle& bundle) {
+  std::unordered_set<const t_type*> checked;
+  for (auto& td :
+       bundle.root_program()->global_scope()->placeholder_typedefs()) {
+    if (checked.count(td.get_type())) {
+      continue;
+    }
+    std::unordered_set<const t_type*> visited;
+    std::vector<const t_type*> chain;
+    if (nullptr != t_typedef::find_type_if(td.get_type(), [&](const t_type* t) {
+          // Find the first typedef which insertion failed
+          checked.insert(t);
+          chain.push_back(t);
+          return visited.insert(t).second == false;
+        })) {
+      std::string msg;
+      for (const auto& i : chain) {
+        msg += i->name() + (&i != &chain.back() ? " --> " : "");
+      }
+      diags.error(td, "Circular typedef: {}", msg);
+      return false;
+    }
+  }
+  return true;
 }
 
 sema::result sema::run(sema_context& ctx, t_program_bundle& bundle) {
   // Resolve types in the root program.
   if (!use_legacy_type_ref_resolution_) {
-    type_ref_resolver().run(ctx, bundle);
+    type_ref_resolver(ctx, bundle).run();
   }
 
   t_program& root_program = *bundle.root_program();
@@ -622,9 +848,9 @@ sema::result sema::run(sema_context& ctx, t_program_bundle& bundle) {
 
   result ret;
   for (t_placeholder_typedef& t :
-       root_program.scope()->placeholder_typedefs()) {
+       root_program.global_scope()->placeholder_typedefs()) {
     if (!t.resolve() && t.name().find(program_prefix) == 0) {
-      ctx.error(t, "Type `{}` not defined.", t.name());
+      report_missing_type(ctx, t);
       ret.unresolved_types = true;
     }
   }
