@@ -56,13 +56,13 @@ const t_structured* get_mixin_type(const t_field& field) {
 }
 
 bool has_experimental_annotation(sema_context& ctx, const t_named& node) {
-  if (node.find_structured_annotation_or_null(kExperimentalUri)) {
+  if (node.has_structured_annotation(kExperimentalUri)) {
     return true;
   }
   for (int pos = ctx.nodes().size() - 1; pos >= 0; --pos) {
     const auto* parent = dynamic_cast<const t_named*>(ctx.nodes().at(pos));
     if (parent != nullptr &&
-        parent->find_structured_annotation_or_null(kExperimentalUri)) {
+        parent->has_structured_annotation(kExperimentalUri)) {
       return true;
     }
   }
@@ -212,7 +212,7 @@ class adapter_or_wrapper_checker {
     auto type = field.type().get_type();
 
     bool structured_annotation_on_field =
-        field.find_structured_annotation_or_null(structured_adapter_annotation);
+        field.has_structured_annotation(structured_adapter_annotation);
 
     bool structured_annotation_on_typedef =
         t_typedef::get_first_structured_annotation_or_null(
@@ -236,7 +236,7 @@ class adapter_or_wrapper_checker {
       const char* structured_adapter_annotation_error_name,
       const char* structured_wrapper_annotation_error_name) {
     bool has_adapter_annotation =
-        node.find_structured_annotation_or_null(structured_adapter_annotation);
+        node.has_structured_annotation(structured_adapter_annotation);
     if (!has_adapter_annotation) {
       return;
     }
@@ -253,11 +253,11 @@ class adapter_or_wrapper_checker {
     }
 
     auto has_wrapper =
-        type->find_structured_annotation_or_null(structured_wrapper_annotation);
+        type->has_structured_annotation(structured_wrapper_annotation);
     std::string typedef_name = type->name();
     while (!has_wrapper) {
       if (const auto* inner_typedf = dynamic_cast<const t_typedef*>(type)) {
-        has_wrapper = inner_typedf->find_structured_annotation_or_null(
+        has_wrapper = inner_typedf->has_structured_annotation(
             structured_wrapper_annotation);
         typedef_name = inner_typedf->name();
         type = inner_typedf->get_type();
@@ -269,9 +269,9 @@ class adapter_or_wrapper_checker {
         } else {
           break;
         }
-      } else if (type->is_struct()) {
-        has_wrapper = type->find_structured_annotation_or_null(
-            structured_wrapper_annotation);
+      } else if (type->is_struct_or_union()) {
+        has_wrapper =
+            type->has_structured_annotation(structured_wrapper_annotation);
         typedef_name = type->name();
         break;
       } else {
@@ -334,8 +334,7 @@ void validate_identifier_is_not_reserved(
   //  1. the node was generated.
   //  2. @thrift.AllowedReservedIdentifier is present.
   if (node.generated() ||
-      node.find_structured_annotation_or_null(kAllowReservedIdentifierUri) !=
-          nullptr) {
+      node.has_structured_annotation(kAllowReservedIdentifierUri)) {
     return;
   }
   ctx.check(
@@ -349,8 +348,7 @@ void validate_filename_is_not_reserved(sema_context& ctx, const t_named& node) {
   //  1. the node was generated.
   //  2. @thrift.AllowReservedFilename is present.
   if (node.generated() ||
-      node.find_structured_annotation_or_null(kAllowReservedFilenameUri) !=
-          nullptr) {
+      node.has_structured_annotation(kAllowReservedFilenameUri)) {
     return;
   }
   ctx.check(
@@ -359,6 +357,47 @@ void validate_filename_is_not_reserved(sema_context& ctx, const t_named& node) {
       node.name());
 }
 
+/**
+ * Checks that the given program has the recommended python namespaces, i.e.
+ * either:
+ * 1. no Python namespace at all, or
+ * 2. at least namespace py3 (in addition to any py-deprecated namespace)
+ */
+void validate_python_namespaces(sema_context& ctx, const t_program& program) {
+  // If a (non-empty) package is defined, all namespaces are effectively
+  // defined, so return.
+  if (!program.package().empty()) {
+    return;
+  }
+
+  const std::map<std::string, std::string>& namespaces = program.namespaces();
+
+  // If there are no Python namespaces, there is nothing to do.
+  if (std::none_of(
+          namespaces.begin(),
+          namespaces.end(),
+          [](const std::pair<const std::string, std::string>& it) {
+            const std::string& key = it.first;
+            return key.find("py") == 0; // i.e., key.starts_with("py")
+          })) {
+    return;
+  }
+
+  // There is at least one namespace that begins with "py", which is used as a
+  // heuristic for "Python target language".
+
+  // Require a "py3" namespace
+  if (namespaces.find("py3") != namespaces.end()) {
+    return;
+  }
+
+  ctx.warning(
+      program,
+      "File has namespaces only for deprecated Thrift Python variants. Please "
+      "add `namespace py3` (or a non-empty `package`) to allow generation of "
+      "the only officially supported variant: thrift-python.");
+}
+
 void validate_interface_function_name_uniqueness(
     sema_context& ctx, const t_interface& node) {
   // Check for a redefinition of a function in the same interface.
@@ -388,15 +427,12 @@ void validate_extends_service_function_name_uniqueness(
   }
 }
 
-void validate_throws_exceptions(sema_context& ctx, const t_throws& node) {
-  for (const auto& except : node.fields()) {
-    auto except_type = except.type()->get_true_type();
-    ctx.check(
-        dynamic_cast<const t_exception*>(except_type),
-        except,
-        "Non-exception type, `{}`, in throws.",
-        except_type->name());
-  }
+void validate_throws_exceptions(sema_context& ctx, const t_field& except) {
+  auto except_type = except.type()->get_true_type();
+  ctx.check(
+      dynamic_cast<const t_exception*>(except_type),
+      "Non-exception type, `{}`, in throws.",
+      except_type->name());
 }
 
 // Checks for a redefinition of a field in the same t_structured, including
@@ -424,7 +460,7 @@ void validate_field_names_uniqueness(
 void validate_exception_message_annotation_is_only_in_exceptions(
     sema_context& ctx, const t_structured& node) {
   for (const auto& f : node.fields()) {
-    if (f.find_structured_annotation_or_null(kExceptionMessageUri)) {
+    if (f.has_structured_annotation(kExceptionMessageUri)) {
       ctx.check(
           node.is_exception(),
           f,
@@ -446,7 +482,7 @@ void validate_union_field_attributes(sema_context& ctx, const t_union& node) {
           field.qualifier() == t_field_qualifier::required ? "required"
                                                            : "optional",
           field.name());
-    } else if (field.find_structured_annotation_or_null(kTerseWriteUri)) {
+    } else if (field.has_structured_annotation(kTerseWriteUri)) {
       ctx.error(
           field,
           "`@thrift.TerseWrite` cannot be applied to union fields (in `{}`).",
@@ -455,50 +491,77 @@ void validate_union_field_attributes(sema_context& ctx, const t_union& node) {
   }
 }
 
+/**
+ * Validates fields with C++ "reference types" (i.e., Box, cpp.Ref, etc.).
+ */
 void validate_boxed_field_attributes(sema_context& ctx, const t_field& node) {
   if (gen::cpp::find_ref_type(node) == gen::cpp::reference_type::none) {
     return;
   }
 
-  bool ref = node.has_annotation({
-                 "cpp.ref",
-                 "cpp2.ref",
-                 "cpp.ref_type",
-                 "cpp2.ref_type",
-             }) ||
-      node.find_structured_annotation_or_null(kCppRefUri);
+  const bool ref = node.has_unstructured_annotation({
+                       "cpp.ref",
+                       "cpp2.ref",
+                       "cpp.ref_type",
+                       "cpp2.ref_type",
+                   }) ||
+      node.has_structured_annotation(kCppRefUri);
 
-  bool box = node.has_annotation({
-                 "cpp.box",
-                 "thrift.box",
-             }) ||
-      node.find_structured_annotation_or_null(kBoxUri);
+  const bool box = node.has_unstructured_annotation({
+                       "cpp.box",
+                       "thrift.box",
+                   }) ||
+      node.has_structured_annotation(kBoxUri);
 
-  bool intern_box = node.find_structured_annotation_or_null(kInternBoxUri);
+  const bool intern_box = node.has_structured_annotation(kInternBoxUri);
 
   if (ref + box + intern_box > 1) {
     ctx.error(
         node,
-        "The {} annotation cannot be combined with the other reference annotations. "
-        "Only annotate a single reference annotations from `{}`.",
+        "The {} annotation cannot be combined with the other reference "
+        "annotations. Only annotate a single reference annotation from `{}`.",
         intern_box ? "`@thrift.InternBox`"
             : box  ? "`@thrift.Box`"
                    : "`@cpp.Ref`",
         node.name());
   }
 
-  if (box) {
-    ctx.check(
-        dynamic_cast<const t_union*>(ctx.parent()) ||
-            node.qualifier() == t_field_qualifier::optional,
-        "The `thrift.box` annotation can only be used with optional fields. "
-        "Make sure `{}` is optional.",
-        node.name());
+  const t_structured& parent_node =
+      dynamic_cast<const t_structured&>(*ctx.parent());
+
+  if (node.qualifier() != t_field_qualifier::optional &&
+      !parent_node.is_union()) {
+    // Field is not optional (and not in a union)
+    // Reminder: all fields in a union are effectively optional.
+
+    if (box) {
+      // For thrift.Box, optional fields are always forbidden.
+      ctx.error(
+          "The `thrift.box` annotation can only be used with optional fields. "
+          "Make sure `{}` is optional.",
+          node.name());
+    } else if (ref) {
+      // For @cpp.Ref (and cpp[2].ref[_type]), non-optional fields result in
+      // either a warning or an error, depending on whether the field is
+      // annotated with `@cpp.AllowLegacyNonOptionalRef`.
+
+      const bool report_error =
+          !node.has_structured_annotation(kCppAllowLegacyNonOptionalRefUri);
+
+      ctx.report(
+          node,
+          report_error ? diagnostic_level::error : diagnostic_level::warning,
+          "Field with @cpp.Ref (or similar) annotation {} be optional: "
+          "`{}` (in `{}`).",
+          report_error ? "must" : "should",
+          node.name(),
+          parent_node.name());
+    }
   }
 
   if (intern_box) {
     ctx.check(
-        node.type()->get_true_type()->is_struct(),
+        node.type()->get_true_type()->is_struct_or_union(),
         "The `@thrift.InternBox` annotation can only be used with a struct field.");
     // TODO(dokwon): Add support for custom defaults and remove this check.
     ctx.check(
@@ -578,18 +641,83 @@ void validate_enum_value(sema_context& ctx, const t_enum_value& node) {
 }
 
 void validate_const_type_and_value(sema_context& ctx, const t_const& node) {
-  detail::check_initializer(ctx, node, node.type(), node.value());
+  if (detail::check_initializer(ctx, node, node.type(), node.value())) {
+    detail::check_duplicate_keys(ctx, node);
+  }
   ctx.check(
-      !node.find_structured_annotation_or_null(kCppAdapterUri) ||
+      !node.has_structured_annotation(kCppAdapterUri) ||
           has_experimental_annotation(ctx, node),
       "Using adapters on const `{}` is only allowed in the experimental mode.",
       node.name());
 }
 
+std::string_view field_qualifier_to_string(t_field_qualifier qualifier) {
+  switch (qualifier) {
+    case t_field_qualifier::none:
+      return "unqualified";
+    case t_field_qualifier::required:
+      return "required";
+    case t_field_qualifier::optional:
+      return "optional";
+    case t_field_qualifier::terse:
+      return "terse";
+  }
+  abort();
+}
+
 void validate_field_default_value(sema_context& ctx, const t_field& field) {
-  if (field.get_default_value() != nullptr) {
-    detail::check_initializer(
-        ctx, field, &field.type().deref(), field.default_value());
+  if (field.default_value() == nullptr) {
+    return;
+  }
+
+  if (!detail::check_initializer(
+          ctx, field, &field.type().deref(), field.default_value())) {
+    // If initializer is not valid to begin with, stop checks and return error.
+    return;
+  }
+  detail::check_duplicate_keys(ctx, field);
+
+  const t_structured& parent_node =
+      dynamic_cast<const t_structured&>(*ctx.parent());
+
+  const t_field_qualifier field_qualifier = field.qualifier();
+
+  if (ctx.sema_parameters().warn_on_redundant_custom_default_values &&
+      detail::is_initializer_default_value(
+          field.type().deref(), *field.default_value())) {
+    ctx.warning(
+        field,
+        "Explicit default value is redundant for ({}) field: "
+        "`{}` (in `{}`).",
+        field_qualifier_to_string(field_qualifier),
+        field.name(),
+        parent_node.name());
+  }
+
+  switch (field_qualifier) {
+    case t_field_qualifier::optional:
+      ctx.warning(
+          field,
+          "Optional field should not have custom default value: "
+          "`{}` (in `{}`).",
+          field.name(),
+          parent_node.name());
+      break;
+    case t_field_qualifier::terse: {
+      if (detail::is_initializer_default_value(
+              field.type().deref(), *field.default_value())) {
+        return;
+      }
+      ctx.error(
+          field,
+          "Terse field should not have custom default value: "
+          "`{}` (in `{}`).",
+          field.name(),
+          parent_node.name());
+      break;
+    }
+    default:
+      break;
   }
 }
 
@@ -649,33 +777,18 @@ void validate_uri_uniqueness(sema_context& ctx, const t_program& prog) {
   visit(prog);
 }
 
-void limit_terse_write_on_experimental_mode(
-    sema_context& ctx, const t_named& node) {
-  ctx.check(
-      !node.find_structured_annotation_or_null(kTerseWriteUri) ||
-          has_experimental_annotation(ctx, node),
-      "Using @thrift.TerseWrite on field `{}` is only allowed in the experimental mode.",
-      node.name());
-}
-
 void validate_field_id(sema_context& ctx, const t_field& node) {
-  if (node.explicit_id() != node.id()) {
-    if (ctx.sema_parameters().forbid_implicit_field_ids) {
-      ctx.error(node, "No field id specified for `{}`", node.name());
-    } else {
-      ctx.warning(
-          node,
-          "No field id specified for `{}`, resulting protocol may have conflicts "
-          "or not be backwards compatible!",
-          node.name());
-    }
-  }
+  ctx.check(
+      node.explicit_id() == node.id(),
+      "No field id specified for `{}`",
+      node.name());
 
   ctx.check(
       node.id() != 0 ||
-          node.has_annotation("cpp.deprecated_allow_zero_as_field_id"),
+          node.has_unstructured_annotation(
+              "cpp.deprecated_allow_zero_as_field_id"),
       "Zero value (0) not allowed as a field id for `{}`",
-      node.get_name());
+      node.name());
 
   ctx.check(
       node.id() >= t_field::min_id || node.is_injected(),
@@ -686,22 +799,42 @@ void validate_field_id(sema_context& ctx, const t_field& node) {
 
 void validate_compatibility_with_lazy_field(
     sema_context& ctx, const t_structured& node) {
-  if (has_lazy_field(node) && node.has_annotation("cpp.methods")) {
+  if (has_lazy_field(node) && node.has_unstructured_annotation("cpp.methods")) {
     ctx.error(
         "cpp.methods is incompatible with lazy deserialization in struct `{}`",
         node.get_name());
   }
 }
 
+/**
+ * Checks that the given field does not have both the (newer) structured
+ * @cpp.Ref annotation and one of the legacy unstructured annotations (cpp.ref,
+ * cpp.ref_type, etc.).
+ */
 void validate_ref_annotation(sema_context& ctx, const t_field& node) {
-  if (node.find_structured_annotation_or_null(kCppRefUri) &&
-      node.has_annotation(
-          {"cpp.ref", "cpp2.ref", "cpp.ref_type", "cpp2.ref_type"})) {
-    ctx.error(
-        "The @cpp.Ref annotation cannot be combined with the `cpp.ref` or "
-        "`cpp.ref_type` annotations. Remove one of the annotations from `{}`.",
+  const bool hasStructuredAnnotation =
+      node.has_structured_annotation(kCppRefUri);
+
+  const bool hasUnstructuredAnnotation = node.has_unstructured_annotation(
+      {"cpp.ref", "cpp2.ref", "cpp.ref_type", "cpp2.ref_type"});
+
+  const int count = hasStructuredAnnotation + hasUnstructuredAnnotation;
+  if (count == 0) {
+    // Neither @cpp.Ref nor cpp[2].ref[_type].
+    // Check that there is no @cpp.AllowedLegacyNonOptionalRef
+    ctx.check(
+        !node.has_structured_annotation(kCppAllowLegacyNonOptionalRefUri),
+        "Cannot annotate field with @cpp.AllowLegacyNonOptionalRef unless it "
+        "is a reference field (i.e., @cpp.Ref): `{}`.",
         node.name());
+    return;
   }
+
+  ctx.check(
+      count == 1,
+      "The @cpp.Ref annotation cannot be combined with the `cpp.ref` or "
+      "`cpp.ref_type` annotations. Remove one of the annotations from `{}`.",
+      node.name());
 }
 
 void validate_cpp_adapter_annotation(sema_context& ctx, const t_named& node) {
@@ -741,58 +874,64 @@ void validate_java_wrapper_and_adapter_annotation(
       node, kJavaAdapterUri, kJavaWrapperUri, "@java.Adapter", "@java.Wrapper");
 }
 
+/**
+ * Suggest @thrift.Box as a replacement for unique reference fields (@cpp.Ref,
+ * etc.). Require it for adapted fields with a reference annotation.
+ */
 void validate_ref_unique_and_box_annotation(
     sema_context& ctx, const t_field& node) {
   const t_const* adapter_annotation =
-      node.find_structured_annotation_or_null(kCppAdapterUri);
+      node.find_structured_annotation_or_null(kCppAdapterUri); // @cpp.Adapter
 
-  if (cpp2::is_unique_ref(&node)) {
-    if (node.has_annotation({"cpp.ref", "cpp2.ref"})) {
-      if (adapter_annotation) {
-        ctx.error(
-            "cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box "
-            "annotation instead in `{}` with @cpp.Adapter.",
-            node.name());
-      } else if (node.qualifier() == t_field_qualifier::optional) {
-        ctx.warning(
-            "cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box "
-            "annotation instead in `{}`.",
-            node.name());
-      }
+  if (!cpp2::is_unique_ref(&node)) {
+    return;
+  }
+
+  if (node.has_unstructured_annotation({"cpp.ref", "cpp2.ref"})) {
+    if (adapter_annotation) {
+      ctx.error(
+          "cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box "
+          "annotation instead in `{}` with @cpp.Adapter.",
+          node.name());
+    } else if (node.qualifier() == t_field_qualifier::optional) {
+      ctx.warning(
+          "cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box "
+          "annotation instead in `{}`.",
+          node.name());
     }
-    if (node.has_annotation({"cpp.ref_type", "cpp2.ref_type"})) {
-      if (adapter_annotation) {
-        ctx.error(
-            "cpp.ref_type = `unique`, cpp2.ref_type = `unique` are deprecated. "
-            "Please use @thrift.Box annotation instead in `{}` with "
-            "@cpp.Adapter.",
-            node.name());
-      } else if (node.qualifier() == t_field_qualifier::optional) {
-        ctx.warning(
-            "cpp.ref_type = `unique`, cpp2.ref_type = `unique` are deprecated. "
-            "Please use @thrift.Box annotation instead in `{}`.",
-            node.name());
-      }
+  }
+  if (node.has_unstructured_annotation({"cpp.ref_type", "cpp2.ref_type"})) {
+    if (adapter_annotation) {
+      ctx.error(
+          "cpp.ref_type = `unique`, cpp2.ref_type = `unique` are deprecated. "
+          "Please use @thrift.Box annotation instead in `{}` with "
+          "@cpp.Adapter.",
+          node.name());
+    } else if (node.qualifier() == t_field_qualifier::optional) {
+      ctx.warning(
+          "cpp.ref_type = `unique`, cpp2.ref_type = `unique` are deprecated. "
+          "Please use @thrift.Box annotation instead in `{}`.",
+          node.name());
     }
-    if (node.find_structured_annotation_or_null(kCppRefUri) != nullptr) {
-      if (adapter_annotation) {
-        ctx.error(
-            "@cpp.Ref{{type = cpp.RefType.Unique}} is deprecated. Please use "
-            "@thrift.Box annotation instead in `{}` with @cpp.Adapter.",
-            node.name());
-      } else if (node.qualifier() == t_field_qualifier::optional) {
-        ctx.warning(
-            "@cpp.Ref{{type = cpp.RefType.Unique}} is deprecated. Please use "
-            "@thrift.Box annotation instead in `{}`.",
-            node.name());
-      }
+  }
+  if (node.has_structured_annotation(kCppRefUri)) {
+    if (adapter_annotation) {
+      ctx.error(
+          "@cpp.Ref{{type = cpp.RefType.Unique}} is deprecated. Please use "
+          "@thrift.Box annotation instead in `{}` with @cpp.Adapter.",
+          node.name());
+    } else if (node.qualifier() == t_field_qualifier::optional) {
+      ctx.warning(
+          "@cpp.Ref{{type = cpp.RefType.Unique}} is deprecated. Please use "
+          "@thrift.Box annotation instead in `{}`.",
+          node.name());
     }
   }
 }
 
 void validate_function_priority_annotation(
     sema_context& ctx, const t_node& node) {
-  if (auto* priority = node.find_annotation_or_null("priority")) {
+  if (auto* priority = node.find_unstructured_annotation_or_null("priority")) {
     const std::string choices[] = {
         "HIGH_IMPORTANT", "HIGH", "IMPORTANT", "NORMAL", "BEST_EFFORT"};
     auto* end = choices + sizeof(choices) / sizeof(choices[0]);
@@ -811,14 +950,14 @@ void validate_exception_message_annotation(
   // - of type STRING
   const t_field* field = nullptr;
   for (const auto& f : node.fields()) {
-    if (f.find_structured_annotation_or_null(kExceptionMessageUri)) {
+    if (f.has_structured_annotation(kExceptionMessageUri)) {
       ctx.check(!field, f, "Duplicate message annotation.");
       field = &f;
     }
   }
-  if (node.has_annotation("message")) {
+  if (node.has_unstructured_annotation("message")) {
     ctx.check(!field, "Duplicate message annotation.");
-    const std::string& v = node.get_annotation("message");
+    const std::string& v = node.get_unstructured_annotation("message");
     field = node.get_field_by_name(v);
     ctx.check(
         field,
@@ -856,20 +995,42 @@ void validate_interaction_annotations(
     sema_context& ctx, const t_interaction& node) {
   for (auto* func : node.get_functions()) {
     ctx.check(
-        !func->has_annotation("thread") &&
-            !func->find_structured_annotation_or_null(kCppProcessInEbThreadUri),
+        !func->has_unstructured_annotation("thread") &&
+            !func->has_structured_annotation(kCppProcessInEbThreadUri),
         "Interaction methods cannot be individually annotated with "
         "thread='eb'. Use process_in_event_base on the interaction instead.");
   }
-  if (node.has_annotation("process_in_event_base") ||
-      node.find_structured_annotation_or_null(kCppProcessInEbThreadUri)) {
+  if (node.has_unstructured_annotation("process_in_event_base") ||
+      node.has_structured_annotation(kCppProcessInEbThreadUri)) {
     ctx.check(
-        !node.has_annotation("serial") &&
-            !node.find_structured_annotation_or_null(kSerialUri),
+        !node.has_unstructured_annotation("serial") &&
+            !node.has_structured_annotation(kSerialUri),
         "EB interactions are already serial");
   }
 }
 
+void validate_cpp_deprecated_terse_write_annotation(
+    sema_context& ctx, const t_field& field) {
+  if (!field.has_structured_annotation(kCppDeprecatedTerseWriteUri)) {
+    return;
+  }
+  if (field.qualifier() != t_field_qualifier::none) {
+    ctx.error(
+        field,
+        "@cpp.DeprecatedTerseWrite can be only used on unqualified field.");
+  }
+  if (gen::cpp::find_ref_type(field) == gen::cpp::reference_type::unique) {
+    return;
+  }
+  const t_type* type = field.get_type()->get_true_type();
+  if (type->is_struct_or_union() || type->is_exception()) {
+    ctx.error(
+        field,
+        "@cpp.DeprecatedTerseWrite is not supported for structured types.");
+  }
+  return;
+}
+
 void validate_cpp_field_interceptor_annotation(
     sema_context& ctx, const t_field& field) {
   if (const t_const* annot =
@@ -1045,7 +1206,13 @@ void validate_reserved_ids_enum(sema_context& ctx, const t_enum& node) {
 }
 
 bool owns_annotations(const t_type* type) {
-  if (type->annotations().empty()) {
+  if (std::none_of(
+          type->unstructured_annotations().begin(),
+          type->unstructured_annotations().end(),
+          [](const auto& a) {
+            return a.second.from ==
+                deprecated_annotation_value::origin::unstructured;
+          })) {
     return false;
   }
   if (dynamic_cast<const t_container*>(type)) {
@@ -1065,12 +1232,10 @@ bool owns_annotations(t_type_ref type) {
 
 void validate_custom_cpp_type_annotations(
     sema_context& ctx, const t_named& node) {
-  const bool hasAdapter =
-      node.find_structured_annotation_or_null(kCppAdapterUri);
-  bool hasCppType = node.has_annotation(
+  const bool hasAdapter = node.has_structured_annotation(kCppAdapterUri);
+  bool hasCppType = node.has_unstructured_annotation(
       {"cpp.type", "cpp2.type", "cpp.template", "cpp2.template"});
-  const bool hasStructuredCppType =
-      node.find_structured_annotation_or_null(kCppTypeUri);
+  const bool hasStructuredCppType = node.has_structured_annotation(kCppTypeUri);
 
   ctx.check(
       !(hasCppType && hasAdapter),
@@ -1084,7 +1249,8 @@ void validate_custom_cpp_type_annotations(
     }
     std::set<std::string> names{
         "cpp.type", "cpp2.type", "cpp.template", "cpp2.template"};
-    for (const auto& [k, v] : node.type().get_type()->annotations()) {
+    for (const auto& [k, v] :
+         node.type().get_type()->unstructured_annotations()) {
       if (names.count(k) && v.src_range.begin != source_location{}) {
         return true;
       }
@@ -1103,10 +1269,6 @@ void validate_custom_cpp_type_annotations(
       hasUnnamedCppType = true;
     }
   }
-  if (hasUnnamedCppType) {
-    ctx.warning(
-        "The cpp.type/cpp.template annotations are deprecated, use @cpp.Type instead");
-  }
   if (hasAdapter && (hasUnnamedCppType || hasStructuredCppType)) {
     // TODO (T169470476): make this an error
     ctx.warning(
@@ -1145,6 +1307,14 @@ struct ValidateAnnotationPositions {
       err(ctx);
     }
   }
+  void operator()(sema_context& ctx, const t_typedef& node) {
+    if (!ctx.sema_parameters().forbid_unstructured_annotations) {
+      return;
+    }
+    if (owns_annotations(node.type())) {
+      err(ctx);
+    }
+  }
   void operator()(sema_context& ctx, const t_function& node) {
     if (owns_annotations(node.return_type())) {
       err(ctx);
@@ -1167,6 +1337,17 @@ struct ValidateAnnotationPositions {
         err(ctx);
       }
     }
+
+    if (auto* exs = node.exceptions()) {
+      for (auto& ex : exs->fields()) {
+        if (owns_annotations(ex.type())) {
+          err(ctx);
+        }
+        if (!ex.unstructured_annotations().empty()) {
+          err(ctx);
+        }
+      }
+    }
   }
   void operator()(sema_context& ctx, const t_container& type) {
     switch (type.get_type_value()) {
@@ -1192,11 +1373,10 @@ struct ValidateAnnotationPositions {
     }
   }
   void operator()(sema_context& ctx, const t_field& node) {
-    if (ctx.sema_parameters().forbid_unstructured_annotations_on_field_types &&
-        owns_annotations(node.type()) &&
+    if (owns_annotations(node.type()) &&
         std::any_of(
-            node.type()->annotations().begin(),
-            node.type()->annotations().end(),
+            node.type()->unstructured_annotations().begin(),
+            node.type()->unstructured_annotations().end(),
             [](const auto& pair) {
               return pair.second.src_range.begin != source_location{};
             })) {
@@ -1254,12 +1434,48 @@ void deprecate_annotations(sema_context& ctx, const t_named& node) {
       {"erl.default_value", erlang("DefaultValue")},
       {"iq.node_type", erlang("Iq")},
   };
+  // Add a replacement to `deprecations` map if a removed unstructured
+  // annotation has a structured annotation replacement.
+  std::set<std::string> removed_annotations = {
+      "code",
+      "cpp.indirection",
+      "cpp2.declare_bitwise_ops",
+      "cpp2.enum_type",
+      "cpp2.deprecated_enum_unscoped",
+      "process_in_event_base",
+  };
+  std::map<std::string, std::string> removed_prefixes = {{"rust.", "rust"}};
 
-  for (const auto& [k, v] : node.annotations()) {
-    if (deprecations.count(k)) {
+  for (const auto& [k, v] : node.unstructured_annotations()) {
+    // Exclude lowered type annotations.
+    if (v.from == deprecated_annotation_value::origin::lowered_cpp_type) {
+      continue;
+    }
+    std::optional<std::string> prefix;
+    std::string lang;
+    for (const auto& [p, l] : removed_prefixes) {
+      if (k.find(p) == 0) {
+        prefix = p;
+        lang = l;
+        break;
+      }
+    }
+    bool directly_deprecated = deprecations.count(k) != 0;
+    if (!prefix && !directly_deprecated) {
+      if (removed_annotations.count(k) != 0) {
+        ctx.error("invalid annotation {}", k);
+        continue;
+      }
+      if (v.from == deprecated_annotation_value::origin::unstructured &&
+          ctx.sema_parameters().forbid_unstructured_annotations) {
+        ctx.error("Unstructured annotations are not allowed.");
+      }
+      continue;
+    }
+    std::string replacement;
+    if (directly_deprecated) {
       std::vector<std::string> parts;
       boost::split(parts, deprecations.at(k), [](char c) { return c == '/'; });
-      std::string replacement;
       if (parts.size() == 1) {
         replacement = parts[0];
       } else if (parts.size() == 4) {
@@ -1268,16 +1484,34 @@ void deprecate_annotations(sema_context& ctx, const t_named& node) {
         assert(parts.size() == 5);
         replacement = fmt::format("@{}.{}", parts[3], parts[4]);
       }
-
-      if (node.find_structured_annotation_or_null(deprecations.at(k).c_str())) {
-        ctx.error("Duplicate annotations {} and {}.", k, replacement);
-      } else {
-        ctx.warning(
-            "The annotation {} is deprecated. Please use {} instead.",
-            k,
-            replacement);
-      }
+    } else {
+      replacement = fmt::format(
+          "a structured annotation from thrift/annotation/{}.thrift", lang);
     }
+
+    if (directly_deprecated &&
+        node.has_structured_annotation(deprecations.at(k).c_str())) {
+      ctx.error("Duplicate annotations {} and {}.", k, replacement);
+    } else if (
+        removed_annotations.count(k) != 0 || prefix ||
+        (v.from == deprecated_annotation_value::origin::unstructured &&
+         ctx.sema_parameters().forbid_unstructured_annotations)) {
+      ctx.error(
+          "The annotation {} has been removed. Please use {} instead.",
+          k,
+          replacement);
+    } else {
+      ctx.warning(
+          "The annotation {} is deprecated. Please use {} instead.",
+          k,
+          replacement);
+    }
+  }
+}
+void deprecate_typedef_type_annotations(
+    sema_context& ctx, const t_typedef& node) {
+  if (owns_annotations(node.type())) {
+    deprecate_annotations(ctx, *node.type());
   }
 }
 
@@ -1331,6 +1565,20 @@ void validate_cursor_serialization_adapter_in_container(
   }
 }
 
+void validate_py3_enable_cpp_adapter(sema_context& ctx, const t_typedef& node) {
+  if (node.has_structured_annotation(kPythonPy3EnableCppAdapterUri)) {
+    const auto& true_type = *node.get_true_type();
+    if (!true_type.is_container() && !true_type.is_string_or_binary()) {
+      ctx.error(
+          "The @python.Py3EnableCppAdapter annotation can only be used on containers and strings.");
+    }
+    if (!node.has_structured_annotation(kCppAdapterUri)) {
+      ctx.error(
+          "The @python.Py3EnableCppAdapter annotation requires the @cpp.Adapter annotation to be present in the same typedef.");
+    }
+  }
+}
+
 // TODO (T191018859): forbid as field type too
 void forbid_exception_as_method_type(
     sema_context& ctx, const t_function& node) {
@@ -1355,6 +1603,7 @@ ast_validator standard_validator() {
   ast_validator validator;
   validator.add_definition_visitor(&validate_identifier_is_not_reserved);
   validator.add_program_visitor(&validate_filename_is_not_reserved);
+  validator.add_program_visitor(&validate_python_namespaces);
 
   validator.add_interface_visitor(&validate_interface_function_name_uniqueness);
   validator.add_interface_visitor(&validate_function_priority_annotation);
@@ -1362,7 +1611,7 @@ ast_validator standard_validator() {
       &validate_extends_service_function_name_uniqueness);
   validator.add_interaction_visitor(&validate_interaction_nesting);
   validator.add_interaction_visitor(&validate_interaction_annotations);
-  validator.add_throws_visitor(&validate_throws_exceptions);
+  validator.add_thrown_exception_visitor(&validate_throws_exceptions);
   validator.add_function_visitor(&validate_function_priority_annotation);
   validator.add_function_visitor(ValidateAnnotationPositions{});
 
@@ -1386,6 +1635,7 @@ ast_validator standard_validator() {
   validator.add_field_visitor(&validate_hack_field_adapter_annotation);
   validator.add_field_visitor(&validate_java_field_adapter_annotation);
   validator.add_field_visitor(&validate_cpp_field_interceptor_annotation);
+  validator.add_field_visitor(&validate_cpp_deprecated_terse_write_annotation);
   validator.add_field_visitor(&validate_required_field);
   validator.add_field_visitor(&validate_cpp_type_annotation<t_field>);
   validator.add_field_visitor(&validate_field_name);
@@ -1405,11 +1655,13 @@ ast_validator standard_validator() {
   validator.add_named_visitor(&validate_java_adapter_annotation);
   validator.add_named_visitor(&validate_java_wrapper_annotation);
   validator.add_named_visitor(&validate_java_wrapper_and_adapter_annotation);
-  validator.add_named_visitor(&limit_terse_write_on_experimental_mode);
   validator.add_named_visitor(&validate_custom_cpp_type_annotations);
   validator.add_named_visitor(&deprecate_annotations);
 
   validator.add_typedef_visitor(&validate_cpp_type_annotation<t_typedef>);
+  validator.add_typedef_visitor(&validate_py3_enable_cpp_adapter);
+  validator.add_typedef_visitor(&deprecate_typedef_type_annotations);
+  validator.add_typedef_visitor(ValidateAnnotationPositions());
 
   validator.add_container_visitor(ValidateAnnotationPositions());
   validator.add_enum_visitor(&validate_cpp_enum_type);
@@ -1423,6 +1675,7 @@ ast_validator standard_validator() {
   validator.add_container_visitor(
       &validate_cursor_serialization_adapter_in_container);
   validator.add_function_visitor(&forbid_exception_as_method_type);
+
   validator.add_const_visitor(&forbid_exception_as_const_type);
 
   add_explicit_include_validators(validator);
