@@ -23,17 +23,6 @@
 using apache::thrift::compiler::test::check_compile;
 using apache::thrift::compiler::test::check_compile_options;
 
-namespace {
-
-// Change the current directory to a subdirectory to avoid relative annotations
-// being found via relative includes. The exact directory doesn't matter so
-// pick "thrift" since we know it exists.
-void disable_relative_includes() {
-  std::filesystem::current_path("thrift");
-}
-
-} // namespace
-
 // Note: To see a reference on the expected lint message format: see the regex
 // in thrift/compiler/test/compiler.cc
 TEST(CompilerTest, diagnostic_in_last_line) {
@@ -61,7 +50,7 @@ TEST(CompilerTest, missing_type_definition) {
   check_compile(R"(
     struct S {
       1: i32 i;
-      2: MyStruct ms; # expected-error: Type `test.MyStruct` not defined.
+      2: MyStruct ms; # expected-error: Type `MyStruct` not defined.
     }
   )");
 }
@@ -83,18 +72,20 @@ TEST(CompilerTest, redefinition) {
 TEST(CompilerTest, zero_as_field_id) {
   check_compile(R"(
     struct Foo {
-      0: i32 field; # expected-warning: Nonpositive field id (0) differs from what is auto-assigned by thrift. The id must be positive or -1.
-                    # expected-warning@-1:  No field id specified for `field`, resulting protocol may have conflicts or not be backwards compatible!
+        0: i32 field; # expected-warning: Nonpositive field id (0) differs from what would be auto-assigned by thrift (if 'allow-neg-keys' was disabled): -1
+                      # expected-error@-1: Zero value (0) not allowed as a field id for `field`
       1: list<i32> other;
     }
   )");
 }
 
+// NOTE: As of Nov 2024, allow-neg-keys is ignored, so this should be identical
+// to the "zero_as_field_id" case above.
 TEST(CompilerTest, zero_as_field_id_neg_keys) {
   check_compile(
       R"(
       struct Foo {
-        0: i32 field; # expected-warning: Nonpositive field id (0) differs from what would be auto-assigned by thrift (-1).
+        0: i32 field; # expected-warning: Nonpositive field id (0) differs from what would be auto-assigned by thrift (if 'allow-neg-keys' was disabled): -1
                       # expected-error@-1: Zero value (0) not allowed as a field id for `field`
         1: list<i32> other;
       }
@@ -103,20 +94,6 @@ TEST(CompilerTest, zero_as_field_id_neg_keys) {
 }
 
 TEST(CompilerTest, no_field_id) {
-  check_compile(R"(
-    struct Experimental {} (thrift.uri = "facebook.com/thrift/annotation/Experimental") # expected-warning: The annotation thrift.uri is deprecated. Please use @thrift.Uri instead.
-    struct Foo {
-      @Experimental
-      i32 field2; # expected-warning@-1: No field id specified for `field2`, resulting protocol may have conflicts or not be backwards compatible!
-    }
-
-    struct Bar {
-      i32 field4; # expected-warning: No field id specified for `field4`, resulting protocol may have conflicts or not be backwards compatible!
-    }
-  )");
-}
-
-TEST(CompilerTest, no_field_id_with_validation) {
   check_compile(
       R"(
     struct Experimental {} (thrift.uri = "facebook.com/thrift/annotation/Experimental") # expected-warning: The annotation thrift.uri is deprecated. Please use @thrift.Uri instead.
@@ -127,52 +104,35 @@ TEST(CompilerTest, no_field_id_with_validation) {
 
     struct Bar {
       i32 field4; # expected-error: No field id specified for `field4`
-    }
-  )",
-      {"--extra-validation", "implicit_field_ids"});
+    })");
 }
 
 TEST(CompilerTest, zero_as_field_id_annotation) {
   check_compile(R"(
     struct Foo {
       0: i32 field (cpp.deprecated_allow_zero_as_field_id);
-        # expected-warning@-1: Nonpositive field id (0) differs from what is auto-assigned by thrift. The id must be positive or -1.
-        # expected-warning@-2: No field id specified for `field`, resulting protocol may have conflicts or not be backwards compatible!
+        # expected-warning@-1: Nonpositive field id (0) differs from what would be auto-assigned by thrift (if 'allow-neg-keys' was disabled): -1
 
       1: list<i32> other;
     }
   )");
 }
 
-TEST(CompilerTest, zero_as_field_id_allow_neg_keys) {
-  check_compile(
-      R"(
-      struct Foo {
-        0: i32 field (cpp.deprecated_allow_zero_as_field_id);
-          # expected-warning@-1: Nonpositive field id (0) differs from what would be auto-assigned by thrift (-1).
-
-        1: list<i32> other;
-      }
-      )",
-      {"--allow-neg-keys"});
-}
-
 TEST(CompilerTest, neg_field_ids) {
   check_compile(
       R"(
       struct Foo {
         i32 f1;  // auto id = -1
-          # expected-warning@-1: No field id specified for `f1`, resulting protocol may have conflicts or not be backwards compatible!
+          # expected-error@-1: No field id specified for `f1`
 
         -2: i32 f2; // auto and manual id = -2
         -32: i32 f3; // min value.
-          # expected-warning@-1: Nonpositive field id (-32) differs from what would be auto-assigned by thrift (-3).
+          # expected-warning@-1: Nonpositive field id (-32) differs from what would be auto-assigned by thrift (if 'allow-neg-keys' was disabled): -3
 
         -33: i32 f4; // min value - 1.
           # expected-error@-1: Reserved field id (-33) cannot be used for `f4`.
       }
-      )",
-      {"--allow-neg-keys"});
+      )");
 }
 
 TEST(CompilerTest, exhausted_neg_field_ids) {
@@ -180,13 +140,12 @@ TEST(CompilerTest, exhausted_neg_field_ids) {
       R"(
       struct Foo {
         -32: i32 f1; // min value.
-          # expected-warning@-1: Nonpositive field id (-32) differs from what would be auto-assigned by thrift (-1).
+          # expected-warning@-1: Nonpositive field id (-32) differs from what would be auto-assigned by thrift (if 'allow-neg-keys' was disabled): -1
 
         i32 f2; // auto id = -2 or min value - 1
           # expected-error@-1: Cannot allocate an id for `f2`. Automatic field ids are exhausted.
       }
-      )",
-      {"--allow-neg-keys"});
+      )");
 }
 
 TEST(CompilerTest, exhausted_pos_field_ids) {
@@ -205,10 +164,11 @@ TEST(CompilerTest, exhausted_pos_field_ids) {
 TEST(CompilerTest, out_of_range_field_ids_overflow) {
   check_compile(R"(
     struct Foo {
-      -32768: i32 f1; # expected-warning: Nonpositive field id (-32768) differs from what is auto-assigned by thrift. The id must be positive or -1.
+      -32768: i32 f1; # expected-warning: Nonpositive field id (-32768) differs from what would be auto-assigned by thrift (if 'allow-neg-keys' was disabled): -1
       32767: i32 f2;
       32768: i32 f3; # expected-error: Integer constant 32768 outside the range of field ids ([-32768, 32767]).
-                     # expected-warning@-1: Nonpositive field id (-32768) differs from what is auto-assigned by thrift. The id must be positive or -2.
+        # expected-error@-1: Field id -32768 for `f3` has already been used.
+        # expected-warning@-2: Nonpositive field id (-32768) differs from what would be auto-assigned by thrift (if 'allow-neg-keys' was disabled): 32767
     }
   )");
 }
@@ -216,7 +176,7 @@ TEST(CompilerTest, out_of_range_field_ids_overflow) {
 TEST(CompilerTest, out_of_range_field_ids_underflow) {
   check_compile(R"(
     struct Foo {
-      -32768: i32 f1; # expected-warning: Nonpositive field id (-32768) differs from what is auto-assigned by thrift. The id must be positive or -1.
+      -32768: i32 f1; # expected-warning: Nonpositive field id (-32768) differs from what would be auto-assigned by thrift (if 'allow-neg-keys' was disabled): -1
       32767: i32 f2;
 
       -32769: i32 f3; # expected-error: Integer constant -32769 outside the range of field ids ([-32768, 32767]).
@@ -667,7 +627,7 @@ TEST(CompilerTest, undefined_type) {
 
 TEST(CompilerTest, undefined_const) {
   check_compile(R"(
-    const Type c = 42; # expected-error: Type `test.Type` not defined.
+    const Type c = 42; # expected-error: Type `Type` not defined.
   )");
 }
 
@@ -679,7 +639,7 @@ TEST(CompilerTest, undefined_const_external) {
 
 TEST(CompilerTest, undefined_annotation) {
   check_compile(R"(
-    @BadAnnotation # expected-error: Type `test.BadAnnotation` not defined.
+    @BadAnnotation # expected-error: Type `BadAnnotation` not defined.
     struct S {}
   )");
 }
@@ -728,12 +688,16 @@ TEST(CompilerTest, annotation_positions) {
     service S {
       i32 (annot) foo() # expected-error: Annotations are not allowed in this position. Extract the type into a named typedef instead.
       void bar(1: i32 (annot) p) # expected-error: Annotations are not allowed in this position. Extract the type into a named typedef instead.
+      void qux() throws (1: E (annot) e) # expected-error: Annotations are not allowed in this position. Extract the type into a named typedef instead.
+      void baz() throws (1: E e (annot)) # expected-error: Annotations are not allowed in this position. Extract the type into a named typedef instead.
     }
     struct Foo {
       1: i32 (annot) f
+      # expected-error@-1: Annotations are not allowed in this position. Extract the type into a named typedef instead.
       @Type{name="foo"}
       2: i32 g
     }
+    exception E {}
   )");
 }
 
@@ -744,8 +708,7 @@ TEST(CompilerTest, annotation_positions_field) {
       1: i32 (annot) f # expected-error: Annotations are not allowed in this position. Extract the type into a named typedef instead.
       2: i32 g
     }
-  )",
-      {"--extra-validation", "unstructured_annotations_on_field_type"});
+  )");
 }
 
 TEST(CompilerTest, performs_in_interaction) {
@@ -797,7 +760,9 @@ TEST(CompilerTest, interactions_as_first_response_type) {
 
 TEST(CompilerTest, deprecated_annotations) {
   check_compile(R"(
+    include "thrift/annotation/cpp.thrift"
     include "thrift/annotation/hack.thrift"
+    include "thrift/annotation/thrift.thrift"
 
     @hack.Attributes{attributes=[]} # expected-error: Duplicate annotations hack.attributes and @hack.Attributes.
     struct A {
@@ -805,6 +770,46 @@ TEST(CompilerTest, deprecated_annotations) {
         2: i64 with (py3.name = "w", go.name = "w") # expected-warning: The annotation py3.name is deprecated. Please use @python.Name instead.
         # expected-warning@-1: The annotation go.name is deprecated. Please use @go.Name instead.
     } (hack.attributes = "")
+
+    typedef i64 (cpp.type = "std::uint64_t") T # expected-warning: The annotation cpp.type is deprecated. Please use @cpp.Type instead.
+
+    # This should not produce a warning even though the annotation is currently lowered.
+    @cpp.Type{name = "std::uint64_t"}
+    typedef i64 T2
+  )");
+}
+
+TEST(CompilerTest, removed_annotations) {
+  check_compile(R"(
+    include "thrift/annotation/thrift.thrift"
+
+    enum E1 {} (cpp2.declare_bitwise_ops)
+    # expected-error@-1: The annotation cpp2.declare_bitwise_ops has been removed. Please use @thrift.BitmaskEnum instead.
+
+    @thrift.DeprecatedUnvalidatedAnnotations{items = {"cpp2.declare_bitwise_ops": "1"}}
+    enum E2 {}
+    # expected-error@-2: The annotation cpp2.declare_bitwise_ops has been removed. Please use @thrift.BitmaskEnum instead.
+
+    enum E3 {} (cpp2.deprecated_enum_unscoped)
+    # expected-error@-1: invalid annotation cpp2.deprecated_enum_unscoped
+
+    struct A {
+      1: i64 removed_unstructured (rust.foo)
+      # expected-error@-1: The annotation rust.foo has been removed. Please use a structured annotation from thrift/annotation/rust.thrift instead.
+      @thrift.DeprecatedUnvalidatedAnnotations{items = {"rust.foo": "1"}}
+      2: i64 removed_catchall
+      # expected-error@-2: The annotation rust.foo has been removed. Please use a structured annotation from thrift/annotation/rust.thrift instead.
+    }
+
+    typedef map<string, string> (rust.type = "HashMap") HashMap
+    # expected-error@-1: The annotation rust.type has been removed. Please use a structured annotation from thrift/annotation/rust.thrift instead.
+
+    struct S {} (cpp.type = "foo") # expected-warning: The annotation cpp.type is deprecated. Please use @cpp.Type instead.
+    typedef S T
+
+    service J {
+      i64 foo(1: i64 arg (rust.name = "bar")) # expected-error: The annotation rust.name has been removed. Please use a structured annotation from thrift/annotation/rust.thrift instead.
+    }
   )");
 }
 
@@ -836,7 +841,8 @@ TEST(CompilerTest, cpp_type_compatibility) {
 
     struct B {
       @cpp.Adapter{name="Adapter"} # expected-warning: At most one of @cpp.Type/@cpp.Adapter/cpp.type/cpp.template can be specified on a definition.
-      1: i32 (cpp.type = "std::uint32_t") field; # expected-warning@-1: The cpp.type/cpp.template annotations are deprecated, use @cpp.Type instead
+      1: i32 (cpp.type = "std::uint32_t") field;
+      # expected-error@-2: Annotations are not allowed in this position. Extract the type into a named typedef instead.
       @cpp.Adapter{name="Adapter"} # expected-warning: At most one of @cpp.Type/@cpp.Adapter/cpp.type/cpp.template can be specified on a definition.
       @cpp.Type{name="std::uint32_t"}
       2: i32 field2;
@@ -1225,7 +1231,6 @@ TEST(CompilerTest, boxed_ref_and_optional) {
     include "thrift/annotation/cpp.thrift"
     include "thrift/annotation/thrift.thrift"
 
-    @thrift.Experimental
     package "apache.org/thrift/test"
 
     struct MyStruct {
@@ -1236,27 +1241,32 @@ TEST(CompilerTest, boxed_ref_and_optional) {
 
     struct A {
         1: optional i64 field (cpp.ref, thrift.box);
-          # expected-error@-1: The `@thrift.Box` annotation cannot be combined with the other reference annotations. Only annotate a single reference annotations from `field`.
+          # expected-error@-1: The `@thrift.Box` annotation cannot be combined with the other reference annotations. Only annotate a single reference annotation from `field`.
           # expected-warning@-2: The annotation thrift.box is deprecated. Please use @thrift.Box instead.
+
         @thrift.Box
-          # expected-error@-1: The `@thrift.InternBox` annotation cannot be combined with the other reference annotations. Only annotate a single reference annotations from `field2`.
+          # expected-error@-1: The `@thrift.InternBox` annotation cannot be combined with the other reference annotations. Only annotate a single reference annotation from `field2`.
           # expected-error@-2: The `@thrift.InternBox` annotation can only be used with a struct field.
           # expected-error@-3: The `@thrift.InternBox` annotation can only be used with unqualified or terse fields. Make sure `field2` is unqualified or annotated with `@thrift.TerseWrite`.
         @thrift.InternBox
         2: optional i64 field2;
+
         @thrift.InternBox
           # expected-error@-1: The `@thrift.InternBox` annotation currently does not support a field with custom default.
         3: MyStruct field3 = {"field1" : 1};
+
         @cpp.Ref
-          # expected-error@-1: The `@thrift.InternBox` annotation cannot be combined with the other reference annotations. Only annotate a single reference annotations from `field4`.
+          # expected-error@-1: The `@thrift.InternBox` annotation cannot be combined with the other reference annotations. Only annotate a single reference annotation from `field4`.
           # expected-error@-2: The `thrift.box` annotation can only be used with optional fields. Make sure `field4` is optional.
         @thrift.Box
         @thrift.InternBox
         @thrift.TerseWrite
         4: MyStruct field4;
+
         @thrift.InternBox
         @thrift.TerseWrite
         5: MyStruct field5;
+
         @thrift.InternBox
         @thrift.TerseWrite
         6: MyStruct2 field6;
@@ -1271,26 +1281,38 @@ TEST(CompilerTest, unique_ref) {
     struct Bar {
       1: optional Foo field1 (cpp.ref);
         # expected-warning@-1: cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box annotation instead in `field1`.
+
       2: optional Foo field2 (cpp2.ref);
         # expected-warning@-1: cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box annotation instead in `field2`.
+
       @cpp.Ref{type = cpp.RefType.Unique}
       3: optional Foo field3;
         # expected-warning@-2: @cpp.Ref{type = cpp.RefType.Unique} is deprecated. Please use @thrift.Box annotation instead in `field3`.
+
       @cpp.Ref{type = cpp.RefType.Shared}
       4: optional Foo field4;
+
       @cpp.Ref{type = cpp.RefType.SharedMutable}
       5: optional Foo field5;
+
       6: optional Foo field6 (cpp.ref_type = "unique");
         # expected-warning@-1: cpp.ref_type = `unique`, cpp2.ref_type = `unique` are deprecated. Please use @thrift.Box annotation instead in `field6`.
+
       7: optional Foo field7 (cpp2.ref_type = "unique");
         # expected-warning@-1: cpp.ref_type = `unique`, cpp2.ref_type = `unique` are deprecated. Please use @thrift.Box annotation instead in `field7`.
+
       8: optional Foo field8 (cpp.ref_type = "shared");
+
       9: optional Foo field9 (cpp2.ref_type = "shared");
+
       10: optional Foo field10 (cpp.ref = "true");
         # expected-warning@-1: cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box annotation instead in `field10`.
+
       11: optional Foo field11 (cpp2.ref = "true");
         # expected-warning@-1: cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box annotation instead in `field11`.
+
       12: optional Foo field12;
+
       13: optional Foo field13;
     }
 
@@ -1298,6 +1320,79 @@ TEST(CompilerTest, unique_ref) {
   )");
 }
 
+TEST(CompilerTest, non_optional_ref_and_box_forbidden) {
+  check_compile(
+      R"(
+    include "thrift/annotation/cpp.thrift"
+    include "thrift/annotation/thrift.thrift"
+
+    struct Bar {
+      1: Foo field1 (cpp.ref);
+        # expected-error@-1: Field with @cpp.Ref (or similar) annotation must be optional: `field1` (in `Bar`).
+
+      2: Foo field2 (cpp2.ref);
+        # expected-error@-1: Field with @cpp.Ref (or similar) annotation must be optional: `field2` (in `Bar`).
+
+      @cpp.Ref{type = cpp.RefType.Unique}
+      3: Foo field3;
+        # expected-error@-2: Field with @cpp.Ref (or similar) annotation must be optional: `field3` (in `Bar`).
+
+      @cpp.Ref{type = cpp.RefType.Shared}
+      4: Foo field4;
+        # expected-error@-2: Field with @cpp.Ref (or similar) annotation must be optional: `field4` (in `Bar`).
+
+      @cpp.Ref{type = cpp.RefType.SharedMutable}
+      5: Foo field5;
+        # expected-error@-2: Field with @cpp.Ref (or similar) annotation must be optional: `field5` (in `Bar`).
+
+      6: Foo field6 (cpp.ref_type = "unique");
+        # expected-error@-1: Field with @cpp.Ref (or similar) annotation must be optional: `field6` (in `Bar`).
+
+      7: Foo field7 (cpp2.ref_type = "unique");
+        # expected-error@-1: Field with @cpp.Ref (or similar) annotation must be optional: `field7` (in `Bar`).
+
+      8: Foo field8 (cpp.ref_type = "shared");
+        # expected-error@-1: Field with @cpp.Ref (or similar) annotation must be optional: `field8` (in `Bar`).
+
+      9: Foo field9 (cpp2.ref_type = "shared");
+        # expected-error@-1: Field with @cpp.Ref (or similar) annotation must be optional: `field9` (in `Bar`).
+
+      10: Foo field10 (cpp.ref = "true");
+        # expected-error@-1: Field with @cpp.Ref (or similar) annotation must be optional: `field10` (in `Bar`).
+
+      11: Foo field11 (cpp2.ref = "true");
+        # expected-error@-1: Field with @cpp.Ref (or similar) annotation must be optional: `field11` (in `Bar`).
+
+      12: Foo field12;
+
+      @thrift.Box
+      13: Foo field13;
+        # expected-error@-2: The `thrift.box` annotation can only be used with optional fields. Make sure `field13` is optional.
+    }
+
+    struct Foo {}
+  )");
+}
+
+TEST(CompilerTest, non_optional_ref_legacy_allowed_annotation_on_wrong_field) {
+  check_compile(
+      R"(
+    include "thrift/annotation/cpp.thrift"
+    include "thrift/annotation/thrift.thrift"
+
+    struct Bar {
+      1: Foo field1 (cpp.ref);
+        # expected-error@-1: Field with @cpp.Ref (or similar) annotation must be optional: `field1` (in `Bar`).
+
+      @cpp.AllowLegacyNonOptionalRef
+      2: Foo field2;
+        # expected-error@-2: Cannot annotate field with @cpp.AllowLegacyNonOptionalRef unless it is a reference field (i.e., @cpp.Ref): `field2`.
+    }
+
+    struct Foo {}
+  )");
+}
+
 TEST(CompilerTest, nonexistent_field_name) {
   check_compile(R"(
     struct Foo {}
@@ -1563,11 +1658,11 @@ TEST(CompilerTest, inject_metadata_fields_annotation) {
     struct Injected2 {}
 
     @internal.InjectMetadataFields{type="UnionFields"}
-      # expected-error@-1: `bar.UnionFields` is not a struct type. `@internal.InjectMetadataFields` can be only used with a struct type.
+      # expected-error@-1: `UnionFields` is not a struct type. `@internal.InjectMetadataFields` can be only used with a struct type.
     struct Injected3 {}
 
     @internal.InjectMetadataFields{type="MyI64"}
-      # expected-error@-1: `bar.MyI64` is not a struct type. `@internal.InjectMetadataFields` can be only used with a struct type.
+      # expected-error@-1: `MyI64` is not a struct type. `@internal.InjectMetadataFields` can be only used with a struct type.
     struct Injected4 {}
 
     @internal.InjectMetadataFields{type="Fields"}
@@ -1579,7 +1674,6 @@ TEST(CompilerTest, inject_metadata_fields_annotation) {
     // the field id gets implicitly converted -1.
     struct BoundaryFields {
       -1: i64 underflow;
-        # expected-warning@-1: Nonpositive value (-1) not allowed as a field id.
       1: i64 lower_boundary;
       999: i64 upper_boundary;
       1000: i64 overflow;
@@ -1810,21 +1904,7 @@ TEST(CompilerTest, required_key_specified_in_structured_annotation) {
 TEST(CompilerTest, nonexist_type_in_variable) {
   check_compile(R"(
     const map<i8, string> foo = {1: "str"}
-      # expected-error@-1: Type `test.i8` not defined.
-  )");
-}
-
-TEST(CompilerTest, terse_write_outside_experimental_mode) {
-  check_compile(R"(
-    include "thrift/annotation/thrift.thrift"
-
-    package "meta.com/thrift/test"
-
-    struct MyStruct {
-        @thrift.TerseWrite
-        1: i32 field1 = 1;
-          # expected-error@-2: Using @thrift.TerseWrite on field `field1` is only allowed in the experimental mode.
-    }
+      # expected-error@-1: Type `i8` not defined.
   )");
 }
 
@@ -1947,7 +2027,7 @@ TEST(CompilerTest, py3_invalid_field_names) {
       {"--gen", "mstch_py3"});
 }
 
-TEST(CompilerTest, warn_on_non_explicit_includes) {
+TEST(CompilerTest, error_on_non_explicit_includes) {
   std::map<std::string, std::string> name_contents_map;
   name_contents_map["path/to/upstream.thrift"] = R"(
     struct TransitiveStruct {}
@@ -1980,7 +2060,7 @@ TEST(CompilerTest, warn_on_non_explicit_includes) {
   )";
 
   name_contents_map["path/to/transitive_struct_field.thrift"] = R"(
-    # expected-warning@3#original[]#replacement[include "path/to/upstream.thrift"\n]@5: Your thrift file depends on a type that it did not include. Please add the following include. [implicit-include]
+    # expected-error@3#original[]#replacement[include "path/to/upstream.thrift"\n]@5: Your thrift file depends on a type that it did not include. Please add the following include. [implicit-include]
     include "path/to/direct.thrift"
 
     struct A {
@@ -2157,12 +2237,501 @@ TEST(CompilerTest, duplicate_include) {
 TEST(CompilerTest, not_bundled_annotation) {
   auto options = check_compile_options();
   options.add_standard_includes = false;
-  disable_relative_includes();
+
+  // Change the current directory to a temp directory to avoid annotations
+  // being found via relative includes.
+  std::filesystem::current_path(std::filesystem::temp_directory_path());
+
   check_compile(
       R"(
-      include "thrift/annotation/cpp.thrift"
-      # expected-error-1: Could not find include file thrift/annotation/cpp.thrift
+      include "thrift/annotation/baz.thrift"
+      # expected-error-1: Could not find include file thrift/annotation/baz.thrift
       )",
       {},
       options);
 }
+
+TEST(CompilerTest, circular_typedef) {
+  check_compile(R"(
+    typedef Foo Bar # expected-error: Circular typedef: Foo --> Bar --> Foo
+    typedef Bar Foo
+  )");
+}
+
+TEST(CompilerTest, combining_unstructured_annotations) {
+  check_compile(R"(
+    include "thrift/annotation/thrift.thrift"
+
+    @thrift.DeprecatedUnvalidatedAnnotations{items = {"foo": "bar"}}
+    struct Foo {} (hs.foo)
+
+    @thrift.DeprecatedUnvalidatedAnnotations{items = {"foo": "bar"}}
+    struct Bar {} (rust.foo)
+    # expected-error@-2: Cannot combine @thrift.DeprecatedUnvalidatedAnnotations with legacy annotation syntax.
+  )");
+}
+
+TEST(Compilertest, custom_default_values) {
+  check_compile(
+      R"(
+    include "thrift/annotation/thrift.thrift"
+
+    union TestUnion {}
+
+    struct TestStruct {}
+
+    exception TestException {}
+
+
+    struct Widget {
+      1: i32 a = 42;
+
+      2: bool b = false;
+        # expected-warning@-1: Explicit default value is redundant for (unqualified) field: `b` (in `Widget`).
+
+      3: i32 c = 0;
+        # expected-warning@-1: Explicit default value is redundant for (unqualified) field: `c` (in `Widget`).
+
+      4: optional bool d = true;
+        # expected-warning@-1: Optional field should not have custom default value: `d` (in `Widget`).
+
+      @thrift.TerseWrite
+      5: i32 e = 43;
+        # expected-error@-2: Terse field should not have custom default value: `e` (in `Widget`).
+
+      6: TestUnion f = {};
+        # expected-warning@-1: Explicit default value is redundant for (unqualified) field: `f` (in `Widget`).
+
+      7: TestStruct g = {};
+        # expected-warning@-1: Explicit default value is redundant for (unqualified) field: `g` (in `Widget`).
+
+      8: TestException h = {};
+        # expected-warning@-1: Explicit default value is redundant for (unqualified) field: `h` (in `Widget`).
+
+      @thrift.TerseWrite
+      9: i32 i = 0;
+        # expected-warning@-2: Explicit default value is redundant for (terse) field: `i` (in `Widget`).
+
+      10: optional bool j = false;
+        # expected-warning@-1: Explicit default value is redundant for (optional) field: `j` (in `Widget`).
+        # expected-warning@-2: Optional field should not have custom default value: `j` (in `Widget`).
+    }
+  )",
+      {"--extra-validation", "warn_on_redundant_custom_default_values"});
+}
+
+TEST(CompilerTest, cpp_deprecated_terse_write) {
+  check_compile(R"(
+    include "thrift/annotation/cpp.thrift"
+    include "thrift/annotation/thrift.thrift"
+
+    struct Bar {}
+
+    struct Foo {
+      @cpp.DeprecatedTerseWrite
+      1: i32 field1;
+
+      @cpp.DeprecatedTerseWrite
+      2: optional i32 field2;
+        # expected-error@-2: @cpp.DeprecatedTerseWrite can be only used on unqualified field.
+
+      @cpp.DeprecatedTerseWrite
+      3: required i32 field3;
+        # expected-warning@-2: The 'required' qualifier is deprecated and ignored by most language implementations. Leave the field unqualified instead.
+        # expected-error@-3: @cpp.DeprecatedTerseWrite can be only used on unqualified field.
+
+      @thrift.TerseWrite
+      @cpp.DeprecatedTerseWrite
+      4: i32 field4;
+        # expected-error@-3: @cpp.DeprecatedTerseWrite can be only used on unqualified field.
+
+      @cpp.DeprecatedTerseWrite
+      5: Bar field5;
+        # expected-error@-2: @cpp.DeprecatedTerseWrite is not supported for structured types.
+
+      @cpp.AllowLegacyNonOptionalRef
+      @cpp.DeprecatedTerseWrite
+      @cpp.AllowLegacyDeprecatedTerseWritesRef
+      @cpp.Ref{type = cpp.RefType.Unique}
+      6: Bar field6;
+        # expected-warning@-5: Field with @cpp.Ref (or similar) annotation should be optional: `field6` (in `Foo`).
+    }
+  )");
+}
+
+TEST(CompilerTest, cpp_deprecated_terse_write_ref) {
+  check_compile(R"(
+    include "thrift/annotation/cpp.thrift"
+    include "thrift/annotation/thrift.thrift"
+
+    struct Foo {
+      @cpp.AllowLegacyNonOptionalRef
+      @cpp.DeprecatedTerseWrite
+      @cpp.Ref{type = cpp.RefType.Unique}
+      1: string field1;
+        # expected-warning@-4: Field with @cpp.Ref (or similar) annotation should be optional: `field1` (in `Foo`).
+        # expected-error@-5: @cpp.Ref{Unique} can not be applied to `field1` since it's cpp.DeprecatedTerseWrite field.
+
+      @cpp.AllowLegacyNonOptionalRef
+      @cpp.DeprecatedTerseWrite
+      @cpp.AllowLegacyDeprecatedTerseWritesRef
+      @cpp.Ref{type = cpp.RefType.Unique}
+      2: string field2;
+        # expected-warning@-5: Field with @cpp.Ref (or similar) annotation should be optional: `field2` (in `Foo`).
+
+      @cpp.DeprecatedTerseWrite
+      @cpp.AllowLegacyDeprecatedTerseWritesRef
+      3: string field3;
+        # expected-error@-3: @cpp.AllowLegacyDeprecatedTerseWritesRef can not be applied to `field3` since it's not cpp.Ref{Unique} field.
+
+      @cpp.AllowLegacyNonOptionalRef
+      @cpp.AllowLegacyDeprecatedTerseWritesRef
+      @cpp.Ref{type = cpp.RefType.Unique}
+      4: string field4;
+        # expected-warning@-4: Field with @cpp.Ref (or similar) annotation should be optional: `field4` (in `Foo`).
+        # expected-error@-5: @cpp.AllowLegacyDeprecatedTerseWritesRef can not be applied to `field4` since it's not cpp.DeprecatedTerseWrite field.
+    }
+  )");
+}
+
+TEST(CompilerTest, base_service_defined_after_use) {
+  check_compile(R"(
+    typedef Base BaseAlias
+
+    service Derived extends Base {}  # expected-error: Service "Base" has not been defined.
+    service DerivedWithAlias extends BaseAlias {}  # expected-error: Service "BaseAlias" has not been defined.
+
+    service Base {}
+  )");
+}
+
+TEST(CompilerTest, invalid_include_alias) {
+  std::map<std::string, std::string> name_contents_map;
+
+  name_contents_map["foo.thrift"] = "";
+  name_contents_map["bar.thrift"] = "";
+  name_contents_map["baz.thrift"] = "";
+  name_contents_map["main.thrift"] = R"(
+    include "baz.thrift" as const; # expected-error: Invalid include alias 'const'
+    include "foo.thrift" as ; # expected-error: Include alias cannot be empty
+    include "bar.thrift" as 123; # expected-error: Invalid include alias 'int literal'
+  )";
+
+  check_compile(name_contents_map, "main.thrift");
+}
+
+TEST(CompilerTest, duplicate_include_alias) {
+  std::map<std::string, std::string> name_contents_map;
+
+  name_contents_map["a/b/c/foo.thrift"] = "";
+  name_contents_map["d/e/bar.thrift"] = "";
+  name_contents_map["main.thrift"] = R"(
+    include "a/b/c/foo.thrift" as foo;
+    include "d/e/bar.thrift" as foo; # expected-error: 'foo' is already an alias for 'a/b/c/foo.thrift'
+    include "a/b/c/foo.thrift" as foo2; # expected-error: Include 'a/b/c/foo.thrift' has multiple aliases: 'foo' vs 'foo2'
+  )";
+
+  check_compile(name_contents_map, "main.thrift");
+}
+
+TEST(CompilerTest, resolve_enum_typedefs_after_use) {
+  std::map<std::string, std::string> name_contents_map;
+  name_contents_map["remote.thrift"] = R"(
+    typedef Remote LazyRemoteAlias;
+
+    enum Remote {
+      FOO = 1,
+    }
+
+    typedef Remote RemoteAlias;
+  )";
+  name_contents_map["main.thrift"] = R"(
+    include "remote.thrift"
+
+    typedef Local LazyLocalAlias;
+
+    struct S {
+      1: remote.Remote field_v1 = remote.Remote.FOO;
+      2: remote.Remote field_v2 = remote.RemoteAlias.FOO;
+      3: remote.Remote field_v3 = remote.FOO;
+      4: remote.Remote field_v4 = LocalAliasForRemote.FOO;
+      5: Local field_v5 = Local.BAR;
+      6: Local field_v6 = LocalAlias.BAR;
+      7: Local field_v8 = BAR;
+    }
+
+    const remote.Remote var_v1 = remote.Remote.FOO;
+    const remote.Remote var_v2 = remote.RemoteAlias.FOO;
+    const remote.Remote var_v3 = remote.FOO;
+    const remote.Remote var_v4 = remote.LazyRemoteAlias.FOO;
+    const remote.Remote var_v5 = LocalAliasForRemote.FOO;
+
+    const Local LAZY_BAR_ALIAS = Local.BAR;
+    const Local LAZY_BAR_ALIAS2 = main.BAR;
+
+    const Local var_v6 = Local.BAR;
+    const Local var_v7 = LocalAlias.BAR;
+    const Local var_v8 = BAR;
+    const Local var_v9 = LazyLocalAlias.BAR;
+    const Local var_v10 = main.LazyLocalAlias.BAR;
+    const Local var_v11 = LAZY_BAR_ALIAS;
+    const Local var_v12 = main.LAZY_BAR_ALIAS;
+    const Local var_v13 = LAZY_BAR_ALIAS2;
+    const Local var_v14 = main.LAZY_BAR_ALIAS2;
+
+    enum Local {
+      BAR = 2,
+    }
+
+    typedef remote.Remote LocalAliasForRemote;
+    typedef Local LocalAlias;
+  )";
+  check_compile(name_contents_map, "main.thrift");
+}
+
+TEST(CompilerTest, include_alias) {
+  std::map<std::string, std::string> name_contents_map;
+  name_contents_map["foo.thrift"] = R"(
+    struct FooBar {
+      1: string name;
+    }
+  )";
+  name_contents_map["bar.thrift"] = R"(
+    struct FooBar {
+      1: i32 id;
+    }
+  )";
+  name_contents_map["main.thrift"] = R"(
+    include "foo.thrift" as foo_alias
+    include "bar.thrift" as bar_alias
+  )";
+  check_compile(name_contents_map, "main.thrift");
+}
+
+// [TEMPORARY] This verifies that unscoped identifiers in injected metadata
+// annotations are resolved to the program they're defined in.
+TEST(CompilerTest, scope_resolution_for_unscoped_injected_metadata) {
+  std::map<std::string, std::string> name_contents_map;
+  name_contents_map["module.thrift"] = R"(
+    include "thrift/annotation/internal.thrift"
+
+    struct Foo {
+      1: map<i16, i32> metadata;
+    }
+
+    @internal.InjectMetadataFields{type = "Foo"}
+    struct MyInjected {}
+
+    @internal.InjectMetadataFields{type = "module.Foo"}
+    struct MyInjected2 {}
+
+  )";
+  name_contents_map["main.thrift"] = R"(
+    include "module.thrift"
+
+    @module.MyInjected
+    struct S1 {
+    }
+
+    @module.MyInjected2
+    struct S2 {
+    }
+  )";
+  check_compile(name_contents_map, "main.thrift");
+}
+
+TEST(CompilerTest, local_vs_global_resolution_sanity) {
+  for (const bool use_global_resolution : {true, false}) {
+    std::map<std::string, std::string> name_content_map;
+
+    name_content_map["c/foo.thrift"] = R"(
+      struct Something {
+        1: i32 val;
+      }
+
+      struct Bar {
+        1: Something field;
+      }
+    )";
+
+    name_content_map["a/foo.thrift"] = R"(
+      include "c/foo.thrift"
+      struct Bar {
+        1: string field;
+      }
+    )";
+
+    name_content_map["b/foo.thrift"] = R"(
+      include "a/foo.thrift"
+
+      struct Bar {
+        1: i32 field;
+      }
+    )";
+
+    name_content_map["local/foo.thrift"] = R"(
+      include "a/foo.thrift"
+      include "b/foo.thrift"
+      include "c/foo.thrift"
+
+      struct Inner {
+        1: string val;
+      }
+
+      struct Bar {
+        1: Inner field;
+      }
+
+      // This should always resolve to the local Bar
+      const Bar bar = Bar{field = Inner{val = "hello"}};
+    )";
+
+    name_content_map["get_b/foo.thrift"] = R"(
+      include "a/foo.thrift"
+      include "b/foo.thrift"
+      include "c/foo.thrift"
+
+      // This should resolve to b's Bar, as it was parsed last
+      // in the hierarchy
+      const foo.Bar bar = foo.Bar{field = 12};
+    )";
+
+    auto options = check_compile_options();
+    options.use_global_resolution = use_global_resolution;
+
+    check_compile(name_content_map, "local/foo.thrift", {}, options);
+    check_compile(name_content_map, "get_b/foo.thrift", {}, options);
+  }
+}
+
+TEST(CompilerTest, include_alias_resolution_priority) {
+  std::map<std::string, std::string> name_contents_map;
+  name_contents_map["foo.thrift"] = R"(
+    struct Bar {
+      1: string name;
+    }
+  )";
+  name_contents_map["other/Foo.thrift"] = R"(
+    struct Bar {
+      1: i32 code;
+    }
+  )";
+  name_contents_map["bar.thrift"] = R"(
+    include "other/Foo.thrift"
+  )";
+
+  name_contents_map["main.thrift"] = R"(
+    include "foo.thrift" as Foo
+    // This transitively includes other/Foo.thrift & should overwrite Foo.Bar
+    // Which clashes with the alias above
+    include "bar.thrift"
+
+    const Foo.Bar MY_BAR = Foo.Bar{name = "hello"};
+  )";
+  check_compile(name_contents_map, "main.thrift");
+}
+
+TEST(CompilerTest, include_alias_resolution_does_not_fallback) {
+  std::map<std::string, std::string> name_contents_map;
+  name_contents_map["foo.thrift"] = R"(
+    struct Annot {
+      1: i32 val;
+    }
+
+    const i32 MY_VAL = 11;
+  )";
+  name_contents_map["a/Foo.thrift"] = R"(
+    const string MY_VAL = "hello";
+  )";
+
+  name_contents_map["bar.thrift"] = R"(
+    include "a/Foo.thrift"
+  )";
+
+  name_contents_map["main.thrift"] = R"(
+    include "foo.thrift" as Foo
+    include "bar.thrift"
+
+    @Foo.Annot{ val = Foo.MY_VAL }
+    struct S {
+    }
+  )";
+  check_compile(name_contents_map, "main.thrift");
+}
+
+TEST(CompilerTest, scope_resolution_duplicate_include_as_alias) {
+  std::map<std::string, std::string> name_contents_map;
+  name_contents_map["v1/foo.thrift"] = R"(
+    struct Bar {
+      1: string name;
+    }
+  )";
+
+  name_contents_map["foo.thrift"] = R"(
+    include "v1/foo.thrift"
+    include "v1/foo.thrift" as FooV2 # expected-warning: Duplicate include of `v1/foo.thrift`
+
+    struct Bar {
+      1: i32 id;
+    }
+
+    // The `foo.Bar` identifier is overriden by the local definition, despite an explicit include
+    const foo.Bar MY_REMOTE_CONST_V1 = foo.Bar{name = "foo"}; # expected-error: no field named `name` in `Bar`
+    const FooV2.Bar MY_REMOTE_CONST_V2 = FooV2.Bar{name = "foo"};
+    const Bar MY_LOCAL_CONST = Bar{id = 1};
+  )";
+
+  check_compile(name_contents_map, "foo.thrift");
+}
+
+TEST(CompilerTest, report_unresolved_identifiers_in_structured_annotations) {
+  std::map<std::string, std::string> name_contents_map;
+  name_contents_map["foo.thrift"] = R"(
+    struct MyAnnot {
+      1: string name;
+      2: i32 id;
+      3: list<float> values;
+    }
+  )";
+
+  name_contents_map["main.thrift"] = R"(
+    include "foo.thrift"
+
+    @foo.MyAnnot{ name = foo.DOES_NOT_EXIST } # expected-error: use of undeclared identifier 'foo.DOES_NOT_EXIST'
+    struct S {}
+
+    @foo.MyAnnot{ id = foo.DOES_NOT_EXIST } # expected-error: use of undeclared identifier 'foo.DOES_NOT_EXIST'
+    struct S2 {}
+
+    @foo.MyAnnot{ values = [
+      foo.DOES_NOT_EXIST, # expected-error: use of undeclared identifier 'foo.DOES_NOT_EXIST'
+      ALSO_DOES_NOT_EXIST # expected-error: use of undeclared identifier 'ALSO_DOES_NOT_EXIST'
+    ] }
+    struct S3 {}
+  )";
+  check_compile(name_contents_map, "main.thrift");
+}
+
+TEST(CompilerTest, reported_unresolved_identifiers_in_constants) {
+  std::map<std::string, std::string> name_contents_map;
+  name_contents_map["foo.thrift"] = R"(
+  )";
+
+  name_contents_map["main.thrift"] = R"(
+    include "foo.thrift"
+
+    const i32 MY_CONST = foo.MY_CONST; # expected-error: use of undeclared identifier 'foo.MY_CONST'
+    const string MY_CONST2 = foo.MY_CONST; # expected-error: use of undeclared identifier 'foo.MY_CONST'
+    const list<i32> MY_CONST3 = foo.MY_CONST; # expected-error: use of undeclared identifier 'foo.MY_CONST'
+    const list<i32> MY_CONST4 = [
+      foo.MY_CONST, # expected-error: use of undeclared identifier 'foo.MY_CONST'
+      foo.MY_CONST2 # expected-error: use of undeclared identifier 'foo.MY_CONST2'
+    ];
+    const map<i32, i32> MY_CONST5 = foo.MY_CONST; # expected-error: use of undeclared identifier 'foo.MY_CONST'
+    const map<i32, i32> MY_CONST6 = { foo.MY_CONST_KEY: # expected-error: use of undeclared identifier 'foo.MY_CONST_KEY'
+      foo.MY_CONST_VALUE }; # expected-error: use of undeclared identifier 'foo.MY_CONST_VALUE'
+    const set<i32> MY_CONST7 = foo.MY_CONST; # expected-error: use of undeclared identifier 'foo.MY_CONST'
+    const set<i32> MY_CONST8 = [ foo.MY_CONST ]; # expected-error: use of undeclared identifier 'foo.MY_CONST'
+  )";
+  check_compile(name_contents_map, "main.thrift");
+}
