@@ -28,7 +28,13 @@ DECLARE_int32(io_max_submit);
 DECLARE_int32(io_registers);
 DECLARE_int32(io_prov_buffs_size);
 DECLARE_int32(io_prov_buffs);
+DECLARE_bool(io_zcrx);
+DECLARE_int32(io_zcrx_num_pages);
+DECLARE_int32(io_zcrx_refill_entries);
+DECLARE_string(io_zcrx_ifname);
+DECLARE_int32(io_zcrx_queue_id);
 
+#if FOLLY_HAS_LIBURING
 namespace apache {
 namespace thrift {
 namespace stress {
@@ -36,3 +42,4 @@ folly::IoUringBackend::Options getIoUringOptions();
 } // namespace stress
 } // namespace thrift
 } // namespace apache
+#endif
