@@ -23,6 +23,7 @@
 #include <random>
 #include <thread>
 
+#include <fmt/chrono.h>
 #include <folly/CPortability.h>
 #include <folly/Synchronized.h>
 #include <folly/executors/Codel.h>
@@ -36,7 +37,6 @@
 #include <thrift/lib/cpp/concurrency/FunctionRunner.h>
 #include <thrift/lib/cpp/concurrency/PosixThreadFactory.h>
 #include <thrift/lib/cpp/concurrency/ThreadManager.h>
-#include <thrift/lib/cpp/concurrency/Util.h>
 
 using namespace apache::thrift::concurrency;
 
@@ -48,20 +48,18 @@ class ThreadManagerTest : public testing::Test {
   gflags::FlagSaver flagsaver_;
 };
 
+#if FOLLY_HAVE_WEAK_SYMBOLS
 static folly::WorkerProvider* kWorkerProviderGlobal = nullptr;
 
 namespace folly {
-
-#if FOLLY_HAVE_WEAK_SYMBOLS
 FOLLY_KEEP std::unique_ptr<folly::QueueObserverFactory>
 make_queue_observer_factory(
     const std::string&, size_t, folly::WorkerProvider* workerProvider) {
   kWorkerProviderGlobal = workerProvider;
   return {};
 }
-#endif
-
 } // namespace folly
+#endif
 
 // Loops until x==y for up to timeout ms.
 // The end result is the same as of {EXPECT,ASSERT}_EQ(x,y)
@@ -87,24 +85,25 @@ make_queue_observer_factory(
 #define CHECK_EQUAL_TIMEOUT(x, y) CHECK_EQUAL_SPECIFIC_TIMEOUT(1000, x, y)
 #define REQUIRE_EQUAL_TIMEOUT(x, y) REQUIRE_EQUAL_SPECIFIC_TIMEOUT(1000, x, y)
 
+using TimePoint = std::chrono::time_point<std::chrono::steady_clock>;
+
+std::chrono::milliseconds toMilliseconds(TimePoint::duration d) {
+  return std::chrono::duration_cast<std::chrono::milliseconds>(d);
+}
+
 class LoadTask : public Runnable {
  public:
   LoadTask(
       std::mutex* mutex,
       std::condition_variable* cond,
       size_t* count,
-      int64_t timeout)
-      : mutex_(mutex),
-        cond_(cond),
-        count_(count),
-        timeout_(timeout),
-        startTime_(0),
-        endTime_(0) {}
+      std::chrono::milliseconds timeout)
+      : mutex_(mutex), cond_(cond), count_(count), timeout_(timeout) {}
 
   void run() override {
-    startTime_ = Util::currentTime();
-    usleep(timeout_ * Util::US_PER_MS);
-    endTime_ = Util::currentTime();
+    startTime_ = std::chrono::steady_clock::now();
+    usleep(std::chrono::microseconds(timeout_).count());
+    endTime_ = std::chrono::steady_clock::now();
 
     {
       std::unique_lock<std::mutex> l(*mutex_);
@@ -119,9 +118,9 @@ class LoadTask : public Runnable {
   std::mutex* mutex_;
   std::condition_variable* cond_;
   size_t* count_;
-  int64_t timeout_;
-  int64_t startTime_;
-  int64_t endTime_;
+  std::chrono::milliseconds timeout_;
+  TimePoint startTime_;
+  TimePoint endTime_;
 };
 
 /**
@@ -129,7 +128,8 @@ class LoadTask : public Runnable {
  * completes. Verify that all tasks completed and that thread manager cleans
  * up properly on delete.
  */
-static void loadTest(size_t numTasks, int64_t timeout, size_t numWorkers) {
+static void loadTest(
+    size_t numTasks, std::chrono::milliseconds timeout, size_t numWorkers) {
   std::mutex mutex;
   std::condition_variable cond;
   size_t tasksLeft = numTasks;
@@ -145,12 +145,12 @@ static void loadTest(size_t numTasks, int64_t timeout, size_t numWorkers) {
         std::make_shared<LoadTask>(&mutex, &cond, &tasksLeft, timeout));
   }
 
-  int64_t startTime = Util::currentTime();
+  TimePoint startTime = std::chrono::steady_clock::now();
   for (const auto& task : tasks) {
     threadManager->add(task);
   }
 
-  int64_t tasksStartedTime = Util::currentTime();
+  TimePoint tasksStartedTime = std::chrono::steady_clock::now();
 
   {
     std::unique_lock<std::mutex> l(mutex);
@@ -158,55 +158,64 @@ static void loadTest(size_t numTasks, int64_t timeout, size_t numWorkers) {
       cond.wait(l);
     }
   }
-  int64_t endTime = Util::currentTime();
+  TimePoint endTime = std::chrono::steady_clock::now();
 
-  int64_t firstTime = std::numeric_limits<int64_t>::max();
-  int64_t lastTime = 0;
-  double averageTime = 0;
-  int64_t minTime = std::numeric_limits<int64_t>::max();
-  int64_t maxTime = 0;
+  TimePoint firstTime = TimePoint::max();
+  TimePoint lastTime = TimePoint::min();
+  double averageTimeMs = 0;
+  using Duration = TimePoint::duration;
+  Duration minTime = Duration::max();
+  Duration maxTime = Duration::min();
 
   for (const auto& task : tasks) {
-    EXPECT_GT(task->startTime_, 0);
-    EXPECT_GT(task->endTime_, 0);
+    EXPECT_GT(task->startTime_, TimePoint());
+    EXPECT_GT(task->endTime_, TimePoint());
 
-    int64_t delta = task->endTime_ - task->startTime_;
-    assert(delta > 0);
+    Duration delta = task->endTime_ - task->startTime_;
+    assert(delta > Duration());
 
     firstTime = std::min(firstTime, task->startTime_);
     lastTime = std::max(lastTime, task->endTime_);
     minTime = std::min(minTime, delta);
     maxTime = std::max(maxTime, delta);
 
-    averageTime += delta;
+    averageTimeMs += toMilliseconds(delta).count();
   }
-  averageTime /= numTasks;
+  averageTimeMs /= numTasks;
 
-  LOG(INFO) << "first start: " << firstTime << "ms " << "last end: " << lastTime
-            << "ms " << "min: " << minTime << "ms " << "max: " << maxTime
-            << "ms " << "average: " << averageTime << "ms";
+  GTEST_LOG_(INFO) << fmt::format(
+      "first start: {} last end: {} min: {} max {} average: {}ms",
+      toMilliseconds(firstTime.time_since_epoch()),
+      toMilliseconds(lastTime.time_since_epoch()),
+      toMilliseconds(minTime),
+      toMilliseconds(maxTime),
+      averageTimeMs);
 
-  double idealTime = ((numTasks + (numWorkers - 1)) / numWorkers) * timeout;
-  double actualTime = endTime - startTime;
-  double taskStartTime = tasksStartedTime - startTime;
+  double idealTimeMs =
+      ((numTasks + (numWorkers - 1)) / numWorkers) * averageTimeMs;
+  double actualTimeMs = toMilliseconds(endTime - startTime).count();
+  Duration taskStartTime = tasksStartedTime - startTime;
 
-  double overheadPct = (actualTime - idealTime) / idealTime;
+  double overheadPct = (actualTimeMs - idealTimeMs) / idealTimeMs;
   if (overheadPct < 0) {
     overheadPct *= -1.0;
   }
 
-  LOG(INFO) << "ideal time: " << idealTime << "ms "
-            << "actual time: " << actualTime << "ms "
-            << "task startup time: " << taskStartTime << "ms "
-            << "overhead: " << overheadPct * 100.0 << "%";
+  GTEST_LOG_(INFO) << fmt::format(
+      "ideal time: {:.2f}ms actual time: {:.2f}ms "
+      "task startup time: {} overhead: {:.2f}ms",
+      idealTimeMs,
+      actualTimeMs,
+      toMilliseconds(taskStartTime),
+      overheadPct * 100);
 
-  // Fail if the test took 10% more time than the ideal time
+  // Fail if the test took 10% more time than the ideal time.
   EXPECT_LT(overheadPct, 0.10);
 }
 
 TEST_F(ThreadManagerTest, LoadTest) {
   size_t numTasks = 10000;
-  int64_t timeout = 50;
+  std::chrono::milliseconds timeout(50);
   size_t numWorkers = 100;
   loadTest(numTasks, timeout, numWorkers);
 }
@@ -267,7 +276,8 @@ static void expireTestCallback(
   }
 }
 
-static void expireTest(size_t numWorkers, int64_t expirationTimeMs) {
+static void expireTest(
+    size_t numWorkers, std::chrono::milliseconds expirationTime) {
   size_t maxPendingTasks = numWorkers;
   size_t activeTasks = numWorkers + maxPendingTasks;
   std::mutex mutex;
@@ -291,11 +301,11 @@ static void expireTest(size_t numWorkers, int64_t expirationTimeMs) {
     auto task = std::make_shared<BlockTask>(
         &mutex, &cond, &bmutex, &bcond, &blocked, &activeTasks);
     tasks.push_back(task);
-    threadManager->add(task, 0, expirationTimeMs);
+    threadManager->add(task, 0, expirationTime.count());
   }
 
   // Sleep for more than the expiration time
-  usleep(expirationTimeMs * Util::US_PER_MS * 1.10);
+  usleep(std::chrono::microseconds(expirationTime).count() * 1.10);
 
   // Unblock the tasks
   {
@@ -325,8 +335,8 @@ static void expireTest(size_t numWorkers, int64_t expirationTimeMs) {
 // DO_BEFORE(aristidis,20250715): Test is flaky. Find owner or remove.
 TEST_F(ThreadManagerTest, DISABLED_ExpireTest) {
   size_t numWorkers = 100;
-  int64_t expireTimeMs = 50;
-  expireTest(numWorkers, expireTimeMs);
+  std::chrono::milliseconds expireTime(50);
+  expireTest(numWorkers, expireTime);
 }
 
 class AddRemoveTask : public Runnable,
@@ -691,7 +701,8 @@ TEST_F(ThreadManagerTest, ObserverTest) {
   threadManager->threadFactory(std::make_shared<PosixThreadFactory>());
   threadManager->start();
 
-  auto task = std::make_shared<LoadTask>(&mutex, &cond, &tasks, 1000);
+  auto task = std::make_shared<LoadTask>(
+      &mutex, &cond, &tasks, std::chrono::milliseconds(1000));
   threadManager->add(task);
   threadManager->join();
   EXPECT_EQ(1, observer->timesCalled);
@@ -810,7 +821,7 @@ TEST_F(ThreadManagerTest, DISABLED_PriorityQueueThreadManagerExecutor) {
   // block the TM
   threadManager->add([&] { reqSyncBaton.wait(); });
 
-  std::string foo = "";
+  std::string foo;
   threadManager->addWithPriority(
       [&] {
         foo += "a";
@@ -879,6 +890,7 @@ TEST_P(JoinTest, DISABLED_Join) {
 INSTANTIATE_TEST_CASE_P(
     ThreadManagerTest, JoinTest, ::testing::ValuesIn(factories));
 
+#if FOLLY_HAVE_WEAK_SYMBOLS
 class TMThreadIDCollectorTest : public ::testing::Test {
  protected:
   void SetUp() override { kWorkerProviderGlobal = nullptr; }
@@ -987,6 +999,7 @@ TEST_F(TMThreadIDCollectorTest, CollectIDBlocksThreadExitTest) {
   EXPECT_EQ(tm->workerCount(), 2);
   tm->join();
 }
+#endif
 
 //
 // =============================================================================
