@@ -191,7 +191,7 @@ class THeader final {
   // Info headers
   // NB: we're using F14NodeMap here because it's a very≥ general map, which
   // sometimes requires reference stability guarantees.
-  typedef folly::F14NodeMap<std::string, std::string> StringToStringMap;
+  using StringToStringMap = folly::F14NodeMap<std::string, std::string>;
 
   /**
    * We know we got a packet in header format here, try to parse the header
@@ -211,7 +211,10 @@ class THeader final {
    * @return IOBuf output data section
    */
   static std::unique_ptr<folly::IOBuf> untransform(
-      std::unique_ptr<folly::IOBuf>, std::vector<uint16_t>& readTrans);
+      std::unique_ptr<folly::IOBuf> buf, std::vector<TTransform>& tTransforms);
+  [[deprecated("Use TTransform version instead.")]]
+  static std::unique_ptr<folly::IOBuf> untransform(
+      std::unique_ptr<folly::IOBuf> buf, std::vector<uint16_t>& readTrans);
 
   /**
    * Transform the data based on our write transform flags
@@ -222,7 +225,12 @@ class THeader final {
    * @return transformed data IOBuf
    */
   static std::unique_ptr<folly::IOBuf> transform(
-      std::unique_ptr<folly::IOBuf>,
+      std::unique_ptr<folly::IOBuf> buf,
+      std::vector<TTransform>& tTransforms,
+      size_t minCompressBytes = 0);
+  [[deprecated("Use TTransform version instead.")]]
+  static std::unique_ptr<folly::IOBuf> transform(
+      std::unique_ptr<folly::IOBuf> buf,
       std::vector<uint16_t>& writeTrans,
       size_t minCompressBytes = 0);
 
@@ -231,17 +239,54 @@ class THeader final {
    */
   void copyMetadataFrom(const THeader& src);
 
+  static uint16_t getNumTTransforms(
+      const std::vector<TTransform>& tTransforms) {
+    return folly::to_narrow(tTransforms.size());
+  }
+  [[deprecated("Use getNumTTransforms instead.")]]
   static uint16_t getNumTransforms(const std::vector<uint16_t>& transforms) {
     return folly::to_narrow(transforms.size());
   }
 
-  void setTransform(uint16_t transId);
-  void setReadTransform(uint16_t transId);
+  void setTTransform(TTransform transId);
+  [[deprecated("Use setTTransform instead.")]]
+  void setTransform(uint16_t transId) {
+    setTTransform(static_cast<TTransform>(transId));
+  }
+
+  void setReadTTransform(TTransform transId);
+  [[deprecated("Use setReadTTransform instead.")]]
+  void setReadTransform(uint16_t transId) {
+    setReadTTransform(static_cast<TTransform>(transId));
+  }
+
+  void setTTransforms(const std::vector<TTransform>& tTransforms) {
+    c_.writeTrans_ = tTransforms;
+  }
+  [[deprecated("Use setTTransforms instead.")]]
   void setTransforms(const std::vector<uint16_t>& trans) {
-    c_.writeTrans_ = trans;
+    static_assert(
+        std::is_same_v<std::underlying_type<TTransform>::type, uint16_t>);
+    setTTransforms(reinterpret_cast<const std::vector<TTransform>&>(trans));
+  }
+
+  const std::vector<TTransform>& getTTransforms() const {
+    return c_.readTrans_;
+  }
+  [[deprecated("Use getTTransforms instead.")]]
+  const std::vector<uint16_t>& getTransforms() const {
+    static_assert(
+        std::is_same_v<std::underlying_type<TTransform>::type, uint16_t>);
+    return reinterpret_cast<const std::vector<uint16_t>&>(getTTransforms());
+  }
+
+  std::vector<TTransform>& getWriteTTransforms() { return c_.writeTrans_; }
+  [[deprecated("Use getWriteTransforms instead.")]]
+  std::vector<uint16_t>& getWriteTransforms() {
+    static_assert(
+        std::is_same_v<std::underlying_type<TTransform>::type, uint16_t>);
+    return reinterpret_cast<std::vector<uint16_t>&>(getWriteTTransforms());
   }
-  const std::vector<uint16_t>& getTransforms() const { return c_.readTrans_; }
-  std::vector<uint16_t>& getWriteTransforms() { return c_.writeTrans_; }
 
   void setClientMetadata(const ClientMetadata& clientMetadata);
   std::optional<ClientMetadata> extractClientMetadata();
@@ -281,16 +326,14 @@ class THeader final {
   void setSequenceNumber(uint32_t sid) { c_.seqId_ = sid; }
 
   enum TRANSFORMS {
-    NONE = 0x00,
-    ZLIB_TRANSFORM = 0x01,
+    NONE = folly::to_underlying(TTransform::NONE),
+    ZLIB_TRANSFORM = folly::to_underlying(TTransform::ZLIB),
     // HMAC_TRANSFORM = 0x02, Deprecated and no longer supported
     // SNAPPY_TRANSFORM = 0x03, Deprecated and no longer supported
     // QLZ_TRANSFORM = 0x04, Deprecated and no longer supported
-    ZSTD_TRANSFORM = 0x05,
-
-    // DO NOT USE. Sentinel value for enum count. Always keep as last value.
-    TRANSFORM_LAST_FIELD = 0x06,
+    ZSTD_TRANSFORM = folly::to_underlying(TTransform::ZSTD),
   };
+  [[deprecated("Use apache::thrift::TTransform instead")]]
 
   /* IOBuf interface */
 
@@ -358,6 +401,14 @@ class THeader final {
 
   folly::Optional<int64_t> getServerLoad() const { return c_.serverLoad_; }
 
+  void setServerSecondaryLoad(folly::Optional<int64_t> load) {
+    c_.serverSecondaryLoad_ = load;
+  }
+
+  folly::Optional<int64_t> getServerSecondaryLoad() const {
+    return c_.serverSecondaryLoad_;
+  }
+
   apache::thrift::concurrency::PRIORITY getCallPriority() const;
 
   std::chrono::milliseconds getTimeoutFromHeader(std::string_view header) const;
@@ -390,10 +441,6 @@ class THeader final {
   void setTenantId(const std::string& tenantId);
   void setServiceTraceMeta(const std::string& serviceTraceMeta);
 
-  // Utility method for converting TRANSFORMS enum to string
-  static const folly::StringPiece getStringTransform(
-      const TRANSFORMS transform);
-
   static CLIENT_TYPE tryGetClientType(const folly::IOBuf& data);
 
   void setRoutingData(std::shared_ptr<void> data) {
@@ -427,6 +474,8 @@ class THeader final {
   static constexpr std::string_view CLIENT_TIMEOUT_HEADER = "client_timeout";
   static constexpr std::string_view QUEUE_TIMEOUT_HEADER = "queue_timeout";
   static constexpr std::string_view QUERY_LOAD_HEADER = "load";
+  static constexpr std::string_view QUERY_SECONDARY_LOAD_HEADER =
+      "secondary_load";
   static constexpr std::string_view kClientId = "client_id";
   static constexpr std::string_view kServiceTraceMeta = "service_trace_meta";
   static constexpr std::string_view kTenantId = "tenant_id";
@@ -494,8 +543,8 @@ class THeader final {
     uint16_t flags_;
     std::string identity_;
 
-    std::vector<uint16_t> readTrans_;
-    std::vector<uint16_t> writeTrans_;
+    std::vector<TTransform> readTrans_;
+    std::vector<TTransform> writeTrans_;
 
     // Map to use for headers
     std::optional<StringToStringMap> readHeaders_;
@@ -524,6 +573,7 @@ class THeader final {
     folly::Optional<uint32_t> crc32c_;
     std::optional<Checksum> checksum_;
     folly::Optional<int64_t> serverLoad_;
+    folly::Optional<int64_t> serverSecondaryLoad_;
 
     std::optional<ProxiedPayloadMetadata> proxiedPayloadMetadata_;
 
