@@ -46,6 +46,21 @@
 #include <immintrin.h>
 #endif
 
+#if defined(__BMI2__) && !defined(__APPLE__)
+// apple silicon can run most x86-64 instructions, but not necessarily all
+#define THRIFT_UTIL_VARINTUTILS_BRANCH_FREE_ENCODER 1
+#elif defined(__ARM_FEATURE_SVE) && defined(__ARM_FEATURE_SVE2_BITPERM) && \
+    __has_include(<arm_neon_sve_bridge.h>)
+#define THRIFT_UTIL_VARINTUTILS_BRANCH_FREE_ENCODER 1
+#else
+#define THRIFT_UTIL_VARINTUTILS_BRANCH_FREE_ENCODER 0
+#endif
+
+#if THRIFT_UTIL_VARINTUTILS_BRANCH_FREE_ENCODER && FOLLY_AARCH64
+#include <arm_neon_sve_bridge.h> // @manual
+#include <arm_sve.h>
+#endif
+
 namespace apache {
 namespace thrift {
 
@@ -95,7 +110,7 @@ void readVarintSlow(CursorT& c, T& value) {
 [[noreturn]] void throwInvalidVarint();
 
 template <class T>
-size_t readVarintMediumSlowUnrolled(T& value, const uint8_t* p) {
+size_t readVarintMediumSlowUnrolledX86(T& value, const uint8_t* p) {
   uint64_t result;
   const uint8_t* start = p;
   do {
@@ -121,6 +136,45 @@ size_t readVarintMediumSlowUnrolled(T& value, const uint8_t* p) {
   return p - start;
 }
 
+// This way of structuring code branching guarantees that only one conditional
+// jump will be taken. It provides a straight-line execution path, while
+// breaking from it when no more bytes need to be read.
+template <class T>
+size_t readVarintMediumSlowUnrolledAarch64(T& result, const uint8_t* p) {
+  T byte;
+  // clang-format off
+  byte = *p++; result  = (byte & 0x7f);       if (UNLIKELY(!(byte & 0x80))) return 1;
+  byte = *p++; result |= (byte & 0x7f) <<  7; if (UNLIKELY(!(byte & 0x80))) return 2;
+  if constexpr (sizeof(T) <= 1) {throwInvalidVarint();} 
+  else {
+    byte = *p++; result |= (byte & 0x7f) << 14; if (UNLIKELY(!(byte & 0x80))) return 3;
+    if constexpr (sizeof(T) <= 2) {throwInvalidVarint();}
+    else {
+      byte = *p++; result |= (byte & 0x7f) << 21; if (UNLIKELY(!(byte & 0x80))) return 4;
+      byte = *p++; result |= (byte & 0x7f) << 28; if (UNLIKELY(!(byte & 0x80))) return 5;
+      if constexpr (sizeof(T) <= 4) {throwInvalidVarint();}
+      else {
+        byte = *p++; result |= (byte & 0x7f) << 35; if (UNLIKELY(!(byte & 0x80))) return 6;
+        byte = *p++; result |= (byte & 0x7f) << 42; if (UNLIKELY(!(byte & 0x80))) return 7;
+        byte = *p++; result |= (byte & 0x7f) << 49; if (UNLIKELY(!(byte & 0x80))) return 8;
+        byte = *p++; result |= (byte & 0x7f) << 56; if (UNLIKELY(!(byte & 0x80))) return 9;
+        byte = *p++; result |= (byte & 0x7f) << 63; if (UNLIKELY(!(byte & 0x80))) return 10;
+        // clang-format on
+        throwInvalidVarint();
+      }
+    }
+  }
+}
+
+template <class T>
+size_t readVarintMediumSlowUnrolled(T& result, const uint8_t* p) {
+  if constexpr (folly::kIsArchAArch64) {
+    return readVarintMediumSlowUnrolledAarch64(result, p);
+  } else {
+    return readVarintMediumSlowUnrolledX86(result, p);
+  }
+}
+
 // The fast-path of the optimized medium-slow paths. Decodes the first two bytes
 // of the varint (into result).
 // Returns true if parsing is finished after those two bytes, and false
@@ -334,23 +388,19 @@ uint8_t writeVarintSlow(Cursor& c, T value) {
   uint8_t* p = c.writableData();
   uint8_t* orig_p = p;
   // precondition: (value & ~0x7f) != 0
+
+#if FOLLY_X64 && !defined(__OPTIMIZE_SIZE__)
+  FOLLY_PRAGMA_UNROLL_N(16)
+#endif
   do {
-    // clang-format off
-    *p++ = ((unval & 0x7f) | 0x80); unval = unval >> 7; if ((unval & ~0x7f) == 0) break;
-    *p++ = ((unval & 0x7f) | 0x80); unval = unval >> 7; if ((unval & ~0x7f) == 0) break;
-    *p++ = ((unval & 0x7f) | 0x80); unval = unval >> 7; if ((unval & ~0x7f) == 0) break;
-    *p++ = ((unval & 0x7f) | 0x80); unval = unval >> 7; if ((unval & ~0x7f) == 0) break;
-    *p++ = ((unval & 0x7f) | 0x80); unval = unval >> 7; if ((unval & ~0x7f) == 0) break;
-    *p++ = ((unval & 0x7f) | 0x80); unval = unval >> 7; if ((unval & ~0x7f) == 0) break;
-    *p++ = ((unval & 0x7f) | 0x80); unval = unval >> 7; if ((unval & ~0x7f) == 0) break;
-    *p++ = ((unval & 0x7f) | 0x80); unval = unval >> 7; if ((unval & ~0x7f) == 0) break;
-    *p++ = ((unval & 0x7f) | 0x80); unval = unval >> 7;
-    // clang-format on
-  } while (false);
+    *p++ = ((unval & 0x7f) | 0x80);
+    unval >>= 7;
+  } while (unval > 0x7f);
 
   *p++ = static_cast<uint8_t>(unval);
-  c.append(p - orig_p);
-  return static_cast<uint8_t>(p - orig_p);
+  uint8_t res = static_cast<uint8_t>(p - orig_p);
+  c.append(res);
+  return res;
 }
 
 } // namespace detail
@@ -365,10 +415,22 @@ uint8_t writeVarintUnrolled(Cursor& c, T value) {
   return detail::writeVarintSlow(c, value);
 }
 
-#ifdef __BMI2__
+#if THRIFT_UTIL_VARINTUTILS_BRANCH_FREE_ENCODER
+
+inline uint64_t compressBits(uint64_t value, uint64_t mask) {
+#if FOLLY_X64
+  return _pdep_u64(value, mask);
+#elif FOLLY_AARCH64
+  // See https://godbolt.org/z/nhc443acd
+  const auto vec = svbdep_u64(svdup_n_u64(value), svdup_n_u64(mask));
+  return vgetq_lane_u64(svget_neonq_u64(vec), 0);
+#else
+  static_assert(0, "no pdep-equivalent instruction is available");
+#endif // __BMI2__, __ARM_FEATURE_SVE2_BITPERM
+}
 
 template <class Cursor, class T>
-uint8_t writeVarintBMI2(Cursor& c, T valueS) {
+uint8_t writeVarintBranchFree(Cursor& c, T valueS) {
   auto value = folly::to_unsigned(valueS);
   if (FOLLY_LIKELY((value & ~0x7f) == 0)) {
     c.template write<uint8_t>(static_cast<uint8_t>(value));
@@ -399,7 +461,7 @@ uint8_t writeVarintBMI2(Cursor& c, T valueS) {
 
   auto clzll = __builtin_clzll(static_cast<uint64_t>(value));
   // Only the first 56 bits of @value will be deposited in @v.
-  uint64_t v = _pdep_u64(value, ~kMask) | (kMask >> kShift[clzll]);
+  uint64_t v = compressBits(value, ~kMask) | (kMask >> kShift[clzll]);
   uint8_t size = kSize[clzll];
 
   if /* constexpr */ (sizeof(T) < sizeof(uint64_t)) {
@@ -421,17 +483,17 @@ uint8_t writeVarintBMI2(Cursor& c, T valueS) {
 
 template <class Cursor, class T>
 uint8_t writeVarint(Cursor& c, T value) {
-  return writeVarintBMI2(c, value);
+  return writeVarintBranchFree(c, value);
 }
 
-#else // __BMI2__
+#else // THRIFT_UTIL_VARINTUTILS_BRANCH_FREE_ENCODER
 
 template <class Cursor, class T>
 uint8_t writeVarint(Cursor& c, T value) {
   return writeVarintUnrolled(c, value);
 }
 
-#endif // __BMI2__
+#endif // THRIFT_UTIL_VARINTUTILS_BRANCH_FREE_ENCODER
 
 inline int32_t zigzagToI32(uint32_t n) {
   return detail::zigzagToSignedInt(n);
