@@ -22,6 +22,7 @@
 #include <utility>
 
 #include <folly/Traits.h>
+#include <folly/lang/Ordering.h>
 #include <thrift/lib/cpp/Field.h>
 #include <thrift/lib/cpp2/Thrift.h>
 
@@ -110,6 +111,20 @@ template <typename Adapter, typename AdaptedT, typename R = void>
 using if_not_less_adapter =
     std::enable_if_t<!is_less_adapter_v<Adapter, AdaptedT>, R>;
 
+// Used to detect if Adapter has a three-way comparison override.
+template <typename Adapter, typename AdaptedT>
+using CompareThreeWayType = decltype(Adapter::compareThreeWay(
+    std::declval<const AdaptedT&>(), std::declval<const AdaptedT&>()));
+template <typename Adapter, typename AdaptedT>
+constexpr bool is_compare_three_way_adapter_v =
+    folly::is_detected_v<CompareThreeWayType, Adapter, AdaptedT>;
+template <typename Adapter, typename AdaptedT, typename R = void>
+using if_compare_three_way_adapter =
+    std::enable_if_t<is_compare_three_way_adapter_v<Adapter, AdaptedT>, R>;
+template <typename Adapter, typename AdaptedT, typename R = void>
+using if_not_compare_three_way_adapter =
+    std::enable_if_t<!is_compare_three_way_adapter_v<Adapter, AdaptedT>, R>;
+
 // Used to detect if Adapter has a clear function override.
 template <typename Adapter, typename AdaptedT>
 using ClearType = decltype(Adapter::clear(std::declval<AdaptedT&>()));
@@ -283,6 +298,57 @@ struct adapter_less<
   }
 };
 
+// CompareThreeWay op based on the thrift types.
+template <typename Adapter, typename AdaptedT>
+struct thrift_compare_three_way {
+  constexpr folly::ordering operator()(
+      const AdaptedT& lhs, const AdaptedT& rhs) const {
+    if (Adapter::toThrift(lhs) == Adapter::toThrift(rhs)) {
+      return folly::ordering::eq;
+    } else if (Adapter::toThrift(lhs) < Adapter::toThrift(rhs)) {
+      return folly::ordering::lt;
+    }
+    return folly::ordering::gt;
+  }
+};
+
+// CompareThreeWay op based on the adapted types, with a fallback on
+// thrift_compare_three_way.
+template <typename Adapter, typename AdaptedT, typename = void>
+struct adapted_compare_three_way : thrift_compare_three_way<Adapter, AdaptedT> {
+};
+template <typename Adapter, typename AdaptedT>
+struct adapted_compare_three_way<
+    Adapter,
+    AdaptedT,
+    folly::void_t<decltype(cr<AdaptedT>() < cr<AdaptedT>())>> {
+  constexpr folly::ordering operator()(
+      const AdaptedT& lhs, const AdaptedT& rhs) const {
+    if (lhs == rhs) {
+      return folly::ordering::eq;
+    } else if (lhs < rhs) {
+      return folly::ordering::lt;
+    }
+    return folly::ordering::gt;
+  }
+};
+
+// CompareThreeWay op based on the adapter, with a fallback on
+// adapted_compare_three_way.
+template <typename Adapter, typename AdaptedT, typename = void>
+struct adapter_compare_three_way
+    : adapted_compare_three_way<Adapter, AdaptedT> {};
+template <typename Adapter, typename AdaptedT>
+struct adapter_compare_three_way<
+    Adapter,
+    AdaptedT,
+    folly::void_t<CompareThreeWayType<Adapter, AdaptedT>>> {
+  constexpr folly::ordering operator()(
+      const AdaptedT& lhs, const AdaptedT& rhs) const {
+    return Adapter::compareThreeWay(lhs, rhs);
+  }
+};
+
 // Hash based on the thrift type.
 template <typename Adapter, typename AdaptedT>
 struct thrift_hash {
