@@ -40,6 +40,7 @@
 #include <thrift/lib/cpp2/async/ClientBufferedStream.h>
 #include <thrift/lib/cpp2/async/ClientSinkBridge.h>
 #include <thrift/lib/cpp2/async/RequestChannel.h>
+#include <thrift/lib/cpp2/async/ServiceInfoHolder.h>
 #include <thrift/lib/cpp2/async/Sink.h>
 #include <thrift/lib/cpp2/async/StreamCallbacks.h>
 #include <thrift/lib/cpp2/detail/meta.h>
@@ -125,9 +126,9 @@ template <int16_t Fid, typename TC, typename T, typename Tag = void>
 struct FieldData {
   static const constexpr int16_t fid = Fid;
   static const constexpr protocol::TType ttype = protocol_type_v<TC, T>;
-  typedef TC type_class;
-  typedef T type;
-  typedef std::remove_pointer_t<T> value_type;
+  using type_class = TC;
+  using type = T;
+  using value_type = std::remove_pointer_t<T>;
   using Ops = Cpp2Ops<value_type>;
   T value;
   static_assert(std::is_pointer_v<T> != std::is_base_of_v<TException, T>, "");
@@ -202,9 +203,9 @@ class ThriftPresult
       public apache::thrift::detail::IsSetHelper<hasIsSet, sizeof...(Field)> {
   // The fields tuple and IsSetHelper are base classes (rather than members)
   // to employ the empty base class optimization when they are empty
-  typedef std::tuple<Field...> Fields;
-  typedef apache::thrift::detail::IsSetHelper<hasIsSet, sizeof...(Field)>
-      CurIsSetHelper;
+  using Fields = std::tuple<Field...>;
+  using CurIsSetHelper =
+      apache::thrift::detail::IsSetHelper<hasIsSet, sizeof...(Field)>;
 
  public:
   using size = std::tuple_size<Fields>;
@@ -328,7 +329,7 @@ struct ThriftPResultSink {
 template <bool hasIsSet, class... Args>
 class Cpp2Ops<ThriftPresult<hasIsSet, Args...>> {
  public:
-  typedef ThriftPresult<hasIsSet, Args...> Presult;
+  using Presult = ThriftPresult<hasIsSet, Args...>;
   static constexpr protocol::TType thriftType() { return protocol::T_STRUCT; }
   template <class Protocol>
   static uint32_t write(Protocol* prot, const Presult* value) {
@@ -1251,8 +1252,7 @@ void populateMethodMetadataMap(
   for (const auto& [methodName, processFuncs] :
        CurrentProcessor::getOwnProcessMap()) {
     const auto& requestInfo = requestInfoMap.at(methodName);
-    std::optional<std::string_view> interactionName =
-        requestInfo.interactionName;
+    std::optional<std::string> interactionName = requestInfo.interactionName;
     if (!interactionName) {
       // If this is a normal RPC that creates an interaction
       interactionName = requestInfo.createdInteraction;
@@ -1430,8 +1430,8 @@ folly::exception_wrapper decode_stream_exception(folly::exception_wrapper ew) {
         auto& payload = err.encoded;
         DCHECK_EQ(payload.metadata.payloadMetadata().has_value(), true);
         DCHECK_EQ(
-            payload.metadata.payloadMetadata()->getType(),
-            PayloadMetadata::Type::exceptionMetadata);
+            folly::to_underlying(payload.metadata.payloadMetadata()->getType()),
+            folly::to_underlying(PayloadMetadata::Type::exceptionMetadata));
         auto& exceptionMetadataBase =
             payload.metadata.payloadMetadata()->get_exceptionMetadata();
         if (auto exceptionMetadataRef = exceptionMetadataBase.metadata()) {
