@@ -24,6 +24,29 @@
 #include <thrift/lib/cpp2/async/ReplyInfo.h>
 #include <thrift/lib/cpp2/server/IResourcePoolAcceptor.h>
 
+// Default to ture, so it can be used for killswitch.
+THRIFT_FLAG_DEFINE_bool(thrift_enable_streaming_tracking, false);
+#if defined(__linux__) && !FOLLY_MOBILE
+/**
+ * TALK TO THE THRIFT TEAM
+ * BEFORE FLIPPING THIS FLAG!
+ */
+DEFINE_bool(
+    EXPERIMENTAL_thrift_enable_streaming_tracking,
+    true,
+    "Enable Thrift Streaming Tracking.");
+#else
+static constexpr bool FLAGS_EXPERIMENTAL_thrift_enable_streaming_tracking =
+    true;
+#endif
+
+namespace {
+bool isStreamTrackingEnabled() {
+  return FLAGS_EXPERIMENTAL_thrift_enable_streaming_tracking &&
+      THRIFT_FLAG(thrift_enable_streaming_tracking);
+}
+} // namespace
+
 namespace apache::thrift {
 
 thread_local RequestParams ServerInterface::requestParams_;
@@ -191,19 +214,21 @@ bool GeneratedAsyncProcessorBase::createInteraction(ServerRequest& req) {
     }
     return tile;
   };
-  auto& conn = *req.requestContext()->getConnectionContext();
+  auto* reqCtx = req.requestContext();
+  auto& conn = *reqCtx->getConnectionContext();
   bool isFactoryFunction = req.methodMetadata()->createsInteraction;
-  auto interactionCreate = req.requestContext()->getInteractionCreate();
+  auto interactionCreate = reqCtx->getInteractionCreate();
   auto isEbMethod =
       req.methodMetadata()->executorType == MethodMetadata::ExecutorType::EVB;
-  auto id = req.requestContext()->getInteractionId();
+  auto id = reqCtx->getInteractionId();
 
   // In the eb model with old-style constructor we create the interaction
   // inline.
   if (isEbMethod && !isFactoryFunction) {
     auto tile = folly::makeTryWith([&] {
       return nullthrows(createInteractionImpl(
-          std::move(*interactionCreate->interactionName_ref()).str()));
+          std::move(*interactionCreate->interactionName_ref()).str(),
+          reqCtx->getHeader()->getProtocolId()));
     });
     if (tile.hasException()) {
       req.request()->sendErrorWrapped(
@@ -213,6 +238,8 @@ bool GeneratedAsyncProcessorBase::createInteraction(ServerRequest& req) {
           kInteractionConstructorErrorErrorCode);
       return true; // Not a duplicate; caller will see missing tile.
     }
+    tile.value()->setOverloadPolicy(
+        InteractionOverloadPolicy::createFromThriftFlag());
     return conn.addTile(id, {tile->release(), &eb});
   }
 
@@ -228,11 +255,20 @@ bool GeneratedAsyncProcessorBase::createInteraction(ServerRequest& req) {
   // Old-style constructor + tm : schedule constructor and return
   if (!isFactoryFunction) {
     apache::thrift::detail::ServerRequestHelper::executor(req)->add(
-        [=, &eb, &conn] {
+        [this,
+         &eb,
+         &conn,
+         nullthrows,
+         interactionCreate,
+         promisePtr,
+         executor,
+         id,
+         reqCtx] {
           std::exception_ptr ex;
           try {
             auto tilePtr = nullthrows(createInteractionImpl(
-                std::move(*interactionCreate->interactionName_ref()).str()));
+                std::move(*interactionCreate->interactionName_ref()).str(),
+                reqCtx->getHeader()->getProtocolId()));
             eb.add([=, &conn, &eb, t = std::move(tilePtr)]() mutable {
               TilePtr tile{t.release(), &eb};
               promisePtr->fulfill(*tile, executor, eb);
@@ -283,8 +319,10 @@ bool GeneratedAsyncProcessorBase::createInteraction(
   if (!tm && !isFactoryFunction) {
     si->setEventBase(&eb);
     si->setRequestContext(&ctx);
-    auto tile = folly::makeTryWith(
-        [&] { return nullthrows(createInteractionImpl(name)); });
+    auto tile = folly::makeTryWith([&] {
+      return nullthrows(
+          createInteractionImpl(name, ctx.getHeader()->getProtocolId()));
+    });
     if (tile.hasException()) {
       req->sendErrorWrapped(
           folly::make_exception_wrapper<TApplicationException>(
@@ -293,6 +331,8 @@ bool GeneratedAsyncProcessorBase::createInteraction(
           kInteractionConstructorErrorErrorCode);
       return true; // Not a duplicate; caller will see missing tile.
     }
+    tile.value()->setOverloadPolicy(
+        InteractionOverloadPolicy::createFromThriftFlag());
     return conn.addTile(id, {tile->release(), &eb});
   }
 
@@ -305,14 +345,24 @@ bool GeneratedAsyncProcessorBase::createInteraction(
 
   // Old-style constructor + tm : schedule constructor and return
   if (!isFactoryFunction) {
-    tm->add([=, &eb, &ctx, name = std::move(name), &conn] {
+    tm->add([this,
+             &eb,
+             &ctx,
+             name = std::move(name),
+             &conn,
+             si,
+             tm,
+             nullthrows,
+             promisePtr,
+             id] {
       si->setEventBase(&eb);
       si->setThreadManager(tm);
       si->setRequestContext(&ctx);
 
       std::exception_ptr ex;
       try {
-        auto tilePtr = nullthrows(createInteractionImpl(name));
+        auto tilePtr = nullthrows(
+            createInteractionImpl(name, ctx.getHeader()->getProtocolId()));
         eb.add([=, &conn, &eb, t = std::move(tilePtr)]() mutable {
           TilePtr tile{t.release(), &eb};
           promisePtr->fulfill(*tile, tm, eb);
@@ -340,12 +390,12 @@ bool GeneratedAsyncProcessorBase::createInteraction(
 }
 
 std::unique_ptr<Tile> GeneratedAsyncProcessorBase::createInteractionImpl(
-    const std::string&) {
+    const std::string&, int16_t) {
   return nullptr;
 }
 
 namespace {
-constexpr std::string_view kNONE = "NONE";
+const std::string kNONE = "NONE";
 /**
  * Call this version to only invoke handlers that explicitly okays
  * callbacks from non-per-request contexts.
@@ -581,7 +631,7 @@ bool GeneratedAsyncProcessorBase::setUpRequestProcessing(ServerRequest& req) {
     if (auto interactionCreate = ctx->getInteractionCreate()) {
       if (!interactionName ||
           *interactionCreate->interactionName_ref() !=
-              interactionName.value()) {
+              std::string_view(interactionName.value())) {
         interactionMetadataValid = false;
       } else if (!createInteraction(req)) {
         // Duplicate id is a contract violation so close the connection.
@@ -874,6 +924,7 @@ void HandlerCallbackBase::doExceptionWrapped(folly::exception_wrapper ew) {
 }
 
 void HandlerCallbackBase::sendReply(SerializedResponse response) {
+  this->ctx_.reset();
   folly::Optional<uint32_t> crc32c = checksumIfNeeded(response);
   auto payload = std::move(response).extractPayload(
       req_->includeEnvelope(),
@@ -895,6 +946,9 @@ void HandlerCallbackBase::sendReply(SerializedResponse response) {
 
 void HandlerCallbackBase::sendReply(
     ResponseAndServerStreamFactory&& responseAndStream) {
+  if (!isStreamTrackingEnabled()) {
+    this->ctx_.reset();
+  }
   folly::Optional<uint32_t> crc32c =
       checksumIfNeeded(responseAndStream.response);
   auto payload = std::move(responseAndStream.response)
@@ -907,6 +961,7 @@ void HandlerCallbackBase::sendReply(
   payload = transform(std::move(payload));
   auto& stream = responseAndStream.stream;
   stream.setInteraction(std::move(interaction_));
+  stream.setContextStack(std::move(this->ctx_));
   if (getEventBase()->isInEventBaseThread()) {
     StreamReplyInfo(
         std::move(req_), std::move(stream), std::move(payload), crc32c)();
@@ -924,6 +979,9 @@ void HandlerCallbackBase::sendReply(
     [[maybe_unused]] std::pair<
         SerializedResponse,
         apache::thrift::detail::SinkConsumerImpl>&& responseAndSinkConsumer) {
+  if (!isStreamTrackingEnabled()) {
+    this->ctx_.reset();
+  }
 #if FOLLY_HAS_COROUTINES
   folly::Optional<uint32_t> crc32c =
       checksumIfNeeded(responseAndSinkConsumer.first);
@@ -937,6 +995,7 @@ void HandlerCallbackBase::sendReply(
   payload = transform(std::move(payload));
   auto& sinkConsumer = responseAndSinkConsumer.second;
   sinkConsumer.interaction = std::move(interaction_);
+  sinkConsumer.contextStack = std::move(this->ctx_);
 
   if (getEventBase()->isInEventBaseThread()) {
     SinkConsumerReplyInfo(
@@ -993,6 +1052,7 @@ HandlerCallback<void>::HandlerCallback(
     ResponseChannelRequest::UniquePtr req,
     ContextStack::UniquePtr ctx,
     const char* serviceName,
+    const char* definingServiceName,
     const char* methodName,
     cob_ptr cp,
     exnw_ptr ewp,
@@ -1005,6 +1065,7 @@ HandlerCallback<void>::HandlerCallback(
           std::move(req),
           std::move(ctx),
           serviceName,
+          definingServiceName,
           methodName,
           ewp,
           eb,
@@ -1019,6 +1080,7 @@ HandlerCallback<void>::HandlerCallback(
     ResponseChannelRequest::UniquePtr req,
     ContextStack::UniquePtr ctx,
     const char* serviceName,
+    const char* definingServiceName,
     const char* methodName,
     cob_ptr cp,
     exnw_ptr ewp,
@@ -1034,6 +1096,7 @@ HandlerCallback<void>::HandlerCallback(
           std::move(req),
           std::move(ctx),
           serviceName,
+          definingServiceName,
           methodName,
           ewp,
           eb,
@@ -1121,11 +1184,11 @@ HandlerCallbackBase::processServiceInterceptorsOnRequest(
   const apache::thrift::server::ServerConfigs* server =
       reqCtx_->getConnectionContext()->getWorkerContext()->getServerContext();
   DCHECK(server);
-  const std::vector<server::ServerConfigs::ServiceInterceptorInfo>&
-      serviceInterceptorsInfo = server->getServiceInterceptors();
+  const std::vector<std::shared_ptr<ServiceInterceptorBase>>&
+      serviceInterceptors = server->getServiceInterceptors();
   std::vector<std::pair<std::size_t, std::exception_ptr>> exceptions;
 
-  for (std::size_t i = 0; i < serviceInterceptorsInfo.size(); ++i) {
+  for (std::size_t i = 0; i < serviceInterceptors.size(); ++i) {
     auto* connectionCtx = reqCtx_->getConnectionContext();
     auto connectionInfo = ServiceInterceptorBase::ConnectionInfo{
         connectionCtx,
@@ -1135,10 +1198,12 @@ HandlerCallbackBase::processServiceInterceptorsOnRequest(
         reqCtx_->getStorageForServiceInterceptorOnRequestByIndex(i),
         arguments,
         serviceName_,
-        methodName_};
+        definingServiceName_,
+        methodName_,
+        reqCtx_->getInterceptorFrameworkMetadata()};
     try {
-      co_await serviceInterceptorsInfo[i].interceptor->internal_onRequest(
-          std::move(connectionInfo), std::move(requestInfo));
+      co_await serviceInterceptors[i]->internal_onRequest(
+          connectionInfo, requestInfo, server->getInterceptorMetricCallback());
     } catch (...) {
       exceptions.emplace_back(i, folly::current_exception());
     }
@@ -1146,12 +1211,14 @@ HandlerCallbackBase::processServiceInterceptorsOnRequest(
   if (!exceptions.empty()) {
     std::string message = fmt::format(
         "ServiceInterceptor::onRequest threw exceptions:\n[{}] {}\n",
-        serviceInterceptorsInfo[exceptions[0].first].qualifiedName,
+        serviceInterceptors[exceptions[0].first]->getQualifiedName().toString(),
         folly::exceptionStr(exceptions[0].second));
     for (std::size_t i = 1; i < exceptions.size(); ++i) {
       message += fmt::format(
           "[{}] {}\n",
-          serviceInterceptorsInfo[exceptions[i].first].qualifiedName,
+          serviceInterceptors[exceptions[i].first]
+              ->getQualifiedName()
+              .toString(),
           folly::exceptionStr(exceptions[i].second));
     }
     co_yield folly::coro::co_error(TApplicationException(message));
@@ -1167,12 +1234,11 @@ HandlerCallbackBase::processServiceInterceptorsOnResponse(
   const apache::thrift::server::ServerConfigs* server =
       reqCtx_->getConnectionContext()->getWorkerContext()->getServerContext();
   DCHECK(server);
-  const std::vector<server::ServerConfigs::ServiceInterceptorInfo>&
-      serviceInterceptorsInfo = server->getServiceInterceptors();
+  const std::vector<std::shared_ptr<ServiceInterceptorBase>>&
+      serviceInterceptors = server->getServiceInterceptors();
   std::vector<std::pair<std::size_t, std::exception_ptr>> exceptions;
 
-  for (auto i = std::ptrdiff_t(serviceInterceptorsInfo.size()) - 1; i >= 0;
-       --i) {
+  for (auto i = std::ptrdiff_t(serviceInterceptors.size()) - 1; i >= 0; --i) {
     auto* connectionCtx = reqCtx_->getConnectionContext();
     auto connectionInfo = ServiceInterceptorBase::ConnectionInfo{
         connectionCtx,
@@ -1182,9 +1248,10 @@ HandlerCallbackBase::processServiceInterceptorsOnResponse(
         reqCtx_->getStorageForServiceInterceptorOnRequestByIndex(i),
         resultOrActiveException,
         serviceName_,
+        definingServiceName_,
         methodName_};
     try {
-      co_await serviceInterceptorsInfo[i].interceptor->internal_onResponse(
+      co_await serviceInterceptors[i]->internal_onResponse(
           std::move(connectionInfo), std::move(responseInfo));
     } catch (...) {
       exceptions.emplace_back(i, folly::current_exception());
@@ -1194,12 +1261,14 @@ HandlerCallbackBase::processServiceInterceptorsOnResponse(
   if (!exceptions.empty()) {
     std::string message = fmt::format(
         "ServiceInterceptor::onResponse threw exceptions:\n[{}] {}\n",
-        serviceInterceptorsInfo[exceptions[0].first].qualifiedName,
+        serviceInterceptors[exceptions[0].first]->getQualifiedName().toString(),
         folly::exceptionStr(exceptions[0].second));
     for (std::size_t i = 1; i < exceptions.size(); ++i) {
       message += fmt::format(
           "[{}] {}\n",
-          serviceInterceptorsInfo[exceptions[i].first].qualifiedName,
+          serviceInterceptors[exceptions[i].first]
+              ->getQualifiedName()
+              .toString(),
           folly::exceptionStr(exceptions[i].second));
     }
     co_yield folly::coro::co_error(TApplicationException(message));
