@@ -43,6 +43,7 @@
 #include <thrift/lib/cpp2/SerializationSwitch.h>
 #include <thrift/lib/cpp2/Thrift.h>
 #include <thrift/lib/cpp2/TrustedServerException.h>
+#include <thrift/lib/cpp2/async/AsyncProcessorFactory.h>
 #include <thrift/lib/cpp2/async/Interaction.h>
 #include <thrift/lib/cpp2/async/ReplyInfo.h>
 #include <thrift/lib/cpp2/async/ResponseChannel.h>
@@ -77,285 +78,9 @@ template <typename T>
 struct HandlerCallbackHelper;
 }
 
-class AsyncProcessor;
-class ServiceHandlerBase;
 class ServerRequest;
 class IResourcePoolAcceptor;
 
-// This contains information about a request that is required in the thrift
-// server prior to the AsyncProcessor::executeRequest interface.
-struct ServiceRequestInfo {
-  bool isSync; // True if this has thread='eb'
-  RpcKind rpcKind; // Type of this request
-  // The qualified function name is currently an input to TProcessorEventHandler
-  // callbacks. We will refactor TProcessorEventHandler to remove the
-  // requirement to pass this as a single string. T112104402
-  const char* functionName_deprecated; // Qualified function name (includes
-                                       // service name)
-  std::optional<std::string_view>
-      interactionName; // Interaction name if part of an interaction
-  concurrency::PRIORITY priority; // Method priority set in the IDL
-  std::optional<std::string_view>
-      createdInteraction; // The name of the interaction created by the RPC
-};
-
-using ServiceRequestInfoMap =
-    folly::F14ValueMap<std::string, ServiceRequestInfo>;
-
-// The base class for generated code that contains information about a service.
-// Each service generates a subclass of this.
-class ServiceInfoHolder {
- public:
-  virtual ~ServiceInfoHolder() = default;
-
-  // This function is generated from the thrift IDL.
-  virtual const ServiceRequestInfoMap& requestInfoMap() const = 0;
-};
-
-// Returned by resource pool components when a request is rejected.
-class ServerRequestRejection {
- public:
-  explicit ServerRequestRejection(TApplicationException&& exception)
-      : impl_(std::move(exception)) {}
-
-  const TApplicationException& applicationException() const& { return impl_; }
-
-  TApplicationException applicationException() && { return std::move(impl_); }
-
- private:
-  TApplicationException impl_;
-};
-
-// Returned to choose a resource pool
-using SelectPoolResult = std::variant<
-    std::monostate, // No opinion (use a reasonable default)
-    std::reference_wrapper<const ResourcePoolHandle>, // Use this ResourcePool
-    ServerRequestRejection>; // Reject the request with this reason
-
-/**
- * Descriptor of a Thrift service - its methods and how they should be handled.
- */
-class AsyncProcessorFactory {
- public:
-#if defined(THRIFT_SCHEMA_AVAILABLE)
-  /**
-   * Reflects on the current service's methods, associated structs etc. at
-   * runtime. This is useful to, for example, a tool that can send requests to a
-   * service without knowing its schemata ahead of time.
-   *
-   * This is analogous to GraphQL introspection
-   * (https://graphql.org/learn/introspection/) and can be used to build a tool
-   * like GraphiQL.
-   */
-  virtual std::optional<schema::DefinitionsSchema> getServiceSchema() {
-    return {};
-  }
-#endif
-  /**
-   * Creates a per-connection processor that will handle requests for this
-   * service. The returned AsyncProcessor has an implicit contract with the
-   * result of createMethodMetadata() - they are tightly coupled. Typically,
-   * both will need to be overridden.
-   */
-  virtual std::unique_ptr<AsyncProcessor> getProcessor() = 0;
-  /**
-   * Returns the known list of user-implemented service handlers. For generated
-   * services, the AsyncProcessorFactory also serves as a ServiceHandlerBase.
-   * However, custom implementations may wrap one or more other
-   * AsyncProcessorFactory's, in which case, they MUST return their combined
-   * result.
-   */
-  virtual std::vector<ServiceHandlerBase*> getServiceHandlers() = 0;
-
-  struct WildcardMethodMetadata;
-
-  struct MethodMetadata {
-    enum class ExecutorType { UNKNOWN, EVB, ANY };
-
-    enum class InteractionType { UNKNOWN, NONE, INTERACTION_V1 };
-
-    MethodMetadata() = default;
-
-   private:
-    explicit MethodMetadata(ExecutorType executor) : executorType(executor) {}
-
-    friend struct WildcardMethodMetadata;
-
-   public:
-    MethodMetadata(
-        ExecutorType executor,
-        InteractionType interaction,
-        RpcKind kind,
-        concurrency::PRIORITY prio,
-        const std::optional<std::string_view> interactName,
-        bool createsInteract)
-        : executorType(executor),
-          interactionType(interaction),
-          rpcKind(kind),
-          priority(prio),
-          interactionName(interactName),
-          createsInteraction(createsInteract) {}
-
-   protected:
-    MethodMetadata(const MethodMetadata& other)
-        : executorType(other.executorType),
-          interactionType(other.interactionType),
-          rpcKind(other.rpcKind),
-          priority(other.priority),
-          interactionName(other.interactionName),
-          createsInteraction(other.createsInteraction) {}
-
-    std::string describeFields() const;
-
-   public:
-    virtual ~MethodMetadata() = default;
-
-    bool isWildcard() const {
-      if (auto status = isWildcard_.load(); status != WildcardStatus::UNKNOWN) {
-        return status == WildcardStatus::YES;
-      }
-      auto status = dynamic_cast<const WildcardMethodMetadata*>(this)
-          ? WildcardStatus::YES
-          : WildcardStatus::NO;
-      auto expected = WildcardStatus::UNKNOWN;
-      isWildcard_.compare_exchange_strong(expected, status);
-      return status == WildcardStatus::YES;
-    }
-
-    virtual std::string describe() const;
-
-    const ExecutorType executorType{ExecutorType::UNKNOWN};
-    const InteractionType interactionType{InteractionType::UNKNOWN};
-    const std::optional<RpcKind> rpcKind{};
-    const std::optional<concurrency::PRIORITY> priority{};
-    const std::optional<std::string_view> interactionName;
-    const bool createsInteraction{false};
-
-   private:
-    enum class WildcardStatus : std::uint8_t { UNKNOWN, NO, YES };
-    mutable folly::relaxed_atomic<WildcardStatus> isWildcard_{
-        WildcardStatus::UNKNOWN};
-  };
-
-  /**
-   * The concrete metadata type that will be passed if createMethodMetadata
-   * returns WildcardMethodMetadataMap and the current method is not in its
-   * knownMethods. This will carry all the information shared by wildcard
-   * methods.
-   */
-  struct WildcardMethodMetadata final : public MethodMetadata {
-    explicit WildcardMethodMetadata(ExecutorType executorType)
-        : MethodMetadata(executorType) {}
-    WildcardMethodMetadata() : WildcardMethodMetadata(ExecutorType::UNKNOWN) {}
-    WildcardMethodMetadata(const WildcardMethodMetadata&) = delete;
-    WildcardMethodMetadata& operator=(const WildcardMethodMetadata&) = delete;
-
-    std::string describe() const override;
-  };
-
-  /**
-   * A map of method names to some loosely typed metadata that will be
-   * passed to AsyncProcessor::processSerializedRequest. The concrete type of
-   * the entries in the map is a contract between the AsyncProcessorFactory and
-   * the AsyncProcessor returned by getProcessor.
-   */
-  using MethodMetadataMap =
-      folly::F14FastMap<std::string, std::shared_ptr<const MethodMetadata>>;
-
-  static std::string describe(const MethodMetadataMap&);
-
-  /**
-   * A marker struct indicating that the AsyncProcessor supports any method, or
-   * a list of methods that is not enumerable. This applies to AsyncProcessor
-   * implementations such as proxies to external services.
-   * The implementation may optionally enumerate a subset of known methods.
-   */
-  struct WildcardMethodMetadataMap {
-    /**
-     * Shared metadata that will be used for all methods
-     * not present in knownMethods.
-     */
-    std::shared_ptr<const WildcardMethodMetadata> wildcardMetadata{};
-    MethodMetadataMap knownMethods;
-  };
-
-  static std::string describe(const WildcardMethodMetadataMap&);
-
-  using CreateMethodMetadataResult =
-      std::variant<MethodMetadataMap, WildcardMethodMetadataMap>;
-
-  static std::string describe(const CreateMethodMetadataResult&);
-
-  /**
-   * This function enumerates the list of methods supported by the
-   * AsyncProcessor returned by getProcessor(), if possible. The return value
-   * represents one of the following states:
-   *   1. This API is supported and there is a static list of known methods.
-   *      This applies to all generated AsyncProcessors.
-   *   2. This API is supported but the complete set of methods is not known or
-   *      is not enumerable (e.g. all method names supported). This applies, for
-   *      example, to AsyncProcessors that proxy to external services.
-   *
-   * If returning (1), Thrift server will lookup the method metadata in the map.
-   * If the method name is not found, a not-found error will be sent and
-   * getProcessor will not be called. Any metadata passed to the processor will
-   * always be a reference from the map.
-   *
-   * If returning (2), Thrift server will lookup the method metadata in the map.
-   * If the method name is not found, Thrift will pass a WildcardMethodMetadata
-   * object instead (kWildcardMethodMetadata). Any metadata passed to the
-   * processor will always be a reference from the map (or be
-   * kWildcardMethodMetadata).
-   */
-  virtual CreateMethodMetadataResult createMethodMetadata() {
-    // For generated APFs, this will lead to fallback to old logic
-    // For custom APFs, we shouldn't use this default but owner shoul
-    // override this function to provide dedicated executorType of their
-    // service
-    WildcardMethodMetadataMap wildcardMap;
-    wildcardMap.wildcardMetadata = std::make_shared<WildcardMethodMetadata>();
-    wildcardMap.knownMethods = {};
-
-    return wildcardMap;
-  }
-
-  /**
-   * Override to return a pre-initialized RequestContext.
-   * Its content will be copied in the RequestContext initialized at
-   * the beginning of each thrift request processing.
-   *
-   * The method metadata (per createMethodMetadata's contract) is also passed
-   * in. If createMethodMetadata is unimplemented or the method is not found,
-   * then this function will not be called.
-   */
-  virtual std::shared_ptr<folly::RequestContext> getBaseContextForRequest(
-      const MethodMetadata&) {
-    return nullptr;
-  }
-
-  /**
-   * Selects the ResourcePool to be used for requests handled by the processor
-   * returned by getProcessor.
-   *
-   * The default implementation of this method has no effect (i.e., default
-   * ResourcePool selection process will be used).
-   *
-   * NOTE: Users of this API must ensure that ResourcePool(s) corresponding to
-   * the ResourcePoolHandle(s) returned by this method exist in the
-   * ResourcePoolSet of the ThriftServer in which the AsyncProcessorFactory is
-   * installed. Failure to do so will result in undefined behavior.
-   */
-  virtual SelectPoolResult selectResourcePool(const ServerRequest&) const {
-    return std::monostate{};
-  }
-
-  virtual ~AsyncProcessorFactory() = default;
-
-  virtual bool isThriftGenerated() const { return false; }
-};
-
-class AsyncProcessor;
-
 /**
  * A class that is created once per-connection and handles incoming requests.
  * This is the hand-off point from Thrift's IO threads to user code - the
@@ -585,7 +310,7 @@ class ServerRequest {
     return sr.resourcePool_ ? sr.resourcePool_ : nullptr;
   }
 
-  static InternalPriority internalPriority(ServerRequest& sr) {
+  static InternalPriority internalPriority(const ServerRequest& sr) {
     return sr.priority_;
   }
 
@@ -789,7 +514,11 @@ class GeneratedAsyncProcessorBase : public AsyncProcessor {
   bool createInteraction(ServerRequest& req);
 
  protected:
-  virtual std::unique_ptr<Tile> createInteractionImpl(const std::string& name);
+  virtual std::unique_ptr<Tile> createInteractionImpl(
+      const std::string& name,
+      // This is only used by Rust, since Rust implementations of interaction is
+      // not fully compatible with standard interaction contract.
+      int16_t protocol);
 
  public:
   void terminateInteraction(
@@ -953,23 +682,20 @@ class ServiceHandlerBase {
 
   virtual folly::SemiFuture<folly::Unit> semifuture_onStartServing() {
 #if FOLLY_HAS_COROUTINES
-    if constexpr (folly::kIsLinux) {
-      return co_onStartServing().semi();
-    }
-#endif
+    return co_onStartServing().semi();
+#else
     return folly::makeSemiFuture();
+#endif
   }
 
   virtual folly::SemiFuture<folly::Unit> semifuture_onStopRequested() {
 #if FOLLY_HAS_COROUTINES
-    if constexpr (folly::kIsLinux) {
-      // TODO(srir): onStopRequested should be implemented similar to
-      // onStartServing
-      try {
-        return co_onStopRequested().semi();
-      } catch (MethodNotImplemented&) {
-        // If co_onStopRequested() is not implemented we just return
-      }
+    // TODO(srir): onStopRequested should be implemented similar to
+    // onStartServing
+    try {
+      return co_onStopRequested().semi();
+    } catch (MethodNotImplemented&) {
+      // If co_onStopRequested() is not implemented we just return
     }
 #endif
     return folly::makeSemiFuture();
@@ -1010,7 +736,7 @@ class ServiceHandlerBase {
  * the same object for both.
  */
 class ServerInterface : public virtual AsyncProcessorFactory,
-                        public ServiceHandlerBase {
+                        public virtual ServiceHandlerBase {
  public:
   ServerInterface() = default;
   ServerInterface(const ServerInterface&) = delete;
@@ -1126,7 +852,7 @@ class ServerInterface : public virtual AsyncProcessorFactory,
         InteractionType interaction,
         RpcKind rpcKind,
         concurrency::PRIORITY priority,
-        const std::optional<std::string_view> interactionName,
+        const std::optional<std::string>& interactionName,
         const bool createsInteraction)
         : MethodMetadata(
               executor,
@@ -1280,6 +1006,7 @@ class HandlerCallbackBase {
       ResponseChannelRequest::UniquePtr req,
       ContextStack::UniquePtr ctx,
       const char* serviceName,
+      const char* definingServiceName,
       const char* methodName,
       exnw_ptr ewp,
       folly::EventBase* eb,
@@ -1290,6 +1017,7 @@ class HandlerCallbackBase {
         ctx_(std::move(ctx)),
         interaction_(std::move(interaction)),
         serviceName_(serviceName),
+        definingServiceName_(definingServiceName),
         methodName_(methodName),
         ewp_(ewp),
         eb_(eb),
@@ -1306,6 +1034,7 @@ class HandlerCallbackBase {
       ResponseChannelRequest::UniquePtr req,
       ContextStack::UniquePtr ctx,
       const char* serviceName,
+      const char* definingServiceName,
       const char* methodName,
       exnw_ptr ewp,
       folly::EventBase* eb,
@@ -1319,6 +1048,7 @@ class HandlerCallbackBase {
         ctx_(std::move(ctx)),
         interaction_(std::move(interaction)),
         serviceName_(serviceName),
+        definingServiceName_(definingServiceName),
         methodName_(methodName),
         ewp_(ewp),
         eb_(eb),
@@ -1525,7 +1255,11 @@ class HandlerCallbackBase {
   ResponseChannelRequest::UniquePtr req_;
   ContextStack::UniquePtr ctx_;
   TilePtr interaction_;
+  // The "leaf" or most derived service name
   const char* const serviceName_ = "";
+  // The service name where the incoming method name is defined (could be
+  // different if inherited service)
+  const char* const definingServiceName_ = "";
   const char* const methodName_ = "";
 
   // May be null in a oneway call
@@ -1626,6 +1360,7 @@ class HandlerCallback : public HandlerCallbackBase {
       ResponseChannelRequest::UniquePtr req,
       ContextStack::UniquePtr ctx,
       const char* serviceName,
+      const char* definingServiceName,
       const char* methodName,
       cob_ptr cp,
       exnw_ptr ewp,
@@ -1639,6 +1374,7 @@ class HandlerCallback : public HandlerCallbackBase {
       ResponseChannelRequest::UniquePtr req,
       ContextStack::UniquePtr ctx,
       const char* serviceName,
+      const char* definingServiceName,
       const char* methodName,
       cob_ptr cp,
       exnw_ptr ewp,
@@ -1716,6 +1452,7 @@ class HandlerCallback<void> : public HandlerCallbackBase {
       ResponseChannelRequest::UniquePtr req,
       ContextStack::UniquePtr ctx,
       const char* serviceName,
+      const char* definingServiceName,
       const char* methodName,
       cob_ptr cp,
       exnw_ptr ewp,
@@ -1729,6 +1466,7 @@ class HandlerCallback<void> : public HandlerCallbackBase {
       ResponseChannelRequest::UniquePtr req,
       ContextStack::UniquePtr ctx,
       const char* serviceName,
+      const char* definingServiceName,
       const char* methodName,
       cob_ptr cp,
       exnw_ptr ewp,
@@ -2078,6 +1816,7 @@ HandlerCallback<T>::HandlerCallback(
     ResponseChannelRequest::UniquePtr req,
     ContextStack::UniquePtr ctx,
     const char* serviceName,
+    const char* definingServiceName,
     const char* methodName,
     cob_ptr cp,
     exnw_ptr ewp,
@@ -2090,6 +1829,7 @@ HandlerCallback<T>::HandlerCallback(
           std::move(req),
           std::move(ctx),
           serviceName,
+          definingServiceName,
           methodName,
           ewp,
           eb,
@@ -2105,6 +1845,7 @@ HandlerCallback<T>::HandlerCallback(
     ResponseChannelRequest::UniquePtr req,
     ContextStack::UniquePtr ctx,
     const char* serviceName,
+    const char* definingServiceName,
     const char* methodName,
     cob_ptr cp,
     exnw_ptr ewp,
@@ -2120,6 +1861,7 @@ HandlerCallback<T>::HandlerCallback(
           std::move(req),
           std::move(ctx),
           serviceName,
+          definingServiceName,
           methodName,
           ewp,
           eb,
@@ -2159,7 +1901,6 @@ void HandlerCallback<T>::doResult(InputType r) {
       this->ctx_.get(),
       executor_ ? executor_.get() : eb_,
       std::forward<InputType>(r));
-  this->ctx_.reset();
   sendReply(std::move(reply));
 }
 
@@ -2169,11 +1910,11 @@ namespace detail {
 // unique_ptr<S>, in which case it typedefs type to S.
 template <class S>
 struct inner_type {
-  typedef S type;
+  using type = S;
 };
 template <class S>
 struct inner_type<std::unique_ptr<S>> {
-  typedef S type;
+  using type = S;
 };
 
 template <typename T>
