@@ -193,7 +193,8 @@ void HeaderClientChannel::sendRequestNoResponse(
     MethodMetadata&& methodMetadata,
     SerializedRequest&& serializedRequest,
     std::shared_ptr<THeader> header,
-    RequestClientCallback::Ptr cb) {
+    RequestClientCallback::Ptr cb,
+    std::unique_ptr<folly::IOBuf> /*unsupported - header has no RpcMetadata*/) {
   preprocessHeader(header.get());
 
   auto buf = LegacySerializedRequest(
@@ -281,7 +282,8 @@ void HeaderClientChannel::sendRequestResponse(
     MethodMetadata&& methodMetadata,
     SerializedRequest&& serializedRequest,
     std::shared_ptr<THeader> header,
-    RequestClientCallback::Ptr cb) {
+    RequestClientCallback::Ptr cb,
+    std::unique_ptr<folly::IOBuf> /*unsupported - header has no RpcMetadata*/) {
   preprocessHeader(header.get());
 
   auto buf = LegacySerializedRequest(
@@ -509,7 +511,8 @@ void HeaderClientChannel::RocketUpgradeChannel::sendRequestResponse(
     apache::thrift::MethodMetadata&& methodMetadata,
     SerializedRequest&& serializedRequest,
     std::shared_ptr<apache::thrift::transport::THeader> header,
-    RequestClientCallback::Ptr cb) {
+    RequestClientCallback::Ptr cb,
+    std::unique_ptr<folly::IOBuf> frameworkMetadata) {
   preprocessHeader(header.get());
 
   initUpgradeIfNeeded(rpcOptions);
@@ -520,6 +523,7 @@ void HeaderClientChannel::RocketUpgradeChannel::sendRequestResponse(
         std::move(serializedRequest),
         std::move(header),
         std::move(cb),
+        std::move(frameworkMetadata),
         false /* oneWay */);
     return;
   }
@@ -531,7 +535,8 @@ void HeaderClientChannel::RocketUpgradeChannel::sendRequestResponse(
       std::move(methodMetadata),
       std::move(serializedRequest),
       std::move(header),
-      std::move(cb));
+      std::move(cb),
+      std::move(frameworkMetadata));
 }
 
 void HeaderClientChannel::RocketUpgradeChannel::sendRequestNoResponse(
@@ -539,7 +544,8 @@ void HeaderClientChannel::RocketUpgradeChannel::sendRequestNoResponse(
     apache::thrift::MethodMetadata&& methodMetadata,
     SerializedRequest&& serializedRequest,
     std::shared_ptr<apache::thrift::transport::THeader> header,
-    RequestClientCallback::Ptr cb) {
+    RequestClientCallback::Ptr cb,
+    std::unique_ptr<folly::IOBuf> frameworkMetadata) {
   preprocessHeader(header.get());
 
   initUpgradeIfNeeded(rpcOptions);
@@ -550,6 +556,7 @@ void HeaderClientChannel::RocketUpgradeChannel::sendRequestNoResponse(
         std::move(serializedRequest),
         std::move(header),
         std::move(cb),
+        std::move(frameworkMetadata),
         true /* oneWay */);
     return;
   }
@@ -561,7 +568,8 @@ void HeaderClientChannel::RocketUpgradeChannel::sendRequestNoResponse(
       std::move(methodMetadata),
       std::move(serializedRequest),
       std::move(header),
-      std::move(cb));
+      std::move(cb),
+      std::move(frameworkMetadata));
 }
 
 void HeaderClientChannel::RocketUpgradeChannel::setCloseCallback(
@@ -720,14 +728,16 @@ void HeaderClientChannel::RocketUpgradeChannel::BufferedRequest::send(
         std::move(methodMetadata_),
         std::move(serializedRequest_),
         std::move(header_),
-        std::move(callback_));
+        std::move(callback_),
+        std::move(frameworkMetadata_));
   } else {
     channel.sendRequestResponse(
         rpcOptions_,
         std::move(methodMetadata_),
         std::move(serializedRequest_),
         std::move(header_),
-        std::move(callback_));
+        std::move(callback_),
+        std::move(frameworkMetadata_));
   }
 }
 
