@@ -63,7 +63,7 @@ const int64_t kRocketClientMinVersion = 8;
 THRIFT_FLAG_DEFINE_bool(rocket_client_new_protocol_key, true);
 THRIFT_FLAG_DEFINE_int64(rocket_client_max_version, kRocketClientMaxVersion);
 THRIFT_FLAG_DEFINE_bool(rocket_client_rocket_skip_protocol_key, false);
-THRIFT_FLAG_DEFINE_bool(rocket_client_enable_bidirectional_propagation, true);
+THRIFT_FLAG_DEFINE_bool(rocket_client_enable_bidirectional_propagation, false);
 
 using namespace apache::thrift::transport;
 
@@ -112,7 +112,9 @@ struct ResponseSerializationHandler {
 
 template <class Handler>
 folly::Try<FirstResponsePayload> decodeResponseError(
-    rocket::RocketException&& ex, Handler&& handler) noexcept {
+    rocket::RocketException&& ex,
+    Handler&& handler,
+    rocket::PayloadSerializer& payloadSerializer) noexcept {
   switch (ex.getErrorCode()) {
     case rocket::ErrorCode::CANCELED:
     case rocket::ErrorCode::INVALID:
@@ -127,8 +129,7 @@ folly::Try<FirstResponsePayload> decodeResponseError(
 
   ResponseRpcError responseError;
   try {
-    rocket::PayloadSerializer::getInstance().unpack(
-        responseError, ex.moveErrorData().get(), false);
+    payloadSerializer.unpack(responseError, ex.moveErrorData().get(), false);
   } catch (...) {
     return folly::Try<FirstResponsePayload>(
         folly::make_exception_wrapper<TApplicationException>(fmt::format(
@@ -206,6 +207,22 @@ folly::Try<FirstResponsePayload> decodeResponseError(
       exCode = kTenantBlocklistedErrorCode;
       exType = TApplicationException::TENANT_BLOCKLISTED;
       break;
+    case ResponseRpcErrorCode::INTERACTION_LOADSHEDDED:
+      exCode = kInteractionLoadsheddedErrorCode;
+      exType = TApplicationException::LOADSHEDDING;
+      break;
+    case ResponseRpcErrorCode::INTERACTION_LOADSHEDDED_OVERLOAD:
+      exCode = kInteractionLoadsheddedOverloadErrorCode;
+      exType = TApplicationException::LOADSHEDDING;
+      break;
+    case ResponseRpcErrorCode::INTERACTION_LOADSHEDDED_APP_OVERLOAD:
+      exCode = kInteractionLoadsheddedAppOverloadErrorCode;
+      exType = TApplicationException::LOADSHEDDING;
+      break;
+    case ResponseRpcErrorCode::INTERACTION_LOADSHEDDED_QUEUE_TIMEOUT:
+      exCode = kInteractionLoadsheddedQueueTimeoutErrorCode;
+      exType = TApplicationException::LOADSHEDDING;
+      break;
     default:
       exCode = kUnknownErrorCode;
       break;
@@ -377,11 +394,13 @@ class FirstRequestProcessorStream : public StreamClientCallback,
       uint16_t protocolId,
       apache::thrift::ManagedStringView&& methodName,
       StreamClientCallback* clientCallback,
-      folly::EventBase* evb)
+      folly::EventBase* evb,
+      rocket::RocketClient::DestructionGuardedClient guardedClient)
       : protocolId_(protocolId),
         methodName_(std::move(methodName)),
         clientCallback_(clientCallback),
-        evb_(evb) {}
+        evb_(evb),
+        guardedClient_(std::move(guardedClient)) {}
 
   FOLLY_NODISCARD bool onFirstResponse(
       FirstResponsePayload&& firstResponse,
@@ -413,7 +432,10 @@ class FirstRequestProcessorStream : public StreamClientCallback,
         [&](rocket::RocketException& ex) {
           LegacyResponseSerializationHandler handler(
               protocolId_, methodName_.view());
-          auto response = decodeResponseError(std::move(ex), handler);
+          auto response = decodeResponseError(
+              std::move(ex),
+              handler,
+              *guardedClient_.client->getPayloadSerializer());
           if (response.hasException()) {
             clientCallback_->onFirstResponseError(
                 std::move(response).exception());
@@ -445,6 +467,7 @@ class FirstRequestProcessorStream : public StreamClientCallback,
   const ManagedStringView methodName_;
   StreamClientCallback* clientCallback_;
   folly::EventBase* evb_;
+  rocket::RocketClient::DestructionGuardedClient guardedClient_;
 };
 
 class FirstRequestProcessorSink : public SinkClientCallback,
@@ -454,11 +477,13 @@ class FirstRequestProcessorSink : public SinkClientCallback,
       uint16_t protocolId,
       apache::thrift::ManagedStringView&& methodName,
       SinkClientCallback* clientCallback,
-      folly::EventBase* evb)
+      folly::EventBase* evb,
+      rocket::RocketClient::DestructionGuardedClient guardedClient)
       : protocolId_(protocolId),
         methodName_(std::move(methodName)),
         clientCallback_(clientCallback),
-        evb_(evb) {}
+        evb_(evb),
+        guardedClient_(std::move(guardedClient)) {}
 
   FOLLY_NODISCARD bool onFirstResponse(
       FirstResponsePayload&& firstResponse,
@@ -492,7 +517,10 @@ class FirstRequestProcessorSink : public SinkClientCallback,
         [&](rocket::RocketException& ex) {
           LegacyResponseSerializationHandler handler(
               protocolId_, methodName_.view());
-          auto response = decodeResponseError(std::move(ex), handler);
+          auto response = decodeResponseError(
+              std::move(ex),
+              handler,
+              *guardedClient_.client->getPayloadSerializer());
           if (response.hasException()) {
             clientCallback_->onFirstResponseError(
                 std::move(response).exception());
@@ -537,6 +565,7 @@ class FirstRequestProcessorSink : public SinkClientCallback,
   const ManagedStringView methodName_;
   SinkClientCallback* clientCallback_;
   folly::EventBase* evb_;
+  rocket::RocketClient::DestructionGuardedClient guardedClient_;
 };
 } // namespace
 
@@ -552,7 +581,8 @@ class RocketClientChannel::SingleRequestSingleResponseCallback final
       size_t requestSerializedSize,
       size_t requestWireSize,
       size_t requestMetadataAndPayloadSize,
-      bool encodeMetadataUsingBinary)
+      bool encodeMetadataUsingBinary,
+      rocket::RocketClient::DestructionGuardedClient guardedClient)
       : cb_(std::move(cb)),
         protocolId_(protocolId),
         methodName_(std::move(methodName)),
@@ -560,7 +590,8 @@ class RocketClientChannel::SingleRequestSingleResponseCallback final
         requestWireSize_(requestWireSize),
         requestMetadataAndPayloadSize_(requestMetadataAndPayloadSize),
         timeBeginSend_(clock::now()),
-        encodeMetadataUsingBinary_(encodeMetadataUsingBinary) {}
+        encodeMetadataUsingBinary_(encodeMetadataUsingBinary),
+        guardedClient_(std::move(guardedClient)) {}
 
   void onWriteSuccess() noexcept override { timeEndSend_ = clock::now(); }
 
@@ -579,7 +610,10 @@ class RocketClientChannel::SingleRequestSingleResponseCallback final
     if (payload.hasException()) {
       if (!payload.exception().with_exception<rocket::RocketException>(
               [&](auto& ex) {
-                response = decodeResponseError(std::move(ex), handler);
+                response = decodeResponseError(
+                    std::move(ex),
+                    handler,
+                    *guardedClient_.client->getPayloadSerializer());
               })) {
         cb_.release()->onResponseError(std::move(payload.exception()));
         return;
@@ -600,9 +634,9 @@ class RocketClientChannel::SingleRequestSingleResponseCallback final
       stats.responseWireSizeBytes =
           payload->metadataAndDataSize() - payload->metadataSize();
 
-      response =
-          rocket::PayloadSerializer::getInstance().unpack<FirstResponsePayload>(
-              std::move(*payload), encodeMetadataUsingBinary_);
+      response = guardedClient_.client->getPayloadSerializer()
+                     ->unpack<FirstResponsePayload>(
+                         std::move(*payload), encodeMetadataUsingBinary_);
       if (response.hasException()) {
         cb_.release()->onResponseError(std::move(response.exception()));
         return;
@@ -669,6 +703,7 @@ class RocketClientChannel::SingleRequestSingleResponseCallback final
   const std::chrono::time_point<clock> timeBeginSend_;
   std::chrono::time_point<clock> timeEndSend_;
   const bool encodeMetadataUsingBinary_;
+  rocket::RocketClient::DestructionGuardedClient guardedClient_;
 };
 
 class RocketClientChannel::SingleRequestNoResponseCallback final
@@ -722,9 +757,14 @@ rocket::SetupFrame RocketClientChannel::makeSetupFrame(
 
   uint32_t serialized_size;
   folly::IOBufQueue paramQueue;
+
   bool encodeMetadataUsingBinary =
       THRIFT_FLAG(rocket_client_binary_rpc_metadata_encoding);
 
+  if (meta.encodeMetadataUsingBinary().has_value()) {
+    encodeMetadataUsingBinary = meta.encodeMetadataUsingBinary().value();
+  }
+
   // TODO: migrate this to
   if (encodeMetadataUsingBinary) {
     BinaryProtocolWriter binaryProtocolWriter;
@@ -806,14 +846,16 @@ void RocketClientChannel::sendRequestResponse(
     apache::thrift::MethodMetadata&& methodMetadata,
     SerializedRequest&& request,
     std::shared_ptr<transport::THeader> header,
-    RequestClientCallback::Ptr cb) {
+    RequestClientCallback::Ptr cb,
+    std::unique_ptr<folly::IOBuf> frameworkMetadata) {
   sendThriftRequest(
       rpcOptions,
       RpcKind::SINGLE_REQUEST_SINGLE_RESPONSE,
       std::move(methodMetadata).name_managed(),
       std::move(request),
       std::move(header),
-      std::move(cb));
+      std::move(cb),
+      std::move(frameworkMetadata));
 }
 
 void RocketClientChannel::sendRequestNoResponse(
@@ -821,14 +863,16 @@ void RocketClientChannel::sendRequestNoResponse(
     apache::thrift::MethodMetadata&& methodMetadata,
     SerializedRequest&& request,
     std::shared_ptr<transport::THeader> header,
-    RequestClientCallback::Ptr cb) {
+    RequestClientCallback::Ptr cb,
+    std::unique_ptr<folly::IOBuf> frameworkMetadata) {
   sendThriftRequest(
       rpcOptions,
       RpcKind::SINGLE_REQUEST_NO_RESPONSE,
       std::move(methodMetadata).name_managed(),
       std::move(request),
       std::move(header),
-      std::move(cb));
+      std::move(cb),
+      std::move(frameworkMetadata));
 }
 
 void RocketClientChannel::sendRequestStream(
@@ -836,7 +880,8 @@ void RocketClientChannel::sendRequestStream(
     apache::thrift::MethodMetadata&& methodMetadata,
     SerializedRequest&& request,
     std::shared_ptr<THeader> header,
-    StreamClientCallback* clientCallback) {
+    StreamClientCallback* clientCallback,
+    std::unique_ptr<folly::IOBuf> frameworkMetadata) {
   DestructorGuard dg(this);
   if (!canHandleRequest(clientCallback)) {
     return;
@@ -853,9 +898,11 @@ void RocketClientChannel::sendRequestStream(
       getInteractionHandle(rpcOptions),
       getServerZstdSupported(),
       buf->computeChainDataLength(),
-      *header);
+      *header,
+      std::move(frameworkMetadata),
+      customCompressor_ != nullptr);
 
-  auto payload = rocket::PayloadSerializer::getInstance().packWithFds(
+  auto payload = getPayloadSerializer()->packWithFds(
       &metadata,
       std::move(buf),
       rpcOptions.copySocketFdsToSend(),
@@ -870,7 +917,8 @@ void RocketClientChannel::sendRequestStream(
           header->getProtocolId(),
           std::move(*metadata.name_ref()),
           clientCallback,
-          evb_));
+          evb_,
+          DestructionGuardedClient(this)));
 }
 
 void RocketClientChannel::sendRequestSink(
@@ -878,7 +926,8 @@ void RocketClientChannel::sendRequestSink(
     apache::thrift::MethodMetadata&& methodMetadata,
     SerializedRequest&& request,
     std::shared_ptr<transport::THeader> header,
-    SinkClientCallback* clientCallback) {
+    SinkClientCallback* clientCallback,
+    std::unique_ptr<folly::IOBuf> frameworkMetadata) {
   DestructorGuard dg(this);
   if (!canHandleRequest(clientCallback)) {
     return;
@@ -895,9 +944,11 @@ void RocketClientChannel::sendRequestSink(
       getInteractionHandle(rpcOptions),
       getServerZstdSupported(),
       buf->computeChainDataLength(),
-      *header);
+      *header,
+      std::move(frameworkMetadata),
+      customCompressor_ != nullptr);
 
-  auto payload = rocket::PayloadSerializer::getInstance().packWithFds(
+  auto payload = getPayloadSerializer()->packWithFds(
       &metadata,
       std::move(buf),
       rpcOptions.copySocketFdsToSend(),
@@ -910,7 +961,8 @@ void RocketClientChannel::sendRequestSink(
           header->getProtocolId(),
           std::move(*metadata.name_ref()),
           clientCallback,
-          evb_),
+          evb_,
+          DestructionGuardedClient(this)),
       header->getDesiredCompressionConfig());
 }
 
@@ -920,7 +972,8 @@ void RocketClientChannel::sendThriftRequest(
     apache::thrift::ManagedStringView&& methodName,
     SerializedRequest&& request,
     std::shared_ptr<transport::THeader> header,
-    RequestClientCallback::Ptr clientCallback) {
+    RequestClientCallback::Ptr clientCallback,
+    std::unique_ptr<folly::IOBuf> frameworkMetadata) {
   DestructorGuard dg(this);
   if (!canHandleRequest(clientCallback)) {
     return;
@@ -937,7 +990,9 @@ void RocketClientChannel::sendThriftRequest(
       getInteractionHandle(rpcOptions),
       getServerZstdSupported(),
       buf->computeChainDataLength(),
-      *header);
+      *header,
+      std::move(frameworkMetadata),
+      customCompressor_ != nullptr);
   header.reset();
 
   switch (kind) {
@@ -968,13 +1023,13 @@ void RocketClientChannel::sendSingleRequestNoResponse(
     RequestRpcMetadata&& metadata,
     std::unique_ptr<folly::IOBuf> buf,
     RequestClientCallback::Ptr cb) {
-  auto requestPayload = rocket::PayloadSerializer::getInstance().packWithFds(
+  auto requestPayload = getPayloadSerializer()->packWithFds(
       &metadata,
       std::move(buf),
       rpcOptions.copySocketFdsToSend(),
       encodeMetadataUsingBinary(),
       getTransportWrapper());
-  const bool isSync = cb->isSync();
+  const bool isSync = cb->isSync() || rpcOptions.getForceSyncOnFiber();
   SingleRequestNoResponseCallback callback(std::move(cb));
 
   if (isSync && folly::fibers::onFiber()) {
@@ -994,16 +1049,26 @@ void RocketClientChannel::sendSingleRequestSingleResponse(
     std::unique_ptr<folly::IOBuf> buf,
     RequestClientCallback::Ptr cb) {
   const auto requestSerializedSize = buf->computeChainDataLength();
-  auto requestPayload = rocket::PayloadSerializer::getInstance().packWithFds(
-      &metadata,
-      std::move(buf),
-      rpcOptions.copySocketFdsToSend(),
-      encodeMetadataUsingBinary(),
-      getTransportWrapper());
+  rocket::Payload requestPayload;
+  try {
+    requestPayload = getPayloadSerializer()->packWithFds(
+        &metadata,
+        std::move(buf),
+        rpcOptions.copySocketFdsToSend(),
+        encodeMetadataUsingBinary(),
+        getTransportWrapper());
+  } catch (std::exception const& ex) {
+    cb.release()->onResponseError(
+        folly::make_exception_wrapper<transport::TTransportException>(
+            transport::TTransportException::TTransportExceptionType::UNKNOWN,
+            fmt::format("Failed to pack request payload: {}", ex.what())));
+    return;
+  }
+
   const auto requestWireSize = requestPayload.dataSize();
   const auto requestMetadataAndPayloadSize =
       requestPayload.metadataAndDataSize();
-  const bool isSync = cb->isSync();
+  const bool isSync = cb->isSync() || rpcOptions.getForceSyncOnFiber();
   SingleRequestSingleResponseCallback callback(
       std::move(cb),
       static_cast<uint16_t>(*metadata.protocol_ref()),
@@ -1011,7 +1076,8 @@ void RocketClientChannel::sendSingleRequestSingleResponse(
       requestSerializedSize,
       requestWireSize,
       requestMetadataAndPayloadSize,
-      encodeMetadataUsingBinary());
+      encodeMetadataUsingBinary(),
+      DestructionGuardedClient(this));
 
   if (isSync && folly::fibers::onFiber()) {
     callback.onResponsePayload(
