@@ -930,6 +930,31 @@ CO_TEST_P(ClientInterceptorTestP, Headers) {
   EXPECT_EQ(interceptor->onResponseHeader, "dummy value");
 }
 
+CO_TEST_P(ClientInterceptorTestP, RpcOptions) {
+  class ClientInterceptorWithRpcOptions
+      : public NamedClientInterceptor<folly::Unit> {
+   public:
+    using RequestState = folly::Unit;
+
+    using NamedClientInterceptor::NamedClientInterceptor;
+
+    std::optional<RequestState> onRequest(RequestInfo requestInfo) override {
+      readContextPropMask = requestInfo.rpcOptions->getContextPropMask();
+      return std::nullopt;
+    }
+
+    uint8_t readContextPropMask = 0;
+  };
+  auto interceptor =
+      std::make_shared<ClientInterceptorWithRpcOptions>("WithRpcOptions");
+  auto client = makeClient(makeInterceptorsList(interceptor));
+
+  RpcOptions rpcOptions;
+  rpcOptions.setContextPropMask(5);
+  co_await client->noop(rpcOptions);
+  EXPECT_EQ(interceptor->readContextPropMask, 5);
+}
+
 CO_TEST_P(ClientInterceptorTestP, BasicStream) {
   if (transportType() != TransportType::ROCKET) {
     // only rocket supports streaming
