@@ -224,6 +224,12 @@ struct SerializedSize<false, type::binary_t> {
   uint32_t operator()(Protocol& prot, const folly::IOBuf& s) const {
     return prot.serializedSizeBinary(s);
   }
+
+  template <typename Protocol>
+  uint32_t operator()(
+      Protocol& prot, const std::unique_ptr<folly::IOBuf>& s) const {
+    return prot.serializedSizeBinary(s);
+  }
 };
 
 template <>
@@ -236,6 +242,11 @@ struct SerializedSize<true, type::binary_t> {
   uint32_t operator()(Protocol& prot, const folly::IOBuf& s) const {
     return prot.serializedSizeZCBinary(s);
   }
+  template <typename Protocol>
+  uint32_t operator()(
+      Protocol& prot, const std::unique_ptr<folly::IOBuf>& s) const {
+    return prot.serializedSizeZCBinary(s);
+  }
 };
 
 template <typename T>
@@ -843,6 +854,9 @@ struct Decode<type::list<Tag>> {
         consumeElem();
       }
     } else if (typeTagToTType<Tag> == t) {
+      if (!canReadNElements(prot, s, {t})) {
+        TProtocolException::throwTruncatedData();
+      }
 #ifndef _MSC_VER
       constexpr auto should_resize_without_initialization =
           std::is_trivial_v<typename ListType::value_type> &&
@@ -889,9 +903,7 @@ struct Decode<type::list<Tag>> {
         }
       }
     } else {
-      while (s--) {
-        prot.skip(t);
-      }
+      apache::thrift::skip_n(prot, s, {t});
     }
 
     prot.readListEnd();
@@ -991,11 +1003,12 @@ struct Decode<type::set<Tag>> {
         consumeElem();
       }
     } else if (typeTagToTType<Tag> == t) {
+      if (!canReadNElements(prot, s, {t})) {
+        TProtocolException::throwTruncatedData();
+      }
       decode_known_length_set(prot, set, s);
     } else {
-      while (s--) {
-        prot.skip(t);
-      }
+      apache::thrift::skip_n(prot, s, {t});
     }
 
     prot.readSetEnd();
@@ -1073,12 +1086,12 @@ struct Decode<type::map<Key, Value>> {
       }
     } else if (
         typeTagToTType<Key> == keyType && typeTagToTType<Value> == valueType) {
+      if (!canReadNElements(prot, s, {keyType, valueType})) {
+        TProtocolException::throwTruncatedData();
+      }
       decode_known_length_map(prot, map, s);
     } else {
-      while (s--) {
-        prot.skip(keyType);
-        prot.skip(valueType);
-      }
+      apache::thrift::skip_n(prot, s, {keyType, valueType});
     }
     prot.readMapEnd();
   }
