@@ -23,12 +23,47 @@
 #include <thrift/lib/cpp2/op/Create.h>
 #include <thrift/lib/cpp2/op/Get.h>
 #include <thrift/lib/cpp2/op/detail/BasePatch.h>
+#include <thrift/lib/cpp2/op/detail/PatchTraits.h>
+#include <thrift/lib/cpp2/patch/detail/FieldPatchCustomVisit.h>
 #include <thrift/lib/cpp2/type/Field.h>
 #include <thrift/lib/cpp2/type/Id.h>
-#include <thrift/lib/cpp2/type/NativeType.h>
 
 namespace apache::thrift::op::detail {
 
+template <typename T>
+struct SafePatchType;
+
+template <typename SafePatch, typename Patch>
+SafePatch toSafePatchImpl(const Patch& patch) {
+  SafePatch safePatch;
+
+  folly::IOBufQueue queue;
+  CompactProtocolWriter writer;
+  writer.setOutput(&queue);
+  op::encode<type::infer_tag<Patch>>(writer, patch);
+
+  safePatch.data() = queue.move();
+  safePatch.version() = calculateMinSafePatchVersion(patch);
+
+  return safePatch;
+}
+
+template <typename Patch, typename SafePatch>
+Patch fromSafePatchImpl(const SafePatch& safePatch) {
+  if (safePatch.version() == 0) {
+    throw std::runtime_error("Invalid Safe Patch");
+  }
+  if (safePatch.version() > detail::kThriftStaticPatchVersion) {
+    throw std::runtime_error(
+        fmt::format("Unsupported patch version: {}", *safePatch.version()));
+  }
+  Patch patch;
+  CompactProtocolReader reader;
+  reader.setInput(safePatch.data()->get());
+  op::decode<type::infer_tag<Patch>>(reader, patch);
+  return patch;
+}
+
 struct FieldIdListToSetAdapter {
   using FieldIdSet = std::unordered_set<FieldId>;
   using FieldIdList = std::vector<std::int16_t>;
@@ -128,14 +163,52 @@ class FieldPatch : public BasePatch<Patch, FieldPatch<Patch>> {
 
   template <typename Visitor>
   void customVisit(Visitor&& v) const {
-    for_each_field_id<Patch>(
-        [&](auto id) { v.template patchIfSet<decltype(id)>(*get(id)); });
+    if (patch::detail::enableFieldPatchCustomVisitShortCircuit()) {
+      // Don't visit the field if it is empty.
+      for_each_field_id<Patch>([&](auto id) {
+        if (auto& p = *get(id); !p.empty()) {
+          v.template patchIfSet<decltype(id)>(p);
+        }
+      });
+    } else {
+      for_each_field_id<Patch>(
+          [&](auto id) { v.template patchIfSet<decltype(id)>(*get(id)); });
+    }
   }
 
  private:
   using Base::data_;
 };
 
+// This constexpr function is only defined in the generated patch.cpp
+// This is used for an additional check to ensure we only instantiated
+// `patch<Id>()` in in the generated patch.cpp.
+template <class>
+constexpr bool only_defined_in_generated_patch_cpp();
+
+template <FieldOrdinal Ord, class Patch>
+void* typeErasedPatchImpl(Patch& patch) {
+  using FieldPatchType = typename Patch::patch_type::underlying_type;
+  if constexpr (folly::to_underlying(Ord) <= size_v<FieldPatchType>) {
+    // Sanity check to ensure `patch<Ordinal>()` returns the correct type.
+    // Otherwise it's a bug in the patch library.
+    using Id = op::get_field_id<FieldPatchType, type::ordinal_tag<Ord>>;
+    static_assert(std::is_same_v<
+                  decltype(patch.template patchImpl<Id>()),
+                  decltype(patch.template patch<Id>())>);
+    return &patch.template patchImpl<Id>();
+  } else {
+    // This code path should never be executed since the caller should already
+    // validate whether the field is patchable.
+    // However, this is needed for explicit instantiation when we can't validate
+    // patchable field size in codegen.
+    folly::throw_exception<std::logic_error>(fmt::format(
+        "Ordinal ({}) exceeds patchable field size ({})",
+        folly::to_underlying(Ord),
+        size_v<FieldPatchType>));
+  }
+}
+
 /// Create a base patch that supports Ensure operator.
 ///
 /// The `Patch` template parameter must be a Thrift struct with the following
@@ -157,6 +230,9 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
   template <class>
   friend class FieldPatch;
 
+  template <FieldOrdinal, class P>
+  friend void* typeErasedPatchImpl(P& patch);
+
   struct Applier {
     T& v;
 
@@ -179,6 +255,17 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
         op::ensure<Id>(v) = def;
       }
     }
+    template <class Id>
+    void ensure(const std::unique_ptr<folly::IOBuf>& def) {
+      if (!isAbsent(op::get<Id>(v))) {
+        return;
+      }
+      if (def) {
+        op::ensure<Id>(v) = std::make_unique<folly::IOBuf>(*def);
+      } else {
+        op::ensure<Id>(v) = std::make_unique<folly::IOBuf>();
+      }
+    }
 
     template <class Id>
     void remove() {
@@ -228,6 +315,7 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
     return !isAbsent(getEnsure<Id>(data_));
   }
 
+  /// Returns if the patch is no-op.
   bool empty() const {
     bool b = true;
     op::for_each_ordinal<T>([&](auto id) {
@@ -240,6 +328,7 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
     return b;
   }
 
+  /// Remove the given field. This is only valid for optional fields.
   template <typename Id>
   std::enable_if_t<
       type::is_optional_or_union_field_v<T, Id> && !is_thrift_union_v<T>>
@@ -301,11 +390,25 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
       }
     }
   }
+  template <typename Id>
+  std::enable_if_t<type::is_optional_or_union_field_v<T, Id>> ensure(
+      const std::unique_ptr<folly::IOBuf>& defaultVal) {
+    if (defaultVal) {
+      ensure<Id>(std::make_unique<folly::IOBuf>(*defaultVal));
+    } else {
+      ensure<Id>(std::make_unique<folly::IOBuf>());
+    }
+  }
+
   /// Ensures the given field is initalized, and return the associated patch
   /// object.
-  template <typename Id>
+  template <class Id>
   decltype(auto) patch() {
-    return (maybeEnsure<Id>(), patchAfter<Id>());
+    using FieldPatchType = typename patch_type::underlying_type;
+    using Ret = op::get_native_type<FieldPatchType, Id>;
+    return *reinterpret_cast<Ret*>(
+        typeErasedPatchImpl<op::get_ordinal_v<FieldPatchType, Id>>(
+            static_cast<Derived&>(*this)));
   }
 
   /// Returns the proper patch object for the given field.
@@ -326,7 +429,9 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
                          : getRawPatch<Id>(data_.patchPrior());
   }
 
-  /// @copybrief AssignPatch::customVisit
+  /// @brief This API uses the Visitor pattern to describe how Patch is applied.
+  /// For each operation that will be performed by the patch, the corresponding
+  /// method (that matches the write API) will be invoked.
   ///
   /// Users should provide a visitor with the following methods
   ///
@@ -338,6 +443,8 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
   ///       template<class Id> void ensure(const op::get_native_type<Id>&);
   ///       // For non-optional fields in structs
   ///       template<class Id> void ensure();
+  ///       // For optional fields in structs and fields in unions
+  ///       template<class Id> void remove();
   ///     }
   ///
   /// For example, let's assume you have the following thrift struct:
@@ -381,7 +488,7 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
       });
     }
 
-    if (Base::template customVisitAssignAndClear(std::forward<Visitor>(v))) {
+    if (Base::customVisitAssignAndClear(std::forward<Visitor>(v))) {
       return;
     }
 
@@ -438,6 +545,27 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
 
   void apply(T& val) const { return customVisit(Applier{val}); }
 
+  /**
+   * Returns a Thrift Patch instance corresponding to the (decoded) `SafePatch`.
+   *
+   * @throws std::runtime_error if the given `SafePatch` cannot be successfully
+   * decoded or safely applied in this process (eg. if the version of the Thrift
+   * Patch library in this process is not compatible with the minimum version
+   * required by `SafePatch`).
+   */
+  static auto fromSafePatch(
+      const typename SafePatchType<type::infer_tag<T>>::type& safePatch) {
+    return fromSafePatchImpl<Derived>(safePatch);
+  }
+
+  /**
+   * Returns a `SafePatch` instance corresponding to the encoded Thrift Patch.
+   */
+  auto toSafePatch() const {
+    return toSafePatchImpl<typename SafePatchType<type::infer_tag<T>>::type>(
+        Base::derived());
+  }
+
  protected:
   using Base::apply;
   using Base::data_;
@@ -574,6 +702,15 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
   }
 
  private:
+  template <typename Id>
+  decltype(auto) patchImpl() {
+    // since only_defined_in_generated_patch_cpp(...) is only defined in the
+    // generated Patch.cpp, if anyone instantiates this function outside
+    // the generated Patch.cpp, they will get a build failure.
+    static_assert(only_defined_in_generated_patch_cpp<Id>());
+    return (maybeEnsure<Id>(), patchAfter<Id>());
+  }
+
   template <typename Id, typename U>
   decltype(auto) getRawPatch(U&& patch) const {
     // Field Ids must always be used to access patch(Prior).
@@ -596,6 +733,7 @@ class BaseEnsurePatch : public BaseClearPatch<Patch, Derived> {
 /// * `terse P patchPrior`
 /// * `terse T ensure`
 /// * `terse P patch`
+/// * `terse list<i16> remove`
 /// Where `P` is the field patch type for the struct type `T`.
 template <typename Patch>
 class StructPatch : public BaseEnsurePatch<Patch, StructPatch<Patch>> {
@@ -633,6 +771,7 @@ class StructPatch : public BaseEnsurePatch<Patch, StructPatch<Patch>> {
     }
   }
 
+  /// @cond
   template <class Protocol>
   uint32_t encode(Protocol& prot) const {
     // PatchOp::Remove
@@ -670,6 +809,7 @@ class StructPatch : public BaseEnsurePatch<Patch, StructPatch<Patch>> {
     s += prot.writeStructEnd();
     return s;
   }
+  /// @endcond
 
  private:
   using Base::data_;
