@@ -244,7 +244,7 @@ inline uint32_t CompactProtocolWriter::writeSetEnd() {
 inline uint32_t CompactProtocolWriter::writeBool(bool value) {
   uint32_t wsize = 0;
 
-  value = detail::validate_bool(value);
+  value = apache::thrift::detail::validate_bool(value);
   if (booleanField_.name != nullptr) {
     // we haven't written the field header yet
     wsize += writeFieldBeginInternal(
@@ -736,25 +736,8 @@ inline void CompactProtocolReader::readStringSize(int32_t& size) {
 
 template <typename StrType>
 inline void CompactProtocolReader::readStringBody(StrType& str, int32_t size) {
-  if (static_cast<int32_t>(in_.length()) < size) {
-    if (!in_.canAdvance(size)) {
-      protocol::TProtocolException::throwTruncatedData();
-    }
-    str.reserve(size); // only reserve for multi iter case below
-  }
-  str.clear();
-  size_t size_left = size;
-  while (size_left > 0) {
-    auto data = in_.peekBytes();
-    auto data_avail = std::min(data.size(), size_left);
-    if (data.empty()) {
-      TProtocolException::throwTruncatedData();
-    }
-
-    str.append((const char*)data.data(), data_avail);
-    size_left -= data_avail;
-    in_.skipNoAdvance(data_avail);
-  }
+  assert(size >= 0); // Checked in readStringSize
+  apache::thrift::detail::readStringBody(str, in_, size);
 }
 
 template <typename StrType>
