@@ -17,11 +17,11 @@
 #pragma once
 
 #include <algorithm>
-#include <initializer_list>
 #include <variant>
 #include <folly/io/Cursor.h>
 #include <folly/io/IOBufQueue.h>
 #include <thrift/lib/cpp2/protocol/BinaryProtocol.h>
+#include <thrift/lib/cpp2/protocol/Serializer.h>
 #include <thrift/lib/cpp2/protocol/detail/CursorBasedSerialization.h>
 #include <thrift/lib/cpp2/type/NativeType.h>
 #include <thrift/lib/cpp2/type/ThriftType.h>
@@ -44,11 +44,34 @@
  * end function, though scalars and materialized types can be read/written
  * directly.
  *
+ * endRead() may be replaced with abandonRead() if the caller doesn't intend to
+ * read any more data from the cursor. This is a faster operation than calling
+ * endRead() as it does not skip to the position of the next valid read.
+ * Once a child reader has been abandoned, abandonRead is the only method that
+ * can be called on its parents.
+ *
  * test/CursorBasedSerializerTest.cpp has several complete examples of usage.
  */
 
 namespace apache::thrift {
 
+struct CursorWriteOpts {
+  size_t minGrowth;
+  size_t maxGrowth;
+  bool padBuffer;
+  size_t padSize;
+  ExternalBufferSharing sharing;
+};
+
+constexpr CursorWriteOpts defaultCursorWriteOpts() {
+  return CursorWriteOpts{
+      (1 << 14) - 16,
+      (1 << 14) - 16,
+      false,
+      128,
+      ExternalBufferSharing::SHARE_EXTERNAL_BUFFER};
+}
+
 /**
  * Manages the lifetime of a Thrift object being used with cursor
  * de/serialization.
@@ -70,6 +93,7 @@ class CursorSerializationWrapper {
   static_assert(
       std::is_same_v<ProtocolWriter, BinaryProtocolWriter>,
       "ProtocolWriter must be BinaryProtocolReader");
+  using Serializer = Serializer<ProtocolReader, ProtocolWriter>;
 
  public:
   CursorSerializationWrapper() = default;
@@ -91,7 +115,9 @@ class CursorSerializationWrapper {
    */
   /* implicit */ CursorSerializationWrapper(
       const T& t, ExternalBufferSharing sharing = COPY_EXTERNAL_BUFFER) {
-    t.write(writer(sharing));
+    auto opts = defaultCursorWriteOpts();
+    opts.sharing = sharing;
+    t.write(writer(opts));
     serializedData_ = queue_.move();
     done();
   }
@@ -100,12 +126,13 @@ class CursorSerializationWrapper {
    * Object read path (traditional Thrift deserialization)
    * Deserializes into a (returned) Thrift object.
    */
-  T deserialize() {
+  T deserialize() const {
+    if (std::holds_alternative<ProtocolWriter>(protocol_)) {
+      folly::throw_exception<std::runtime_error>(
+          "Concurrent reads/writes not supported");
+    }
     checkHasData();
-    T ret;
-    ret.read(reader());
-    done();
-    return ret;
+    return Serializer::template deserialize<T>(serializedData_.get());
   }
 
   /**
@@ -131,17 +158,46 @@ class CursorSerializationWrapper {
     done();
   }
 
-  /** Cursor write path */
-  StructuredCursorWriter<Tag> beginWrite() {
-    serializedData_.reset(); // Prevent concurrent read from seeing wrong data.
-    return StructuredCursorWriter<Tag>(writer());
+  /**
+   * A faster version of endRead for when the caller doesn't intend to read any
+   * more. Once a child reader has been abandoned, abandonRead is the only
+   * method that can be called on any of its parents.
+   */
+  template <bool Contiguous>
+  void abandonRead(
+      StructuredCursorReader<Tag, ProtocolReader, Contiguous>&& reader) {
+    reader.abandon();
+    done();
   }
+
+  /** Cursor write path */
+  StructuredCursorWriter<Tag> beginWriteWithOpts(const CursorWriteOpts& opts) {
+    serializedData_.reset(); // Prevent concurrent read from seeing wrong data.
+    return StructuredCursorWriter<Tag>(writer(opts));
+  }
+
+  StructuredCursorWriter<Tag> beginWrite() {
+    return beginWriteWithOpts(defaultCursorWriteOpts());
+  }
+
   void endWrite(StructuredCursorWriter<Tag>&& writer) {
     writer.finalize();
     serializedData_ = queue_.move();
     done();
   }
 
+  /**
+   * Allows writing to be aborted. This is useful when you have an error
+   * condition and you want to avoid writing to the buffer. Prevents
+   * CursorSerializationWrapper dtor from throwing if the write was not
+   * completed.
+   */
+  void abandonWrite(StructuredCursorWriter<Tag>&& writer) {
+    writer.abandon();
+    queue_.reset();
+    done();
+  }
+
   /** Access to serialized data */
   const folly::IOBuf& serializedData() const& {
     checkHasData();
@@ -160,13 +216,20 @@ class CursorSerializationWrapper {
     reader.setInput(cursor);
     return &reader;
   }
-  ProtocolWriter* writer(
-      ExternalBufferSharing sharing = SHARE_EXTERNAL_BUFFER) {
+
+  ProtocolWriter* writer(const CursorWriteOpts& opts) {
     checkInactive();
-    auto& writer = protocol_.template emplace<ProtocolWriter>(sharing);
-    writer.setOutput(&queue_);
+    auto& writer = protocol_.template emplace<ProtocolWriter>(opts.sharing);
+    if (opts.padBuffer) {
+      queue_.preallocate(opts.minGrowth, opts.minGrowth);
+      queue_.trimStart(opts.padSize);
+    }
+    writer.setOutput(
+        folly::io::QueueAppender{&queue_, opts.minGrowth, opts.maxGrowth});
+
     return &writer;
   }
+
   void done() { protocol_.template emplace<std::monostate>(); }
 
   void checkInactive() const {
@@ -333,6 +396,18 @@ class StructuredCursorReader : detail::BaseCursorReader<ProtocolReader> {
     state_ = State::Active;
   }
 
+  /**
+   * A faster version of endRead for when the caller doesn't intend to read any
+   * more. Once a child reader has been abandoned, abandonRead is the only
+   * method that can be called on any of its parents.
+   */
+  template <typename CTag>
+  void abandonRead(
+      ContainerCursorReader<CTag, ProtocolReader, Contiguous>&& child) {
+    child.abandon();
+    state_ = State::Abandoned;
+  }
+
   /** structured types
    *
    * Note: when beginRead returns a reader, that reader must be passed to
@@ -361,6 +436,19 @@ class StructuredCursorReader : detail::BaseCursorReader<ProtocolReader> {
     state_ = State::Active;
   }
 
+  /**
+   * A faster version of endRead for when the caller doesn't intend to read any
+   * more. Once a child reader has been abandoned, abandonRead is the only
+   * method that can be called on any of its parents.
+   */
+  template <typename CTag>
+  void abandonRead(
+      StructuredCursorReader<CTag, ProtocolReader, Contiguous>&& child) {
+    checkState(State::Child);
+    child.abandon();
+    state_ = State::Abandoned;
+  }
+
   /** union type accessor */
 
   template <
@@ -392,6 +480,14 @@ class StructuredCursorReader : detail::BaseCursorReader<ProtocolReader> {
     state_ = State::Done;
   }
 
+  void abandon() {
+    if (state_ != State::Active && state_ != State::Abandoned) {
+      folly::throw_exception<std::runtime_error>(
+          "Unexpected state when abandoning reader");
+    }
+    state_ = State::Done;
+  }
+
   template <typename Ident>
   bool beforeReadField() {
     checkState(State::Active);
@@ -457,7 +553,7 @@ class StructuredCursorReader : detail::BaseCursorReader<ProtocolReader> {
   // Last field id the caller tried to read.
   FieldId fieldId_{0};
   // Contains last field id read from the buffer.
-  typename ProtocolReader::StructReadState readState_;
+  typename ProtocolReader::StructReadState readState_{};
 
   template <typename T>
   T copy(const T& in) {
@@ -586,6 +682,19 @@ class ContainerCursorReader : detail::BaseCursorReader<ProtocolReader> {
     --remaining_;
   }
 
+  /**
+   * A faster version of endRead for when the caller doesn't intend to read any
+   * more. Once a child reader has been abandoned, abandonRead is the only
+   * method that can be called on any of its parents.
+   */
+  template <typename CTag>
+  void abandonRead(
+      ContainerCursorReader<CTag, ProtocolReader, Contiguous>&& child) {
+    checkState(State::Child);
+    child.abandon();
+    state_ = State::Abandoned;
+  }
+
   /**
    * Manual read path for containers of structured types.
    *
@@ -633,12 +742,33 @@ class ContainerCursorReader : detail::BaseCursorReader<ProtocolReader> {
     --remaining_;
   }
 
+  /**
+   * A faster version of endRead for when the caller doesn't intend to read any
+   * more. Once a child reader has been abandoned, abandonRead is the only
+   * method that can be called on any of its parents.
+   */
+  template <typename CTag>
+  void abandonRead(
+      StructuredCursorReader<CTag, ProtocolReader, Contiguous>&& child) {
+    checkState(State::Child);
+    child.abandon();
+    state_ = State::Abandoned;
+  }
+
  private:
   explicit ContainerCursorReader(ProtocolReader* p);
   ContainerCursorReader() { remaining_ = 0; }
 
   void finalize();
 
+  void abandon() {
+    if (state_ != State::Active && state_ != State::Abandoned) {
+      folly::throw_exception<std::runtime_error>(
+          "Unexpected state when abandoning reader");
+    }
+    state_ = State::Done;
+  }
+
   bool advance() {
     checkState(State::Active);
     DCHECK_GT(remaining_, 0);
@@ -708,8 +838,14 @@ class ContainerCursorIterator {
   // Dereference
   // These return non-const to allow moving the deserialzed value out.
   // Modifying them does not affect the underlying buffer.
-  reference operator*() { return reader_->currentValue(); }
-  pointer operator->() { return &reader_->currentValue(); }
+  reference operator*() {
+    FOLLY_SAFE_DCHECK(reader_);
+    return reader_->currentValue();
+  }
+  pointer operator->() {
+    FOLLY_SAFE_DCHECK(reader_);
+    return &reader_->currentValue();
+  }
 
   // Equality is only defined against itself and the end iterator
   bool operator==(ContainerCursorIterator other) const {
@@ -885,6 +1021,23 @@ class StructuredCursorWriter : detail::BaseCursorWriter {
         [&] { op::encode<type_tag<Ident>>(*protocol_, value); }, value);
   }
 
+  /** This is a niche API to support writing the fields of a struct temporally
+   * out of order in cases where both fields need to be written using CurSe, the
+   * computation of the second field that produces its serialized value must
+   * happen before the computation of the first field can proceed, and
+   * reordering the fields in the struct is not feasible. In other cases,
+   * migrating one of the fields to change the order in the struct or performing
+   * the computation in field order is preferable due to the added cost and
+   * complexity of using this API. */
+  template <typename Ident, enable_for<type::structured_c, Ident> = 0>
+  void writeSerialized(
+      CursorSerializationWrapper<native_type<Ident>>&& cursorValue) {
+    beforeWriteField<Ident>();
+    // Will fail if the cursor value is not finalized
+    protocol_->writeRaw(cursorValue.serializedData());
+    afterWriteField();
+  }
+
  private:
   explicit StructuredCursorWriter(BinaryProtocolWriter* p)
       : BaseCursorWriter(p) {
@@ -1193,7 +1346,11 @@ void ContainerCursorReader<Tag, ProtocolReader, Contiguous>::finalize() {
   checkState(State::Active);
   state_ = State::Done;
 
-  if (remaining_ > 1) {
+  if (remaining_ > 0) {
+    if (!lastRead_) {
+      lastRead_.emplace();
+      readItem();
+    }
     // Skip remaining unread elements
     skip_n(*protocol_, remaining_ - 1, detail::ContainerTraits<Tag>::wireTypes);
   }
