@@ -18,9 +18,34 @@
 
 #include <folly/Utility.h>
 
+#include <folly/algorithm/simd/find_first_of.h>
+
 namespace apache {
 namespace thrift {
 
+namespace detail::json {
+
+inline constexpr auto json_ws_alphabet = std::array{
+    char(kJSONSpace),
+    char(kJSONNewline),
+    char(kJSONTab),
+    char(kJSONCarriageReturn),
+};
+using json_ws_scalar_needle_t = folly::conditional_t<
+    folly::kIsMobile,
+    folly::simd::default_scalar_finder_first_not_of,
+    folly::simd::ltindex_scalar_finder_first_not_of>;
+using json_ws_vector_needle_t = folly::conditional_t<
+    folly::kIsMobile || !folly::kIsArchAmd64,
+    folly::simd::default_vector_finder_first_not_of,
+    folly::simd::shuffle_vector_finder_first_not_of>;
+inline constexpr auto json_ws_vector_needle =
+    json_ws_vector_needle_t{json_ws_alphabet};
+inline constexpr auto json_ws_scalar_needle =
+    json_ws_scalar_needle_t{json_ws_alphabet};
+
+} // namespace detail::json
+
 // Return the hex character representing the integer val. The value is masked
 // to make sure it is in the correct range.
 inline uint8_t JSONProtocolWriterCommon::hexChar(uint8_t val) {
@@ -261,40 +286,31 @@ inline uint32_t JSONProtocolWriterCommon::writeJSONEscapeChar(uint8_t ch) {
   return 6;
 }
 
-inline uint32_t JSONProtocolWriterCommon::writeJSONChar(uint8_t ch) {
-  if (ch >= 0x30) {
-    // Only special character >= 0x30 is '\'
-    if (ch == apache::thrift::detail::json::kJSONBackslash) {
-      out_.write(apache::thrift::detail::json::kJSONBackslash);
-      out_.write(apache::thrift::detail::json::kJSONBackslash);
-      return 2;
-    } else {
-      out_.write(ch);
-      return 1;
-    }
-  } else {
-    uint8_t outCh = kJSONCharTable[ch];
-    // Check if regular character, backslash escaped, or JSON escaped
-    if (outCh == 1) {
-      out_.write(ch);
-      return 1;
-    } else if (outCh > 1) {
-      out_.write(apache::thrift::detail::json::kJSONBackslash);
-      out_.write(outCh);
-      return 2;
-    } else {
-      return writeJSONEscapeChar(ch);
-    }
-  }
-}
-
 inline uint32_t JSONProtocolWriterCommon::writeJSONString(
     folly::StringPiece str) {
-  uint32_t ret = 2;
-
   out_.write(apache::thrift::detail::json::kJSONStringDelimiter);
-  for (auto c : str) {
-    ret += writeJSONChar(c);
+  uint32_t ret = 2;
+  for (uint8_t ch : str) {
+    // Only special characters >= 32 are '\' and '"'
+    if (ch == apache::thrift::detail::json::kJSONBackslash ||
+        ch == apache::thrift::detail::json::kJSONStringDelimiter) {
+      out_.write(apache::thrift::detail::json::kJSONBackslash);
+      ret += 1;
+    }
+    if (ch >= 32) {
+      out_.write(ch);
+      ret += 1;
+    } else {
+      uint8_t outCh = kJSONCharTable[ch];
+      // Check if regular character, backslash escaped, or JSON escaped
+      if (outCh != 0) {
+        out_.write(apache::thrift::detail::json::kJSONBackslash);
+        out_.write(outCh);
+        ret += 2;
+      } else {
+        ret += writeJSONEscapeChar(ch);
+      }
+    }
   }
   out_.write(apache::thrift::detail::json::kJSONStringDelimiter);
 
@@ -435,20 +451,26 @@ inline void JSONProtocolReaderCommon::readBinary(folly::IOBuf& str) {
  */
 
 inline void JSONProtocolReaderCommon::skipWhitespace() {
-  for (auto peek = in_.peekBytes(); !peek.empty(); peek = in_.peekBytes()) {
-    uint32_t size = 0;
-    for (char ch : peek) {
-      if (ch != apache::thrift::detail::json::kJSONSpace &&
-          ch != apache::thrift::detail::json::kJSONNewline &&
-          ch != apache::thrift::detail::json::kJSONTab &&
-          ch != apache::thrift::detail::json::kJSONCarriageReturn) {
-        in_.skip(size);
-        return;
-      }
-      ++skippedWhitespace_;
-      ++size;
+  constexpr auto& vector_needle = detail::json::json_ws_vector_needle;
+  constexpr auto& scalar_needle = detail::json::json_ws_scalar_needle;
+  while (true) { // for loop generates larger code with 2 calls to peekBytesSlow
+    auto const peek = folly::reinterpret_span_cast<char const>(in_.peek());
+    if (peek.empty()) {
+      return;
     }
+    auto const newl = apache::thrift::detail::json::kJSONNewline;
+    // if 0th char is newline then it is likely that indentation follows; vector
+    // algorithm is better for indentation but worse for single-char whitespace
+    auto const usevec = !folly::kIsMobile /* has vector acceleration */ && //
+        folly::kIsArchAmd64 && peek.size() > 16 && peek[0] == newl;
+    auto const vecskip = size_t(usevec);
+    auto const size =
+        vecskip + vector_needle(scalar_needle, usevec, peek.subspan(vecskip));
+    skippedWhitespace_ += size;
     in_.skip(size);
+    if (size < peek.size()) {
+      return;
+    }
   }
 }
 
@@ -743,8 +765,7 @@ inline void JSONProtocolReaderCommon::readJSONString(StrType& val) {
   if (fullDecodeRequired) {
     json += "\"";
     try {
-      folly::dynamic parsed = folly::parseJson(json);
-      val += parsed.getString();
+      val += readJSONStringViaDynamic(json);
     } catch (const std::exception& e) {
       throwUnrecognizableAsString(json, e);
     }
