@@ -24,14 +24,17 @@
 #include <folly/MapUtil.h>
 #include <folly/io/IOBufQueue.h>
 #include <folly/lang/Exception.h>
+#include <folly/logging/xlog.h>
 #include <thrift/lib/cpp/util/EnumUtils.h>
 #include <thrift/lib/cpp/util/SaturatingMath.h>
 #include <thrift/lib/cpp/util/VarintUtils.h>
 #include <thrift/lib/cpp2/op/Get.h>
+#include <thrift/lib/cpp2/patch/detail/Scuba.h>
 #include <thrift/lib/cpp2/protocol/BinaryProtocol.h>
 #include <thrift/lib/cpp2/protocol/CompactProtocol.h>
 #include <thrift/lib/cpp2/protocol/FieldMask.h>
 #include <thrift/lib/cpp2/protocol/Object.h>
+#include <thrift/lib/cpp2/protocol/detail/Patch.h>
 #include <thrift/lib/cpp2/type/Id.h>
 #include <thrift/lib/cpp2/type/NativeType.h>
 #include <thrift/lib/cpp2/type/Tag.h>
@@ -50,7 +53,8 @@ using value_field_id =
     type::field_id_tag<static_cast<FieldId>(type::base_type_v<Tag>)>;
 
 template <typename Tag>
-using value_native_type = op::get_native_type<Value, value_field_id<Tag>>;
+using value_native_type =
+    op::get_native_type<detail::Value, value_field_id<Tag>>;
 
 constexpr FieldId kSafePatchVersionId = FieldId{1};
 constexpr FieldId kSafePatchDataId = FieldId{2};
@@ -76,7 +80,7 @@ PatchOp toOp(FieldId id) {
 void checkOps(
     const Object& patch,
     Value::Type valueType,
-    folly::F14FastSet<PatchOp> supportedOps) {
+    folly::F14VectorSet<PatchOp> supportedOps) {
   checkNotSafePatch(patch);
   for (const auto& field : patch) {
     auto op = toOp(FieldId{field.first});
@@ -108,7 +112,7 @@ decltype(auto) argAs(V&& value) {
         util::enumNameSafe<Value::Type>(expected),
         util::enumNameSafe<Value::Type>(value.getType())));
   }
-  return *op::get<Id, Value>(std::forward<V>(value));
+  return *op::get<Id, Value::ThriftValue>(std::forward<V>(value).toThrift());
 }
 
 template <typename Tag>
@@ -285,7 +289,7 @@ void ApplyPatch::operator()(
 }
 
 void ApplyPatch::operator()(
-    const Object& patch, folly::F14FastSet<Value>& value) const {
+    const Object& patch, folly::F14VectorSet<Value>& value) const {
   checkOps(
       patch,
       Value::Type::setValue,
@@ -328,6 +332,9 @@ void ApplyPatch::operator()(
   }
 
   if (auto* put = findOp(patch, PatchOp::Put)) {
+    auto msg = "SetPatch::Put should be migrated to SetPatch::Add";
+    patch::detail::logDeprecatedOperation("SetPatch::Put", "[dynamic]", msg);
+    XLOG(DFATAL) << msg;
     insert_set(*validate_if_set(put, "put"));
   }
 }
@@ -499,6 +506,10 @@ void impl(Patch&& patch, Object& value) {
 
     if (const auto* p_set = to_remove->if_set()) {
       // TODO: Remove this after migrating to List
+      auto msg = "StructPatch::Remove should be migrated from `set` to `list`";
+      patch::detail::logDeprecatedOperation(
+          "StructPatch::Remove (set)", "[dynamic]", msg);
+      XLOG(DFATAL) << msg;
       remove(*p_set);
     } else if (const auto* p_list = to_remove->if_list()) {
       remove(*p_list);
@@ -608,227 +619,89 @@ void ApplyPatch::operator()(Object&& patch, Object& value) const {
   impl(std::move(patch), value);
 }
 
-// Inserts the next mask with union operator to getIncludesRef(mask)[id].
-// Skips if mask is allMask (already includes all fields), or next is noneMask.
-template <typename Id, typename F>
-void insertMaskUnion(
-    Mask& mask, Id id, const Mask& next, const F& getIncludesRef) {
-  if (mask != allMask() && next != noneMask()) {
-    Mask& current = getIncludesRef(mask)
-                        .ensure()
-                        .emplace(std::move(id), noneMask())
-                        .first->second;
-    current = current | next;
-  }
-}
-
 // Insert allMask to getIncludesRef(mask)[id] if id does not exist.
 template <typename Id, typename F>
 void tryInsertAllMask(Mask& mask, Id id, const F& getIncludesRef) {
   getIncludesRef(mask).ensure().emplace(std::move(id), allMask());
 }
 
+// TODO(dokwon): Remove this once view is deprecated.
 template <typename Id, typename F>
-void insertNextMask(
+void insertNextMaskDeprecated(
     ExtractedMasksFromPatch& masks,
     const Value& nextPatch,
     Id readId,
     Id writeId,
-    bool recursive,
     bool view,
     const F& getIncludesRef) {
-  if (recursive) {
-    auto nextMasks = view
-        ? protocol::extractMaskViewFromPatch(nextPatch.as_object())
-        : protocol::extractMaskFromPatch(nextPatch.as_object());
-    insertMaskUnion(
-        masks.read, std::move(readId), nextMasks.read, getIncludesRef);
-    insertMaskUnion(
-        masks.write, std::move(writeId), nextMasks.write, getIncludesRef);
-  } else {
-    insertMaskUnion(masks.read, std::move(readId), allMask(), getIncludesRef);
-    insertMaskUnion(masks.write, std::move(writeId), allMask(), getIncludesRef);
+  auto nextMasks = view
+      ? protocol::extractMaskViewFromPatch(nextPatch.as_object())
+      : protocol::extractMaskFromPatch(nextPatch.as_object());
+  insertNextMask(
+      masks, nextMasks, std::move(readId), std::move(writeId), getIncludesRef);
+}
+
+// Constructs the field mask from the patch object for the field.
+void insertNextFieldsToMask(
+    ExtractedMasksFromPatch& masks, const Object& patch, bool view) {
+  auto getIncludesObjRef = [&](Mask& mask) { return mask.includes_ref(); };
+  for (const auto& [id, value] : patch) {
+    // Object patch can get here only StructPatch::Patch(Prior|After)
+    // operations, which require reading existing value to know if/how given
+    // operations can/should be applied. Generate allMask() read mask for them
+    // if the recursively extracted masks from patch does not include the
+    // field.
+    insertNextMaskDeprecated(masks, value, id, id, view, getIncludesObjRef);
+    insertFieldsToMask(masks.read, FieldId{id});
   }
 }
 
-// Ensure requires reading existing value to know whether the field is set or
-// not. Insert allMask() if the field was never included in read mask
-// before.
-void insertEnsureReadFieldsToMask(Mask& mask, const Value& ensureFields) {
-  const auto& obj = ensureFields.as_object();
-  auto getIncludesObjRef = [&](Mask& m) { return m.includes_ref(); };
-  for (const auto& [id, value] : obj) {
-    tryInsertAllMask(mask, id, getIncludesObjRef);
-  }
-}
-
-// Ensure only requires writing fields that exists in the patch. Put allMask()
-// iff current mask is not allMask and the field was never included in the write
-// mask before.
-void insertEnsureWriteFieldsToMask(Mask& mask, const Value& ensureFields) {
-  if (mask == allMask()) {
-    return;
-  }
-  const auto& obj = ensureFields.as_object();
-  for (const auto& [id, value] : obj) {
-    mask.includes_ref().ensure().emplace(id, allMask());
-  }
-}
-
-// Ensure requires reading existing value to know whether keys exist in the
-// map. Insert allMask() if the key was never included in read mask
-// before.
-void insertReadKeysToMapMask(
-    Mask& mask, const folly::F14FastMap<Value, Value>& map, bool view) {
+// Constructs the map mask from the patch object for the map. If view, it uses
+// address of Value to populate map mask. If not view, it uses the appropriate
+// integer map mask and string map mask after parsing from Value.
+void insertNextMapItemsToMask(
+    ExtractedMasksFromPatch& masks,
+    const folly::F14FastMap<protocol::Value, protocol::Value>& patch,
+    bool view) {
   auto getIncludesMapRef = [&](Mask& mask) { return mask.includes_map_ref(); };
   auto getIncludesStringMapRef = [&](Mask& mask) {
     return mask.includes_string_map_ref();
   };
+  // Map patch can get here only MapPatch::Patch(Prior|After) operations,
+  // which require reading existing value to know if/how given operations
+  // can/should be applied. Generate allMask() read map mask if the
+  // recursively extracted masks from patch do not include the key.
   if (view) {
-    auto readValueIndex = buildValueIndex(mask);
-    for (const auto& [key, _] : map) {
-      auto id = static_cast<int64_t>(
+    auto readValueIndex = buildValueIndex(masks.read);
+    auto writeValueIndex = buildValueIndex(masks.write);
+    for (const auto& [key, value] : patch) {
+      auto readId = static_cast<int64_t>(
           getMapIdValueAddressFromIndex(readValueIndex, key));
-      tryInsertAllMask(mask, id, getIncludesMapRef);
+      auto writeId = static_cast<int64_t>(
+          getMapIdValueAddressFromIndex(writeValueIndex, key));
+
+      insertNextMaskDeprecated(
+          masks, value, readId, writeId, view, getIncludesMapRef);
+      tryInsertAllMask(masks.read, readId, getIncludesMapRef);
     }
     return;
   }
-  for (const auto& [key, _] : map) {
+  for (const auto& [key, value] : patch) {
     if (getArrayKeyFromValue(key) == ArrayKey::Integer) {
-      tryInsertAllMask(
-          mask,
-          static_cast<int64_t>(getMapIdFromValue(key)),
-          getIncludesMapRef);
+      auto id = static_cast<int64_t>(getMapIdFromValue(key));
+      insertNextMaskDeprecated(masks, value, id, id, view, getIncludesMapRef);
+      insertKeysToMask(masks.read, id);
     } else {
-      tryInsertAllMask(mask, getStringFromValue(key), getIncludesStringMapRef);
+      auto id = getStringFromValue(key);
+      insertNextMaskDeprecated(
+          masks, value, id, id, view, getIncludesStringMapRef);
+      insertKeysToMask(masks.read, id);
     }
   }
 }
 
-void insertTypeToMask(Mask& mask, const type::Type& type) {
-  if (mask == allMask()) {
-    return;
-  }
-  mask.includes_type_ref().ensure().emplace(type, allMask());
-}
-
-template <typename T>
-const T& getKeyOrElem(const T& value) {
-  return value;
-}
-template <typename K, typename V>
-const K& getKeyOrElem(const std::pair<const K, V>& value) {
-  return value.first;
-}
-
-// Put allMask() iff current mask is not allMask and the key was never included
-// in the write mask before. `view` specifies whether to use address of Value to
-// populate map mask (deprecated).
-template <typename Container>
-void insertWriteKeysToMapMask(Mask& mask, const Container& c, bool view) {
-  if (mask == allMask()) {
-    return;
-  }
-
-  if (view) {
-    auto writeValueIndex = buildValueIndex(mask);
-    for (const auto& elem : c) {
-      const auto& v = getKeyOrElem(elem);
-      auto id = static_cast<int64_t>(
-          getMapIdValueAddressFromIndex(writeValueIndex, v));
-      mask.includes_map_ref().ensure().emplace(id, allMask());
-    }
-    return;
-  }
-
-  for (const auto& elem : c) {
-    const auto& v = getKeyOrElem(elem);
-
-    if (getArrayKeyFromValue(v) == ArrayKey::Integer) {
-      mask.includes_map_ref().ensure().emplace(
-          static_cast<int64_t>(getMapIdFromValue(v)), allMask());
-    } else {
-      mask.includes_string_map_ref().ensure().emplace(
-          getStringFromValue(v), allMask());
-    }
-  }
-}
-
-// Remove only requires writing fields that exists in the patch.
-void insertRemoveWriteFieldsToMask(Mask& mask, const std::vector<Value>& ids) {
-  if (mask == allMask()) {
-    return;
-  }
-  for (const Value& id : ids) {
-    mask.includes_ref().ensure().emplace(id.as_i16(), allMask());
-  }
-}
-
-// If recursive, it constructs the mask from the patch object for the field.
-// If view, it uses address of Value to populate map mask. If not view, it
-// uses the appropriate integer map mask and string map mask after parsing
-// from Value.
-void insertFieldsToMask(
-    ExtractedMasksFromPatch& masks,
-    const Value& patchFields,
-    bool recursive,
-    bool view) {
-  auto getIncludesMapRef = [&](Mask& mask) { return mask.includes_map_ref(); };
-  auto getIncludesStringMapRef = [&](Mask& mask) {
-    return mask.includes_string_map_ref();
-  };
-
-  if (const auto* obj = patchFields.if_object()) {
-    auto getIncludesObjRef = [&](Mask& mask) { return mask.includes_ref(); };
-    for (const auto& [id, value] : *obj) {
-      // Object patch can get here only StructPatch::Patch(Prior|After
-      // operations, which require reading existing value to know if/how given
-      // operations can/should be applied. Generate allMask() read mask for them
-      // if the recursively extracted masks from patch does not include the
-      // field.
-      insertNextMask(masks, value, id, id, recursive, view, getIncludesObjRef);
-      tryInsertAllMask(masks.read, id, getIncludesObjRef);
-    }
-  } else if (const auto* map = patchFields.if_map()) {
-    // Map patch can get here only MapPatch::Patch(Prior|After) operations,
-    // which require reading existing value to know if/how given operations
-    // can/should be applied. Generate allMask() read map mask if the
-    // recursively extracted masks from patch do not include the key.
-    if (view) {
-      auto readValueIndex = buildValueIndex(masks.read);
-      auto writeValueIndex = buildValueIndex(masks.write);
-      for (const auto& [key, value] : *map) {
-        auto readId = static_cast<int64_t>(
-            getMapIdValueAddressFromIndex(readValueIndex, key));
-        auto writeId = static_cast<int64_t>(
-            getMapIdValueAddressFromIndex(writeValueIndex, key));
-        insertNextMask(
-            masks, value, readId, writeId, recursive, view, getIncludesMapRef);
-        tryInsertAllMask(masks.read, readId, getIncludesMapRef);
-      }
-      return;
-    }
-    for (const auto& [key, value] : *map) {
-      if (getArrayKeyFromValue(key) == ArrayKey::Integer) {
-        auto id = static_cast<int64_t>(getMapIdFromValue(key));
-        insertNextMask(
-            masks, value, id, id, recursive, view, getIncludesMapRef);
-        tryInsertAllMask(masks.read, id, getIncludesMapRef);
-      } else {
-        auto id = getStringFromValue(key);
-        insertNextMask(
-            masks, value, id, id, recursive, view, getIncludesStringMapRef);
-        tryInsertAllMask(masks.read, id, getIncludesStringMapRef);
-      }
-    }
-  }
-}
-
-void insertTypesToMask(
-    ExtractedMasksFromPatch& masks,
-    const Value& patchTypes,
-    bool recursive,
-    bool view) {
+void insertNextTypesToMask(
+    ExtractedMasksFromPatch& masks, const Value& patchTypes, bool view) {
   auto getIncludesTypeRef = [&](Mask& mask) {
     return mask.includes_type_ref();
   };
@@ -845,25 +718,23 @@ void insertTypesToMask(
     }
     auto dynPatch =
         protocol::detail::parseValueFromAny(type_to_patch.patch().value());
-    insertNextMask(
+    insertNextMaskDeprecated(
         masks,
         dynPatch,
         type_to_patch.type().value(),
         type_to_patch.type().value(),
-        recursive,
         view,
         getIncludesTypeRef);
-    insertTypeToMask(masks.read, type_to_patch.type().value());
+    insertTypeToMaskIfNotAllMask(masks.read, type_to_patch.type().value());
   }
 }
 
+// We cannot distinguish SetPatch::remove and MapPatch::remove without
+// additional schema information. However, we can disambiguate if the same
+// patch contains other map operations which is tracked by `isMap`.
 ExtractedMasksFromPatch extractMaskFromPatch(
-    const protocol::Object& patch, bool view) {
+    const protocol::Object& patch, bool view, bool isMap = false) {
   ExtractedMasksFromPatch masks = {noneMask(), noneMask()};
-  // We cannot distinguish SetPatch::remove and MapPatch::remove without
-  // additional schema information. However, we can disambiguate if the same
-  // patch contains other map operations.
-  bool isMap = false;
 
   // If Assign, it is a write operation
   if (findOp(patch, PatchOp::Assign)) {
@@ -887,59 +758,67 @@ ExtractedMasksFromPatch extractMaskFromPatch(
   if (auto* value = findOp(patch, PatchOp::Put)) {
     if (value->is_map()) {
       isMap = true;
-      insertWriteKeysToMapMask(masks.write, value->as_map(), view);
+      insertKeysToMaskIfNotAllMask(masks.write, value->as_map(), view);
     } else if (!isIntrinsicDefault(*value)) {
       return {allMask(), allMask()};
     }
   }
 
-  // If PatchPrior or PatchAfter, recursively constructs the mask for the
-  // fields.
+  // If PatchPrior or PatchAfter, recursively constructs a field mask or map
+  // mask.
   for (auto op : {PatchOp::PatchPrior, PatchOp::PatchAfter}) {
-    if (auto* patchFields = findOp(patch, op)) {
-      if (patchFields->is_map()) {
+    if (auto* subpatch = findOp(patch, op)) {
+      if (subpatch->is_object()) {
+        insertNextFieldsToMask(masks, subpatch->as_object(), view);
+      } else if (subpatch->is_map()) {
         isMap = true;
+        insertNextMapItemsToMask(masks, subpatch->as_map(), view);
       }
-      insertFieldsToMask(masks, *patchFields, true, view);
     }
   }
 
   // If EnsureStruct, add fields/keys to mask.
   if (auto* ensureStruct = findOp(patch, PatchOp::EnsureStruct)) {
-    if (ensureStruct->if_object()) {
-      insertEnsureReadFieldsToMask(masks.read, *ensureStruct);
-      insertEnsureWriteFieldsToMask(masks.write, *ensureStruct);
-    } else if (ensureStruct->if_map()) {
+    if (ensureStruct->is_object()) {
+      // Ensure requires reading existing value to know whether the field is set
+      // or not. Insert allMask() if the field was never included in read mask
+      // before.
+      insertFieldsToMask(masks.read, ensureStruct->as_object());
+      insertFieldsToMaskIfNotAllMask(masks.write, ensureStruct->as_object());
+    } else if (ensureStruct->is_map()) {
       isMap = true;
-      insertReadKeysToMapMask(masks.read, ensureStruct->as_map(), view);
-      insertWriteKeysToMapMask(masks.write, ensureStruct->as_map(), view);
+      insertKeysToMask(masks.read, ensureStruct->as_map(), view);
+      insertKeysToMaskIfNotAllMask(masks.write, ensureStruct->as_map(), view);
     }
   }
 
   // If EnsureUnion, add fields to mask for read mask and all mask for write
   // mask.
   if (auto* ensureUnion = findOp(patch, PatchOp::EnsureUnion)) {
-    insertEnsureReadFieldsToMask(masks.read, *ensureUnion);
+    // Ensure requires reading existing value to know whether the field is set
+    // or not. Insert allMask() if the field was never included in read mask
+    // before.
+    insertFieldsToMask(masks.read, ensureUnion->as_object());
     masks.write = allMask();
   }
 
   // We can only distinguish struct. For struct, add removed fields to write
   // mask. Both set and map use a set for Remove, so they are indistinguishable.
-  // For set and map, we cannot distinguish them. For view, we always add
-  // removed keys to write map mask. For non-view, if any of map operations is
-  // used we can correctly distinguish between map and set, then we add removed
-  // keys to write map mask. For set or if we cannot distinguish, it is a
-  // read-write operation if not intristic default.
+  // For view, we always add removed keys to write map mask. For non-view, if
+  // any of map operations is used we can correctly distinguish between map and
+  // set, then we add removed keys to write map mask. For set or any of map
+  // operations is not used, it is a read-write operation if not intristic
+  // default.
   if (auto* value = findOp(patch, PatchOp::Remove)) {
     if (value->is_list()) {
       // struct patch
-      insertRemoveWriteFieldsToMask(masks.write, value->as_list());
+      insertFieldsToMaskIfNotAllMask(masks.write, value->as_list());
     } else if (!isIntrinsicDefault(*value)) {
       if (!view && !isMap) {
         // cannot distinguish between set/map patch
         return {allMask(), allMask()};
       }
-      insertWriteKeysToMapMask(masks.write, value->as_set(), view);
+      insertKeysToMaskIfNotAllMask(masks.write, value->as_set(), view);
     }
   }
 
@@ -947,7 +826,7 @@ ExtractedMasksFromPatch extractMaskFromPatch(
   // mask each type patch.
   for (auto op : {PatchOp::PatchIfTypeIsPrior, PatchOp::PatchIfTypeIsAfter}) {
     if (auto* patchTypes = findOp(patch, op)) {
-      insertTypesToMask(masks, *patchTypes, true, view);
+      insertNextTypesToMask(masks, *patchTypes, view);
     }
   }
 
@@ -961,16 +840,11 @@ ExtractedMasksFromPatch extractMaskFromPatch(
     if (!type::AnyData::isValid(anyStruct)) {
       throw std::runtime_error("Invalid AnyStruct");
     }
-    insertTypeToMask(masks.read, anyStruct.type().value());
+    insertTypeToMaskIfNotAllMask(masks.read, anyStruct.type().value());
     masks.write = allMask();
   }
 
-  // Read mask should be always subset of write mask. If not, make read mask
-  // equal to write mask. This can happen for struct or map fields with patch
-  // operations that returns noneMask for read mask (i.e. assign).
-  if ((masks.read | masks.write) != masks.write) {
-    masks.read = masks.write;
-  }
+  ensureRWMaskInvariant(masks);
 
   return masks;
 }
@@ -1021,6 +895,128 @@ int32_t calculateMinSafePatchVersion(const protocol::Object& patch) {
   return version;
 }
 
+ExtractedMasksFromPatch extractMapMaskFromPatchImpl(
+    const protocol::Value& patch, const Mask& mask);
+ExtractedMasksFromPatch extractMapMaskFromPatchImpl(
+    const protocol::Object& patch, const Mask& mask);
+
+template <typename K>
+ExtractedMasksFromPatch extractMapMaskFromPatchMapImpl(
+    const protocol::Object& patch, const K& k, const Mask& v) {
+  if (auto* removeSet = findOp(patch, PatchOp::Remove)) {
+    const auto& set = removeSet->as_set();
+    if (!set.empty()) {
+      if (set.contains(getValueAs(k, *set.begin()))) {
+        return {noneMask(), allMask()};
+      }
+    }
+  }
+
+  ExtractedMasksFromPatch rwmask = {noneMask(), noneMask()};
+  std::optional<Value> key;
+  if (auto* putMap = findOp(patch, PatchOp::Put)) {
+    const auto& map = putMap->as_map();
+    if (!map.empty()) {
+      key = getValueAs(k, map.begin()->first);
+      if (map.contains(*key)) {
+        rwmask = {noneMask(), allMask()};
+      }
+    }
+  }
+
+  for (auto patchOp : {PatchOp::PatchPrior, PatchOp::PatchAfter}) {
+    if (auto* itemPatch = findOp(patch, patchOp)) {
+      const auto& map = itemPatch->as_map();
+      if (!map.empty()) {
+        if (!key.has_value()) {
+          key = getValueAs(k, map.begin()->first);
+        }
+        if (map.contains(*key)) {
+          rwmask = rwmask | extractMapMaskFromPatchImpl(map.at(*key), v);
+        }
+      }
+    }
+  }
+
+  if (auto* ensureMap = findOp(patch, PatchOp::EnsureStruct)) {
+    const auto& map = ensureMap->as_map();
+    if (!map.empty()) {
+      if (!key.has_value()) {
+        key = getValueAs(k, map.begin()->first);
+      }
+      if (map.contains(*key)) {
+        rwmask.read = rwmask.read & allMask();
+        rwmask.write = rwmask.write & allMask();
+      }
+    }
+  }
+  return rwmask;
+}
+
+ExtractedMasksFromPatch extractMapMaskFromPatchImpl(
+    const protocol::Object& patch, const Mask& mask) {
+  if (isAllMask(mask)) {
+    return extractMaskFromPatch(patch, false, true);
+  }
+
+  // If there is an assign or clear, we can short-circut the logic.
+  if (findOp(patch, PatchOp::Assign)) {
+    return {noneMask(), allMask()};
+  }
+  if (auto* clear = findOp(patch, PatchOp::Clear)) {
+    if (!isIntrinsicDefault(*clear)) {
+      return {noneMask(), allMask()};
+    }
+  }
+
+  ExtractedMasksFromPatch rwmask = {noneMask(), noneMask()};
+  if (mask.includes_ref()) {
+    const auto& [id, v] = *mask.includes_ref()->begin();
+    // If there is a StructPatch::remove, we can short-circuit the logic.
+    if (auto* value = findOp(patch, PatchOp::Remove)) {
+      if (std::find(
+              value->as_list().begin(),
+              value->as_list().end(),
+              asValueStruct<type::i16_t>(id)) != value->as_list().end()) {
+        return {noneMask(), allMask()};
+      }
+    }
+
+    for (auto patchOp : {PatchOp::PatchPrior, PatchOp::PatchAfter}) {
+      if (auto* fieldPatch = findOp(patch, patchOp)) {
+        if (fieldPatch->as_object().contains(FieldId{id})) {
+          rwmask = rwmask |
+              extractMapMaskFromPatchImpl(
+                       fieldPatch->as_object().at(FieldId{id}), v);
+        }
+      }
+    }
+
+    if (auto* ensureStruct = findOp(patch, PatchOp::EnsureStruct)) {
+      if (ensureStruct->as_object().contains(FieldId{id})) {
+        rwmask.read = rwmask.read & allMask();
+        rwmask.write = rwmask.write & allMask();
+      }
+    }
+  } else if (mask.includes_map_ref()) {
+    const auto& [k, v] = *mask.includes_map_ref()->begin();
+    return extractMapMaskFromPatchMapImpl(patch, MapId{k}, v);
+  } else if (mask.includes_string_map_ref()) {
+    const auto& [k, v] = *mask.includes_string_map_ref()->begin();
+    return extractMapMaskFromPatchMapImpl(patch, k, v);
+  } else if (mask.includes_type_ref()) {
+    folly::throw_exception<std::runtime_error>("not implemented");
+  } else {
+    folly::throw_exception<std::runtime_error>("Invalid mask");
+  }
+  return rwmask;
+}
+
+ExtractedMasksFromPatch extractMapMaskFromPatchImpl(
+    const protocol::Value& patch, const Mask& mask) {
+  return extractMapMaskFromPatchImpl(patch.as_object(), mask);
+}
+
 } // namespace detail
 
 ExtractedMasksFromPatch extractMaskViewFromPatch(
@@ -1034,6 +1030,13 @@ ExtractedMasksFromPatch extractMaskFromPatch(const protocol::Object& patch) {
   return detail::extractMaskFromPatch(patch, false);
 }
 
+ExtractedMasksFromPatch extractMapMaskFromPatch_DO_NOT_USE(
+    const protocol::Object& patch, const Mask& mask) {
+  detail::validateSinglePath(mask);
+  auto masks = detail::extractMapMaskFromPatchImpl(patch, mask);
+  return masks;
+}
+
 template <type::StandardProtocol Protocol>
 std::unique_ptr<folly::IOBuf> applyPatchToSerializedData(
     const protocol::Object& patch, const folly::IOBuf& buf) {
