@@ -31,6 +31,7 @@
 #include <folly/Utility.h>
 #include <folly/container/Reserve.h>
 #include <folly/container/View.h>
+#include <folly/container/range_traits.h>
 #include <folly/functional/Invoke.h>
 #include <folly/io/IOBuf.h>
 #include <folly/memory/UninitializedMemoryHacks.h>
@@ -594,17 +595,28 @@ struct protocol_methods<type_class::list<ElemClass>, Type> {
         // resizeWithoutInitialization first, then resize.
         if constexpr (should_resize_without_initialization) {
           folly::resizeWithoutInitialization(out, list_size);
-          auto outIt = out.begin();
-          const auto outEnd = out.end();
-          try {
-            for (; outIt != outEnd; ++outIt) {
-              elem_methods::read(protocol, *outIt);
+          // Check if we can do a fast path (memcpy that reverses byte order)
+          // instead of processing elements sequentially
+          if constexpr (kShouldProcessAsArithmeticVector<
+                            Protocol,
+                            Type,
+                            elem_type>) {
+            protocol.template readArithmeticVector<elem_type>(
+                out.data(), out.size());
+          } else {
+            // fallback: process element by element
+            auto outIt = out.begin();
+            const auto outEnd = out.end();
+            try {
+              for (; outIt != outEnd; ++outIt) {
+                elem_methods::read(protocol, *outIt);
+              }
+            } catch (...) {
+              // For behaviour parity, initialize the leftover elements when
+              // exceptions happen
+              std::fill(outIt, outEnd, elem_type());
+              throw;
             }
-          } catch (...) {
-            // For behaviour parity, initialize the leftover elements when
-            // exceptions happen
-            std::fill(outIt, outEnd, elem_type());
-            throw;
           }
         } else if constexpr (should_resize) {
           out.resize(list_size);
@@ -627,6 +639,22 @@ struct protocol_methods<type_class::list<ElemClass>, Type> {
     read(protocol, out);
   }
 
+  template <typename Protocol, typename = void>
+  struct SupportsArithmeticVectors : std::false_type {};
+
+  template <typename Protocol>
+  struct SupportsArithmeticVectors<
+      Protocol,
+      std::void_t<decltype(Protocol::kSupportsArithmeticVectors())>>
+      : std::bool_constant<Protocol::kSupportsArithmeticVectors()> {};
+
+  template <typename Protocol, typename ContainerType, typename ElementType>
+  static constexpr bool kShouldProcessAsArithmeticVector =
+      !std::is_same_v<elem_type, bool> &&
+      folly::is_contiguous_range_v<ContainerType> &&
+      std::is_arithmetic_v<ElementType> &&
+      SupportsArithmeticVectors<Protocol>::value;
+
   template <typename Protocol>
   static std::size_t write(Protocol& protocol, const Type& out) {
     std::size_t xfer = 0;
@@ -634,8 +662,13 @@ struct protocol_methods<type_class::list<ElemClass>, Type> {
     xfer += protocol.writeListBegin(
         elem_ttype::value, checked_container_size(out.size()));
 
-    for (const auto& elem : out) {
-      xfer += elem_methods::write(protocol, elem);
+    if constexpr (kShouldProcessAsArithmeticVector<Protocol, Type, elem_type>) {
+      xfer += protocol.template writeArithmeticVector<elem_type>(
+          out.data(), out.size());
+    } else {
+      for (const auto& elem : out) {
+        xfer += elem_methods::write(protocol, elem);
+      }
     }
     xfer += protocol.writeListEnd();
     return xfer;
