@@ -16,7 +16,6 @@
 
 #include <thrift/lib/cpp2/protocol/CursorBasedSerializer.h>
 
-#include <ranges>
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
 #include <thrift/lib/cpp2/protocol/test/gen-cpp2/cursor_clients.h>
@@ -69,7 +68,7 @@ TEST(CursorSerializer, QualifierRead) {
   auto reader = wrapper.beginRead();
   EXPECT_EQ(reader.read<ident::opt>(), 3);
   EXPECT_EQ(reader.read<ident::unq>(), 1);
-  EXPECT_EQ(reader.read<ident::terse>(), 2);
+  EXPECT_EQ(reader.read<ident::terse>(), 0);
   wrapper.endRead(std::move(reader));
 
   // Reading from a serialized empty object applies the appropriate default
@@ -340,6 +339,27 @@ TEST(CursorSerializer, NestedStructRead) {
   wrapper.endRead(std::move(outerReader));
 }
 
+TEST(CursorSerializer, AbandonNestedStructRead) {
+  CursorSerializationWrapper<Meal> wrapper(Meal{});
+  auto outerReader = wrapper.beginRead();
+  auto innerReader = outerReader.beginRead<ident::cookie>();
+
+  outerReader.abandonRead(std::move(innerReader));
+
+  // Can't use parent reader after abandoning child.
+  EXPECT_THAT(
+      [&] { outerReader.read<ident::dessert>(); },
+      ThrowsMessage<std::runtime_error>("Reader abandoned"));
+
+  // Can't call endRead after abandoning child.
+  EXPECT_THAT(
+      [&] { wrapper.endRead(std::move(outerReader)); },
+      ThrowsMessage<std::runtime_error>("Reader abandoned"));
+
+  // @lint-ignore CLANGTIDY bugprone-use-after-move
+  wrapper.abandonRead(std::move(outerReader));
+}
+
 TEST(CursorSerializer, CursorReadInContainer) {
   Struct s;
   Stringish inner;
@@ -623,6 +643,50 @@ TEST(CursorSerializer, NestedStructWrite) {
   EXPECT_THAT(*obj.list_field(), ElementsAre(42));
 }
 
+TEST(CursorSerializer, NestedStructWriteWithSeparateCursor) {
+  CursorSerializationWrapper<Meal> mealCursor;
+  CursorSerializationWrapper<Cookie> cookieCursor;
+  CursorSerializationWrapper<Struct> structCursor;
+
+  auto cookieWriter = cookieCursor.beginWrite();
+  cookieWriter.write<ident::id>(42);
+  cookieWriter.write<ident::fortune>("Meaning of life...");
+  cookieWriter.write<ident::flavor>("None");
+  cookieCursor.endWrite(std::move(cookieWriter));
+
+  auto structWriter = structCursor.beginWrite();
+  structWriter.write<ident::i32_field>(42);
+  structCursor.endWrite(std::move(structWriter));
+
+  auto mealWriter = mealCursor.beginWrite();
+  mealWriter.write<ident::appetizer>(2);
+  mealWriter.write<ident::main>(4);
+  mealWriter.writeSerialized<ident::cookie>(std::move(cookieCursor));
+  // Doesn't compile
+  // mealWriter.writeSerialized<ident::cookie>(std::move(structCursor));
+  mealCursor.endWrite(std::move(mealWriter));
+
+  auto deserialized = mealCursor.deserialize();
+  EXPECT_EQ(*deserialized.appetizer(), 2);
+  EXPECT_EQ(*deserialized.main(), 4);
+  EXPECT_EQ(*deserialized.cookie()->flavor(), "None");
+  EXPECT_EQ(*deserialized.cookie()->id(), 42);
+  EXPECT_EQ(*deserialized.cookie()->fortune(), "Meaning of life...");
+
+  {
+    auto mealReader = mealCursor.beginRead();
+    EXPECT_EQ(mealReader.read<ident::appetizer>(), 2);
+    EXPECT_EQ(mealReader.read<ident::main>(), 4);
+
+    auto cookieReader = mealReader.beginRead<ident::cookie>();
+    EXPECT_EQ(cookieReader.read<ident::id>(), 42);
+    EXPECT_EQ(cookieReader.read<ident::fortune>(), "Meaning of life...");
+    EXPECT_EQ(cookieReader.read<ident::flavor>(), "None");
+    mealReader.endRead(std::move(cookieReader));
+    mealCursor.endRead(std::move(mealReader));
+  }
+}
+
 TEST(CursorSerializer, CursorWriteInContainer) {
   StructCursor wrapper;
   auto writer = wrapper.beginWrite();
@@ -640,3 +704,71 @@ TEST(CursorSerializer, CursorWriteInContainer) {
       *obj.set_nested_field(),
       Contains(Contains(IsThriftUnionWith<ident::string_field>(Eq("foo")))));
 }
+
+void doCursorReadRemainEndTest(int count) {
+  std::unique_ptr<folly::IOBuf> buf;
+  {
+    ReadRemainingWrapper wrapper;
+    auto writer = wrapper.beginWrite();
+
+    auto listWriter = writer.beginWrite<ident::aaa>();
+    for (int i = 0; i < count; ++i) {
+      auto s = "a string " + std::to_string(i);
+      listWriter.write(s);
+    }
+    writer.endWrite(std::move(listWriter));
+
+    auto bwriter = writer.beginWrite<ident::bbb>();
+    for (int i = 0; i < count; ++i) {
+      bwriter.write(i);
+    }
+    writer.endWrite(std::move(bwriter));
+
+    writer.write<ident::ccc>(true);
+
+    wrapper.endWrite(std::move(writer));
+
+    buf = std::move(wrapper).serializedData();
+  }
+
+  {
+    ReadRemainingWrapper wrapper = ReadRemainingWrapper(std::move(buf));
+    auto reader = wrapper.beginRead();
+    auto listReader = reader.beginRead<ident::aaa>();
+    listReader.remaining();
+    reader.endRead(std::move(listReader));
+    auto breader = reader.beginRead<ident::bbb>();
+    reader.endRead(std::move(breader));
+    wrapper.endRead(std::move(reader));
+  }
+}
+
+TEST(CursorBasedSerializer, CursorReadRemainingEndOne) {
+  doCursorReadRemainEndTest(1);
+}
+
+TEST(CursorBasedSerializer, CursorReadRemainingEndMany) {
+  doCursorReadRemainEndTest(10);
+}
+
+TEST(CursorBasedSerializer, ConcurrentAccess) {
+  EmptyWrapper wrapper;
+  auto writer = wrapper.beginWrite();
+  EXPECT_THROW(wrapper.beginRead(), std::runtime_error);
+  EXPECT_THROW(wrapper.deserialize(), std::runtime_error);
+  EXPECT_THROW(wrapper.beginWrite(), std::runtime_error);
+  wrapper.endWrite(std::move(writer));
+
+  auto reader = wrapper.beginRead();
+  EXPECT_THROW(wrapper.beginRead(), std::runtime_error);
+  EXPECT_EQ(wrapper.deserialize(), Empty{});
+  EXPECT_THROW(wrapper.beginWrite(), std::runtime_error);
+  wrapper.endRead(std::move(reader));
+}
+
+TEST(CursorSerializer, AbandonedWrite) {
+  CursorSerializationWrapper<Qualifiers> wrapper;
+  auto writer = wrapper.beginWrite();
+  writer.write<ident::opt>(3);
+  wrapper.abandonWrite(std::move(writer));
+}
