@@ -16,16 +16,35 @@
 
 #pragma once
 
+#include <unordered_set>
+#include <folly/synchronization/RelaxedAtomic.h>
+#include <thrift/lib/cpp2/runtime/BaseSchemaRegistry.h>
 #include <thrift/lib/thrift/gen-cpp2/schema_types.h>
 
 namespace apache::thrift {
 
 class SchemaRegistry {
  public:
-  static const type::Schema& getMergedSchema();
+  // Access the global registry.
+  static SchemaRegistry& get();
+
+  using Ptr = std::shared_ptr<type::Schema>;
+
+  // Access all data registered
+  Ptr getMergedSchema();
+
+  // Helpers for working with schemas.
   static type::Schema mergeSchemas(
       folly::Range<const std::string_view*> schemas);
   static type::Schema mergeSchemas(std::vector<type::Schema>&& schemas);
+
+  explicit SchemaRegistry(BaseSchemaRegistry& base) : base_(base) {}
+
+ private:
+  BaseSchemaRegistry& base_;
+  Ptr mergedSchema_;
+  folly::relaxed_atomic<bool> mergedSchemaAccessed_{false};
+  std::unordered_set<type::ProgramId> includedPrograms_;
 };
 
 } // namespace apache::thrift
