@@ -40,8 +40,6 @@ THRIFT_FLAG_DEFINE_bool(server_header_reject_all, true);
 
 THRIFT_FLAG_DEFINE_int64(monitoring_over_header_logging_sample_rate, 1'000'000);
 
-THRIFT_FLAG_DECLARE_bool(enforce_header_transport_valid_protocol);
-
 namespace apache::thrift {
 
 using namespace std;
@@ -168,14 +166,13 @@ Cpp2Connection::~Cpp2Connection() {
 
 void Cpp2Connection::invokeServiceInterceptorsOnConnectionClosed() noexcept {
 #if FOLLY_HAS_COROUTINES
-  const auto& serviceInterceptorsInfo =
+  const auto& serviceInterceptors =
       worker_->getServer()->getServiceInterceptors();
-  for (std::size_t i = 0; i < serviceInterceptorsInfo.size(); ++i) {
+  for (std::size_t i = 0; i < serviceInterceptors.size(); ++i) {
     ServiceInterceptorBase::ConnectionInfo connectionInfo{
         &context_,
         context_.getStorageForServiceInterceptorOnConnectionByIndex(i)};
-    serviceInterceptorsInfo[i].interceptor->internal_onConnectionClosed(
-        connectionInfo);
+    serviceInterceptors[i]->internal_onConnectionClosed(connectionInfo);
   }
 #endif // FOLLY_HAS_COROUTINES
 }
@@ -256,11 +253,21 @@ void Cpp2Connection::setServerHeaders(
   auto& writeHeaders = request.getHeader()->mutableWriteHeaders();
   setServerHeaders(writeHeaders);
   const auto& readHeaders = request.getHeader()->getHeaders();
+  // set load header
   auto ptr = folly::get_ptr(readHeaders, THeader::QUERY_LOAD_HEADER);
   if (ptr) {
     auto load = getWorker()->getServer()->getLoad(*ptr);
     writeHeaders[THeader::QUERY_LOAD_HEADER] = folly::to<std::string>(load);
   }
+
+  // set secondary load header
+  auto ptrSl =
+      folly::get_ptr(readHeaders, THeader::QUERY_SECONDARY_LOAD_HEADER);
+  if (ptrSl) {
+    auto load = getWorker()->getServer()->getLoad(*ptrSl);
+    writeHeaders[THeader::QUERY_SECONDARY_LOAD_HEADER] =
+        folly::to<std::string>(load);
+  }
 }
 
 void Cpp2Connection::requestTimeoutExpired() {
@@ -425,8 +432,7 @@ void Cpp2Connection::requestReceived(
   auto msgBegin = apache::thrift::detail::ap::deserializeMessageBegin(
       *hreq->getBuf(), protoId);
 
-  if (THRIFT_FLAG(enforce_header_transport_valid_protocol) &&
-      !msgBegin.metadata.isValid) {
+  if (!msgBegin.metadata.isValid) {
     disconnect("Rejecting unparseable message begin");
     return;
   }
@@ -551,8 +557,8 @@ void Cpp2Connection::requestReceived(
     server->getGetHeaderHandler()(hreq->getHeader(), context_.getPeerAddress());
   }
 
-  if (auto overloadResult = server->checkOverload(
-          &hreq->getHeader()->getHeaders(), &methodName)) {
+  if (auto overloadResult =
+          server->checkOverload(hreq->getHeader()->getHeaders(), methodName)) {
     killRequestServerOverloaded(std::move(hreq), std::move(*overloadResult));
     return;
   }
@@ -894,7 +900,8 @@ void Cpp2Connection::Cpp2Request::sendTimeoutResponse(
   cancelTimeout();
 }
 
-void Cpp2Connection::Cpp2Request::sendQueueTimeoutResponse() {
+void Cpp2Connection::Cpp2Request::sendQueueTimeoutResponse(
+    bool /*interactionIsTerminated*/) {
   sendTimeoutResponse(
       HeaderServerChannel::HeaderRequest::TimeoutResponseType::QUEUE);
   connection_->queueTimeoutExpired();
