@@ -19,15 +19,12 @@
 #include <vector>
 
 #include <folly/GLog.h>
-#include <folly/Overload.h>
-#include <folly/String.h>
 #include <folly/experimental/io/AsyncIoUringSocketFactory.h>
 #include <folly/io/async/AsyncSSLSocket.h>
 #include <folly/io/async/AsyncSocket.h>
 #include <folly/io/async/EventBaseLocal.h>
 #include <folly/io/async/fdsock/AsyncFdSocket.h>
 #include <folly/portability/Sockets.h>
-#include <thrift/lib/cpp/concurrency/Util.h>
 #include <thrift/lib/cpp2/Flags.h>
 #include <thrift/lib/cpp2/async/ResponseChannel.h>
 #include <thrift/lib/cpp2/security/extensions/ThriftParametersContext.h>
@@ -56,7 +53,12 @@ void Cpp2Worker::initRequestsRegistry() {
   auto memPerWorker = server_->getMaxDebugPayloadMemoryPerWorker();
   auto maxFinished = server_->getMaxFinishedDebugPayloadsPerWorker();
   std::weak_ptr<Cpp2Worker> self_weak = shared_from_this();
-  evb->runInEventBaseThread([=, self_weak = std::move(self_weak)]() {
+  evb->runInEventBaseThread([this,
+                             evb,
+                             memPerReq,
+                             memPerWorker,
+                             maxFinished,
+                             self_weak = std::move(self_weak)]() {
     if (auto self = self_weak.lock()) {
       self->requestsRegistry_ = &registry.get().try_emplace(
           *evb, memPerReq, memPerWorker, maxFinished);
@@ -199,16 +201,15 @@ void Cpp2Worker::invokeServiceInterceptorsOnConnectionForHeader(
     [[maybe_unused]] Cpp2Connection& connection) noexcept {
 #if FOLLY_HAS_COROUTINES
   Cpp2ConnContext& context = connection.getContext();
-  const auto& serviceInterceptorsInfo = server_->getServiceInterceptors();
+  const auto& serviceInterceptors = server_->getServiceInterceptors();
   bool didServiceInterceptorOnConnectionThrow = false;
 
-  for (std::size_t i = 0; i < serviceInterceptorsInfo.size(); ++i) {
+  for (std::size_t i = 0; i < serviceInterceptors.size(); ++i) {
     ServiceInterceptorBase::ConnectionInfo connectionInfo{
         &context,
         context.getStorageForServiceInterceptorOnConnectionByIndex(i)};
     try {
-      serviceInterceptorsInfo[i].interceptor->internal_onConnection(
-          std::move(connectionInfo));
+      serviceInterceptors[i]->internal_onConnection(std::move(connectionInfo));
     } catch (...) {
       didServiceInterceptorOnConnectionThrow = true;
     }
