@@ -51,10 +51,10 @@
 #include <thrift/lib/cpp2/Flags.h>
 #include <thrift/lib/cpp2/async/MultiplexAsyncProcessor.h>
 #include <thrift/lib/cpp2/runtime/Init.h>
+#include <thrift/lib/cpp2/server/ConcurrencyControllerInterfaceUnsafeAPI.h>
 #include <thrift/lib/cpp2/server/Cpp2Connection.h>
 #include <thrift/lib/cpp2/server/Cpp2Worker.h>
 #include <thrift/lib/cpp2/server/ExecutorToThreadManagerAdaptor.h>
-#include <thrift/lib/cpp2/server/InternalPriorityRequestPile.h>
 #include <thrift/lib/cpp2/server/LegacyHeaderRoutingHandler.h>
 #include <thrift/lib/cpp2/server/LoggingEvent.h>
 #include <thrift/lib/cpp2/server/ServerFlags.h>
@@ -92,7 +92,6 @@ THRIFT_FLAG_DEFINE_bool(server_enable_stoptls, false);
 
 THRIFT_FLAG_DEFINE_bool(enable_mrl_check_for_thrift_server, false);
 THRIFT_FLAG_DEFINE_bool(enforce_mrl_check_for_thrift_server, false);
-THRIFT_FLAG_DEFINE_bool(migrate_mrl_initializer_for_thrift_server, false);
 
 THRIFT_FLAG_DEFINE_bool(dump_snapshot_on_long_shutdown, true);
 
@@ -267,10 +266,10 @@ ThriftServer::ThriftServer()
           apache::thrift::detail::makeAdaptiveConcurrencyConfig(),
           thriftConfig_.getMaxRequests().getObserver(),
           detail::getThriftServerConfig(*this)},
-      cpuConcurrencyController_{
+      cpuConcurrencyController_{std::make_shared<CPUConcurrencyController>(
           makeCPUConcurrencyControllerConfigInternal(),
           *this,
-          detail::getThriftServerConfig(*this)},
+          detail::getThriftServerConfig(*this))},
       addresses_(1),
       wShutdownSocketSet_(folly::tryGetShutdownSocketSet()),
       lastRequestTime_(
@@ -343,7 +342,7 @@ void ThriftServer::initializeDefaults() {
 std::unique_ptr<RequestPileInterface> ThriftServer::makeStandardRequestPile(
     RoundRobinRequestPile::Options options) {
   if (runtimeServerActions_.interactionInService) {
-    return std::make_unique<InternalPriorityRequestPile>(std::move(options));
+    options = RoundRobinRequestPile::addInternalPriorities(std::move(options));
   }
   return std::make_unique<RoundRobinRequestPile>(std::move(options));
 }
@@ -551,21 +550,27 @@ class ThriftServer::ConnectionEventCallback
   void onConnectionEnqueuedForAcceptorCallback(
       const folly::NetworkSocket,
       const folly::SocketAddress& clientAddr) noexcept override {
-    THRIFT_CONNECTION_EVENT(connection_enqueued_acceptor)
-        .log(thriftServer_, clientAddr);
+    // Increment counters prior to logging Scuba events to mitigate the risk of
+    // race conditions between increment and decrement operations.
     if (pendingConnectionsMetrics_) {
       pendingConnectionsMetrics_->onConnectionEnqueuedToIoWorker();
     }
+
+    THRIFT_CONNECTION_EVENT(connection_enqueued_acceptor)
+        .log(thriftServer_, clientAddr);
   }
 
   void onConnectionDequeuedByAcceptorCallback(
       const folly::NetworkSocket,
       const folly::SocketAddress& clientAddr) noexcept override {
-    THRIFT_CONNECTION_EVENT(connection_dequeued_acceptor)
-        .log(thriftServer_, clientAddr);
+    // Increment counters prior to logging Scuba events to mitigate the risk of
+    // race conditions between increment and decrement operations.
     if (pendingConnectionsMetrics_) {
       pendingConnectionsMetrics_->onConnectionDequedByIoWorker();
     }
+
+    THRIFT_CONNECTION_EVENT(connection_dequeued_acceptor)
+        .log(thriftServer_, clientAddr);
   }
 
   void onBackoffStarted() noexcept override {}
@@ -736,7 +741,7 @@ void ThriftServer::setup() {
         socket->setZeroCopy(useZeroCopy);
         socket->setQueueTimeout(getSocketQueueTimeout());
         if (callbackAssignFunc_) {
-          socket->setCallbackAssignFunction(std::move(callbackAssignFunc_));
+          socket->setCallbackAssignFunction(callbackAssignFunc_);
         }
 
         try {
@@ -951,10 +956,11 @@ void ThriftServer::setupThreadManager() {
       }
     } else {
       auto explanation = fmt::format(
-          "thrift flag: {}, enable gflag: {}, disable gflag: {}",
+          "thrift flag: {}, enable gflag: {}, disable gflag: {}, runtime actions: {}",
           THRIFT_FLAG(experimental_use_resource_pools),
           FLAGS_thrift_experimental_use_resource_pools,
-          FLAGS_thrift_disable_resource_pools);
+          FLAGS_thrift_disable_resource_pools,
+          runtimeServerActions_.explain());
       XLOG_IF(INFO, infoLoggingEnabled_)
           << "Using resource pools on address/port " << getAddressAsString()
           << ": " << explanation;
@@ -972,38 +978,63 @@ void ThriftServer::setupThreadManager() {
   // Now do setup that we want to do whether we created these resources or the
   // client did.
   if (!resourcePoolSet().empty()) {
-    // Keep concurrency controller in sync with max requests for now.
+    setConcurrencyLimitCallbackHandle =
+        detail::getThriftServerConfig(*this)
+            .getConcurrencyLimit()
+            .getObserver()
+            .addCallback(
+                [this](const folly::observer::Snapshot<uint32_t>& snapshot) {
+                  auto concurrencyLimit = *snapshot;
+                  if (auto cc =
+                          resourcePoolSet()
+                              .resourcePool(ResourcePoolHandle::defaultAsync())
+                              .concurrencyController()) {
+                    cc.value().get().setExecutionLimitRequests(
+                        concurrencyLimit != 0
+                            ? concurrencyLimit
+                            : std::numeric_limits<
+                                  decltype(concurrencyLimit)>::max());
+                  }
+                });
+    // ThriftServer's maxRequests was historically synced to
+    // ConcurrencyController's executionLimitRequests. This syncing behavior
+    // will be removed, but for now we sync by default.
     setMaxRequestsCallbackHandle =
         detail::getThriftServerConfig(*this)
             .getMaxRequests()
             .getObserver()
-            .addCallback([this](folly::observer::Snapshot<uint32_t> snapshot) {
-              auto maxRequests = *snapshot;
-              if (auto cc =
-                      resourcePoolSet()
-                          .resourcePool(ResourcePoolHandle::defaultAsync())
-                          .concurrencyController()) {
-                cc.value().get().setExecutionLimitRequests(
-                    maxRequests != 0
-                        ? maxRequests
-                        : std::numeric_limits<decltype(maxRequests)>::max());
-              }
-            });
+            .addCallback(
+                [this](const folly::observer::Snapshot<uint32_t>& snapshot) {
+                  auto maxRequests = *snapshot;
+                  if (auto cc =
+                          resourcePoolSet()
+                              .resourcePool(ResourcePoolHandle::defaultAsync())
+                              .concurrencyController()) {
+                    (cc.value().get())
+                        .setExecutionLimitRequests(
+                            maxRequests != 0
+                                ? maxRequests
+                                : std::numeric_limits<
+                                      decltype(maxRequests)>::max());
+                  }
+                });
     setMaxQpsCallbackHandle =
         detail::getThriftServerConfig(*this)
             .getMaxQps()
             .getObserver()
-            .addCallback([this](folly::observer::Snapshot<uint32_t> snapshot) {
-              auto maxQps = *snapshot;
-              if (auto cc =
-                      resourcePoolSet()
-                          .resourcePool(ResourcePoolHandle::defaultAsync())
-                          .concurrencyController()) {
-                cc.value().get().setQpsLimit(
-                    maxQps != 0 ? maxQps
-                                : std::numeric_limits<decltype(maxQps)>::max());
-              }
-            });
+            .addCallback(
+                [this](const folly::observer::Snapshot<uint32_t>& snapshot) {
+                  auto maxQps = *snapshot;
+                  if (auto cc =
+                          resourcePoolSet()
+                              .resourcePool(ResourcePoolHandle::defaultAsync())
+                              .concurrencyController()) {
+                    cc.value().get().setQpsLimit(
+                        maxQps != 0
+                            ? maxQps
+                            : std::numeric_limits<decltype(maxQps)>::max());
+                  }
+                });
     // Create an adapter so calls to getThreadManager_deprecated will work
     // when we are using resource pools
     if (!threadManager_ &&
@@ -1144,36 +1175,10 @@ bool ThriftServer::runtimeResourcePoolsChecks() {
         if (metadata.interactionType ==
             AsyncProcessorFactory::MethodMetadata::InteractionType::
                 INTERACTION_V1) {
-          // We've found an interaction in this service. We check if the
-          // AsyncProcessorFactory is a Thrift-generated one or not.
-          //
-          // If it is a generated AsyncProcessorFactory:
-          //  Check enable_resource_pools_for_interaction AND
-          //  enable_resource_pools_for_interaction_generated_processor_only. If
-          //  BOTH are disabled, we disable Resource Pools.
-          //
-          // If it is a custom AsyncProcessorFactory:
-          //  Check enable_resource_pools_for_interaction. If it is disabled, we
-          //  disable Resource Pools.
+          // We've found an interaction in this service. We use
+          // `interactionInService` to decide if we need to add internal
+          // priorities to the default RoundRobinRequestPile.
           runtimeServerActions_.interactionInService = true;
-          if (auto apf = getProcessorFactory();
-              apf && apf->isThriftGenerated()) {
-            if (!THRIFT_FLAG(enable_resource_pools_for_interaction) &&
-                !THRIFT_FLAG(
-                    enable_resource_pools_for_interaction_generated_processor_only)) {
-              XLOG_IF(INFO, infoLoggingEnabled_)
-                  << "Resource pools disabled on generated processor. Interaction on request "
-                  << methodToMetadataPtr.first;
-              runtimeDisableResourcePoolsDeprecated();
-            }
-          } else {
-            if (!THRIFT_FLAG(enable_resource_pools_for_interaction)) {
-              XLOG_IF(INFO, infoLoggingEnabled_)
-                  << "Resource pools disabled on custom processor. Interaction on request "
-                  << methodToMetadataPtr.first;
-              runtimeDisableResourcePoolsDeprecated();
-            }
-          }
         }
       }
     } else { // we can only have WildcardMethodMetadataMap here
@@ -1727,8 +1732,8 @@ void ThriftServer::callOnStartServing() {
   {
     ServiceInterceptorBase::InitParams initParams;
     std::vector<folly::coro::Task<void>> tasks;
-    for (const auto& info : getServiceInterceptors()) {
-      tasks.emplace_back(info.interceptor->co_onStartServing(initParams));
+    for (const auto& interceptor : getServiceInterceptors()) {
+      tasks.emplace_back(interceptor->co_onStartServing(initParams));
     }
     folly::coro::blockingWait(folly::coro::collectAllRange(std::move(tasks)));
   }
@@ -1955,12 +1960,11 @@ PreprocessResult ThriftServer::preprocess(
 }
 
 folly::Optional<OverloadResult> ThriftServer::checkOverload(
-    const transport::THeader::StringToStringMap* readHeaders,
-    const std::string* method) {
+    const transport::THeader::StringToStringMap& readHeaders,
+    const std::string& method) {
   if (UNLIKELY(
           isOverloaded_ &&
-          (method == nullptr ||
-           !getMethodsBypassMaxRequestsLimit().contains(*method)) &&
+          !getMethodsBypassMaxRequestsLimit().contains(method) &&
           isOverloaded_(readHeaders, method))) {
     return OverloadResult{
         kAppOverloadedErrorCode,
@@ -1970,6 +1974,60 @@ folly::Optional<OverloadResult> ThriftServer::checkOverload(
         LoadShedder::CUSTOM};
   }
 
+  // Log services that might break if ConcurrencyController's executionLimit is
+  // unsynced from maxRequests and is synced with only concurrencyLimit instead.
+  if (resourcePoolSet().empty() ||
+      folly::test_once(cancelSetMaxRequestsCallbackHandleFlag_) ||
+      folly::test_once(serviceReliesOnSyncedMaxRequestsFlag_)) {
+    // This is okay. If we're not using resource pools, syncing was cancelled
+    // because setConcurrencyLimit was explicitly called, or we've already
+    // detected and logged reliance on maxRequests syncing with
+    // concurrencyLimit, we don't need to check anything else.
+  } else if (
+      thriftConfig_.getMaxRequests().get() ==
+      thriftConfig_.getConcurrencyLimit().get()) {
+    // This is okay. When ConcurrencyController's executionLimit is synced with
+    // concurrencyLimit instead of maxRequests, there will be no difference
+    // since the values are the same.
+  } else if (
+      !isActiveRequestsTrackingDisabled() &&
+      !THRIFT_FLAG(enforce_queue_concurrency_resource_pools) &&
+      !getMethodsBypassMaxRequestsLimit().contains(method)) {
+    // This is okay. No bypass method is enabled. ThriftServer is strictly
+    // rejecting requests once there are maxRequests requests on the server.
+    // ConcurrencyController's will never enforce its executionLimit (synced
+    // from maxRequests) since ThriftServer will never pass enough requests into
+    // the resource pool. After ConcurrencyController's executionLimit is synced
+    // from concurrencyLimit instead, the new default value (uint32_t max) will
+    // continue to be greater than the number of requests that can be passed
+    // into the resource pool.
+  } else {
+    // This is not okay. When ConcurrencyController's executionLimit is unsynced
+    // from maxRequests and synced to concurrencyLimit instead, the service
+    // might encounter a behavioral change.
+    folly::call_once(serviceMightRelyOnSyncedMaxRequestsFlag_, [this]() {
+      LOG(WARNING) << "Service might rely on synced max requests.";
+      THRIFT_SERVER_EVENT(serviceMightRelyOnSyncedMaxRequests).log(*this);
+    });
+
+    if (auto concurrencyController =
+            resourcePoolSet()
+                .resourcePool(ResourcePoolHandle::defaultAsync())
+                .concurrencyController()) {
+      if (ConcurrencyControllerInterfaceUnsafeAPI(concurrencyController->get())
+              .getLimitHasBeenEnforced()) {
+        // Cncurrency limit has been enforced while still synced with
+        // maxRequests. This is the kind of case that absolutely must be
+        // migrated before completely decoupling maxRequests from
+        // concurrencyLimit.
+        folly::call_once(serviceReliesOnSyncedMaxRequestsFlag_, [this]() {
+          LOG(WARNING) << "Service relies on synced max requests.";
+          THRIFT_SERVER_EVENT(serviceReliesOnSyncedMaxRequests).log(*this);
+        });
+      }
+    }
+  }
+
   // If active request tracking is disabled or we are using resource pools,
   // skip max requests enforcement here. Resource pools has its own separate
   // concurrency limiting mechanism.
@@ -1977,11 +2035,10 @@ folly::Optional<OverloadResult> ThriftServer::checkOverload(
       THRIFT_FLAG(enforce_queue_concurrency_resource_pools);
   if (!isActiveRequestsTrackingDisabled() && !useQueueConcurrency) {
     if (auto maxRequests = getMaxRequests(); maxRequests > 0 &&
-        (method == nullptr ||
-         !getMethodsBypassMaxRequestsLimit().contains(*method)) &&
+        !getMethodsBypassMaxRequestsLimit().contains(method) &&
         static_cast<uint32_t>(getActiveRequests()) >= maxRequests) {
       LoadShedder loadShedder = LoadShedder::MAX_REQUESTS;
-      if (getCPUConcurrencyController().requestShed(
+      if (notifyCPUConcurrencyControllerOnRequestLoadShed(
               CPUConcurrencyController::Method::MAX_REQUESTS)) {
         loadShedder = LoadShedder::CPU_CONCURRENCY_CONTROLLER;
       } else if (getAdaptiveConcurrencyController().enabled()) {
@@ -1996,11 +2053,10 @@ folly::Optional<OverloadResult> ThriftServer::checkOverload(
 
   if (auto maxQps = getMaxQps(); maxQps > 0 &&
       FLAGS_thrift_server_enforces_qps_limit &&
-      (method == nullptr ||
-       !getMethodsBypassMaxRequestsLimit().contains(*method)) &&
+      !getMethodsBypassMaxRequestsLimit().contains(method) &&
       !qpsTokenBucket_.consume(1.0, maxQps, maxQps)) {
     LoadShedder loadShedder = LoadShedder::MAX_QPS;
-    if (getCPUConcurrencyController().requestShed(
+    if (notifyCPUConcurrencyControllerOnRequestLoadShed(
             CPUConcurrencyController::Method::MAX_QPS)) {
       loadShedder = LoadShedder::CPU_CONCURRENCY_CONTROLLER;
     }
@@ -2223,11 +2279,6 @@ folly::observer::Observer<bool> ThriftServer::enforceTLSCertRevocation() {
   return THRIFT_FLAG_OBSERVE(enforce_mrl_check_for_thrift_server);
 }
 
-folly::observer::Observer<bool>
-ThriftServer::migrateTLSCertRevocationInitializer() {
-  return THRIFT_FLAG_OBSERVE(migrate_mrl_initializer_for_thrift_server);
-}
-
 folly::observer::Observer<bool> ThriftServer::enableReceivingDelegatedCreds() {
   return THRIFT_FLAG_OBSERVE(server_fizz_enable_receiving_dc);
 }
@@ -2446,22 +2497,21 @@ ThriftServer::processModulesSpecification(ModulesSpecification&& specs) {
       std::vector<std::shared_ptr<ServiceInterceptorBase>> serviceInterceptors =
           moduleSpec.module->getServiceInterceptors();
 
-      std::vector<ServiceInterceptorInfo> result;
+      std::vector<std::shared_ptr<ServiceInterceptorBase>> result;
       for (auto& interceptor : serviceInterceptors) {
-        auto qualifiedName =
-            fmt::format("{}.{}", moduleSpec.name, interceptor->getName());
-        if (seenNames_.find(qualifiedName) != seenNames_.end()) {
-          throw std::logic_error(
-              fmt::format("Duplicate ServiceInterceptor: {}", qualifiedName));
+        interceptor->setModuleName(moduleSpec.name);
+        auto qualifiedNameStr = interceptor->getQualifiedName().toString();
+        if (seenNames_.find(qualifiedNameStr) != seenNames_.end()) {
+          throw std::logic_error(fmt::format(
+              "Duplicate ServiceInterceptor: {}", qualifiedNameStr));
         }
-        seenNames_.insert(qualifiedName);
-        result.emplace_back(ServiceInterceptorInfo{
-            std::move(qualifiedName), std::move(interceptor)});
+        seenNames_.insert(qualifiedNameStr);
+        result.emplace_back(std::move(interceptor));
       }
       interceptorsByModule_.emplace_back(std::move(result));
     }
 
-    std::vector<ServiceInterceptorInfo> coalesce() && {
+    std::vector<std::shared_ptr<ServiceInterceptorBase>> coalesce() && {
       if constexpr (folly::kIsDebug) {
         // Our contract guarantees ordering of interceptors within a module, but
         // not across modules. In debug mode, let's shuffle order across modules
@@ -2473,7 +2523,7 @@ ThriftServer::processModulesSpecification(ModulesSpecification&& specs) {
             folly::Random::DefaultGenerator());
       }
 
-      std::vector<ServiceInterceptorInfo> result;
+      std::vector<std::shared_ptr<ServiceInterceptorBase>> result;
       for (auto& interceptors : interceptorsByModule_) {
         result.insert(
             result.end(),
@@ -2484,14 +2534,17 @@ ThriftServer::processModulesSpecification(ModulesSpecification&& specs) {
     }
 
    private:
-    std::vector<std::vector<ServiceInterceptorInfo>> interceptorsByModule_;
+    std::vector<std::vector<std::shared_ptr<ServiceInterceptorBase>>>
+        interceptorsByModule_;
     std::unordered_set<std::string> seenNames_;
   };
 #else
   class ServiceInterceptorsCollector {
    public:
     void addModule(const ModulesSpecification::Info&) {}
-    std::vector<ServiceInterceptorInfo> coalesce() && { return {}; }
+    std::vector<std::shared_ptr<ServiceInterceptorBase>> coalesce() && {
+      return {};
+    }
   };
 #endif // FOLLY_HAS_COROUTINES
   ServiceInterceptorsCollector serviceInterceptorsCollector;
@@ -2574,4 +2627,11 @@ bool ThriftServer::getTaskExpireTimeForRequest(
   }
   return queueTimeout != taskTimeout;
 }
+
+bool ThriftServer::notifyCPUConcurrencyControllerOnRequestLoadShed(
+    std::optional<CPUConcurrencyController::Method> method) {
+  auto* cpuConcurrencyControllerPtr = getCPUConcurrencyController();
+  return cpuConcurrencyControllerPtr != nullptr &&
+      cpuConcurrencyControllerPtr->requestShed(method);
+}
 } // namespace apache::thrift
