@@ -24,20 +24,27 @@
 #include <folly/coro/GtestHelpers.h>
 #include <thrift/lib/cpp2/async/HTTPClientChannel.h>
 #include <thrift/lib/cpp2/async/HeaderClientChannel.h>
+#include <thrift/lib/cpp2/async/MultiplexAsyncProcessor.h>
 #include <thrift/lib/cpp2/async/RocketClientChannel.h>
 #include <thrift/lib/cpp2/server/ServerModule.h>
 #include <thrift/lib/cpp2/server/ServiceInterceptor.h>
 #include <thrift/lib/cpp2/server/ThriftServer.h>
+#include <thrift/lib/cpp2/server/ThriftServerInternals.h>
 #include <thrift/lib/cpp2/server/test/gen-cpp2/ServiceInterceptor_clients.h>
 #include <thrift/lib/cpp2/server/test/gen-cpp2/ServiceInterceptor_handlers.h>
+#include <thrift/lib/cpp2/test/gen-cpp2/MultiplexAsyncProcessor2_clients.h>
+#include <thrift/lib/cpp2/test/gen-cpp2/MultiplexAsyncProcessor2_handlers.h>
+#include <thrift/lib/cpp2/test/gen-cpp2/MultiplexAsyncProcessor_clients.h>
+#include <thrift/lib/cpp2/test/gen-cpp2/MultiplexAsyncProcessor_handlers.h>
 #include <thrift/lib/cpp2/transport/http2/common/HTTP2RoutingHandler.h>
 #include <thrift/lib/cpp2/util/ScopedServerInterfaceThread.h>
 
 using namespace apache::thrift;
+using namespace apache::thrift::test;
+using namespace apache::thrift::test2;
 using namespace ::testing;
 
 namespace {
-
 using TransportType = Cpp2ConnContext::TransportType;
 
 std::unique_ptr<HTTP2RoutingHandler> createHTTP2RoutingHandler(
@@ -66,6 +73,12 @@ class ServiceInterceptorTestP : public ::testing::TestWithParam<TransportType> {
     return runner;
   }
 
+  std::shared_ptr<AsyncProcessorFactory> multiplex(
+      std::vector<std::shared_ptr<AsyncProcessorFactory>> services) {
+    return std::make_shared<MultiplexAsyncProcessorFactory>(
+        std::move(services));
+  }
+
   ScopedServerInterfaceThread::MakeChannelFunc channelFor(
       TransportType transportType) {
     return [transportType](
@@ -249,6 +262,59 @@ struct ServiceInterceptorThrowOnRequest
   int onResponseCount = 0;
 };
 
+// Covering the case where ServiceInterceptor should be able to catch exception
+// thrown from another interceptor
+struct ServiceInterceptorRequestExceptionCatch
+    : public NamedServiceInterceptor<folly::Unit> {
+ public:
+  using NamedServiceInterceptor::NamedServiceInterceptor;
+
+  explicit ServiceInterceptorRequestExceptionCatch(
+      const std::string_view interceptorName,
+      const bool shouldSelfThrow,
+      const std::string_view expectedErrMsg)
+      : NamedServiceInterceptor<folly::Unit>(std::string(interceptorName)),
+        expectedErrMsg_(expectedErrMsg),
+        shouldSelfThrow_(shouldSelfThrow) {}
+
+  folly::coro::Task<std::optional<folly::Unit>> onRequest(
+      folly::Unit*, RequestInfo) override {
+    onRequestCount++;
+    if (shouldSelfThrow_) {
+      throw std::logic_error(
+          "Exception from ServiceInterceptorRequestExceptionCatch");
+    }
+    co_return std::nullopt;
+  }
+
+  folly::coro::Task<void> onResponse(
+      folly::Unit*, folly::Unit*, ResponseInfo responseInfo) override {
+    onResponseCount++;
+    if (std::holds_alternative<folly::exception_wrapper>(
+            responseInfo.resultOrActiveException)) {
+      const auto& exception = std::get<folly::exception_wrapper>(
+          responseInfo.resultOrActiveException);
+      EXPECT_TRUE(exception.get_exception() != nullptr);
+      if (shouldSelfThrow_) {
+        EXPECT_THAT(
+            std::string(exception.get_exception()->what()),
+            HasSubstr(
+                "Exception from ServiceInterceptorRequestExceptionCatch"));
+      } else {
+        EXPECT_THAT(
+            std::string(exception.get_exception()->what()),
+            HasSubstr(expectedErrMsg_));
+      }
+    }
+    co_return;
+  }
+
+  int onRequestCount = 0;
+  int onResponseCount = 0;
+  const std::string expectedErrMsg_;
+  const bool shouldSelfThrow_;
+};
+
 struct ServiceInterceptorThrowOnResponse
     : public NamedServiceInterceptor<folly::Unit> {
  public:
@@ -367,15 +433,32 @@ struct ServiceInterceptorLogResultTypeOnResponse
   std::vector<Entry> results;
 };
 
+class MockInterceptorMetricCallback : public InterceptorMetricCallback {
+ public:
+  MOCK_METHOD(
+      void,
+      onRequestComplete,
+      (const ServiceInterceptorQualifiedName& qualifiedName,
+       std::chrono::microseconds onResponseTime),
+      (override));
+};
+
 } // namespace
 
 CO_TEST_P(ServiceInterceptorTestP, BasicTM) {
   auto interceptor =
       std::make_shared<ServiceInterceptorCountWithRequestState>("Interceptor1");
+  auto interceptorMetricCallback =
+      std::make_shared<MockInterceptorMetricCallback>();
   auto runner =
       makeServer(std::make_shared<TestHandler>(), [&](ThriftServer& server) {
         server.addModule(std::make_unique<TestModule>(interceptor));
+        detail::ThriftServerInternals(server).setInterceptorMetricCallback(
+            interceptorMetricCallback);
       });
+  EXPECT_CALL(*interceptorMetricCallback, onRequestComplete(_, _))
+      .Times(2)
+      .WillRepeatedly(Return());
 
   auto client =
       makeClient<apache::thrift::Client<test::ServiceInterceptorTest>>(*runner);
@@ -720,6 +803,85 @@ CO_TEST_P(ServiceInterceptorTestP, OnRequestException) {
   EXPECT_EQ(interceptor3->onResponseCount, 1);
 }
 
+CO_TEST_P(
+    ServiceInterceptorTestP, InterceptorOnRequestExceptionCatchSelfThrowTest) {
+  auto interceptor1 = std::make_shared<ServiceInterceptorRequestExceptionCatch>(
+      "Interceptor1",
+      true /* self throw */,
+      "Exception from ServiceInterceptorRequestExceptionCatch");
+  auto runner =
+      makeServer(std::make_shared<TestHandler>(), [&](ThriftServer& server) {
+        server.addModule(
+            std::make_unique<TestModule>(InterceptorList{interceptor1}));
+      });
+
+  auto client =
+      makeClient<apache::thrift::Client<test::ServiceInterceptorTest>>(*runner);
+  EXPECT_THROW(
+      {
+        try {
+          co_await client->co_echo("");
+        } catch (const apache::thrift::TApplicationException& ex) {
+          EXPECT_THAT(
+              std::string(ex.what()),
+              HasSubstr("ServiceInterceptor::onRequest threw exceptions"));
+          EXPECT_THAT(
+              std::string(ex.what()),
+              HasSubstr(
+                  "Exception from ServiceInterceptorRequestExceptionCatch"));
+          EXPECT_THAT(
+              std::string(ex.what()), HasSubstr("[TestModule.Interceptor1]"));
+          throw;
+        }
+      },
+      apache::thrift::TApplicationException);
+  EXPECT_EQ(interceptor1->onRequestCount, 1);
+  EXPECT_EQ(interceptor1->onResponseCount, 1);
+}
+
+CO_TEST_P(
+    ServiceInterceptorTestP, InterceptorOnRequestExceptionCatchOtherThrowTest) {
+  auto interceptor1 =
+      std::make_shared<ServiceInterceptorThrowOnRequest>("Interceptor1");
+  auto interceptor2 = std::make_shared<ServiceInterceptorRequestExceptionCatch>(
+      "Interceptor2",
+      false /* self throw */,
+      "Exception from ServiceInterceptorThrowOnRequest::onRequest");
+  auto runner =
+      makeServer(std::make_shared<TestHandler>(), [&](ThriftServer& server) {
+        server.addModule(std::make_unique<TestModule>(
+            InterceptorList{interceptor1, interceptor2}));
+      });
+
+  auto client =
+      makeClient<apache::thrift::Client<test::ServiceInterceptorTest>>(*runner);
+  EXPECT_THROW(
+      {
+        try {
+          co_await client->co_echo("");
+        } catch (const apache::thrift::TApplicationException& ex) {
+          EXPECT_THAT(
+              std::string(ex.what()),
+              HasSubstr("ServiceInterceptor::onRequest threw exceptions"));
+          EXPECT_THAT(
+              std::string(ex.what()),
+              HasSubstr(
+                  "Exception from ServiceInterceptorThrowOnRequest::onRequest"));
+          EXPECT_THAT(
+              std::string(ex.what()), HasSubstr("[TestModule.Interceptor1]"));
+          EXPECT_THAT(
+              std::string(ex.what()),
+              Not(HasSubstr("[TestModule.Interceptor2]")));
+          throw;
+        }
+      },
+      apache::thrift::TApplicationException);
+  EXPECT_EQ(interceptor1->onRequestCount, 1);
+  EXPECT_EQ(interceptor1->onResponseCount, 1);
+  EXPECT_EQ(interceptor2->onRequestCount, 1);
+  EXPECT_EQ(interceptor2->onResponseCount, 1);
+}
+
 CO_TEST_P(ServiceInterceptorTestP, OnRequestExceptionEB) {
   auto interceptor =
       std::make_shared<ServiceInterceptorThrowOnRequest>("Interceptor1");
@@ -1122,27 +1284,53 @@ CO_TEST_P(ServiceInterceptorTestP, RequestArguments) {
   EXPECT_EQ(interceptor->arg3, requestArgs);
 }
 
-CO_TEST_P(ServiceInterceptorTestP, ServiceAndMethodNames) {
-  struct ServiceInterceptorCheckingServiceAndMethodNames
-      : public NamedServiceInterceptor<folly::Unit> {
-   public:
-    using ConnectionState = folly::Unit;
-    using RequestState = folly::Unit;
+namespace {
+struct ServiceNameInfo {
+  std::string serviceName;
+  std::string definingServiceName;
+  std::string methodName;
 
-    ServiceInterceptorCheckingServiceAndMethodNames()
-        : NamedServiceInterceptor("SomeName") {}
+  [[maybe_unused]] friend bool operator==(
+      const ServiceNameInfo& lhs, const ServiceNameInfo& rhs) {
+    return std::tie(lhs.serviceName, lhs.definingServiceName, lhs.methodName) ==
+        std::tie(rhs.serviceName, rhs.definingServiceName, rhs.methodName);
+  }
 
-    folly::coro::Task<std::optional<RequestState>> onRequest(
-        ConnectionState*, RequestInfo requestInfo) override {
-      names.emplace_back(
-          requestInfo.serviceName ? std::string(requestInfo.serviceName) : "",
-          requestInfo.methodName ? std::string(requestInfo.methodName) : "");
-      co_return std::nullopt;
-    }
+  [[maybe_unused]] friend std::ostream& operator<<(
+      std::ostream& out, const ServiceNameInfo& info) {
+    return out << fmt::format(
+               "[{}, {}, {}]",
+               info.serviceName,
+               info.definingServiceName,
+               info.methodName);
+  }
+};
 
-    using Entry = std::pair<std::string, std::string>;
-    std::vector<Entry> names;
-  };
+struct ServiceInterceptorCheckingServiceAndMethodNames
+    : public NamedServiceInterceptor<folly::Unit> {
+ public:
+  using ConnectionState = folly::Unit;
+  using RequestState = folly::Unit;
+
+  ServiceInterceptorCheckingServiceAndMethodNames()
+      : NamedServiceInterceptor("SomeName") {}
+
+  folly::coro::Task<std::optional<RequestState>> onRequest(
+      ConnectionState*, RequestInfo requestInfo) override {
+    names.emplace_back(
+        requestInfo.serviceName ? std::string(requestInfo.serviceName) : "",
+        requestInfo.definingServiceName
+            ? std::string(requestInfo.definingServiceName)
+            : "",
+        requestInfo.methodName ? std::string(requestInfo.methodName) : "");
+    co_return std::nullopt;
+  }
+
+  std::vector<ServiceNameInfo> names;
+};
+} // namespace
+
+CO_TEST_P(ServiceInterceptorTestP, ServiceAndMethodNamesBasic) {
   auto interceptor =
       std::make_shared<ServiceInterceptorCheckingServiceAndMethodNames>();
   auto runner =
@@ -1155,25 +1343,186 @@ CO_TEST_P(ServiceInterceptorTestP, ServiceAndMethodNames) {
   co_await client->co_echo("");
   co_await client->co_noop();
 
-  std::vector<ServiceInterceptorCheckingServiceAndMethodNames::Entry>
-      expectedNames = {
-          {"ServiceInterceptorTest", "echo"},
-          {"ServiceInterceptorTest", "noop"},
-      };
+  std::vector<ServiceNameInfo> expectedNames = {
+      {"ServiceInterceptorTest", "ServiceInterceptorTest", "echo"},
+      {"ServiceInterceptorTest", "ServiceInterceptorTestBase", "noop"},
+  };
 
   if (transportType() == TransportType::ROCKET) {
     // only rocket supports interactions
     auto interaction = co_await client->co_createInteraction();
     co_await interaction.co_echo("");
-    expectedNames.emplace_back("ServiceInterceptorTest", "createInteraction");
     expectedNames.emplace_back(
-        "ServiceInterceptorTest", "SampleInteraction.echo");
+        "ServiceInterceptorTest",
+        "ServiceInterceptorTest",
+        "createInteraction");
+    expectedNames.emplace_back(
+        "ServiceInterceptorTest",
+        "ServiceInterceptorTest",
+        "SampleInteraction.echo");
   }
 
   EXPECT_THAT(interceptor->names, ElementsAreArray(expectedNames));
 }
+/* Service Authorization Platform depends on ServiceInterceptor multiplex
+ * conflict resolution. Breakage in these tests could result in ACL failures.*/
+CO_TEST_P(
+    ServiceInterceptorTestP, ServiceAndMethodNamesMultiplexBasicConflicts) {
+  class SecondHandler : public apache::thrift::ServiceHandler<Second> {
+    int three() override { return 3; }
+    int four() override { return 4; }
+  };
 
-CO_TEST_P(ServiceInterceptorTestP, ResultOrActiveExceptionTypesAreCorrect) {
+  class ThirdHandler : public apache::thrift::ServiceHandler<Third> {
+    int five() override { return 5; }
+    int six() override { return 6; }
+  };
+
+  class ConflictsHandler : public apache::thrift::ServiceHandler<Conflicts> {
+    int four() override { return 444; }
+    int five() override { return 555; }
+  };
+  std::vector<std::shared_ptr<AsyncProcessorFactory>> services = {
+      /* order matters when resolving conflicts */
+      std::make_shared<SecondHandler>(),
+      std::make_shared<ConflictsHandler>(),
+      std::make_shared<ThirdHandler>(),
+  };
+  auto interceptor =
+      std::make_shared<ServiceInterceptorCheckingServiceAndMethodNames>();
+  auto runner = makeServer(multiplex(services), [&](ThriftServer& server) {
+    server.addModule(std::make_unique<TestModule>(interceptor));
+  });
+
+  auto client2 = makeClient<apache::thrift::Client<Second>>(*runner);
+  auto client3 = makeClient<apache::thrift::Client<Third>>(*runner);
+
+  EXPECT_EQ(co_await client2->co_three(), 3);
+  // Second takes precedence
+  EXPECT_EQ(co_await client2->co_four(), 4);
+  // Conflicts takes precedence
+  EXPECT_EQ(co_await client3->co_five(), 555);
+  EXPECT_EQ(co_await client3->co_six(), 6);
+
+  std::vector<ServiceNameInfo> expectedNames = {
+      {"Second", "Second", "three"},
+      {"Second", "Second", "four"},
+      {"Conflicts", "Conflicts", "five"},
+      {"Third", "Third", "six"},
+  };
+
+  EXPECT_THAT(interceptor->names, ElementsAreArray(expectedNames));
+}
+
+CO_TEST_P(
+    ServiceInterceptorTestP,
+    ServiceAndMethodNamesMultiplexInteractionConflicts) {
+  if (transportType() == TransportType::ROCKET) {
+    class Interaction1Handler
+        : public apache::thrift::ServiceHandler<Interaction1> {
+     public:
+      std::unique_ptr<Thing1If> createThing1() override {
+        class Thing1 : public Thing1If {
+         public:
+          void foo() override {}
+        };
+        return std::make_unique<Thing1>();
+      }
+    };
+
+    class ConflictsInteraction1Handler
+        : public apache::thrift::ServiceHandler<
+              apache::thrift::test2::ConflictsInteraction1> {
+     public:
+      std::unique_ptr<Thing1If> createThing1() override {
+        class Thing1 : public Thing1If {
+         public:
+          void foo() override { ADD_FAILURE() << "Should never be called"; }
+          void bar() override { ADD_FAILURE() << "Should never be called"; }
+        };
+        return std::make_unique<Thing1>();
+      }
+    };
+
+    std::vector<std::shared_ptr<AsyncProcessorFactory>> services = {
+        /* order matters when resolving conflicts */
+        std::make_shared<Interaction1Handler>(),
+        std::make_shared<ConflictsInteraction1Handler>(),
+    };
+    auto interceptor =
+        std::make_shared<ServiceInterceptorCheckingServiceAndMethodNames>();
+    auto runner = makeServer(multiplex(services), [&](ThriftServer& server) {
+      server.addModule(std::make_unique<TestModule>(interceptor));
+    });
+
+    auto client1 = makeClient<apache::thrift::Client<Interaction1>>(*runner);
+    auto client2 =
+        makeClient<apache::thrift::Client<ConflictsInteraction1>>(*runner);
+
+    auto thing = client1->createThing1();
+    EXPECT_NO_THROW(co_await thing.co_foo());
+
+    auto thing2 = client2->createThing1();
+
+    // Thing1.bar from ConflictsInteraction1 should not be in MethodMetadataMap
+    // because Interaction1 already added Thing1.foo.
+    EXPECT_THROW(co_await thing2.co_bar(), TApplicationException);
+
+    std::vector<ServiceNameInfo> expectedNames = {
+        {"Interaction1", "Interaction1", "Thing1.foo"},
+    };
+
+    EXPECT_THAT(interceptor->names, ElementsAreArray(expectedNames));
+  }
+}
+
+CO_TEST_P(
+    ServiceInterceptorTestP, ResultOrActiveExceptionTypesAreCorrectBasic) {
+  auto interceptor =
+      std::make_shared<ServiceInterceptorLogResultTypeOnResponse>(
+          "Interceptor1");
+  auto runner =
+      makeServer(std::make_shared<TestHandler>(), [&](ThriftServer& server) {
+        server.addModule(std::make_unique<TestModule>(interceptor));
+      });
+
+  auto client =
+      makeClient<apache::thrift::Client<test::ServiceInterceptorTest>>(*runner);
+  co_await client->co_echo("");
+  co_await client->co_noop();
+
+  EXPECT_THROW(
+      co_await client->co_echo("throw"), apache::thrift::TApplicationException);
+
+  co_await client->co_echo_eb("");
+
+  {
+    test::RequestArgsStruct requestArgs;
+    requestArgs.foo() = 1;
+    requestArgs.bar() = "hello";
+    co_await client->co_echoStruct(requestArgs);
+  }
+
+  using ResultKind = ServiceInterceptorLogResultTypeOnResponse::ResultKind;
+  const std::vector<ServiceInterceptorLogResultTypeOnResponse::Entry>
+      expectedResults = {
+          // echo
+          {ResultKind::OK, typeid(std::string)},
+          // noop
+          {ResultKind::OK, typeid(folly::Unit)},
+          // echo("throw")
+          {ResultKind::EXCEPTION, typeid(std::runtime_error)},
+          // echo_eb
+          {ResultKind::OK, typeid(std::string)},
+          // echoStruct
+          {ResultKind::OK, typeid(test::ResponseArgsStruct)},
+      };
+
+  EXPECT_THAT(interceptor->results, ElementsAreArray(expectedResults));
+}
+
+CO_TEST_P(
+    ServiceInterceptorTestP, ResultOrActiveExceptionTypesAreCorrectAdvanced) {
   if (transportType() != TransportType::ROCKET) {
     // only rocket supports all transport features being tested here
     co_return;
@@ -1189,8 +1538,6 @@ CO_TEST_P(ServiceInterceptorTestP, ResultOrActiveExceptionTypesAreCorrect) {
 
   auto client =
       makeClient<apache::thrift::Client<test::ServiceInterceptorTest>>(*runner);
-  co_await client->co_echo("");
-  co_await client->co_noop();
 
   {
     auto stream = (co_await client->co_iota(1)).toAsyncGenerator();
@@ -1211,25 +1558,9 @@ CO_TEST_P(ServiceInterceptorTestP, ResultOrActiveExceptionTypesAreCorrect) {
     // terminate interaction
   }
 
-  EXPECT_THROW(
-      co_await client->co_echo("throw"), apache::thrift::TApplicationException);
-
-  co_await client->co_echo_eb("");
-
-  {
-    test::RequestArgsStruct requestArgs;
-    requestArgs.foo() = 1;
-    requestArgs.bar() = "hello";
-    co_await client->co_echoStruct(requestArgs);
-  }
-
   using ResultKind = ServiceInterceptorLogResultTypeOnResponse::ResultKind;
-  std::vector<ServiceInterceptorLogResultTypeOnResponse::Entry>
+  const std::vector<ServiceInterceptorLogResultTypeOnResponse::Entry>
       expectedResults = {
-          // echo
-          {ResultKind::OK, typeid(std::string)},
-          // noop
-          {ResultKind::OK, typeid(folly::Unit)},
           // iota
           {ResultKind::OK, typeid(apache::thrift::ServerStream<std::int32_t>)},
           // createInteraction
@@ -1238,12 +1569,6 @@ CO_TEST_P(ServiceInterceptorTestP, ResultOrActiveExceptionTypesAreCorrect) {
           {ResultKind::OK, typeid(std::string)},
           // SampleInteraction2.echo
           {ResultKind::OK, typeid(std::string)},
-          // echo("throw")
-          {ResultKind::EXCEPTION, typeid(std::runtime_error)},
-          // echo_eb
-          {ResultKind::OK, typeid(std::string)},
-          // echoStruct
-          {ResultKind::OK, typeid(test::ResponseArgsStruct)},
       };
 
   EXPECT_THAT(interceptor->results, ElementsAreArray(expectedResults));
