@@ -26,8 +26,8 @@
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
 #include <thrift/lib/cpp2/async/RocketClientChannel.h>
-#include <thrift/lib/cpp2/server/InternalPriorityRequestPile.h>
 #include <thrift/lib/cpp2/server/ParallelConcurrencyController.h>
+#include <thrift/lib/cpp2/server/RoundRobinRequestPile.h>
 #include <thrift/lib/cpp2/server/ThriftServer.h>
 #include <thrift/lib/cpp2/test/gen-cpp2/Calculator.h>
 #include <thrift/lib/cpp2/test/gen-cpp2/Streamer.h>
@@ -103,8 +103,10 @@ struct SemiCalculatorHandler : apache::thrift::ServiceHandler<Calculator> {
 };
 
 TEST(InteractionTest, PrioritizedInteractionRequest) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
-
+  gflags::FlagSaver flag_saver;
+  // Make sure resourcepool is enabled, it's somehow will NOT be enabled in
+  // test.
+  FLAGS_thrift_experimental_use_resource_pools = true;
   struct BlockingCalculatorHandler : public SemiCalculatorHandler {
     folly::Baton<> blockInteraction, blockNormal, startedInteraction;
 
@@ -139,12 +141,11 @@ TEST(InteractionTest, PrioritizedInteractionRequest) {
   // Wait for the interaction requests to reach the server
   handler->startedInteraction.wait();
 
-  auto normal1 = client->semifuture_addPrimitive(1, 2);
-  auto normal2 = client->semifuture_addPrimitive(3, 4);
+  auto normal = client->semifuture_addPrimitive(1, 2);
 
-  // Wait for the 2 requests to reach the server
+  // Wait for the normal requests to reach the server
   auto start = std::chrono::steady_clock::now();
-  while (runner.getThriftServer().resourcePoolSet().numQueued() != 2) {
+  while (runner.getThriftServer().resourcePoolSet().numQueued() != 1) {
     /* sleep override */
     std::this_thread::sleep_for(std::chrono::milliseconds(10));
     if (std::chrono::steady_clock::now() - start > std::chrono::seconds(5)) {
@@ -157,21 +158,15 @@ TEST(InteractionTest, PrioritizedInteractionRequest) {
   handler->blockInteraction.post();
   EXPECT_EQ(std::move(sf1).get(), 42);
 
-  // Unblock the normal request and wait for it to finish
-  handler->blockNormal.post();
-  EXPECT_EQ(std::move(normal1).get(), 3);
-
-  // The continuation to the interaction should have been finished now even
-  // though it was enqueued after the second normal request
+  // Make sure interaction request is executed before normal request.
   EXPECT_EQ(std::move(sf2).get(), 42);
 
-  // The second normal request will be dequeued last
+  // Unblock the normal request and wait for it to finish
   handler->blockNormal.post();
-  EXPECT_EQ(std::move(normal2).get(), 7);
+  EXPECT_EQ(std::move(normal).get(), 3);
 }
 
 TEST(InteractionTest, TerminateUsed) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   folly::EventBase eb;
   CalculatorAsyncClient client(
@@ -183,7 +178,6 @@ TEST(InteractionTest, TerminateUsed) {
 }
 
 TEST(InteractionTest, TerminateActive) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   folly::EventBase eb;
   CalculatorAsyncClient client(
@@ -195,7 +189,6 @@ TEST(InteractionTest, TerminateActive) {
 }
 
 TEST(InteractionTest, TerminateUnused) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   folly::EventBase eb;
   CalculatorAsyncClient client(
@@ -207,7 +200,6 @@ TEST(InteractionTest, TerminateUnused) {
 }
 
 TEST(InteractionTest, TerminateWithoutSetup) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   folly::EventBase eb;
   CalculatorAsyncClient client(
@@ -218,7 +210,6 @@ TEST(InteractionTest, TerminateWithoutSetup) {
 }
 
 TEST(InteractionTest, TerminateUsedPRC) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   auto client = runner.newClient<CalculatorAsyncClient>(
       nullptr, RocketClientChannel::newChannel);
@@ -228,7 +219,6 @@ TEST(InteractionTest, TerminateUsedPRC) {
 }
 
 TEST(InteractionTest, TerminateUnusedPRC) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   auto client = runner.newClient<CalculatorAsyncClient>(
       nullptr, RocketClientChannel::newChannel);
@@ -238,7 +228,6 @@ TEST(InteractionTest, TerminateUnusedPRC) {
 }
 
 TEST(InteractionTest, TerminateWithoutSetupPRC) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   auto client = runner.newClient<CalculatorAsyncClient>(
       nullptr, RocketClientChannel::newChannel);
@@ -247,7 +236,6 @@ TEST(InteractionTest, TerminateWithoutSetupPRC) {
 }
 
 TEST(InteractionTest, TerminateStressPRC) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   auto client = runner.newClient<CalculatorAsyncClient>(
       nullptr, RocketClientChannel::newChannel);
@@ -259,7 +247,6 @@ TEST(InteractionTest, TerminateStressPRC) {
 }
 
 TEST(InteractionTest, IsDetachable) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   folly::EventBase eb;
   CalculatorAsyncClient client(
@@ -285,7 +272,6 @@ TEST(InteractionTest, IsDetachable) {
 }
 
 TEST(InteractionTest, QueueTimeout) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct SlowCalculatorHandler : apache::thrift::ServiceHandler<Calculator> {
     struct SemiAdditionHandler
         : apache::thrift::ServiceHandler<Calculator>::AdditionIf {
@@ -318,7 +304,6 @@ TEST(InteractionTest, QueueTimeout) {
 }
 
 TEST(InteractionTest, OnTermination) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct TerminationHandler : apache::thrift::ServiceHandler<Calculator> {
     folly::coro::Baton create;
     folly::coro::Baton terminate;
@@ -524,7 +509,6 @@ struct CalculatorHandler : apache::thrift::ServiceHandler<Calculator> {
 };
 
 TEST(InteractionCodegenTest, Basic) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<CalculatorHandler>()};
   auto client = runner.newClient<CalculatorAsyncClient>(
       nullptr, RocketClientChannel::newChannel);
@@ -554,7 +538,6 @@ TEST(InteractionCodegenTest, Basic) {
 }
 
 TEST(InteractionCodegenTest, RpcOptions) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<CalculatorHandler>()};
   auto client = runner.newClient<CalculatorAsyncClient>(
       nullptr, RocketClientChannel::newChannel);
@@ -585,7 +568,6 @@ TEST(InteractionCodegenTest, RpcOptions) {
 }
 
 TEST(InteractionCodegenTest, BasicSemiFuture) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   auto client = runner.newClient<CalculatorAsyncClient>(
       nullptr, RocketClientChannel::newChannel);
@@ -611,7 +593,6 @@ TEST(InteractionCodegenTest, BasicSemiFuture) {
 }
 
 TEST(InteractionCodegenTest, BasicSync) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   auto client = runner.newClient<CalculatorAsyncClient>(
       nullptr, RocketClientChannel::newChannel);
@@ -638,7 +619,6 @@ TEST(InteractionCodegenTest, BasicSync) {
 }
 
 TEST(InteractionCodegenTest, Error) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct BrokenCalculatorHandler : CalculatorHandler {
     std::unique_ptr<AdditionIf> createAddition() override {
       throw std::runtime_error("Plus key is broken");
@@ -669,7 +649,6 @@ TEST(InteractionCodegenTest, Error) {
 }
 
 TEST(InteractionCodegenTest, MethodException) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct ExceptionCalculatorHandler
       : apache::thrift::ServiceHandler<Calculator> {
     struct AdditionHandler
@@ -706,7 +685,6 @@ TEST(InteractionCodegenTest, MethodException) {
 }
 
 TEST(InteractionCodegenTest, SlowConstructor) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct SlowCalculatorHandler : CalculatorHandler {
     std::unique_ptr<AdditionIf> createAddition() override {
       b.wait();
@@ -733,7 +711,6 @@ TEST(InteractionCodegenTest, SlowConstructor) {
 }
 
 TEST(InteractionCodegenTest, FastTermination) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
 #if FOLLY_HAS_COROUTINES
   struct SlowCalculatorHandler : CalculatorHandler {
     struct SlowAdditionHandler : AdditionHandler {
@@ -778,7 +755,6 @@ TEST(InteractionCodegenTest, FastTermination) {
 }
 
 TEST(InteractionCodegenTest, ClientCrashDuringInteraction) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
 #if FOLLY_HAS_COROUTINES
   struct SlowCalculatorHandler : CalculatorHandler {
     struct SlowAdditionHandler : AdditionHandler {
@@ -818,7 +794,6 @@ TEST(InteractionCodegenTest, ClientCrashDuringInteraction) {
 }
 
 TEST(InteractionCodegenTest, ClientCrashDuringInteractionConstructor) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
 #if FOLLY_HAS_COROUTINES
   struct SlowCalculatorHandler : CalculatorHandler {
     struct SlowAdditionHandler : AdditionHandler {
@@ -865,7 +840,6 @@ TEST(InteractionCodegenTest, ClientCrashDuringInteractionConstructor) {
 }
 
 TEST(InteractionCodegenTest, ReuseIdDuringConstructor) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct SlowCalculatorHandler : CalculatorHandler {
     std::unique_ptr<AdditionIf> createAddition() override {
       if (first) {
@@ -909,7 +883,6 @@ TEST(InteractionCodegenTest, ReuseIdDuringConstructor) {
 }
 
 TEST(InteractionCodegenTest, ConstructorExceptionPropagated) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct SlowCalculatorHandler : CalculatorHandler {
     std::unique_ptr<AdditionIf> createAddition() override {
       b.wait();
@@ -940,7 +913,6 @@ TEST(InteractionCodegenTest, ConstructorExceptionPropagated) {
 }
 
 TEST(InteractionCodegenTest, SerialInteraction) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
 #if FOLLY_HAS_COROUTINES
   struct SerialCalculatorHandler : CalculatorHandler {
     struct SerialAdditionHandler
@@ -1024,7 +996,6 @@ TEST(InteractionCodegenTest, SerialInteraction) {
 }
 
 TEST(InteractionCodegenTest, StreamExtendsInteractionLifetime) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
 #if FOLLY_HAS_COROUTINES
   struct StreamingHandler : apache::thrift::ServiceHandler<Streamer> {
     StreamingHandler()
@@ -1170,7 +1141,6 @@ TEST(InteractionCodegenTest, StreamExtendsInteractionLifetime) {
 }
 
 TEST(InteractionCodegenTest, ShutdownDuringStreamTeardown) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
 #if FOLLY_HAS_COROUTINES
   struct StreamingHandler : apache::thrift::ServiceHandler<Streamer> {
     struct StreamTile : apache::thrift::ServiceHandler<Streamer>::StreamingIf {
@@ -1201,7 +1171,6 @@ TEST(InteractionCodegenTest, ShutdownDuringStreamTeardown) {
 }
 
 TEST(InteractionCodegenTest, BasicEB) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct ExceptionCalculatorHandler
       : apache::thrift::ServiceHandler<Calculator> {
     struct AdditionHandler
@@ -1238,7 +1207,6 @@ TEST(InteractionCodegenTest, BasicEB) {
 }
 
 TEST(InteractionCodegenTest, ErrorEB) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct ExceptionCalculatorHandler
       : apache::thrift::ServiceHandler<Calculator> {
     std::unique_ptr<AdditionFastIf> createAdditionFast() override {
@@ -1263,7 +1231,6 @@ TEST(InteractionCodegenTest, ErrorEB) {
 }
 
 CO_TEST(InteractionCodegenTest, Factory) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   auto client = makeTestClient<CalculatorAsyncClient>(
       std::make_shared<SemiCalculatorHandler>());
 
@@ -1290,7 +1257,6 @@ CO_TEST(InteractionCodegenTest, Factory) {
 }
 
 CO_TEST(InteractionCodegenTest, FactoryError) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   auto client = makeTestClient<CalculatorAsyncClient>(
       std::make_shared<SemiCalculatorHandler>());
 
@@ -1302,7 +1268,6 @@ CO_TEST(InteractionCodegenTest, FactoryError) {
 }
 
 CO_TEST(InteractionCodegenTest, FactoryEager) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   auto client = makeTestClient<CalculatorAsyncClient>(
       std::make_shared<SemiCalculatorHandler>());
 
@@ -1319,7 +1284,6 @@ CO_TEST(InteractionCodegenTest, FactoryEager) {
 }
 
 TEST(InteractionCodegenTest, FactoryEb) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   auto client = makeTestClient<CalculatorAsyncClient>(
       std::make_shared<SemiCalculatorHandler>());
 
@@ -1361,7 +1325,6 @@ TEST(InteractionCodegenTest, FactoryEb) {
 }
 
 TEST(InteractionCodegenTest, FactoryHandlerCallback) {
-  THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
   struct HandlerResult : apache::thrift::ServiceHandler<Calculator> {
     void async_tm_newAddition(
         apache::thrift::HandlerCallbackPtr<TileAndResponse<AdditionIf, void>>
@@ -1529,51 +1492,17 @@ TEST(InteractionCodegenTest, FactoryHandlerCallback) {
   }
 }
 
-TEST(InteractionResourcePoolsThriftFlags, GeneratedAsyncProcessorFactory) {
+TEST(InteractionResourcePools, GeneratedAsyncProcessorFactory) {
   if (!FLAGS_thrift_experimental_use_resource_pools) {
     // This test is only relevant when resource pools is requested.
     return;
   }
   auto handler = std::make_shared<SemiCalculatorHandler>();
-
-  // both flags true, expect resource pool enabled
-  {
-    THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
-    THRIFT_FLAG_SET_MOCK(
-        enable_resource_pools_for_interaction_generated_processor_only, true);
-    ScopedServerInterfaceThread server(handler);
-    EXPECT_TRUE(server.getThriftServer().resourcePoolEnabled());
-  }
-
-  // only generated processor flag true, expect resource pool enabled
-  {
-    THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, false);
-    THRIFT_FLAG_SET_MOCK(
-        enable_resource_pools_for_interaction_generated_processor_only, true);
-    ScopedServerInterfaceThread server(handler);
-    EXPECT_TRUE(server.getThriftServer().resourcePoolEnabled());
-  }
-
-  // only generated processor flag false, expect resource pool enabled
-  {
-    THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
-    THRIFT_FLAG_SET_MOCK(
-        enable_resource_pools_for_interaction_generated_processor_only, false);
-    ScopedServerInterfaceThread server(handler);
-    EXPECT_TRUE(server.getThriftServer().resourcePoolEnabled());
-  }
-
-  // both flags false, expect resource pool disabled
-  {
-    THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, false);
-    THRIFT_FLAG_SET_MOCK(
-        enable_resource_pools_for_interaction_generated_processor_only, false);
-    ScopedServerInterfaceThread server(handler);
-    EXPECT_FALSE(server.getThriftServer().resourcePoolEnabled());
-  }
+  ScopedServerInterfaceThread server(handler);
+  EXPECT_TRUE(server.getThriftServer().resourcePoolEnabled());
 }
 
-TEST(InteractionResourcePoolsThriftFlags, CustomAsyncProcessorFactory) {
+TEST(InteractionResourcePools, CustomAsyncProcessorFactory) {
   if (!FLAGS_thrift_experimental_use_resource_pools) {
     // This test is only relevant when resource pools is requested.
     return;
@@ -1592,50 +1521,16 @@ TEST(InteractionResourcePoolsThriftFlags, CustomAsyncProcessorFactory) {
     SemiCalculatorHandler handler_;
   };
   auto handler = std::make_shared<CustomAsyncProcessorFactory>();
-
-  // both flags true, expect resource pool enabled
-  {
-    THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
-    THRIFT_FLAG_SET_MOCK(
-        enable_resource_pools_for_interaction_generated_processor_only, true);
-    ScopedServerInterfaceThread server(handler);
-    EXPECT_TRUE(server.getThriftServer().resourcePoolEnabled());
-  }
-
-  // only generated processor flag true, expect resource pool disabled
-  {
-    THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, false);
-    THRIFT_FLAG_SET_MOCK(
-        enable_resource_pools_for_interaction_generated_processor_only, true);
-    ScopedServerInterfaceThread server(handler);
-    EXPECT_FALSE(server.getThriftServer().resourcePoolEnabled());
-  }
-
-  // only generated processor flag false, expect resource pool enabled
-  {
-    THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
-    THRIFT_FLAG_SET_MOCK(
-        enable_resource_pools_for_interaction_generated_processor_only, false);
-    ScopedServerInterfaceThread server(handler);
-    EXPECT_TRUE(server.getThriftServer().resourcePoolEnabled());
-  }
-
-  // both flags false, expect resource pool disabled
-  {
-    THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, false);
-    THRIFT_FLAG_SET_MOCK(
-        enable_resource_pools_for_interaction_generated_processor_only, false);
-    ScopedServerInterfaceThread server(handler);
-    EXPECT_FALSE(server.getThriftServer().resourcePoolEnabled());
-  }
+  ScopedServerInterfaceThread server(handler);
+  EXPECT_TRUE(server.getThriftServer().resourcePoolEnabled());
 }
 
 class InternalPriorityTest : public testing::Test {
  public:
-  class TestRequestPile : public InternalPriorityRequestPile {
+  class TestRequestPile : public RoundRobinRequestPile {
    public:
     TestRequestPile()
-        : InternalPriorityRequestPile(InternalPriorityRequestPile::Options()) {}
+        : RoundRobinRequestPile(RoundRobinRequestPile::Options()) {}
 
     ~TestRequestPile() override {
       EXPECT_TRUE(expectedInternalPriorities_.empty());
@@ -1648,7 +1543,7 @@ class InternalPriorityTest : public testing::Test {
           detail::ServerRequestHelper::internalPriority(request);
       EXPECT_EQ(expectedInternalPriority, internalPriority);
       expectedInternalPriorities_.pop();
-      return InternalPriorityRequestPile::enqueue(std::move(request));
+      return RoundRobinRequestPile::enqueue(std::move(request));
     }
 
    private:
@@ -1662,9 +1557,6 @@ class InternalPriorityTest : public testing::Test {
     if (!FLAGS_thrift_experimental_use_resource_pools) {
       GTEST_SKIP() << "This test is only relevant when resource pools is used";
     }
-    THRIFT_FLAG_SET_MOCK(enable_resource_pools_for_interaction, true);
-    THRIFT_FLAG_SET_MOCK(
-        enable_resource_pools_for_interaction_generated_processor_only, true);
     auto pile = std::make_unique<TestRequestPile>();
     auto cc = std::make_unique<ParallelConcurrencyController>(
         *pile.get(), *folly::getGlobalCPUExecutor().get());
