@@ -17,14 +17,12 @@
 #include <thrift/lib/cpp2/transport/rocket/client/RocketClient.h>
 
 #include <chrono>
-#include <functional>
 #include <limits>
 #include <string>
 #include <utility>
 
 #include <fmt/core.h>
 #include <folly/Conv.h>
-#include <folly/CppAttributes.h>
 #include <folly/ExceptionWrapper.h>
 #include <folly/GLog.h>
 #include <folly/Likely.h>
@@ -44,13 +42,11 @@
 #include <thrift/lib/cpp2/transport/rocket/RocketException.h>
 #include <thrift/lib/cpp2/transport/rocket/Types.h>
 #include <thrift/lib/cpp2/transport/rocket/client/RequestContext.h>
+#include <thrift/lib/cpp2/transport/rocket/compression/CustomCompressorRegistry.h>
 #include <thrift/lib/cpp2/transport/rocket/framing/Frames.h>
 #include <thrift/lib/cpp2/transport/rocket/framing/Util.h>
-#include <thrift/lib/cpp2/transport/rocket/payload/PayloadSerializer.h>
 #include <thrift/lib/thrift/gen-cpp2/RpcMetadata_types.h>
 
-THRIFT_FLAG_DEFINE_int64(rocket_server_version_timeout_ms, 500);
-THRIFT_FLAG_DEFINE_bool(rocket_client_enable_keep_alive, true);
 THRIFT_FLAG_DEFINE_bool(rocket_client_binary_rpc_metadata_encoding, false);
 
 namespace apache::thrift {
@@ -111,7 +107,7 @@ RocketClient::RocketClient(
     socket_2->setCloseOnFailedWrite(false);
   }
 
-  if (THRIFT_FLAG(rocket_client_enable_keep_alive) && keepAliveTimeoutMs > 0) {
+  if (keepAliveTimeoutMs > 0) {
     keepAliveWatcher_ = std::unique_ptr<
         KeepAliveWatcher,
         folly::DelayedDestruction::Destructor>(new KeepAliveWatcher(
@@ -179,8 +175,8 @@ void RocketClient::handleFrame(std::unique_ptr<folly::IOBuf> frame) {
     }
     ServerPushMetadata serverMeta;
     try {
-      PayloadSerializer::getInstance().unpack(
-          serverMeta, std::move(mdPushFrame.metadata()), false);
+      getPayloadSerializer()->unpack(
+          serverMeta, std::move(mdPushFrame.metadata()), false /* useBinary */);
     } catch (...) {
       close(transport::TTransportException(
           transport::TTransportException::CORRUPTED_DATA,
@@ -195,6 +191,16 @@ void RocketClient::handleFrame(std::unique_ptr<folly::IOBuf> frame) {
             (int32_t)std::numeric_limits<int16_t>::max()));
         serverZstdSupported_ =
             serverMeta.setupResponse_ref()->zstdSupported_ref().value_or(false);
+
+        if (auto ref =
+                serverMeta.setupResponse_ref()->compressionSetupResponse()) {
+          auto customCompressionRes =
+              handleSetupResponseCustomCompression(*ref);
+          if (customCompressionRes.hasError()) {
+            close(customCompressionRes.error());
+            return;
+          }
+        }
         break;
       }
       case ServerPushMetadata::Type::streamHeadersPush: {
@@ -232,8 +238,7 @@ void RocketClient::handleFrame(std::unique_ptr<folly::IOBuf> frame) {
 
           close(RocketException(
               ErrorCode::REJECTED,
-              rocket::PayloadSerializer::getInstance().packCompact(
-                  responseRpcError)));
+              getPayloadSerializer()->packCompact(responseRpcError)));
         }
         return;
       }
@@ -251,6 +256,59 @@ void RocketClient::handleFrame(std::unique_ptr<folly::IOBuf> frame) {
   handleStreamChannelFrame(streamId, frameType, std::move(frame));
 }
 
+folly::Expected<folly::Unit, transport::TTransportException>
+RocketClient::handleSetupResponseCustomCompression(
+    CompressionSetupResponse const& setupResponse) {
+  if (!setupResponse.custom_ref()) {
+    return folly::makeUnexpected(transport::TTransportException(
+        transport::TTransportException::NOT_SUPPORTED,
+        "Only 'custom' compressor setup response is supported on client."));
+  }
+  const auto& customSetupResponse = setupResponse.custom_ref().value();
+
+  auto factory =
+      CustomCompressorRegistry::get(*customSetupResponse.compressorName());
+  if (!factory) {
+    return folly::makeUnexpected(transport::TTransportException(
+        transport::TTransportException::NOT_SUPPORTED,
+        fmt::format(
+            "Custom compressor {} is not supported on client.",
+            *customSetupResponse.compressorName())));
+  }
+
+  std::shared_ptr<CustomCompressor> compressor;
+  try {
+    compressor = factory->make(
+        customSetupResponse,
+        CustomCompressorFactory::CompressorLocation::CLIENT);
+  } catch (const std::exception& ex) {
+    return folly::makeUnexpected(transport::TTransportException(
+        transport::TTransportException::INVALID_SETUP,
+        fmt::format(
+            "Failed to make custom compressor on client due to: {}",
+            ex.what())));
+  }
+
+  if (!compressor) {
+    return folly::makeUnexpected(transport::TTransportException(
+        transport::TTransportException::INVALID_SETUP,
+        "Failed to make custom compressor on client."));
+  }
+
+  if (!payloadSerializerHolder_) {
+    payloadSerializerHolder_.emplace();
+  }
+
+  CustomCompressionPayloadSerializerStrategyOptions options;
+  options.compressor = compressor;
+  CustomCompressionPayloadSerializerStrategy<DefaultPayloadSerializerStrategy>
+      strategy{options};
+  payloadSerializerHolder_->initialize(std::move(strategy));
+  customCompressor_ = compressor;
+
+  return folly::unit;
+}
+
 void RocketClient::handleRequestResponseFrame(
     RequestContext& ctx,
     FrameType frameType,
@@ -386,9 +444,8 @@ StreamChannelStatusResponse RocketClient::handleFirstResponse(
     serverCallback.onInitialError(makeContractViolation(kErrorMsg));
     return {StreamChannelStatus::ContractViolation, kErrorMsg};
   }
-  auto firstResponse =
-      rocket::PayloadSerializer::getInstance().unpack<FirstResponsePayload>(
-          std::move(fullPayload), false /* decodeMetadataUsingBinary */);
+  auto firstResponse = getPayloadSerializer()->unpack<FirstResponsePayload>(
+      std::move(fullPayload), encodeMetadataUsingBinary_);
   if (firstResponse.hasException()) {
     serverCallback.onInitialError(std::move(firstResponse.exception()));
     return StreamChannelStatus::Complete;
@@ -442,9 +499,8 @@ StreamChannelStatusResponse RocketClient::handleStreamResponse(
     bool next,
     bool complete) {
   if (next) {
-    auto streamPayload =
-        rocket::PayloadSerializer::getInstance().unpack<StreamPayload>(
-            std::move(fullPayload), false /* decodeMetadataUsingBinary */);
+    auto streamPayload = getPayloadSerializer()->unpack<StreamPayload>(
+        std::move(fullPayload), encodeMetadataUsingBinary_);
     if (streamPayload.hasException()) {
       return serverCallback.onStreamError(std::move(streamPayload.exception()));
     }
@@ -511,9 +567,8 @@ StreamChannelStatusResponse RocketClient::handleSinkResponse(
     return {StreamChannelStatus::ContractViolation, std::move(msg)};
   };
   if (next) {
-    auto streamPayload =
-        rocket::PayloadSerializer::getInstance().unpack<StreamPayload>(
-            std::move(fullPayload), false /* decodeMetadataUsingBinary */);
+    auto streamPayload = getPayloadSerializer()->unpack<StreamPayload>(
+        std::move(fullPayload), encodeMetadataUsingBinary_);
     if (streamPayload.hasException()) {
       return serverCallback.onFinalResponseError(
           std::move(streamPayload.exception()));
@@ -1076,7 +1131,7 @@ bool RocketClient::sendPayload(
   return sendFrame(
       PayloadFrame(
           streamId,
-          rocket::PayloadSerializer::getInstance().pack(
+          getPayloadSerializer()->pack(
               std::move(payload),
               encodeMetadataUsingBinary(),
               getTransportWrapper()),
@@ -1129,8 +1184,7 @@ bool RocketClient::sendHeadersPush(
       std::move(payload.payload);
   return sendFrame(
       MetadataPushFrame::makeFromMetadata(
-          rocket::PayloadSerializer::getInstance().packCompact(
-              std::move(clientMeta))),
+          getPayloadSerializer()->packCompact(clientMeta)),
       std::move(onError));
 }
 
@@ -1517,8 +1571,7 @@ void RocketClient::terminateInteraction(int64_t id) {
   clientMeta.interactionTerminate_ref().ensure().interactionId_ref() = id;
   std::ignore = sendFrame(
       MetadataPushFrame::makeFromMetadata(
-          rocket::PayloadSerializer::getInstance().packCompact(
-              std::move(clientMeta))),
+          getPayloadSerializer()->packCompact(clientMeta)),
       std::move(onError));
 }
 
@@ -1533,8 +1586,7 @@ void RocketClient::onServerVersionRequired() {
   // include a timeout to close the connection if server does not respond
   serverVersionTimeout_.reset(new ServerVersionTimeout(*this));
   evb_->timer().scheduleTimeout(
-      serverVersionTimeout_.get(),
-      std::chrono::milliseconds(THRIFT_FLAG(rocket_server_version_timeout_ms)));
+      serverVersionTimeout_.get(), std::chrono::milliseconds(500));
 }
 
 void RocketClient::setServerVersion(int32_t serverVersion) {
@@ -1560,8 +1612,7 @@ void RocketClient::sendTransportMetadataPush() {
     clientMeta.transportMetadataPush_ref() = std::move(*transportMetadataPush);
     std::ignore = sendFrame(
         MetadataPushFrame::makeFromMetadata(
-            rocket::PayloadSerializer::getInstance().packCompact(
-                std::move(clientMeta))),
+            getPayloadSerializer()->packCompact(clientMeta)),
         std::move(onError));
   }
 }
