@@ -32,6 +32,7 @@
 #include <thrift/lib/cpp2/server/LoggingEvent.h>
 #include <thrift/lib/cpp2/transport/core/RpcMetadataUtil.h>
 #include <thrift/lib/cpp2/transport/core/SendCallbacks.h>
+#include <thrift/lib/cpp2/transport/rocket/compression/CompressionManager.h>
 #include <thrift/lib/cpp2/transport/rocket/framing/Flags.h>
 #include <thrift/lib/cpp2/transport/rocket/payload/PayloadSerializer.h>
 #include <thrift/lib/cpp2/transport/rocket/server/RocketServerConnection.h>
@@ -69,6 +70,10 @@ ResponseRpcError makeResponseRpcError(
       case ResponseRpcErrorCode::QUEUE_OVERLOADED:
       case ResponseRpcErrorCode::QUEUE_TIMEOUT:
       case ResponseRpcErrorCode::APP_OVERLOAD:
+      case ResponseRpcErrorCode::INTERACTION_LOADSHEDDED:
+      case ResponseRpcErrorCode::INTERACTION_LOADSHEDDED_OVERLOAD:
+      case ResponseRpcErrorCode::INTERACTION_LOADSHEDDED_APP_OVERLOAD:
+      case ResponseRpcErrorCode::INTERACTION_LOADSHEDDED_QUEUE_TIMEOUT:
         return ResponseRpcErrorCategory::LOADSHEDDING;
       case ResponseRpcErrorCode::SHUTDOWN:
         return ResponseRpcErrorCategory::SHUTDOWN;
@@ -85,7 +90,9 @@ ResponseRpcError makeResponseRpcError(
   return responseRpcError;
 }
 
-RocketException makeRocketException(const ResponseRpcError& responseRpcError) {
+RocketException makeRocketException(
+    const ResponseRpcError& responseRpcError,
+    PayloadSerializer& payloadSerializer) {
   auto rocketCategory = [&] {
     auto category = ResponseRpcErrorCategory::INTERNAL_ERROR;
     if (auto errorCategory = responseRpcError.category()) {
@@ -103,8 +110,7 @@ RocketException makeRocketException(const ResponseRpcError& responseRpcError) {
   }();
 
   return RocketException(
-      rocketCategory,
-      PayloadSerializer::getInstance().packCompact(responseRpcError));
+      rocketCategory, payloadSerializer.packCompact(responseRpcError));
 }
 
 template <typename Serializer>
@@ -291,7 +297,17 @@ FOLLY_NODISCARD std::optional<ResponseRpcError> processFirstResponseHelper(
                        {kTenantQuotaExceededErrorCode,
                         ResponseRpcErrorCode::TENANT_QUOTA_EXCEEDED},
                        {kTenantBlocklistedErrorCode,
-                        ResponseRpcErrorCode::TENANT_BLOCKLISTED}});
+                        ResponseRpcErrorCode::TENANT_BLOCKLISTED},
+                       {kInteractionLoadsheddedErrorCode,
+                        ResponseRpcErrorCode::INTERACTION_LOADSHEDDED},
+                       {kInteractionLoadsheddedOverloadErrorCode,
+                        ResponseRpcErrorCode::INTERACTION_LOADSHEDDED_OVERLOAD},
+                       {kInteractionLoadsheddedAppOverloadErrorCode,
+                        ResponseRpcErrorCode::
+                            INTERACTION_LOADSHEDDED_APP_OVERLOAD},
+                       {kInteractionLoadsheddedQueueTimeoutErrorCode,
+                        ResponseRpcErrorCode::
+                            INTERACTION_LOADSHEDDED_QUEUE_TIMEOUT}});
                   if (auto errorCode = folly::get_ptr(errorCodeMap, *exPtr)) {
                     return *errorCode;
                   }
@@ -375,7 +391,7 @@ FOLLY_NODISCARD std::optional<ResponseRpcError> processFirstResponse(
 
   // apply compression if client has specified compression codec
   if (compressionConfig.has_value()) {
-    rocket::detail::setCompressionCodec(
+    CompressionManager().setCompressionCodec(
         *compressionConfig, metadata, payload->computeChainDataLength());
   }
 
@@ -476,17 +492,49 @@ void ThriftServerRequestResponse::sendThriftResponse(
   // createRequestLoggingCallback must happen after processFirstResponse.
   cb = createRequestLoggingCallback(std::move(cb), metadata, responseRpcError);
 
+  auto payloadSerializerPtr = context_.connection().getPayloadSerializer();
   if (responseRpcError) {
-    auto ex = makeRocketException(*responseRpcError);
+    auto ex = makeRocketException(*responseRpcError, *payloadSerializerPtr);
     context_.sendError(std::move(ex), std::move(cb));
     return;
   }
-  auto payload = PayloadSerializer::getInstance().packWithFds(
-      &metadata,
-      std::move(data),
-      std::move(getRequestContext()->getHeader()->fds),
-      context_.connection().isDecodingMetadataUsingBinaryProtocol(),
-      context_.connection().getRawSocket());
+
+  // Prevents potential complexity (e.g. using custom compression to
+  // encode custom compression failures, or to that matter, any failure at all),
+  // we use ZSTD to compress exception payloads.
+  // custom compression implies ZSTD, so it is safe to fallback to it.
+  if (metadata.compression().value_or(CompressionAlgorithm::NONE) ==
+      CompressionAlgorithm::CUSTOM) {
+    auto payloadIsException = false;
+    if (!metadata.payloadMetadata()) {
+      // unexpected, handle it conservatively
+      payloadIsException = true;
+    } else if (metadata.payloadMetadata()->exceptionMetadata_ref()) {
+      payloadIsException = true;
+    }
+
+    if (payloadIsException) {
+      metadata.compression() = CompressionAlgorithm::ZSTD;
+    }
+  }
+
+  rocket::Payload payload;
+  try {
+    payload = payloadSerializerPtr->packWithFds(
+        &metadata,
+        std::move(data),
+        std::move(getRequestContext()->getHeader()->fds),
+        context_.connection().isDecodingMetadataUsingBinaryProtocol(),
+        context_.connection().getRawSocket());
+  } catch (std::exception const& ex) {
+    auto error = makeResponseRpcError(
+        ResponseRpcErrorCode::UNKNOWN,
+        fmt::format("Failed to pack or send payload due to: {}", ex.what()),
+        metadata);
+    auto rex = makeRocketException(error, *payloadSerializerPtr);
+    context_.sendError(std::move(rex), std::move(cb));
+    return;
+  }
 
   if (maxResponseWriteTime_ > std::chrono::milliseconds{0}) {
     cb = apache::thrift::MessageChannel::SendCallbackPtr(
@@ -635,7 +683,8 @@ bool ThriftServerRequestStream::sendStreamThriftResponse(
   }
   if (auto responseRpcError = processFirstResponse(
           metadata, data, getProtoId(), version_, getCompressionConfig())) {
-    auto ex = makeRocketException(*responseRpcError);
+    auto ex = makeRocketException(
+        *responseRpcError, *context_.connection().getPayloadSerializer());
     handleStreamError(std::move(ex), clientCallback_);
     return false;
   }
@@ -659,7 +708,8 @@ void ThriftServerRequestStream::sendStreamThriftResponse(
   }
   if (auto responseRpcError = processFirstResponse(
           metadata, data, getProtoId(), version_, getCompressionConfig())) {
-    auto ex = makeRocketException(*responseRpcError);
+    auto ex = makeRocketException(
+        *responseRpcError, *context_.connection().getPayloadSerializer());
     handleStreamError(std::move(ex), clientCallback_);
     return;
   }
@@ -677,7 +727,8 @@ void ThriftServerRequestStream::sendSerializedError(
     std::unique_ptr<folly::IOBuf> exbuf) noexcept {
   if (auto responseRpcError = processFirstResponse(
           metadata, exbuf, getProtoId(), version_, getCompressionConfig())) {
-    auto ex = makeRocketException(*responseRpcError);
+    auto ex = makeRocketException(
+        *responseRpcError, *context_.connection().getPayloadSerializer());
     handleStreamError(std::move(ex), clientCallback_);
     return;
   }
@@ -748,7 +799,8 @@ void ThriftServerRequestSink::sendSerializedError(
     std::unique_ptr<folly::IOBuf> exbuf) noexcept {
   if (auto responseRpcError = processFirstResponse(
           metadata, exbuf, getProtoId(), version_, getCompressionConfig())) {
-    auto ex = makeRocketException(*responseRpcError);
+    auto ex = makeRocketException(
+        *responseRpcError, *context_.connection().getPayloadSerializer());
     handleStreamError(std::move(ex), clientCallback_);
     return;
   }
@@ -769,7 +821,8 @@ void ThriftServerRequestSink::sendSinkThriftResponse(
   }
   if (auto responseRpcError = processFirstResponse(
           metadata, data, getProtoId(), version_, getCompressionConfig())) {
-    auto ex = makeRocketException(*responseRpcError);
+    auto ex = makeRocketException(
+        *responseRpcError, *context_.connection().getPayloadSerializer());
     handleStreamError(std::move(ex), clientCallback_);
     return;
   }
@@ -802,7 +855,8 @@ bool ThriftServerRequestSink::sendSinkThriftResponse(
   }
   if (auto responseRpcError = processFirstResponse(
           metadata, data, getProtoId(), version_, getCompressionConfig())) {
-    auto ex = makeRocketException(*responseRpcError);
+    auto ex = makeRocketException(
+        *responseRpcError, *context_.connection().getPayloadSerializer());
     handleStreamError(std::move(ex), clientCallback_);
     return false;
   }
