@@ -41,8 +41,10 @@
 #include <thrift/lib/cpp2/transport/rocket/FdSocket.h>
 #include <thrift/lib/cpp2/transport/rocket/PayloadUtils.h>
 #include <thrift/lib/cpp2/transport/rocket/RocketException.h>
+#include <thrift/lib/cpp2/transport/rocket/compression/CustomCompressorRegistry.h>
 #include <thrift/lib/cpp2/transport/rocket/framing/ErrorCode.h>
 #include <thrift/lib/cpp2/transport/rocket/framing/Frames.h>
+#include <thrift/lib/cpp2/transport/rocket/server/InteractionOverload.h>
 #include <thrift/lib/cpp2/transport/rocket/server/RocketServerConnection.h>
 #include <thrift/lib/cpp2/transport/rocket/server/RocketServerFrameContext.h>
 #include <thrift/lib/cpp2/transport/rocket/server/RocketSinkClientCallback.h>
@@ -177,7 +179,7 @@ void ThriftRocketServerHandler::handleSetupFrame(
 
   RequestSetupMetadata meta;
   try {
-    if (PayloadSerializer::getInstance().unpack(
+    if (PayloadSerializer::getInstance()->unpack(
             meta, cursor, frame.encodeMetadataUsingBinary()) !=
         frame.payload().metadataSize()) {
       return connection.close(folly::make_exception_wrapper<RocketException>(
@@ -280,11 +282,28 @@ void ThriftRocketServerHandler::handleSetupFrame(
     serverMeta.set_setupResponse();
     serverMeta.setupResponse_ref()->version_ref() = version_;
     serverMeta.setupResponse_ref()->zstdSupported_ref() = true;
-    CompactProtocolWriter compactProtocolWriter;
-    folly::IOBufQueue queue;
-    compactProtocolWriter.setOutput(&queue);
-    serverMeta.write(&compactProtocolWriter);
-    connection.sendMetadataPush(std::move(queue).move());
+
+    if (auto ref = meta.compressionSetupRequest()) {
+      auto compressionSetupRes =
+          handleSetupFrameCustomCompression(*ref, connection);
+      if (compressionSetupRes.hasError()) {
+        LOG(WARNING) << fmt::format(
+            "Error setting up custom compression: {}, fallback to not using custom compression.",
+            compressionSetupRes.error());
+      } else {
+        auto optResponse = std::move(compressionSetupRes.value());
+        if (optResponse) {
+          serverMeta.setupResponse_ref()
+              ->compressionSetupResponse()
+              .ensure()
+              .custom_ref() = std::move(*optResponse);
+        }
+        // otherwise, custom compression is simply not used
+      }
+    }
+
+    connection.sendMetadataPush(
+        PayloadSerializer::getInstance()->packCompact(serverMeta));
   } catch (const std::exception& e) {
     return connection.close(folly::make_exception_wrapper<RocketException>(
         ErrorCode::INVALID_SETUP,
@@ -296,6 +315,57 @@ void ThriftRocketServerHandler::handleSetupFrame(
   invokeServiceInterceptorsOnConnection(connection);
 }
 
+folly::Expected<std::optional<CustomCompressionSetupResponse>, std::string>
+ThriftRocketServerHandler::handleSetupFrameCustomCompression(
+    CompressionSetupRequest const& setupRequest,
+    RocketServerConnection& connection) {
+  if (!setupRequest.custom_ref()) {
+    return folly::makeUnexpected(
+        "Cannot setup compression on server due to unrecognized request type");
+  }
+
+  auto factory = CustomCompressorRegistry::get(
+      *setupRequest.custom_ref()->compressorName());
+  if (!factory) {
+    return folly::makeUnexpected(fmt::format(
+        "Custom compressor {} is not supported on server.",
+        *setupRequest.custom_ref()->compressorName()));
+  }
+
+  std::optional<CustomCompressionSetupResponse> response;
+  try {
+    response = factory->createCustomCompressorNegotiationResponse(
+        *setupRequest.custom_ref());
+  } catch (const std::exception& ex) {
+    return folly::makeUnexpected(fmt::format(
+        "Failed to make create negotiation response on server due to: {}",
+        ex.what()));
+  }
+
+  if (!response) {
+    // custom compression chooses not to be used for this connection
+    return folly::makeExpected<std::string>(response);
+  }
+
+  std::shared_ptr<CustomCompressor> compressor;
+  try {
+    compressor = factory->make(
+        *response, CustomCompressorFactory::CompressorLocation::SERVER);
+  } catch (const std::exception& ex) {
+    return folly::makeUnexpected(fmt::format(
+        "Failed to make create custom compressor on server due to: {}",
+        ex.what()));
+  }
+
+  if (!compressor) {
+    return folly::makeUnexpected(fmt::format(
+        "Failed to make create custom compressor on server due to unknown error."));
+  }
+
+  connection.applyCustomCompression(compressor);
+  return folly::makeExpected<std::string>(std::move(*response));
+}
+
 void ThriftRocketServerHandler::handleRequestResponseFrame(
     RequestResponseFrame&& frame, RocketServerFrameContext&& context) {
   auto makeRequestResponse = [&](RequestRpcMetadata&& md,
@@ -323,7 +393,7 @@ void ThriftRocketServerHandler::handleRequestResponseFrame(
       std::move(frame.payload()),
       std::move(makeRequestResponse),
       RpcKind::SINGLE_REQUEST_SINGLE_RESPONSE,
-      context.connection().isDecodingMetadataUsingBinaryProtocol());
+      context.connection());
 }
 
 void ThriftRocketServerHandler::handleRequestFnfFrame(
@@ -350,7 +420,7 @@ void ThriftRocketServerHandler::handleRequestFnfFrame(
       std::move(frame.payload()),
       std::move(makeRequestFnf),
       RpcKind::SINGLE_REQUEST_NO_RESPONSE,
-      context.connection().isDecodingMetadataUsingBinaryProtocol());
+      context.connection());
 }
 
 void ThriftRocketServerHandler::handleRequestStreamFrame(
@@ -378,7 +448,7 @@ void ThriftRocketServerHandler::handleRequestStreamFrame(
       std::move(frame.payload()),
       std::move(makeRequestStream),
       RpcKind::SINGLE_REQUEST_STREAMING_RESPONSE,
-      context.connection().isDecodingMetadataUsingBinaryProtocol());
+      context.connection());
 }
 
 void ThriftRocketServerHandler::handleRequestChannelFrame(
@@ -406,7 +476,7 @@ void ThriftRocketServerHandler::handleRequestChannelFrame(
       std::move(frame.payload()),
       std::move(makeRequestSink),
       RpcKind::SINK,
-      context.connection().isDecodingMetadataUsingBinaryProtocol());
+      context.connection());
 }
 
 void ThriftRocketServerHandler::connectionClosing() {
@@ -421,16 +491,16 @@ void ThriftRocketServerHandler::handleRequestCommon(
     Payload&& payload,
     F&& makeRequest,
     RpcKind expectedKind,
-    bool decodeMetadataUsingBinary) {
+    RocketServerConnection& connection) {
   std::chrono::steady_clock::time_point readEnd{
       std::chrono::steady_clock::now()};
   auto wiredPayloadSize = payload.metadataAndDataSize();
 
   rocket::Payload debugPayload = payload.clone();
   auto requestPayloadTry =
-      rocket::PayloadSerializer::getInstance()
-          .unpackAsCompressed<RequestPayload>(
-              std::move(payload), decodeMetadataUsingBinary);
+      connection.getPayloadSerializer()->unpackAsCompressed<RequestPayload>(
+          std::move(payload),
+          connection.isDecodingMetadataUsingBinaryProtocol());
 
   auto makeActiveRequest = [&](auto&& md, auto&& payload, auto&& reqCtx) {
     serverConfigs_->incActiveRequests();
@@ -526,7 +596,8 @@ void ThriftRocketServerHandler::handleRequestCommon(
   if (metadata.crc32c_ref()) {
     try {
       if (auto compression = metadata.compression_ref()) {
-        data = uncompressBuffer(std::move(data), *compression);
+        data = connection.getPayloadSerializer()->uncompressBuffer(
+            std::move(data), *compression);
       }
     } catch (...) {
       handleDecompressionFailure(
@@ -602,6 +673,22 @@ void ThriftRocketServerHandler::handleRequestCommon(
       ? (*metadata.frameworkMetadata())->clone()
       : nullptr;
 
+  if (interactionIdOpt &&
+      THRIFT_FLAG(enable_interaction_overload_protection_server)) {
+    if (auto* interaction =
+            apache::thrift::detail::Cpp2ConnContextInternalAPI(connContext_)
+                .findTile(*interactionIdOpt)) {
+      if (auto* overloadPolicy =
+              apache::thrift::detail::TileInternalAPI(*interaction)
+                  .getOverloadPolicy();
+          overloadPolicy && !overloadPolicy->allowNewRequest()) {
+        handleInteractionLoadshedded(makeActiveRequest(
+            std::move(metadata), std::move(debugPayload), std::move(reqCtx)));
+        return;
+      }
+    }
+  }
+
   // A request should not be active until the overload checking is done.
   auto request = makeRequest(
       std::move(metadata), std::move(debugPayload), std::move(reqCtx));
@@ -610,9 +697,16 @@ void ThriftRocketServerHandler::handleRequestCommon(
   const auto& headers = request->getTHeader().getHeaders();
   const auto& name = request->getMethodName();
 
-  auto overloadResult = serverConfigs_->checkOverload(&headers, &name);
+  auto overloadResult = serverConfigs_->checkOverload(headers, name);
   serverConfigs_->incActiveRequests();
   if (UNLIKELY(overloadResult.has_value())) {
+    if ((interactionCreateOpt || interactionIdOpt) &&
+        shouldTerminateInteraction(
+            interactionCreateOpt.has_value(),
+            interactionIdOpt,
+            &connContext_)) {
+      overloadResult->errorCode = mapToTerminalError(overloadResult->errorCode);
+    }
     handleRequestOverloadedServer(
         std::move(request), std::move(*overloadResult));
     return;
@@ -626,36 +720,11 @@ void ThriftRocketServerHandler::handleRequestCommon(
   auto preprocessResult =
       serverConfigs_->preprocess({headers, name, connContext_, request.get()});
   if (UNLIKELY(!std::holds_alternative<std::monostate>(preprocessResult))) {
-    folly::variant_match(
-        preprocessResult,
-        [&](AppClientException& ace) {
-          handleAppError(std::move(request), ace);
-        },
-        [&](const AppServerException& ase) {
-          handleAppError(std::move(request), ase);
-        },
-        [&](AppOverloadedException& aoe) {
-          handleRequestOverloadedServer(
-              std::move(request),
-              OverloadResult{
-                  kAppOverloadedErrorCode,
-                  aoe.getMessage(),
-                  LoadShedder::CUSTOM});
-        },
-        [&](AppQuotaExceededException& aqe) {
-          handleQuotaExceededException(
-              std::move(request),
-              kTenantQuotaExceededErrorCode,
-              aqe.getMessage());
-        },
-        [&](AppTenantBlocklistedException& atb) {
-          handleQuotaExceededException(
-              std::move(request),
-              kTenantBlocklistedErrorCode,
-              atb.getMessage());
-        },
-        [](std::monostate&) { folly::assume_unreachable(); });
-
+    handlePreprocessResult(
+        std::move(request),
+        std::move(preprocessResult),
+        interactionCreateOpt.has_value(),
+        interactionIdOpt);
     return;
   }
 
@@ -706,7 +775,8 @@ void ThriftRocketServerHandler::handleRequestCommon(
       std::move(data),
       crc32Opt ? CompressionAlgorithm::NONE
                : compressionOpt.value_or(CompressionAlgorithm::NONE),
-      checksumAlgorithm);
+      checksumAlgorithm,
+      connection.getPayloadSerializer());
 
   const auto protocolId = request->getProtoId();
   Cpp2Worker::dispatchRequest(
@@ -721,6 +791,42 @@ void ThriftRocketServerHandler::handleRequestCommon(
       worker_->getServer());
 }
 
+void ThriftRocketServerHandler::handlePreprocessResult(
+    ThriftRequestCoreUniquePtr request,
+    PreprocessResult&& preprocessResult,
+    bool isInteractionCreatePresent,
+    std::optional<int64_t>& interactionIdOpt) {
+  folly::variant_match(
+      preprocessResult,
+      [&](AppClientException& ace) { handleAppError(std::move(request), ace); },
+      [&](const AppServerException& ase) {
+        handleAppError(std::move(request), ase);
+      },
+      [&](AppOverloadedException& aoe) {
+        std::string_view exCode = kAppOverloadedErrorCode;
+        if ((isInteractionCreatePresent || interactionIdOpt) &&
+            shouldTerminateInteraction(
+                isInteractionCreatePresent, interactionIdOpt, &connContext_)) {
+          exCode = kInteractionLoadsheddedAppOverloadErrorCode;
+        }
+        handleRequestOverloadedServer(
+            std::move(request),
+            OverloadResult{
+                std::string(exCode), aoe.getMessage(), LoadShedder::CUSTOM});
+      },
+      [&](AppQuotaExceededException& aqe) {
+        handleQuotaExceededException(
+            std::move(request),
+            kTenantQuotaExceededErrorCode,
+            aqe.getMessage());
+      },
+      [&](AppTenantBlocklistedException& atb) {
+        handleQuotaExceededException(
+            std::move(request), kTenantBlocklistedErrorCode, atb.getMessage());
+      },
+      [](std::monostate&) { folly::assume_unreachable(); });
+}
+
 void ThriftRocketServerHandler::handleRequestWithBadMetadata(
     ThriftRequestCoreUniquePtr request) {
   if (auto* observer = serverConfigs_->getObserver()) {
@@ -844,6 +950,19 @@ void ThriftRocketServerHandler::handleServerShutdown(
       kQueueOverloadedErrorCode);
 }
 
+void ThriftRocketServerHandler::handleInteractionLoadshedded(
+    ThriftRequestCoreUniquePtr request) {
+  if (auto* observer = serverConfigs_->getObserver()) {
+    observer->taskKilled();
+  }
+
+  request->sendErrorWrapped(
+      folly::make_exception_wrapper<TApplicationException>(
+          TApplicationException::LOADSHEDDING,
+          "Interaction already loadshedded"),
+      kInteractionLoadsheddedErrorCode);
+}
+
 void ThriftRocketServerHandler::handleInjectedFault(
     ThriftRequestCoreUniquePtr request, InjectedFault fault) {
   switch (fault) {
@@ -886,18 +1005,17 @@ void ThriftRocketServerHandler::onBeforeHandleFrame() {
 void ThriftRocketServerHandler::invokeServiceInterceptorsOnConnection(
     RocketServerConnection& connection) noexcept {
 #if FOLLY_HAS_COROUTINES
-  const auto& serviceInterceptorsInfo =
+  const auto& serviceInterceptors =
       worker_->getServer()->getServiceInterceptors();
   std::vector<std::pair<std::size_t, std::exception_ptr>> exceptions;
   didExecuteServiceInterceptorsOnConnection_ = true;
 
-  for (std::size_t i = 0; i < serviceInterceptorsInfo.size(); ++i) {
+  for (std::size_t i = 0; i < serviceInterceptors.size(); ++i) {
     ServiceInterceptorBase::ConnectionInfo connectionInfo{
         &connContext_,
         connContext_.getStorageForServiceInterceptorOnConnectionByIndex(i)};
     try {
-      serviceInterceptorsInfo[i].interceptor->internal_onConnection(
-          std::move(connectionInfo));
+      serviceInterceptors[i]->internal_onConnection(std::move(connectionInfo));
     } catch (...) {
       exceptions.emplace_back(i, folly::current_exception());
     }
@@ -905,12 +1023,14 @@ void ThriftRocketServerHandler::invokeServiceInterceptorsOnConnection(
   if (!exceptions.empty()) {
     std::string message = fmt::format(
         "ServiceInterceptor::onConnection threw exceptions:\n[{}] {}\n",
-        serviceInterceptorsInfo[exceptions[0].first].qualifiedName,
+        serviceInterceptors[exceptions[0].first]->getQualifiedName().toString(),
         folly::exceptionStr(exceptions[0].second));
     for (std::size_t i = 1; i < exceptions.size(); ++i) {
       message += fmt::format(
           "[{}] {}\n",
-          serviceInterceptorsInfo[exceptions[i].first].qualifiedName,
+          serviceInterceptors[exceptions[i].first]
+              ->getQualifiedName()
+              .toString(),
           folly::exceptionStr(exceptions[i].second));
     }
     return connection.close(folly::make_exception_wrapper<RocketException>(
@@ -923,14 +1043,13 @@ void ThriftRocketServerHandler::
     invokeServiceInterceptorsOnConnectionClosed() noexcept {
 #if FOLLY_HAS_COROUTINES
   if (didExecuteServiceInterceptorsOnConnection_) {
-    const auto& serviceInterceptorsInfo =
+    const auto& serviceInterceptors =
         worker_->getServer()->getServiceInterceptors();
-    for (std::size_t i = 0; i < serviceInterceptorsInfo.size(); ++i) {
+    for (std::size_t i = 0; i < serviceInterceptors.size(); ++i) {
       ServiceInterceptorBase::ConnectionInfo connectionInfo{
           &connContext_,
           connContext_.getStorageForServiceInterceptorOnConnectionByIndex(i)};
-      serviceInterceptorsInfo[i].interceptor->internal_onConnectionClosed(
-          connectionInfo);
+      serviceInterceptors[i]->internal_onConnectionClosed(connectionInfo);
     }
   }
 #endif // FOLLY_HAS_COROUTINES
