@@ -32,7 +32,6 @@
 #include <folly/SocketAddress.h>
 #include <folly/executors/InlineExecutor.h>
 #include <folly/fibers/Baton.h>
-#include <folly/fibers/FiberManager.h>
 #include <folly/fibers/FiberManagerMap.h>
 #include <folly/futures/Future.h>
 #include <folly/io/IOBuf.h>
@@ -45,7 +44,6 @@
 
 #include <thrift/lib/cpp2/async/ServerSinkBridge.h>
 #include <thrift/lib/cpp2/async/StreamCallbacks.h>
-#include <thrift/lib/cpp2/protocol/Serializer.h>
 #include <thrift/lib/cpp2/transport/core/TryUtil.h>
 #include <thrift/lib/cpp2/transport/rocket/Types.h>
 #include <thrift/lib/cpp2/transport/rocket/client/RocketClient.h>
@@ -113,8 +111,7 @@ rocket::SetupFrame RocketTestClient::makeTestSetupFrame(
   *meta.opaque() = std::move(md);
   meta.maxVersion() = kClientVersion;
 
-  auto serializedMeta =
-      PayloadSerializer::getInstance().packCompact(std::move(meta));
+  auto serializedMeta = PayloadSerializer::getInstance()->packCompact(meta);
 
   // Serialize RocketClient's major/minor version (which is separate from the
   // rsocket protocol major/minor version) into setup metadata.
@@ -378,7 +375,7 @@ class RocketTestServer::RocketTestServerHandler : public RocketServerHandler {
     // Validate RequestSetupMetadata
     RequestSetupMetadata meta;
     size_t unpackedSize =
-        PayloadSerializer::getInstance().unpack(meta, cursor, false);
+        PayloadSerializer::getInstance()->unpack(meta, cursor, false);
     EXPECT_EQ(unpackedSize, frame.payload().metadataSize());
     EXPECT_EQ(expectedSetupMetadata_, meta.opaque_ref().value_or({}));
     version_ = std::min(kServerVersion, meta.maxVersion_ref().value_or(0));
@@ -386,7 +383,7 @@ class RocketTestServer::RocketTestServerHandler : public RocketServerHandler {
     serverMeta.set_setupResponse();
     serverMeta.setupResponse_ref()->version_ref() = version_;
     connection.sendMetadataPush(
-        PayloadSerializer::getInstance().packCompact(std::move(serverMeta)));
+        PayloadSerializer::getInstance()->packCompact(serverMeta));
   }
 
   void handleRequestResponseFrame(
