@@ -38,13 +38,11 @@
 #include <folly/io/async/EventBase.h>
 #include <folly/io/async/ScopedEventBaseThread.h>
 
-#include <folly/io/async/AsyncSocket.h>
+#include <folly/stop_watch.h>
 #include <thrift/lib/cpp/transport/TTransport.h>
 #include <thrift/lib/cpp/transport/TTransportException.h>
 #include <thrift/lib/cpp2/async/ClientSinkBridge.h>
 #include <thrift/lib/cpp2/async/FutureRequest.h>
-#include <thrift/lib/cpp2/async/RequestCallback.h>
-#include <thrift/lib/cpp2/async/RequestChannel.h>
 #include <thrift/lib/cpp2/async/RocketClientChannel.h>
 #include <thrift/lib/cpp2/async/Sink.h>
 #include <thrift/lib/cpp2/async/StreamCallbacks.h>
@@ -1080,7 +1078,8 @@ TEST_F(RocketNetworkTest, RequestStreamNewApiBasic) {
       "dummy",
       apache::thrift::SerializedRequest(std::move(payload)),
       std::make_shared<THeader>(),
-      &clientCallback);
+      &clientCallback,
+      /* frameworkMetadata */ nullptr);
 
   evb.loop();
 
@@ -1108,7 +1107,8 @@ TEST_F(RocketNetworkTest, RequestStreamNewApiError) {
       "dummy",
       apache::thrift::SerializedRequest(std::move(payload)),
       std::make_shared<THeader>(),
-      &clientCallback);
+      &clientCallback,
+      /* frameworkMetadata */ nullptr);
 
   evb.loop();
 
@@ -1139,7 +1139,8 @@ TEST_F(RocketNetworkTest, RequestStreamNewApiHeadersPush) {
         "dummy",
         apache::thrift::SerializedRequest(std::move(payload)),
         std::make_shared<THeader>(),
-        &clientCallback);
+        &clientCallback,
+        /* frameworkMetadata */ nullptr);
 
     evb.loop();
 
@@ -1162,7 +1163,8 @@ TEST_F(RocketNetworkTest, RequestStreamNewApiHeadersPush) {
         "dummy",
         apache::thrift::SerializedRequest(std::move(payload)),
         std::make_shared<THeader>(),
-        &clientCallback);
+        &clientCallback,
+        /* frameworkMetadata */ nullptr);
 
     evb.loop();
 
@@ -1429,19 +1431,23 @@ TEST_F(RocketNetworkTest, ObserverIsNotifiedOnWriteSuccessRequestResponse) {
         std::make_unique<NiceMock<MockRocketServerConnectionObserver>>(
             eventSet);
 
-    this->server_->getEventBase().runInEventBaseThreadAndWait([&] {
-      auto connCount = 0;
-      server_->getConnectionManager()->forEachConnection(
-          [&](wangle::ManagedConnection* connection) {
-            if (auto conn = dynamic_cast<RocketServerConnection*>(connection)) {
-              EXPECT_EQ(conn->numObservers(), 0);
-              conn->addObserver(observer.get());
-              EXPECT_EQ(conn->numObservers(), 1);
-              connCount += 1;
-            }
-          });
-      EXPECT_EQ(connCount, 1);
-    });
+    auto connCount = 0;
+    folly::stop_watch<std::chrono::minutes> timer;
+    while (connCount == 0 && timer.elapsed() < std::chrono::minutes(1)) {
+      this->server_->getEventBase().runInEventBaseThreadAndWait([&] {
+        server_->getConnectionManager()->forEachConnection(
+            [&](wangle::ManagedConnection* connection) {
+              if (auto conn =
+                      dynamic_cast<RocketServerConnection*>(connection)) {
+                EXPECT_EQ(conn->numObservers(), 0);
+                conn->addObserver(observer.get());
+                EXPECT_EQ(conn->numObservers(), 1);
+                connCount += 1;
+              }
+            });
+      });
+    }
+    EXPECT_EQ(connCount, 1);
 
     constexpr size_t kSetupFrameSize(14);
     constexpr size_t kSetupFrameStreamId(0);
