@@ -38,6 +38,7 @@ const (
 type compactType byte
 
 const (
+	COMPACT_STOP          = 0x00
 	COMPACT_BOOLEAN_TRUE  = 0x01
 	COMPACT_BOOLEAN_FALSE = 0x02
 	COMPACT_BYTE          = 0x03
@@ -53,26 +54,20 @@ const (
 	COMPACT_FLOAT         = 0x0D
 )
 
-var (
-	typeToCompactType map[types.Type]compactType
-)
-
-func init() {
-	typeToCompactType = map[types.Type]compactType{
-		types.STOP:   types.STOP,
-		types.BOOL:   COMPACT_BOOLEAN_TRUE,
-		types.BYTE:   COMPACT_BYTE,
-		types.I16:    COMPACT_I16,
-		types.I32:    COMPACT_I32,
-		types.I64:    COMPACT_I64,
-		types.DOUBLE: COMPACT_DOUBLE,
-		types.FLOAT:  COMPACT_FLOAT,
-		types.STRING: COMPACT_BINARY,
-		types.LIST:   COMPACT_LIST,
-		types.SET:    COMPACT_SET,
-		types.MAP:    COMPACT_MAP,
-		types.STRUCT: COMPACT_STRUCT,
-	}
+var typeToCompactType = map[types.Type]compactType{
+	types.STOP:   COMPACT_STOP,
+	types.BOOL:   COMPACT_BOOLEAN_TRUE,
+	types.BYTE:   COMPACT_BYTE,
+	types.I16:    COMPACT_I16,
+	types.I32:    COMPACT_I32,
+	types.I64:    COMPACT_I64,
+	types.DOUBLE: COMPACT_DOUBLE,
+	types.FLOAT:  COMPACT_FLOAT,
+	types.STRING: COMPACT_BINARY,
+	types.LIST:   COMPACT_LIST,
+	types.SET:    COMPACT_SET,
+	types.MAP:    COMPACT_MAP,
+	types.STRUCT: COMPACT_STRUCT,
 }
 
 type compactFormat struct {
@@ -83,12 +78,12 @@ type compactFormat struct {
 var _ types.Format = (*compactFormat)(nil)
 
 // NewCompactFormat creates a CompactFormat
-func NewCompactFormat(trans io.ReadWriteCloser) types.Format {
+func NewCompactFormat(readWriter types.ReadWriteSizer) types.Format {
 	p := &compactFormat{}
 	p.compactDecoder.version = COMPACT_VERSION_BE
-	p.compactDecoder.reader = trans
+	p.compactDecoder.reader = readWriter
 	p.compactEncoder.version = COMPACT_VERSION_BE
-	p.compactEncoder.writer = trans
+	p.compactEncoder.writer = readWriter
 	return p
 }
 
@@ -96,7 +91,7 @@ func newCompactEncoder(writer io.Writer) types.Encoder {
 	return &compactEncoder{writer: writer, version: COMPACT_VERSION_BE}
 }
 
-func newCompactDecoder(reader io.Reader) types.Decoder {
+func newCompactDecoder(reader types.ReadSizer) types.Decoder {
 	return &compactDecoder{reader: reader, version: COMPACT_VERSION_BE}
 }
 
@@ -107,7 +102,7 @@ func (p *compactFormat) WriteMessageBegin(name string, typeID types.MessageType,
 }
 
 type compactDecoder struct {
-	reader io.Reader
+	reader types.ReadSizer
 	// Used to keep track of the last field for the current and previous structs,
 	// so we can do the delta stuff.
 	lastFieldRead   []int
@@ -141,6 +136,14 @@ type compactEncoder struct {
 	version int
 }
 
+// IsCompactFramed Check if the magic value corresponds to compact proto
+func IsCompactFramed(magic uint32) bool {
+	protocolID := int8(magic >> 24)
+	protocolVersion := int8((magic >> 16) & uint32(COMPACT_VERSION_MASK))
+	return uint8(protocolID) == uint8(COMPACT_PROTOCOL_ID) && (protocolVersion == int8(COMPACT_VERSION) ||
+		protocolVersion == int8(COMPACT_VERSION_BE))
+}
+
 //
 // Public Writing methods.
 //
@@ -160,9 +163,7 @@ func (p *compactEncoder) WriteMessageBegin(name string, typeID types.MessageType
 	if err != nil {
 		return types.NewProtocolException(err)
 	}
-	e := p.WriteString(name)
-	return e
-
+	return p.WriteString(name)
 }
 
 func (p *compactEncoder) WriteMessageEnd() error { return nil }
@@ -237,7 +238,7 @@ func (p *compactEncoder) writeFieldBeginInternal(name string, typeID types.Type,
 func (p *compactEncoder) WriteFieldEnd() error { return nil }
 
 func (p *compactEncoder) WriteFieldStop() error {
-	err := p.writeByteDirect(types.STOP)
+	err := p.writeByteDirect(COMPACT_STOP)
 	return types.NewProtocolException(err)
 }
 
@@ -344,13 +345,13 @@ func (p *compactEncoder) WriteString(value string) error {
 
 // Write a byte array, using a varint for the size.
 func (p *compactEncoder) WriteBinary(bin []byte) error {
-	_, e := p.writeVarint32(int32(len(bin)))
-	if e != nil {
-		return types.NewProtocolException(e)
+	_, err := p.writeVarint32(int32(len(bin)))
+	if err != nil {
+		return types.NewProtocolException(err)
 	}
 	if len(bin) > 0 {
-		_, e = p.writer.Write(bin)
-		return types.NewProtocolException(e)
+		_, err = p.writer.Write(bin)
+		return types.NewProtocolException(err)
 	}
 	return nil
 }
@@ -399,10 +400,10 @@ func (p *compactDecoder) ReadMessageEnd() error { return nil }
 
 // Read a struct begin. There's nothing on the wire for this, but it is our
 // opportunity to push a new struct begin marker onto the field stack.
-func (p *compactDecoder) ReadStructBegin() (name string, err error) {
+func (p *compactDecoder) ReadStructBegin() (string, error) {
 	p.lastFieldRead = append(p.lastFieldRead, p.lastFieldIDRead)
 	p.lastFieldIDRead = 0
-	return
+	return "", nil
 }
 
 // Doesn't actually consume any wire data, just removes the last field for
@@ -422,7 +423,7 @@ func (p *compactDecoder) ReadFieldBegin() (name string, typeID types.Type, id in
 	}
 
 	// if it's a stop, then we can return immediately, as the struct is over.
-	if (t & 0x0f) == types.STOP {
+	if (t & 0x0f) == COMPACT_STOP {
 		return "", types.STOP, 0, nil
 	}
 	// mask off the 4 MSB of the type header. it could contain a field id delta.
@@ -460,33 +461,30 @@ func (p *compactDecoder) ReadFieldEnd() error { return nil }
 // Read a map header off the wire. If the size is zero, skip reading the key
 // and value type. This means that 0-length maps will yield Maps without the
 // "correct" types.
-func (p *compactDecoder) ReadMapBegin() (keyType types.Type, valueType types.Type, size int, err error) {
-	size32, e := p.readVarint32()
-	if e != nil {
-		err = types.NewProtocolException(e)
-		return
+func (p *compactDecoder) ReadMapBegin() (types.Type /* kType */, types.Type /* vType */, int /* size */, error) {
+	size32, err := p.readVarint32()
+	if err != nil {
+		return 0, 0, 0, types.NewProtocolException(err)
 	}
 	if size32 < 0 {
-		err = invalidDataLength
-		return
+		return 0, 0, 0, invalidDataLength
 	}
-	remainingBytes := remainingBytes(p.reader)
-	if uint64(size32*2) > remainingBytes || remainingBytes == types.UnknownRemaining {
-		err = invalidDataLength
-		return
+	size := int(size32)
+	remainingBytes := p.reader.Len()
+	if size*2 > remainingBytes {
+		return 0, 0, 0, invalidDataLength
 	}
-	size = int(size32)
 
-	keyAndValueType := byte(types.STOP)
+	kvTypeByte := byte(types.STOP)
 	if size != 0 {
-		keyAndValueType, err = p.ReadByte()
+		kvTypeByte, err = p.ReadByte()
 		if err != nil {
-			return
+			return 0, 0, 0, err
 		}
 	}
-	keyType, _ = compactToThriftType(compactType(keyAndValueType >> 4))
-	valueType, _ = compactToThriftType(compactType(keyAndValueType & 0xf))
-	return
+	kType, _ := compactToThriftType(compactType(kvTypeByte >> 4))
+	vType, _ := compactToThriftType(compactType(kvTypeByte & 0xf))
+	return kType, vType, size, nil
 }
 
 func (p *compactDecoder) ReadMapEnd() error { return nil }
@@ -495,36 +493,32 @@ func (p *compactDecoder) ReadMapEnd() error { return nil }
 // be packed into the element type header. If it's a longer list, the 4 MSB
 // of the element type header will be 0xF, and a varint will follow with the
 // true size.
-func (p *compactDecoder) ReadListBegin() (elemType types.Type, size int, err error) {
-	size_and_type, err := p.ReadByte()
+func (p *compactDecoder) ReadListBegin() (types.Type /* elemType */, int /* size */, error) {
+	sizeAndTypeByte, err := p.ReadByte()
 	if err != nil {
-		return
+		return 0, 0, err
 	}
-	size = int((size_and_type >> 4) & 0x0f)
+	size := int((sizeAndTypeByte >> 4) & 0x0f)
 	if size == 15 {
-		size2, e := p.readVarint32()
-		if e != nil {
-			err = types.NewProtocolException(e)
-			return
+		size2, err := p.readVarint32()
+		if err != nil {
+			return 0, 0, types.NewProtocolException(err)
 		}
 		if size2 < 0 {
-			err = invalidDataLength
-			return
+			return 0, 0, invalidDataLength
 		}
 		size = int(size2)
 	}
-	remainingBytes := remainingBytes(p.reader)
-	if uint64(size) > remainingBytes || remainingBytes == types.UnknownRemaining {
-		err = invalidDataLength
-		return
+	remainingBytes := p.reader.Len()
+	if size > remainingBytes {
+		return 0, 0, invalidDataLength
 	}
 
-	elemType, e := compactToThriftType(compactType(size_and_type))
-	if e != nil {
-		err = types.NewProtocolException(e)
-		return
+	elemType, err := compactToThriftType(compactType(sizeAndTypeByte))
+	if err != nil {
+		return 0, 0, types.NewProtocolException(err)
 	}
-	return
+	return elemType, size, nil
 }
 
 func (p *compactDecoder) ReadListEnd() error { return nil }
@@ -533,7 +527,7 @@ func (p *compactDecoder) ReadListEnd() error { return nil }
 // be packed into the element type header. If it's a longer set, the 4 MSB
 // of the element type header will be 0xF, and a varint will follow with the
 // true size.
-func (p *compactDecoder) ReadSetBegin() (elemType types.Type, size int, err error) {
+func (p *compactDecoder) ReadSetBegin() (types.Type /* elemType */, int /* size */, error) {
 	return p.ReadListBegin()
 }
 
@@ -542,7 +536,7 @@ func (p *compactDecoder) ReadSetEnd() error { return nil }
 // Read a boolean off the wire. If this is a boolean field, the value should
 // already have been read during readFieldBegin, so we'll just consume the
 // pre-stored value. Otherwise, read a byte.
-func (p *compactDecoder) ReadBool() (value bool, err error) {
+func (p *compactDecoder) ReadBool() (bool, error) {
 	if p.boolValueIsNotNull {
 		p.boolValueIsNotNull = false
 		return p.boolValue, nil
@@ -557,37 +551,37 @@ func (p *compactDecoder) ReadByte() (byte, error) {
 }
 
 // Read an i16 from the wire as a zigzag varint.
-func (p *compactDecoder) ReadI16() (value int16, err error) {
+func (p *compactDecoder) ReadI16() (int16, error) {
 	v, err := p.ReadI32()
 	return int16(v), err
 }
 
 // Read an i32 from the wire as a zigzag varint.
-func (p *compactDecoder) ReadI32() (value int32, err error) {
-	v, e := p.readVarint32()
-	if e != nil {
-		return 0, types.NewProtocolException(e)
+func (p *compactDecoder) ReadI32() (int32, error) {
+	v, err := p.readVarint32()
+	if err != nil {
+		return 0, types.NewProtocolException(err)
 	}
-	value = p.zigzagToInt32(v)
+	value := p.zigzagToInt32(v)
 	return value, nil
 }
 
 // Read an i64 from the wire as a zigzag varint.
-func (p *compactDecoder) ReadI64() (value int64, err error) {
-	v, e := p.readVarint64()
-	if e != nil {
-		return 0, types.NewProtocolException(e)
+func (p *compactDecoder) ReadI64() (int64, error) {
+	v, err := p.readVarint64()
+	if err != nil {
+		return 0, types.NewProtocolException(err)
 	}
-	value = p.zigzagToInt64(v)
+	value := p.zigzagToInt64(v)
 	return value, nil
 }
 
 // No magic here - just read a double off the wire.
-func (p *compactDecoder) ReadDouble() (value float64, err error) {
+func (p *compactDecoder) ReadDouble() (float64, error) {
 	longBits := p.rBuffer[0:8]
-	_, e := io.ReadFull(p.reader, longBits)
-	if e != nil {
-		return 0.0, types.NewProtocolException(e)
+	_, err := io.ReadFull(p.reader, longBits)
+	if err != nil {
+		return 0.0, types.NewProtocolException(err)
 	}
 	if p.version == COMPACT_VERSION {
 		return math.Float64frombits(binary.LittleEndian.Uint64(longBits)), nil
@@ -596,26 +590,26 @@ func (p *compactDecoder) ReadDouble() (value float64, err error) {
 }
 
 // No magic here - just read a float off the wire.
-func (p *compactDecoder) ReadFloat() (value float32, err error) {
+func (p *compactDecoder) ReadFloat() (float32, error) {
 	bits := p.rBuffer[0:4]
-	_, e := io.ReadFull(p.reader, bits)
-	if e != nil {
-		return 0.0, types.NewProtocolException(e)
+	_, err := io.ReadFull(p.reader, bits)
+	if err != nil {
+		return 0.0, types.NewProtocolException(err)
 	}
 	return math.Float32frombits(binary.BigEndian.Uint32(bits)), nil
 }
 
 // Reads a []byte (via readBinary), and then UTF-8 decodes it.
-func (p *compactDecoder) ReadString() (value string, err error) {
-	length, e := p.readVarint32()
-	if e != nil {
-		return "", types.NewProtocolException(e)
+func (p *compactDecoder) ReadString() (string, error) {
+	length, err := p.readVarint32()
+	if err != nil {
+		return "", types.NewProtocolException(err)
 	}
 	if length < 0 {
 		return "", invalidDataLength
 	}
-	remainingBytes := remainingBytes(p.reader)
-	if uint64(length) > remainingBytes || remainingBytes == types.UnknownRemaining {
+	remainingBytes := p.reader.Len()
+	if int(length) > remainingBytes {
 		return "", invalidDataLength
 	}
 
@@ -628,37 +622,38 @@ func (p *compactDecoder) ReadString() (value string, err error) {
 	} else {
 		buf = make([]byte, length)
 	}
-	_, e = io.ReadFull(p.reader, buf)
-	return string(buf), types.NewProtocolException(e)
+	_, err = io.ReadFull(p.reader, buf)
+	return string(buf), types.NewProtocolException(err)
 }
 
 // Read a []byte from the wire.
-func (p *compactDecoder) ReadBinary() (value []byte, err error) {
-	length, e := p.readVarint32()
-	if e != nil {
-		return nil, types.NewProtocolException(e)
+func (p *compactDecoder) ReadBinary() ([]byte, error) {
+	size32, err := p.readVarint32()
+	if err != nil {
+		return nil, types.NewProtocolException(err)
 	}
-	if length == 0 {
+	if size32 == 0 {
 		return []byte{}, nil
 	}
-	if length < 0 {
+	if size32 < 0 {
 		return nil, invalidDataLength
 	}
-	remainingBytes := remainingBytes(p.reader)
-	if uint64(length) > remainingBytes || remainingBytes == types.UnknownRemaining {
+	size := int(size32)
+	remainingBytes := p.reader.Len()
+	if size > remainingBytes {
 		return nil, invalidDataLength
 	}
 
-	buf := make([]byte, length)
-	_, e = io.ReadFull(p.reader, buf)
-	return buf, types.NewProtocolException(e)
+	buf := make([]byte, size)
+	_, err = io.ReadFull(p.reader, buf)
+	return buf, types.NewProtocolException(err)
 }
 
-func (p *compactEncoder) Flush() (err error) {
+func (p *compactEncoder) Flush() error {
 	return flush(p.writer)
 }
 
-func (p *compactDecoder) Skip(fieldType types.Type) (err error) {
+func (p *compactDecoder) Skip(fieldType types.Type) error {
 	return types.SkipDefaultDepth(p, fieldType)
 }
 
@@ -798,7 +793,7 @@ func (p *compactDecoder) isBoolType(b byte) bool {
 // Type value.
 func compactToThriftType(t compactType) (types.Type, error) {
 	switch byte(t) & 0x0f {
-	case types.STOP:
+	case COMPACT_STOP:
 		return types.STOP, nil
 	case COMPACT_BOOLEAN_FALSE, COMPACT_BOOLEAN_TRUE:
 		return types.BOOL, nil
@@ -825,5 +820,5 @@ func compactToThriftType(t compactType) (types.Type, error) {
 	case COMPACT_STRUCT:
 		return types.STRUCT, nil
 	}
-	return types.STOP, types.Exception(fmt.Errorf("don't know what type: %#x", t&0x0f))
+	return types.STOP, fmt.Errorf("don't know what type: %#x", t&0x0f)
 }
