@@ -17,6 +17,7 @@
 package thrift
 
 import (
+	"bytes"
 	"encoding/base64"
 	"encoding/json"
 	"fmt"
@@ -29,7 +30,7 @@ import (
 
 func TestWriteJSONProtocolBool(t *testing.T) {
 	thetype := "boolean"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	for _, value := range boolValues {
 		if e := p.WriteBool(value); e != nil {
@@ -54,13 +55,12 @@ func TestWriteJSONProtocolBool(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadJSONProtocolBool(t *testing.T) {
 	thetype := "boolean"
 	for _, value := range boolValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewCompactJSONFormat(trans)
 		if value {
 			trans.Write([]byte{'1'}) // not JSON_TRUE
@@ -80,13 +80,12 @@ func TestReadJSONProtocolBool(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, vv)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteJSONProtocolByte(t *testing.T) {
 	thetype := "byte"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	for _, value := range byteValues {
 		if e := p.WriteByte(value); e != nil {
@@ -105,13 +104,12 @@ func TestWriteJSONProtocolByte(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadJSONProtocolByte(t *testing.T) {
 	thetype := "byte"
 	for _, value := range byteValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewCompactJSONFormat(trans)
 		trans.WriteString(strconv.Itoa(int(value)))
 		s := trans.String()
@@ -126,13 +124,12 @@ func TestReadJSONProtocolByte(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, v)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteJSONProtocolI16(t *testing.T) {
 	thetype := "int16"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	for _, value := range int16Values {
 		if e := p.WriteI16(value); e != nil {
@@ -151,13 +148,12 @@ func TestWriteJSONProtocolI16(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadJSONProtocolI16(t *testing.T) {
 	thetype := "int16"
 	for _, value := range int16Values {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewCompactJSONFormat(trans)
 		trans.WriteString(strconv.Itoa(int(value)))
 		s := trans.String()
@@ -172,13 +168,12 @@ func TestReadJSONProtocolI16(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, v)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteJSONProtocolI32(t *testing.T) {
 	thetype := "int32"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	for _, value := range int32Values {
 		if e := p.WriteI32(value); e != nil {
@@ -197,13 +192,12 @@ func TestWriteJSONProtocolI32(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadJSONProtocolI32(t *testing.T) {
 	thetype := "int32"
 	for _, value := range int32Values {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewCompactJSONFormat(trans)
 		trans.WriteString(strconv.Itoa(int(value)))
 		s := trans.String()
@@ -218,13 +212,12 @@ func TestReadJSONProtocolI32(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, v)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteJSONProtocolI64(t *testing.T) {
 	thetype := "int64"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	for _, value := range int64Values {
 		if e := p.WriteI64(value); e != nil {
@@ -243,13 +236,12 @@ func TestWriteJSONProtocolI64(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadJSONProtocolI64(t *testing.T) {
 	thetype := "int64"
 	for _, value := range int64Values {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewCompactJSONFormat(trans)
 		trans.WriteString(strconv.FormatInt(value, 10))
 		s := trans.String()
@@ -264,13 +256,12 @@ func TestReadJSONProtocolI64(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, v)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteJSONProtocolDouble(t *testing.T) {
 	thetype := "double"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	for _, value := range doubleValues {
 		if e := p.WriteDouble(value); e != nil {
@@ -281,16 +272,16 @@ func TestWriteJSONProtocolDouble(t *testing.T) {
 		}
 		s := trans.String()
 		if math.IsInf(value, 1) {
-			if s != jsonQuote(types.JSON_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_INFINITY))
+			if s != jsonQuote(JSON_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_INFINITY))
 			}
 		} else if math.IsInf(value, -1) {
-			if s != jsonQuote(types.JSON_NEGATIVE_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY))
+			if s != jsonQuote(JSON_NEGATIVE_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NEGATIVE_INFINITY))
 			}
 		} else if math.IsNaN(value) {
-			if s != jsonQuote(types.JSON_NAN) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NAN))
+			if s != jsonQuote(JSON_NAN) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NAN))
 			}
 		} else {
 			if s != fmt.Sprint(value) {
@@ -303,15 +294,14 @@ func TestWriteJSONProtocolDouble(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadJSONProtocolDouble(t *testing.T) {
 	thetype := "double"
 	for _, value := range doubleValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewCompactJSONFormat(trans)
-		n := types.NewNumericFromDouble(value)
+		n := NewNumericFromDouble(value)
 		trans.WriteString(n.String())
 		s := trans.String()
 		v, e := p.ReadDouble()
@@ -339,13 +329,12 @@ func TestReadJSONProtocolDouble(t *testing.T) {
 			}
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteJSONProtocolFloat(t *testing.T) {
 	thetype := "float"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	for _, value := range floatValues {
 		if e := p.WriteFloat(value); e != nil {
@@ -356,16 +345,16 @@ func TestWriteJSONProtocolFloat(t *testing.T) {
 		}
 		s := trans.String()
 		if math.IsInf(float64(value), 1) {
-			if s != jsonQuote(types.JSON_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_INFINITY))
+			if s != jsonQuote(JSON_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_INFINITY))
 			}
 		} else if math.IsInf(float64(value), -1) {
-			if s != jsonQuote(types.JSON_NEGATIVE_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY))
+			if s != jsonQuote(JSON_NEGATIVE_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NEGATIVE_INFINITY))
 			}
 		} else if math.IsNaN(float64(value)) {
-			if s != jsonQuote(types.JSON_NAN) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NAN))
+			if s != jsonQuote(JSON_NAN) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NAN))
 			}
 		} else {
 			if s != fmt.Sprint(value) {
@@ -378,15 +367,14 @@ func TestWriteJSONProtocolFloat(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadJSONProtocolFloat(t *testing.T) {
 	thetype := "float"
 	for _, value := range floatValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewCompactJSONFormat(trans)
-		n := types.NewNumericFromFloat(value)
+		n := NewNumericFromFloat(value)
 		trans.WriteString(n.String())
 		s := trans.String()
 		v, e := p.ReadFloat()
@@ -414,13 +402,12 @@ func TestReadJSONProtocolFloat(t *testing.T) {
 			}
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteJSONProtocolString(t *testing.T) {
 	thetype := "string"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	for _, value := range stringValues {
 		if e := p.WriteString(value); e != nil {
@@ -439,13 +426,12 @@ func TestWriteJSONProtocolString(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadJSONProtocolString(t *testing.T) {
 	thetype := "string"
 	for _, value := range stringValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewCompactJSONFormat(trans)
 		trans.WriteString(jsonQuote(value))
 		s := trans.String()
@@ -461,7 +447,6 @@ func TestReadJSONProtocolString(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, *v1)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
@@ -471,7 +456,7 @@ func TestWriteJSONProtocolBinary(t *testing.T) {
 	b64value := make([]byte, base64.StdEncoding.EncodedLen(len(protocolBdata)))
 	base64.StdEncoding.Encode(b64value, value)
 	b64String := string(b64value)
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	if e := p.WriteBinary(value); e != nil {
 		t.Fatalf("Unable to write %s value %v due to error: %s", thetype, value, e.Error())
@@ -496,7 +481,6 @@ func TestWriteJSONProtocolBinary(t *testing.T) {
 			t.Fatalf("Invalid value for binary at %v\nexpected: \"%v\"\n   read: \"%v\"", k, v, v1[k])
 		}
 	}
-	trans.Close()
 }
 
 func TestReadJSONProtocolBinary(t *testing.T) {
@@ -505,7 +489,7 @@ func TestReadJSONProtocolBinary(t *testing.T) {
 	b64value := make([]byte, base64.StdEncoding.EncodedLen(len(protocolBdata)))
 	base64.StdEncoding.Encode(b64value, value)
 	b64String := string(b64value)
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
 	trans.WriteString(jsonQuote(b64String))
 	s := trans.String()
@@ -526,14 +510,13 @@ func TestReadJSONProtocolBinary(t *testing.T) {
 		t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, *v1)
 	}
 	trans.Reset()
-	trans.Close()
 }
 
 func TestWriteJSONProtocolList(t *testing.T) {
 	thetype := "list"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
-	p.WriteListBegin(types.Type(types.DOUBLE), len(doubleValues))
+	p.WriteListBegin(types.DOUBLE, len(doubleValues))
 	for _, value := range doubleValues {
 		if e := p.WriteDouble(value); e != nil {
 			t.Fatalf("Unable to write %s value %v due to error: %s", thetype, value, e.Error())
@@ -544,7 +527,7 @@ func TestWriteJSONProtocolList(t *testing.T) {
 		t.Fatalf("Unable to write %s due to error flushing: %s", thetype, e.Error())
 	}
 	str := trans.String()
-	str1 := new([]interface{})
+	str1 := new([]any)
 	err := json.Unmarshal([]byte(str), str1)
 	if err != nil {
 		t.Fatalf("Unable to decode %s, wrote: %s", thetype, str)
@@ -562,16 +545,16 @@ func TestWriteJSONProtocolList(t *testing.T) {
 	for k, value := range doubleValues {
 		s := l[k+2]
 		if math.IsInf(value, 1) {
-			if s.(string) != types.JSON_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_INFINITY), str)
+			if s.(string) != JSON_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_INFINITY), str)
 			}
 		} else if math.IsInf(value, 0) {
-			if s.(string) != types.JSON_NEGATIVE_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY), str)
+			if s.(string) != JSON_NEGATIVE_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NEGATIVE_INFINITY), str)
 			}
 		} else if math.IsNaN(value) {
-			if s.(string) != types.JSON_NAN {
-				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NAN), str)
+			if s.(string) != JSON_NAN {
+				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NAN), str)
 			}
 		} else {
 			if s.(float64) != value {
@@ -580,14 +563,13 @@ func TestWriteJSONProtocolList(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestWriteJSONProtocolSet(t *testing.T) {
 	thetype := "set"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
-	p.WriteSetBegin(types.Type(types.DOUBLE), len(doubleValues))
+	p.WriteSetBegin(types.DOUBLE, len(doubleValues))
 	for _, value := range doubleValues {
 		if e := p.WriteDouble(value); e != nil {
 			t.Fatalf("Unable to write %s value %v due to error: %s", thetype, value, e.Error())
@@ -598,7 +580,7 @@ func TestWriteJSONProtocolSet(t *testing.T) {
 		t.Fatalf("Unable to write %s due to error flushing: %s", thetype, e.Error())
 	}
 	str := trans.String()
-	str1 := new([]interface{})
+	str1 := new([]any)
 	err := json.Unmarshal([]byte(str), str1)
 	if err != nil {
 		t.Fatalf("Unable to decode %s, wrote: %s", thetype, str)
@@ -616,16 +598,16 @@ func TestWriteJSONProtocolSet(t *testing.T) {
 	for k, value := range doubleValues {
 		s := l[k+2]
 		if math.IsInf(value, 1) {
-			if s.(string) != types.JSON_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_INFINITY), str)
+			if s.(string) != JSON_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_INFINITY), str)
 			}
 		} else if math.IsInf(value, 0) {
-			if s.(string) != types.JSON_NEGATIVE_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY), str)
+			if s.(string) != JSON_NEGATIVE_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NEGATIVE_INFINITY), str)
 			}
 		} else if math.IsNaN(value) {
-			if s.(string) != types.JSON_NAN {
-				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NAN), str)
+			if s.(string) != JSON_NAN {
+				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NAN), str)
 			}
 		} else {
 			if s.(float64) != value {
@@ -634,14 +616,13 @@ func TestWriteJSONProtocolSet(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestWriteJSONProtocolMap(t *testing.T) {
 	thetype := "map"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewCompactJSONFormat(trans)
-	p.WriteMapBegin(types.Type(types.I32), types.Type(types.DOUBLE), len(doubleValues))
+	p.WriteMapBegin(types.I32, types.DOUBLE, len(doubleValues))
 	for k, value := range doubleValues {
 		if e := p.WriteI32(int32(k)); e != nil {
 			t.Fatalf("Unable to write %s key int32 value %v due to error: %s", thetype, k, e.Error())
@@ -686,15 +667,15 @@ func TestWriteJSONProtocolMap(t *testing.T) {
 		s := strconv.FormatFloat(dv, 'g', 10, 64)
 		if math.IsInf(value, 1) {
 			if !math.IsInf(dv, 1) {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(types.JSON_INFINITY))
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(JSON_INFINITY))
 			}
 		} else if math.IsInf(value, 0) {
 			if !math.IsInf(dv, 0) {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY))
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(JSON_NEGATIVE_INFINITY))
 			}
 		} else if math.IsNaN(value) {
 			if !math.IsNaN(dv) {
-				t.Fatalf("Bad value for %s at index %v  %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(types.JSON_NAN))
+				t.Fatalf("Bad value for %s at index %v  %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(JSON_NAN))
 			}
 		} else {
 			expected := strconv.FormatFloat(value, 'g', 10, 64)
@@ -711,7 +692,6 @@ func TestWriteJSONProtocolMap(t *testing.T) {
 	if err != nil {
 		t.Fatalf("Error while reading map end: %s", err.Error())
 	}
-	trans.Close()
 }
 
 func TestCompactJSONProtocolUnmatchedBeginEnd(t *testing.T) {
