@@ -17,6 +17,7 @@
 package thrift
 
 import (
+	"bytes"
 	"encoding/base64"
 	"encoding/json"
 	"fmt"
@@ -30,7 +31,7 @@ import (
 
 func TestWriteSimpleJSONProtocolBool(t *testing.T) {
 	thetype := "boolean"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	for _, value := range boolValues {
 		if e := p.WriteBool(value); e != nil {
@@ -49,18 +50,17 @@ func TestWriteSimpleJSONProtocolBool(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolBool(t *testing.T) {
 	thetype := "boolean"
 	for _, value := range boolValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewSimpleJSONFormat(trans)
 		if value {
-			trans.Write(types.JSON_TRUE)
+			trans.Write(JSON_TRUE)
 		} else {
-			trans.Write(types.JSON_FALSE)
+			trans.Write(JSON_FALSE)
 		}
 
 		s := trans.String()
@@ -75,13 +75,12 @@ func TestReadSimpleJSONProtocolBool(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, v)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteSimpleJSONProtocolByte(t *testing.T) {
 	thetype := "byte"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	for _, value := range byteValues {
 		if e := p.WriteByte(value); e != nil {
@@ -100,13 +99,12 @@ func TestWriteSimpleJSONProtocolByte(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolByte(t *testing.T) {
 	thetype := "byte"
 	for _, value := range byteValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewSimpleJSONFormat(trans)
 		trans.WriteString(strconv.Itoa(int(value)))
 
@@ -122,13 +120,12 @@ func TestReadSimpleJSONProtocolByte(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, v)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteSimpleJSONProtocolI16(t *testing.T) {
 	thetype := "int16"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	for _, value := range int16Values {
 		if e := p.WriteI16(value); e != nil {
@@ -147,13 +144,12 @@ func TestWriteSimpleJSONProtocolI16(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolI16(t *testing.T) {
 	thetype := "int16"
 	for _, value := range int16Values {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewSimpleJSONFormat(trans)
 		trans.WriteString(strconv.Itoa(int(value)))
 
@@ -169,13 +165,12 @@ func TestReadSimpleJSONProtocolI16(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, v)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteSimpleJSONProtocolI32(t *testing.T) {
 	thetype := "int32"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	for _, value := range int32Values {
 		if e := p.WriteI32(value); e != nil {
@@ -194,13 +189,12 @@ func TestWriteSimpleJSONProtocolI32(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolI32(t *testing.T) {
 	thetype := "int32"
 	for _, value := range int32Values {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewSimpleJSONFormat(trans)
 		trans.WriteString(strconv.Itoa(int(value)))
 
@@ -216,7 +210,6 @@ func TestReadSimpleJSONProtocolI32(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, v)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
@@ -224,7 +217,7 @@ func TestReadSimpleJSONProtocolI32Null(t *testing.T) {
 	thetype := "int32"
 	value := "null"
 
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	trans.WriteString(value)
 	s := trans.String()
@@ -237,12 +230,11 @@ func TestReadSimpleJSONProtocolI32Null(t *testing.T) {
 		t.Fatalf("Bad value for %s value %v, wrote: %v, received: %v", thetype, value, s, v)
 	}
 	trans.Reset()
-	trans.Close()
 }
 
 func TestWriteSimpleJSONProtocolI64(t *testing.T) {
 	thetype := "int64"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	for _, value := range int64Values {
 		if e := p.WriteI64(value); e != nil {
@@ -261,13 +253,12 @@ func TestWriteSimpleJSONProtocolI64(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolI64(t *testing.T) {
 	thetype := "int64"
 	for _, value := range int64Values {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewSimpleJSONFormat(trans)
 		trans.WriteString(strconv.FormatInt(value, 10))
 
@@ -283,7 +274,6 @@ func TestReadSimpleJSONProtocolI64(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, v)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
@@ -291,7 +281,7 @@ func TestReadSimpleJSONProtocolI64Null(t *testing.T) {
 	thetype := "int32"
 	value := "null"
 
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	trans.WriteString(value)
 	s := trans.String()
@@ -304,12 +294,11 @@ func TestReadSimpleJSONProtocolI64Null(t *testing.T) {
 		t.Fatalf("Bad value for %s value %v, wrote: %v, received: %v", thetype, value, s, v)
 	}
 	trans.Reset()
-	trans.Close()
 }
 
 func TestWriteSimpleJSONProtocolDouble(t *testing.T) {
 	thetype := "double"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	for _, value := range doubleValues {
 		if e := p.WriteDouble(value); e != nil {
@@ -320,16 +309,16 @@ func TestWriteSimpleJSONProtocolDouble(t *testing.T) {
 		}
 		s := trans.String()
 		if math.IsInf(value, 1) {
-			if s != jsonQuote(types.JSON_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_INFINITY))
+			if s != jsonQuote(JSON_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_INFINITY))
 			}
 		} else if math.IsInf(value, -1) {
-			if s != jsonQuote(types.JSON_NEGATIVE_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY))
+			if s != jsonQuote(JSON_NEGATIVE_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NEGATIVE_INFINITY))
 			}
 		} else if math.IsNaN(value) {
-			if s != jsonQuote(types.JSON_NAN) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NAN))
+			if s != jsonQuote(JSON_NAN) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NAN))
 			}
 		} else {
 			if s != fmt.Sprint(value) {
@@ -342,15 +331,14 @@ func TestWriteSimpleJSONProtocolDouble(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolDouble(t *testing.T) {
 	thetype := "double"
 	for _, value := range doubleValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewSimpleJSONFormat(trans)
-		n := types.NewNumericFromDouble(value)
+		n := NewNumericFromDouble(value)
 		trans.WriteString(n.String())
 
 		s := trans.String()
@@ -379,13 +367,12 @@ func TestReadSimpleJSONProtocolDouble(t *testing.T) {
 			}
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteSimpleJSONProtocolFloat(t *testing.T) {
 	thetype := "float"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	for _, value := range floatValues {
 		if e := p.WriteFloat(value); e != nil {
@@ -396,16 +383,16 @@ func TestWriteSimpleJSONProtocolFloat(t *testing.T) {
 		}
 		s := trans.String()
 		if math.IsInf(float64(value), 1) {
-			if s != jsonQuote(types.JSON_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_INFINITY))
+			if s != jsonQuote(JSON_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_INFINITY))
 			}
 		} else if math.IsInf(float64(value), -1) {
-			if s != jsonQuote(types.JSON_NEGATIVE_INFINITY) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY))
+			if s != jsonQuote(JSON_NEGATIVE_INFINITY) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NEGATIVE_INFINITY))
 			}
 		} else if math.IsNaN(float64(value)) {
-			if s != jsonQuote(types.JSON_NAN) {
-				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(types.JSON_NAN))
+			if s != jsonQuote(JSON_NAN) {
+				t.Fatalf("Bad value for %s %v, wrote: %v, expected: %v", thetype, value, s, jsonQuote(JSON_NAN))
 			}
 		} else {
 			if s != fmt.Sprint(value) {
@@ -418,15 +405,14 @@ func TestWriteSimpleJSONProtocolFloat(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolFloat(t *testing.T) {
 	thetype := "float"
 	for _, value := range floatValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewSimpleJSONFormat(trans)
-		n := types.NewNumericFromFloat(value)
+		n := NewNumericFromFloat(value)
 		trans.WriteString(n.String())
 
 		s := trans.String()
@@ -455,13 +441,12 @@ func TestReadSimpleJSONProtocolFloat(t *testing.T) {
 			}
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 
 func TestWriteSimpleJSONProtocolString(t *testing.T) {
 	thetype := "string"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	for _, value := range stringValues {
 		if e := p.WriteString(value); e != nil {
@@ -480,13 +465,12 @@ func TestWriteSimpleJSONProtocolString(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolString(t *testing.T) {
 	thetype := "string"
 	for _, value := range stringValues {
-		trans := NewMemoryBuffer()
+		trans := new(bytes.Buffer)
 		p := NewSimpleJSONFormat(trans)
 		trans.WriteString(jsonQuote(value))
 
@@ -503,14 +487,13 @@ func TestReadSimpleJSONProtocolString(t *testing.T) {
 			t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, *v1)
 		}
 		trans.Reset()
-		trans.Close()
 	}
 }
 func TestReadSimpleJSONProtocolStringNull(t *testing.T) {
 	thetype := "string"
 	value := "null"
 
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	trans.WriteString(value)
 	s := trans.String()
@@ -522,7 +505,6 @@ func TestReadSimpleJSONProtocolStringNull(t *testing.T) {
 		t.Fatalf("Bad value for %s value %v, wrote: %v, received: %v", thetype, value, s, v)
 	}
 	trans.Reset()
-	trans.Close()
 }
 
 func TestWriteSimpleJSONProtocolBinary(t *testing.T) {
@@ -531,7 +513,7 @@ func TestWriteSimpleJSONProtocolBinary(t *testing.T) {
 	b64value := make([]byte, base64.StdEncoding.EncodedLen(len(protocolBdata)))
 	base64.StdEncoding.Encode(b64value, value)
 	b64String := string(b64value)
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	if e := p.WriteBinary(value); e != nil {
 		t.Fatalf("Unable to write %s value %v due to error: %s", thetype, value, e.Error())
@@ -547,7 +529,6 @@ func TestWriteSimpleJSONProtocolBinary(t *testing.T) {
 	if err := json.Unmarshal([]byte(s), v1); err != nil || *v1 != b64String {
 		t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, *v1)
 	}
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolBinary(t *testing.T) {
@@ -556,7 +537,7 @@ func TestReadSimpleJSONProtocolBinary(t *testing.T) {
 	b64value := make([]byte, base64.StdEncoding.EncodedLen(len(protocolBdata)))
 	base64.StdEncoding.Encode(b64value, value)
 	b64String := string(b64value)
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	trans.WriteString(jsonQuote(b64String))
 	s := trans.String()
@@ -577,14 +558,13 @@ func TestReadSimpleJSONProtocolBinary(t *testing.T) {
 		t.Fatalf("Bad json-decoded value for %s %v, wrote: '%s', expected: '%v'", thetype, value, s, *v1)
 	}
 	trans.Reset()
-	trans.Close()
 }
 
 func TestReadSimpleJSONProtocolBinaryNull(t *testing.T) {
 	thetype := "binary"
 	value := "null"
 
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
 	trans.WriteString(value)
 	s := trans.String()
@@ -598,14 +578,13 @@ func TestReadSimpleJSONProtocolBinaryNull(t *testing.T) {
 		t.Fatalf("Bad value for %s value %v, wrote: %v, received: %v", thetype, value, s, v)
 	}
 	trans.Reset()
-	trans.Close()
 }
 
 func TestWriteSimpleJSONProtocolList(t *testing.T) {
 	thetype := "list"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
-	p.WriteListBegin(types.Type(types.DOUBLE), len(doubleValues))
+	p.WriteListBegin(types.DOUBLE, len(doubleValues))
 	for _, value := range doubleValues {
 		if e := p.WriteDouble(value); e != nil {
 			t.Fatalf("Unable to write %s value %v due to error: %s", thetype, value, e.Error())
@@ -616,7 +595,7 @@ func TestWriteSimpleJSONProtocolList(t *testing.T) {
 		t.Fatalf("Unable to write %s due to error flushing: %s", thetype, e.Error())
 	}
 	str := trans.String()
-	str1 := new([]interface{})
+	str1 := new([]any)
 	err := json.Unmarshal([]byte(str), str1)
 	if err != nil {
 		t.Fatalf("Unable to decode %s, wrote: %s", thetype, str)
@@ -625,7 +604,7 @@ func TestWriteSimpleJSONProtocolList(t *testing.T) {
 	if len(l) < 2 {
 		t.Fatalf("List must be at least of length two to include metadata")
 	}
-	if int(l[0].(float64)) != types.DOUBLE {
+	if int(l[0].(float64)) != int(types.DOUBLE) {
 		t.Fatal("Invalid type for list, expected: ", types.DOUBLE, ", but was: ", l[0])
 	}
 	if int(l[1].(float64)) != len(doubleValues) {
@@ -634,16 +613,16 @@ func TestWriteSimpleJSONProtocolList(t *testing.T) {
 	for k, value := range doubleValues {
 		s := l[k+2]
 		if math.IsInf(value, 1) {
-			if s.(string) != types.JSON_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_INFINITY), str)
+			if s.(string) != JSON_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_INFINITY), str)
 			}
 		} else if math.IsInf(value, 0) {
-			if s.(string) != types.JSON_NEGATIVE_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY), str)
+			if s.(string) != JSON_NEGATIVE_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NEGATIVE_INFINITY), str)
 			}
 		} else if math.IsNaN(value) {
-			if s.(string) != types.JSON_NAN {
-				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NAN), str)
+			if s.(string) != JSON_NAN {
+				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NAN), str)
 			}
 		} else {
 			if s.(float64) != value {
@@ -652,14 +631,13 @@ func TestWriteSimpleJSONProtocolList(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestWriteSimpleJSONProtocolSet(t *testing.T) {
 	thetype := "set"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
-	p.WriteSetBegin(types.Type(types.DOUBLE), len(doubleValues))
+	p.WriteSetBegin(types.DOUBLE, len(doubleValues))
 	for _, value := range doubleValues {
 		if e := p.WriteDouble(value); e != nil {
 			t.Fatalf("Unable to write %s value %v due to error: %s", thetype, value, e.Error())
@@ -670,7 +648,7 @@ func TestWriteSimpleJSONProtocolSet(t *testing.T) {
 		t.Fatalf("Unable to write %s due to error flushing: %s", thetype, e.Error())
 	}
 	str := trans.String()
-	str1 := new([]interface{})
+	str1 := new([]any)
 	err := json.Unmarshal([]byte(str), str1)
 	if err != nil {
 		t.Fatalf("Unable to decode %s, wrote: %s", thetype, str)
@@ -679,7 +657,7 @@ func TestWriteSimpleJSONProtocolSet(t *testing.T) {
 	if len(l) < 2 {
 		t.Fatalf("Set must be at least of length two to include metadata")
 	}
-	if int(l[0].(float64)) != types.DOUBLE {
+	if int(l[0].(float64)) != int(types.DOUBLE) {
 		t.Fatal("Invalid type for set, expected: ", types.DOUBLE, ", but was: ", l[0])
 	}
 	if int(l[1].(float64)) != len(doubleValues) {
@@ -688,16 +666,16 @@ func TestWriteSimpleJSONProtocolSet(t *testing.T) {
 	for k, value := range doubleValues {
 		s := l[k+2]
 		if math.IsInf(value, 1) {
-			if s.(string) != types.JSON_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_INFINITY), str)
+			if s.(string) != JSON_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_INFINITY), str)
 			}
 		} else if math.IsInf(value, 0) {
-			if s.(string) != types.JSON_NEGATIVE_INFINITY {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY), str)
+			if s.(string) != JSON_NEGATIVE_INFINITY {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NEGATIVE_INFINITY), str)
 			}
 		} else if math.IsNaN(value) {
-			if s.(string) != types.JSON_NAN {
-				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(types.JSON_NAN), str)
+			if s.(string) != JSON_NAN {
+				t.Fatalf("Bad value for %s at index %v  %v, wrote: %q, expected: %q, originally wrote: %q", thetype, k, value, s, jsonQuote(JSON_NAN), str)
 			}
 		} else {
 			if s.(float64) != value {
@@ -706,14 +684,13 @@ func TestWriteSimpleJSONProtocolSet(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestWriteSimpleJSONProtocolMap(t *testing.T) {
 	thetype := "map"
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := NewSimpleJSONFormat(trans)
-	p.WriteMapBegin(types.Type(types.I32), types.Type(types.DOUBLE), len(doubleValues))
+	p.WriteMapBegin(types.I32, types.DOUBLE, len(doubleValues))
 	for k, value := range doubleValues {
 		if e := p.WriteI32(int32(k)); e != nil {
 			t.Fatalf("Unable to write %s key int32 value %v due to error: %s", thetype, k, e.Error())
@@ -737,10 +714,10 @@ func TestWriteSimpleJSONProtocolMap(t *testing.T) {
 	expectedKeyType, _ := strconv.Atoi(l[0])
 	expectedValueType, _ := strconv.Atoi(l[1])
 	expectedSize, _ := strconv.Atoi(l[2])
-	if expectedKeyType != types.I32 {
+	if expectedKeyType != int(types.I32) {
 		t.Fatal("Expected map key type ", types.I32, ", but was ", l[0])
 	}
-	if expectedValueType != types.DOUBLE {
+	if expectedValueType != int(types.DOUBLE) {
 		t.Fatal("Expected map value type ", types.DOUBLE, ", but was ", l[1])
 	}
 	if expectedSize != len(doubleValues) {
@@ -758,16 +735,16 @@ func TestWriteSimpleJSONProtocolMap(t *testing.T) {
 		}
 		s := strv
 		if math.IsInf(value, 1) {
-			if s != jsonQuote(types.JSON_INFINITY) {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(types.JSON_INFINITY))
+			if s != jsonQuote(JSON_INFINITY) {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(JSON_INFINITY))
 			}
 		} else if math.IsInf(value, 0) {
-			if s != jsonQuote(types.JSON_NEGATIVE_INFINITY) {
-				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(types.JSON_NEGATIVE_INFINITY))
+			if s != jsonQuote(JSON_NEGATIVE_INFINITY) {
+				t.Fatalf("Bad value for %s at index %v %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(JSON_NEGATIVE_INFINITY))
 			}
 		} else if math.IsNaN(value) {
-			if s != jsonQuote(types.JSON_NAN) {
-				t.Fatalf("Bad value for %s at index %v  %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(types.JSON_NAN))
+			if s != jsonQuote(JSON_NAN) {
+				t.Fatalf("Bad value for %s at index %v  %v, wrote: %v, expected: %v", thetype, k, value, s, jsonQuote(JSON_NAN))
 			}
 		} else {
 			expected := strconv.FormatFloat(value, 'g', 10, 64)
@@ -781,11 +758,10 @@ func TestWriteSimpleJSONProtocolMap(t *testing.T) {
 		}
 		trans.Reset()
 	}
-	trans.Close()
 }
 
 func TestWriteSimpleJSONProtocolSafePeek(t *testing.T) {
-	trans := NewMemoryBuffer()
+	trans := new(bytes.Buffer)
 	p := newSimpleJSONFormat(trans)
 	trans.Write([]byte{'a', 'b'})
 
