@@ -24,9 +24,9 @@ import (
 // order to perform io and message processing
 type ProcessorFunction interface {
 	// Read a serializable message from the protocol.
-	Read(prot Decoder) (Struct, Exception)
+	Read(prot Decoder) (Struct, error)
 	// RunContext processes a message handing it to the client handler.
 	RunContext(ctx context.Context, args Struct) (WritableStruct, ApplicationException)
 	// Write a serializable response
-	Write(seqID int32, result WritableStruct, prot Encoder) Exception
+	Write(seqID int32, result WritableStruct, prot Encoder) error
 }
