@@ -14,12 +14,16 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * @package thrift
  */
 
+// @oss-enable: use namespace FlibSL\{C, Math, Str, Vec};
+
+<<Oncalls('thrift')>> // @oss-disable
 interface IThriftAsyncProcessor extends IThriftProcessor {
   public function processAsync(
     TProtocol $input,
     TProtocol $output,
+    ?string $fname = null,
+    ?int $rseqid = null,
   ): Awaitable<bool>;
 }
