@@ -14,11 +14,13 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * @package thrift
  */
 
+// @oss-enable: use namespace FlibSL\{C, Math, Str, Vec};
+
 /**
  * Base interface for Thrift service handlers/clients
  */
+<<Oncalls('thrift')>> // @oss-disable
 interface IThriftIf {
 }
