@@ -14,17 +14,19 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * @package thrift.protocol
  */
 
+// @oss-enable: use namespace FlibSL\{C, Math, Str, Vec};
+
 /**
  * Protocol factory creates protocol objects from transports
  */
+<<Oncalls('thrift')>> // @oss-disable
 interface TProtocolFactory {
   /**
    * Build a protocol from the base transport
    *
    * @return TProtocol protocol
    */
-  public function getProtocol(TTransport $trans): TProtocol;
+  public function getProtocol(TTransport $trans)[read_globals]: TProtocol;
 }
