@@ -14,9 +14,10 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * @package thrift.protocol.binary
  */
 
+// @oss-enable: use namespace FlibSL\{C, Math, Str, Vec};
+
 /**
  * Do not use this class.
  *
@@ -24,12 +25,6 @@
  * or TBinaryProtocolUnaccelerated
  * @deprecated
  */
-class TBinaryProtocol extends TBinaryProtocolAccelerated {
-  public function __construct(
-    $trans,
-    $strict_read = false,
-    $strict_write = true,
-  ) {
-    parent::__construct($trans, $strict_read, $strict_write);
-  }
+<<Oncalls('thrift')>> // @oss-disable
+final class TBinaryProtocol extends TBinaryProtocolAccelerated {
 }
