@@ -21,9 +21,7 @@
 
 #include <folly/ExceptionWrapper.h>
 
-namespace thrift {
-namespace py3 {
-namespace exception {
+namespace thrift::py3::exception {
 
 // The only place this function is used it needs a shared_ptr, so may as
 // well just return one instead of a unique_ptr
@@ -34,6 +32,14 @@ std::shared_ptr<T> try_make_shared_exception(
   return e ? std::make_shared<T>(*e) : std::shared_ptr<T>();
 }
 
-} // namespace exception
-} // namespace py3
-} // namespace thrift
+template <class T>
+const T& unwrap_exception(const folly::exception_wrapper& exception) {
+  if (exception.is_compatible_with<T>()) {
+    return *exception.get_exception<T>();
+  }
+  // relying on reference lifetime extension ok because only called
+  // synchronously
+  return T();
+}
+
+} // namespace thrift::py3::exception
