@@ -17,12 +17,11 @@
 
 import asyncio
 import socket
-import sys
 import tempfile
 import time
 import unittest
 from pathlib import Path
-from typing import Optional, Sequence
+from typing import cast as typing_cast, Iterable, Optional, Sequence
 
 from derived.clients import DerivedTestingService
 from derived.services import DerivedTestingServiceInterface
@@ -36,11 +35,13 @@ from testing.services import (
     TestingServiceInterface,
 )
 from testing.types import Color, easy, HardError
+
 from thrift.py3.client import ClientType, get_client
 from thrift.py3.common import Priority, Protocol, RpcOptions
 from thrift.py3.exceptions import ApplicationError
 from thrift.py3.server import get_context, ServiceInterface, SocketAddress, ThriftServer
 from thrift.py3.test.cpp_handler import CppHandler
+from thrift.python.server import ServiceInterface as PythonServiceInterface
 
 
 class Handler(TestingServiceInterface):
@@ -108,29 +109,24 @@ class DerivedHandler(Handler, DerivedTestingServiceInterface):
 
 class TestServer:
     server: ThriftServer
-    serve_task: asyncio.Task
+    serve_task: Optional[asyncio.Task]
 
     def __init__(
         self,
         ip: Optional[str] = None,
         path: Optional["Path"] = None,
-        handler: ServiceInterface = Handler(),  # noqa: B008
+        handler: ServiceInterface | PythonServiceInterface = Handler(),  # noqa: B008
     ) -> None:
+        self.serve_task: Optional[asyncio.Task] = None
         self.server = ThriftServer(handler, ip=ip, path=path)
-        # pyre-fixme[4, 8]: The initialization below eliminates
-        #                   the pyre[13] error, but results in
-        #                   pyre[4] and pyre[8] errors.
-        #                   __aenter__ sets the required value.
-        self.serve_task = None
 
     async def __aenter__(self) -> SocketAddress:
         self.serve_task = asyncio.get_event_loop().create_task(self.server.serve())
         return await self.server.get_address()
 
-    # pyre-fixme[2]: Parameter must be annotated.
-    async def __aexit__(self, *exc_info) -> None:
+    async def __aexit__(self, *exc_info: Iterable[object]) -> None:
         self.server.stop()
-        await self.serve_task
+        await typing_cast(asyncio.Task, self.serve_task)
 
 
 # These tests do not cleanly convert to unittest.IsolateAsyncioTestCase
@@ -139,9 +135,6 @@ class ClientServerTests(unittest.TestCase):
     These are tests where a client and server talk to each other
     """
 
-    # pyre-fixme[56]: Argument `sys.version_info[slice(None, 2, None)] < (3, 7)` to
-    #  decorator factory `unittest.skipIf` could not be resolved in a global scope.
-    @unittest.skipIf(sys.version_info[:2] < (3, 7), "Requires py3.7")
     def test_get_context(self) -> None:
         loop = asyncio.get_event_loop()
 
@@ -366,21 +359,6 @@ class ClientServerTests(unittest.TestCase):
 
         loop.run_until_complete(inner_test())
 
-    def test_non_utf8_exception_message(self) -> None:
-        loop = asyncio.get_event_loop()
-
-        async def inner_test() -> None:
-            async with TestServer(handler=CppHandler()) as sa:
-                ip, port = sa.ip, sa.port
-                assert ip and port
-                async with get_client(TestingService, host=ip, port=port) as client:
-                    with self.assertRaises(HardError):
-                        await client.hard_error(True)
-                    with self.assertRaises(UnicodeDecodeError):
-                        await client.hard_error(False)
-
-        loop.run_until_complete(inner_test())
-
     def test_renamed_func(self) -> None:
         loop = asyncio.get_event_loop()
 
@@ -497,6 +475,18 @@ class ClientServerTests(unittest.TestCase):
         loop.run_until_complete(inner_test())
 
 
+class Utf8Test(unittest.IsolatedAsyncioTestCase):
+    async def test_non_utf8_exception_message(self) -> None:
+        async with TestServer(handler=CppHandler()) as sa:
+            ip, port = sa.ip, sa.port
+            assert ip and port
+            async with get_client(TestingService, host=ip, port=port) as client:
+                with self.assertRaises(HardError):
+                    await client.hard_error(True)
+                with self.assertRaises(UnicodeDecodeError):
+                    await client.hard_error(False)
+
+
 class StackHandler(StackServiceInterface):
     async def add_to(self, lst: Sequence[int], value: int) -> Sequence[int]:
         return [x + value for x in lst]
