@@ -19,6 +19,8 @@ import pickle
 import string
 import unittest
 
+from typing import cast, ItemsView, Iterator, KeysView, Optional, ValuesView
+
 from thrift.python.mutable_containers import (
     MapItemsView,
     MapKeysView,
@@ -28,7 +30,7 @@ from thrift.python.mutable_containers import (
 )
 
 from thrift.python.mutable_typeinfos import MutableListTypeInfo
-from thrift.python.mutable_types import to_thrift_list
+from thrift.python.mutable_types import to_thrift_list, to_thrift_map, to_thrift_set
 
 from thrift.python.types import typeinfo_i32, typeinfo_string
 
@@ -42,6 +44,218 @@ def _create_MutableMap_str_i32(map: dict[str, int]) -> MutableMap[str, int]:
     return MutableMap(typeinfo_string, typeinfo_i32, map)
 
 
+class MutableMapTypeHints(unittest.TestCase):
+    """
+    Tests type hints for MutableMap class.
+
+    This test suite checks that Pyre correctly identifies and raises errors
+    when there are type mismatches in function calls or assignments.
+
+    Casting an object to MutableMap[K, V] is sufficient to check the type hints.
+
+    The `pyre-ignore` comments in the test are intended to suppress real errors,
+    as we run Pyre with the `--report-unused-ignores` flag.
+    """
+
+    def test_type_hints(self) -> None:
+        mutable_map = cast(MutableMap[str, int], object())
+        try:
+            ### get() ###
+            v1: Optional[int] = mutable_map.get("key")  # noqa
+
+            # pyre-ignore[9]: v2 is type `int` but is used as type `Optional[int]`
+            v2: int = mutable_map.get("key")  # noqa
+
+            v3: int = mutable_map.get("key", 42)  # noqa
+            v4: int | str = mutable_map.get("key", "value")  # noqa
+
+            # pyre-ignore[6]: 1st positional argument, expected `str` but got `int`
+            v5: int = mutable_map.get(999, 42)  # noqa
+
+            ###################################################################
+
+            ### __setitem__() ###
+            mutable_map["key"] = 1
+
+            # pyre-ignore[6]: expected `int` but got `str`
+            mutable_map["key"] = "value"
+            # pyre-ignore[6]: expected `int` but got `float`
+            mutable_map["key"] = 1.0
+
+            # pyre-ignore[6]: expected `str` but got `int`
+            mutable_map[1] = 1
+            # pyre-ignore[6]: expected `str` but got `bytes`
+            mutable_map[b"key"] = 1
+
+            # pyre-ignore[6]: expected `str` but got `int` and expected `int` but got `str`
+            mutable_map[1] = "value"
+
+            ###################################################################
+
+            ### __contains__() ###
+            _ = 1 in mutable_map
+            _ = "key" in mutable_map
+            _ = b"key" in mutable_map
+
+            ###################################################################
+
+            ### __delitem() ###
+            del mutable_map["key"]
+
+            # pyre-ignore[6]: expected `str` but got `int`
+            del mutable_map[1]
+            # pyre-ignore[6]: expected `str` but got `bytes`
+            del mutable_map[b"key"]
+
+            ###################################################################
+
+            ### pop() ####
+            v6: int = mutable_map.pop("key")  # noqa
+
+            # pyre-ignore[6]: expected `str` but got `bytes`
+            v7: int = mutable_map.pop(b"key")  # noqa
+            # pyre-ignore[9]: v8 type `str` but is used as type `int`
+            v8: str = mutable_map.pop("key")  # noqa
+
+            v9: int = mutable_map.pop("key", 42)  # noqa
+            v10: int | float = mutable_map.pop("key", 42.0)  # noqa
+
+            ###################################################################
+
+            ### keys() ####
+
+            v11: MapKeysView[str] = mutable_map.keys()
+            v12: KeysView[str] = mutable_map.keys()  # noqa
+
+            # pyre-ignore[9]: v13 is type `MapKeysView[int]` but is used as type
+            #  `MapKeysView[str]`
+            v13: MapKeysView[int] = mutable_map.keys()  # noqa
+
+            keys_iter_1: Iterator[str] = iter(v11)  # noqa
+
+            # pyre-ignore[9]: keys_iter_2 is type `Iterator[int]` but is used as
+            #  type `Iterator[str]`
+            keys_iter_2: Iterator[int] = iter(v11)  # noqa
+
+            ###################################################################
+
+            ### items() ####
+
+            v14: MapItemsView[str, int] = mutable_map.items()
+            v15: ItemsView[str, int] = mutable_map.items()  # noqa
+
+            # pyre-ignore[9]: v16 is type `MapItemsView[str, str]` but is used
+            #  as type `MapItemsView[str, int]`
+            v16: MapItemsView[str, str] = mutable_map.items()  # noqa
+
+            items_iter_1: Iterator[tuple[str, int]] = iter(v14)  # noqa
+
+            # pyre-ignore[9]: items_iter_2 is type `Iterator[Tuple[str, str]]`
+            #  but is used as type `Iterator[Tuple[str, int]]`
+            items_iter_2: Iterator[tuple[str, str]] = iter(v14)  # noqa
+
+            ###################################################################
+
+            ### values() ####
+
+            v17: MapValuesView[int] = mutable_map.values()
+            v18: ValuesView[int] = mutable_map.values()  # noqa
+
+            # pyre-ignore[9]: v19 is type `MapValuesView[str]` but is used as
+            #  type `MapValuesView[int]`
+            v19: MapValuesView[str] = mutable_map.values()  # noqa
+
+            values_iter_1: Iterator[int] = iter(v17)  # noqa
+
+            # pyre-ignore[9]: values_iter_2 is type `Iterator[str]` but is used
+            #  as type `Iterator[int]`
+            values_iter_2: Iterator[str] = iter(v17)  # noqa
+
+        except Exception:
+            pass
+
+    def test_type_hints_with_container_value(self) -> None:
+        mutable_map = cast(MutableMap[str, MutableList[int]], object())
+        try:
+            ### get() ###
+            v1: Optional[MutableList[int]] = mutable_map.get("key")  # noqa
+
+            # pyre-ignore[9]: v2 is type `MutableList[int]` but is used as type `Optional[MutableList[int]]`.
+            v2: MutableList[int] = mutable_map.get("key")  # noqa
+
+            default_value_1 = cast(MutableList[int], object())
+            v3: MutableList[int] = mutable_map.get("key", default_value_1)  # noqa
+
+            default_value_2 = cast(MutableList[str], object())
+            v4: MutableList[int] | MutableList[str] = mutable_map.get(  # noqa
+                "key", default_value_2
+            )
+
+            # pyre-ignore[6]: 1st positional argument, expected `str` but got `int`
+            v5: MutableList[int] = mutable_map.get(999, default_value_1)
+
+            ###################################################################
+
+            ### __setitem__() ###
+            mutable_list_int = cast(MutableList[int], object())
+            mutable_list_str = cast(MutableList[str], object())
+
+            mutable_map["key"] = to_thrift_list([])
+            mutable_map["key"] = mutable_list_int
+
+            # pyre-ignore[6]: expected `MutableList[int]` but got `MutableList[str]`
+            mutable_map["key"] = mutable_list_str
+            # pyre-ignore[6]: expected `MutableList[int]` but got `List[int]`
+            mutable_map["key"] = [1, 2, 3]
+
+            # pyre-ignore[6]: expected `str` but got `int`
+            mutable_map[1] = mutable_list_int
+            # pyre-ignore[6]: expected `str` but got `bytes`
+            mutable_map[b"key"] = mutable_list_int
+
+            # pyre-ignore[6]: expected `str` but got `int` and expected `MutableList[int]` but got `MutableList[str]`
+            mutable_map[1] = mutable_list_str
+
+            ###################################################################
+
+            ### __contains__() ###
+            _ = [1, 2, 3] in mutable_map
+            _ = to_thrift_list([]) in mutable_map
+            _ = to_thrift_set(set()) in mutable_map
+            _ = to_thrift_map({}) in mutable_map
+
+            ###################################################################
+
+            ### __delitem() ###
+            del mutable_map["key"]
+
+            # pyre-ignore[6]: expected `str` but got `List[int]`
+            del mutable_map[[1, 2, 3]]
+            # pyre-ignore[6]: expected `str` but got `_ThriftListWrapper`
+            del mutable_map[to_thrift_list([])]
+
+            ###################################################################
+
+            ### pop() ####
+            v6: MutableList[int] = mutable_map.pop("key")  # noqa
+
+            # pyre-ignore[6]: expected `str` but got `bytes`
+            v7: MutableList[int] = mutable_map.pop(b"key")  # noqa
+            # pyre-ignore[9]: type `MutableList[str]` but is used as type `MutableList[int]`
+            v8: MutableList[str] = mutable_map.pop("key")  # noqa
+
+            mutable_list_int = cast(MutableList[int], object())
+            v9: MutableList[int] = mutable_map.pop("key", mutable_list_int)  # noqa
+
+            mutable_list_str = cast(MutableList[str], object())
+            v10: MutableList[int] | MutableList[str] = mutable_map.pop(  # noqa
+                "key", mutable_list_str
+            )
+
+        except Exception:
+            pass
+
+
 class MutableMapTest(unittest.TestCase):
     def test_smoke(self) -> None:
         mutable_map = MutableMap(typeinfo_string, typeinfo_i32, {})
@@ -246,6 +460,7 @@ class MutableMapTest(unittest.TestCase):
         mutable_map = _create_MutableMap_str_i32({})
 
         self.assertIsInstance(mutable_map.keys(), MapKeysView)
+        self.assertIsInstance(mutable_map.keys(), collections.abc.KeysView)
         self.assertEqual(0, len(mutable_map.keys()))
 
         mutable_map["A"] = 65
@@ -275,6 +490,7 @@ class MutableMapTest(unittest.TestCase):
         # made to the map will be reflected in the `keys_view`.
         keys_view = mutable_map.keys()
         self.assertIsInstance(keys_view, MapKeysView)
+        self.assertIsInstance(keys_view, collections.abc.KeysView)
         self.assertEqual(0, len(keys_view))
 
         mutable_map["A"] = 65
@@ -299,6 +515,7 @@ class MutableMapTest(unittest.TestCase):
         mutable_map = _create_MutableMap_str_i32({})
 
         self.assertIsInstance(mutable_map.items(), MapItemsView)
+        self.assertIsInstance(mutable_map.items(), collections.abc.ItemsView)
         self.assertEqual(0, len(mutable_map.items()))
 
         mutable_map["A"] = 65
@@ -331,6 +548,7 @@ class MutableMapTest(unittest.TestCase):
         # made to the map will be reflected in the `items_view`.
         items_view = mutable_map.items()
         self.assertIsInstance(items_view, MapItemsView)
+        self.assertIsInstance(items_view, collections.abc.ItemsView)
         self.assertEqual(0, len(items_view))
 
         mutable_map["A"] = 65
@@ -351,10 +569,45 @@ class MutableMapTest(unittest.TestCase):
         self.assertNotIn(("A", 65), items_view)
         self.assertNotIn(("a", 97), items_view)
 
+    def test_items_view_container_value(self) -> None:
+        mutable_map: MutableMap[str, MutableList[int]] = MutableMap(
+            typeinfo_string,
+            MutableListTypeInfo(typeinfo_i32),
+            {},
+        )
+
+        # The `items()` method returns a view of the map's items. Any modifications
+        # made to the map will be reflected in the `items_view`.
+        items_view = mutable_map.items()
+        self.assertIsInstance(items_view, MapItemsView)
+        self.assertIsInstance(items_view, collections.abc.ItemsView)
+        self.assertEqual(0, len(items_view))
+
+        mutable_map["A"] = to_thrift_list([1, 2, 3])
+
+        self.assertEqual(1, len(items_view))
+        self.assertIn(("A", [1, 2, 3]), items_view)
+        self.assertNotIn(("A", [3, 2, 1]), items_view)
+        self.assertNotIn(("a", [3, 2, 1]), items_view)
+
+        mutable_map["a"] = to_thrift_list([3, 2, 1])
+
+        self.assertEqual(2, len(items_view))
+        self.assertIn(("A", [1, 2, 3]), items_view)
+        self.assertNotIn(("A", [3, 2, 1]), items_view)
+        self.assertIn(("a", [3, 2, 1]), items_view)
+
+        mutable_map.clear()
+
+        self.assertEqual(0, len(items_view))
+        self.assertNotIn(("A", [1, 2, 3]), items_view)
+        self.assertNotIn(("a", [3, 2, 1]), items_view)
+
     def test_values(self) -> None:
         mutable_map = _create_MutableMap_str_i32({})
 
         self.assertIsInstance(mutable_map.values(), MapValuesView)
+        self.assertIsInstance(mutable_map.values(), collections.abc.ValuesView)
         self.assertEqual(0, len(mutable_map.values()))
 
         mutable_map["A"] = 65
@@ -387,6 +640,7 @@ class MutableMapTest(unittest.TestCase):
         # made to the map will be reflected in the `values_view`.
         values_view = mutable_map.values()
         self.assertIsInstance(values_view, MapValuesView)
+        self.assertIsInstance(values_view, collections.abc.ValuesView)
         self.assertEqual(0, len(values_view))
 
         mutable_map["A"] = 65
@@ -411,9 +665,45 @@ class MutableMapTest(unittest.TestCase):
         self.assertNotIn(65, values_view)
         self.assertNotIn(97, values_view)
 
+    def test_values_view_with_container_value(self) -> None:
+        mutable_map: MutableMap[str, MutableList[int]] = MutableMap(
+            typeinfo_string,
+            MutableListTypeInfo(typeinfo_i32),
+            {},
+        )
+
+        mutable_list_1: MutableList[int] = MutableList(typeinfo_i32, [1, 2, 3])
+        mutable_list_2: MutableList[int] = MutableList(typeinfo_i32, [4, 5, 6])
+
+        # The `values()` method returns a view of the map's values. Any modifications
+        # made to the map will be reflected in the `values_view`.
+        values_view = mutable_map.values()
+        self.assertIsInstance(values_view, MapValuesView)
+        self.assertIsInstance(values_view, collections.abc.ValuesView)
+        self.assertEqual(0, len(values_view))
+
+        mutable_map["A"] = to_thrift_list([1, 2, 3])
+
+        self.assertEqual(1, len(values_view))
+        self.assertIn([1, 2, 3], values_view)
+        self.assertIn(mutable_list_1, values_view)
+        self.assertNotIn([4, 5, 6], values_view)
+        self.assertNotIn(mutable_list_2, values_view)
+
+        mutable_map["a"] = to_thrift_list([4, 5, 6])
+
+        self.assertEqual(2, len(values_view))
+        self.assertIn([1, 2, 3], values_view)
+        self.assertIn(mutable_list_1, values_view)
+        self.assertIn([4, 5, 6], values_view)
+        self.assertIn(mutable_list_2, values_view)
+
+        mutable_map.clear()
+        self.assertEqual(0, len(values_view))
+
     def test_mutation_via_values_view(self) -> None:
         internal_map = {1: [1, 2, 3], 2: [4, 5], 3: [6]}
-        mutable_map = MutableMap(
+        mutable_map: MutableMap[int, MutableList[int]] = MutableMap(
             typeinfo_i32,
             MutableListTypeInfo(typeinfo_i32),
             # pyre-ignore[6]
@@ -440,7 +730,7 @@ class MutableMapTest(unittest.TestCase):
 
     def test_pickle_round_trip(self) -> None:
         internal_map = {1: [1, 2, 3], 2: [4, 5], 3: [6]}
-        mutable_map = MutableMap(
+        mutable_map: MutableMap[int, MutableList[int]] = MutableMap(
             typeinfo_i32,
             MutableListTypeInfo(typeinfo_i32),
             # pyre-ignore[6]
@@ -475,6 +765,7 @@ class MutableMapTest(unittest.TestCase):
 
         # Wrong key type raises a KeyError not a TypeError
         with self.assertRaisesRegex(KeyError, "123"):
+            # pyre-ignore[6]: Intentional for test
             mutable_map.pop(123)
 
     def test_popitem(self) -> None:
@@ -501,6 +792,7 @@ class MutableMapTest(unittest.TestCase):
         with self.assertRaisesRegex(
             TypeError, "Expected type <class 'str'>, got: <class 'int'>"
         ):
+            # pyre-ignore[6]: Intentional for test
             mutable_map.setdefault(123, 999)
 
         # Default value for `setdefault()` is `None`, however, `None` is not a
@@ -511,7 +803,7 @@ class MutableMapTest(unittest.TestCase):
             mutable_map.setdefault("new-key")
 
     def test_setdefault_container_value(self) -> None:
-        mutable_map = MutableMap(
+        mutable_map: MutableMap[str, MutableList[int]] = MutableMap(
             typeinfo_string,
             MutableListTypeInfo(typeinfo_i32),
             {},
@@ -570,6 +862,7 @@ class MutableMapTest(unittest.TestCase):
             TypeError, "Expected type <class 'str'>, got: <class 'int'>"
         ):
             # `{"B": 66}` raises before keyword argument
+            # pyre-ignore[6]: Intentional for test
             mutable_map.update({1: 999}, A=65)
 
         self.assertEqual({"A": 65, "a": 97}, mutable_map)
@@ -578,6 +871,37 @@ class MutableMapTest(unittest.TestCase):
             TypeError, "not a <class 'int'>, is actually of type <class 'str'>"
         ):
             # `{"B": 66}` updates the mutable map before keyword argument raises
+            # pyre-ignore[6]: Intentional for test
             mutable_map.update({"B": 66}, x="Not an Integer")
 
         self.assertEqual({"A": 65, "a": 97, "B": 66}, mutable_map)
+
+    def test_match(self) -> None:
+        mutable_map = _create_MutableMap_str_i32({})
+
+        mutable_map["A"] = 65
+        mutable_map["a"] = 97
+
+        match mutable_map:
+            case {"A": 65, "B": 97}:
+                self.fail("shouldn't happen")
+            case {"B": 65}:
+                self.fail("shouldn't happen")
+            case {"A": 66, **rest}:
+                self.fail("shouldn't happen")
+            case {"A": 65, **rest}:
+                self.assertEqual({"a": 97}, rest)
+            case _:
+                self.fail("shouldn't happen")
+
+        match mutable_map:
+            case {"A": 65, "B": 97}:
+                self.fail("shouldn't happen")
+            case {"B": 65}:
+                self.fail("shouldn't happen")
+            case {"A": 66, **rest}:
+                self.fail("shouldn't happen")
+            case {"A": 65, "a": x}:
+                self.assertEqual(x, 97)
+            case _:
+                self.fail("shouldn't happen")
