@@ -21,7 +21,7 @@ import copy
 import math
 import types
 import unittest
-from typing import Type, TypeVar
+from typing import Callable, cast, Type, TypeVar
 from unittest import mock
 
 import testing.thrift_mutable_types as mutable_test_types
@@ -34,10 +34,10 @@ from folly.iobuf import IOBuf
 
 from parameterized import parameterized_class
 
-from testing.thrift_mutable_types import __Reserved as DoubleUnderscoreReservedMutable
+from testing.thrift_mutable_types import _Reserved as _ReservedMutable
 
 from testing.thrift_types import (
-    __Reserved as DoubleUnderscoreReserved,
+    _Reserved,
     Color,
     ComplexRef,
     customized,
@@ -48,6 +48,7 @@ from testing.thrift_types import (
     IOBufListStruct,
     Kind,
     ListTypes,
+    mixed,
     Nested1,
     Nested2,
     Nested3,
@@ -61,6 +62,7 @@ from testing.thrift_types import (
     StructuredAnnotation,
     UnusedError,
 )
+from thrift.python.exceptions import GeneratedError
 from thrift.python.mutable_types import (
     _isset as mutable_isset,
     _ThriftListWrapper,
@@ -71,8 +73,7 @@ from thrift.python.mutable_types import (
     to_thrift_set,
 )
 
-from thrift.python.serializer import deserialize, serialize_iobuf
-from thrift.python.types import isset, update_nested_field
+from thrift.python.types import isset, Struct, StructOrUnion, update_nested_field
 
 ListT = TypeVar("ListT")
 SetT = TypeVar("SetT")
@@ -108,18 +109,20 @@ class StructTestsParameterized(unittest.TestCase):
         self.StructuredAnnotation: Type[StructuredAnnotation] = (
             self.test_types.StructuredAnnotation
         )
+        self.mixed: Type[mixed] = self.test_types.mixed
         self.is_mutable_run: bool = self.test_types.__name__.endswith(
             "thrift_mutable_types"
         )
         # pyre-ignore[8]: Intentional for test
-        self.DoubleUnderscoreReserved: Type[DoubleUnderscoreReserved] = (
-            DoubleUnderscoreReserved
-            if not self.is_mutable_run
-            else DoubleUnderscoreReservedMutable
+        self._Reserved: Type[_Reserved] = (
+            _Reserved if not self.is_mutable_run else _ReservedMutable
         )
         # pyre-ignore[16]: has no attribute `serializer_module`
         self.serializer: types.ModuleType = self.serializer_module
-        self.isset = mutable_isset if self.is_mutable_run else isset
+        # pyre-ignore[8]: has no attribute `serializer_module`
+        self.isset: Callable[[StructOrUnion | GeneratedError], dict[str, bool]] = (
+            mutable_isset if self.is_mutable_run else isset
+        )
 
     def to_list(self, list_data: list[ListT]) -> list[ListT] | _ThriftListWrapper:
         return to_thrift_list(list_data) if self.is_mutable_run else list_data
@@ -139,13 +142,11 @@ class StructTestsParameterized(unittest.TestCase):
 
     def test_isset_Error(self) -> None:
         e = self.UnusedError(message="ACK")
-        # pyre-ignore[6]: `mutable_isset` expected ...
         self.assertTrue(self.isset(e)["message"])
 
     def test_isset_Union(self) -> None:
         i = self.Integers(large=2)
         with self.assertRaises(TypeError):
-            # pyre-ignore[6]: for test
             self.isset(i)["large"]
 
     def test_no_dict(self) -> None:
@@ -153,7 +154,6 @@ class StructTestsParameterized(unittest.TestCase):
             self.easy().__dict__
         # TODO: make it a tuple instead of a list
         # self.assertIsInstance(easy().__class__.__slots__, tuple)
-        # pyre-ignore[16]
         self.assertIs(self.easy().__class__.__slots__, self.easy().__slots__)
         self.assertIs(self.easy(val=5).__slots__, self.easy(name="yo").__slots__)
 
@@ -242,6 +242,7 @@ class StructTestsParameterized(unittest.TestCase):
             self.easy(val=1, an_int=self.Integers(small=300), name=1)
 
     def test_iterate(self) -> None:
+        # pyre-ignore[28]: proving __mangled_str works
         x = self.Reserved(
             from_="hello",
             nonlocal_=3,
@@ -250,7 +251,7 @@ class StructTestsParameterized(unittest.TestCase):
             move="Qh4xe1",
             inst="foo",
             changes="bar",
-            _Reserved__mangled_str="secret",
+            __mangled_str="secret",
             _Reserved__mangled_int=42,
         )
         self.assertEqual(
@@ -305,7 +306,16 @@ class StructTestsParameterized(unittest.TestCase):
         self.assertIsNone(x.name)
         self.assertIsNotNone(x.an_int)
 
+    def test_call_bad_key(self) -> None:
+        x = self.easy()
+
+        err_type = AttributeError if self.is_mutable_run else TypeError
+        with self.assertRaisesRegex(err_type, "'easy' object .* attribute.* 'bad_key'"):
+            # pyre-ignore[28]: bad key for test
+            x = x(bad_key="foo")
+
     def test_reserved(self) -> None:
+        # pyre-ignore[28]: proving the mangling is optional
         x = self.Reserved(
             from_="hello",
             nonlocal_=3,
@@ -315,7 +325,7 @@ class StructTestsParameterized(unittest.TestCase):
             inst="foo",
             changes="bar",
             _Reserved__mangled_str="secret",
-            _Reserved__mangled_int=42,
+            __mangled_int=42,
         )
         self.assertEqual(x.from_, "hello")
         self.assertEqual(x.nonlocal_, 3)
@@ -329,7 +339,7 @@ class StructTestsParameterized(unittest.TestCase):
 
         self.assertEqual(x, x)
 
-        y = self.DoubleUnderscoreReserved(
+        y = self._Reserved(
             _Reserved__mangled_str="secret",
             _Reserved__mangled_int=42,
         )
@@ -366,6 +376,48 @@ class StructTestsParameterized(unittest.TestCase):
         dif_int = copy.copy(x.an_int)
         self.assertEqual(x.an_int, dif_int)
 
+    def test_defaulted_optional_field(self) -> None:
+        def assert_mixed(m: mixed) -> None:
+            self.assertIsNone(m.opt_field)
+            self.assertIsNone(m.opt_float)
+            self.assertIsNone(m.opt_int)
+            self.assertIsNone(m.opt_enum)
+
+        def assert_isset(m: mixed) -> None:
+            isset = self.isset(m)
+            for fld_name, _ in mixed:
+                if not fld_name.startswith("opt_"):
+                    continue
+                self.assertFalse(isset[fld_name], fld_name)
+
+        # constructor
+        m = self.mixed()
+        assert_mixed(m)
+        assert_isset(m)
+
+        # call operator
+        m = m(some_field_="don't care")
+        assert_mixed(m)
+        assert_isset(m)
+
+        # serialization round-trip
+        m = self.serializer.deserialize(self.mixed, self.serializer.serialize(m))
+        assert_mixed(m)
+        assert_isset(m)
+
+        ### Now with explicit `None` set
+        m = self.mixed(opt_field=None)
+        self.assertIsNone(m.opt_field)
+        self.assertFalse(self.isset(m)["opt_field"])
+
+        m = m(opt_field=None)
+        self.assertIsNone(m.opt_field)
+        self.assertFalse(self.isset(m)["opt_field"])
+
+        m = self.serializer.deserialize(self.mixed, self.serializer.serialize(m))
+        self.assertIsNone(m.opt_field)
+        self.assertFalse(self.isset(m)["opt_field"])
+
 
 class StructTestsImmutable(unittest.TestCase):
     """
@@ -695,3 +747,15 @@ class StructDeepcopyTests(unittest.TestCase):
         with self.assertRaises(TypeError):
             # TODO(ffrancet): pyre should complain about this
             z < y  # noqa: B015
+
+    def test_instance_base_class(self) -> None:
+        self.assertIsInstance(Nested1(), Struct)
+        self.assertIsInstance(Nested1(), Nested1)
+        self.assertNotIsInstance(Nested1(), Nested2)
+        self.assertNotIsInstance(3, Nested1)
+        self.assertNotIsInstance(3, Struct)
+        self.assertTrue(issubclass(Nested1, Struct))
+        self.assertFalse(issubclass(int, Struct))
+        self.assertFalse(issubclass(int, Nested1))
+        self.assertFalse(issubclass(Struct, Nested1))
+        self.assertFalse(issubclass(Nested1, Nested2))
