@@ -14,12 +14,15 @@
  * limitations under the License.
  */
 
+#include <thrift/lib/python/types.h>
+
 // To access Cython C APIs, such as create_IOBuf(). See `types.pxd`.
 #include <thrift/lib/python/types_api.h> // @manual
 
 #include <folly/Indestructible.h>
 #include <folly/Range.h>
 #include <folly/ScopeGuard.h>
+#include <folly/io/IOBuf.h>
 #include <folly/lang/New.h>
 #include <folly/python/import.h>
 #include <thrift/lib/cpp2/protocol/TableBasedSerializer.h>
@@ -314,9 +317,10 @@ PyObject* createStructTupleWithDefaultValues(
  *   * `""` (i.e., the empty string) for strings and `binary` fields (or an
  *      empty `IOBuf` if applicable).
  *
- * The default value for container types chang based if they are mutable or
- * not:
- *  Mutable:
+ * The default value for container types change based on whether they are
+ * mutable or not:
+ *
+ * Mutable:
  *    * An empty `list` for lists.
  *    * An empty `dict` for maps.
  *    * An empty `set` for sets.
@@ -832,13 +836,14 @@ detail::OptionalThriftValue getString(
  *
  * @throws if `value` cannot be copied to a new `PyBytesObject`.
  */
-void setString(void* objectPtr, const std::string& value) {
+void* setString(void* objectPtr, const std::string& value) {
   UniquePyObjectPtr bytesObj{
       PyBytes_FromStringAndSize(value.data(), value.size())};
   if (bytesObj == nullptr) {
     THRIFT_PY3_CHECK_ERROR();
   }
   setPyObject(objectPtr, std::move(bytesObj));
+  return nullptr;
 }
 
 detail::OptionalThriftValue getIOBuf(
@@ -850,7 +855,7 @@ detail::OptionalThriftValue getIOBuf(
              : detail::OptionalThriftValue{};
 }
 
-void setIOBuf(void* objectPtr, const folly::IOBuf& value) {
+void* setIOBuf(void* objectPtr, const folly::IOBuf& value) {
   ensureImportOrThrow();
   PyObject* buf = create_IOBuf(value.clone());
   UniquePyObjectPtr iobufObj{buf};
@@ -858,6 +863,7 @@ void setIOBuf(void* objectPtr, const folly::IOBuf& value) {
     THRIFT_PY3_CHECK_ERROR();
   }
   setPyObject(objectPtr, std::move(iobufObj));
+  return nullptr;
 }
 
 // This helper method for `MutableMapTypeInfo::write()` sorts the map keys and
@@ -1073,8 +1079,9 @@ class PrimitiveTypeInfoHelper final {
    *        converted to the corresponding Python type and pointed to by the
    *        given `PyObject*`.
    */
-  static void set(void* objectPtr, TCppType value) {
+  static void* set(void* objectPtr, TCppType value) {
     setPyObject(objectPtr, primitiveCppToPython(value));
+    return nullptr;
   }
 };
 
@@ -1083,7 +1090,7 @@ const detail::TypeInfo
     PrimitiveTypeInfoHelper<TCppType, TThriftProtocolTypeEnum>::kTypeInfo{
         /* .type */ TThriftProtocolTypeEnum,
         /* .get */ get,
-        /* .set */ reinterpret_cast<detail::VoidFuncPtr>(set),
+        /* .set */ reinterpret_cast<detail::VoidPtrFuncPtr>(set),
         /* .typeExt */ nullptr,
     };
 
@@ -1264,7 +1271,7 @@ detail::TypeInfo createImmutableStructTypeInfo(
       /* .type */ protocol::TType::T_STRUCT,
       /* .get */ getStruct,
       /* .set */
-      reinterpret_cast<detail::VoidFuncPtr>(
+      reinterpret_cast<detail::VoidPtrFuncPtr>(
           dynamicStructInfo.isUnion() ? setImmutableUnion : setImmutableStruct),
       /* .typeExt */ &dynamicStructInfo.getStructInfo(),
   };
@@ -1282,7 +1289,7 @@ detail::TypeInfo createMutableStructTypeInfo(
       /* .type */ protocol::TType::T_STRUCT,
       /* .get */ getMutableStruct,
       /* .set */
-      reinterpret_cast<detail::VoidFuncPtr>(
+      reinterpret_cast<detail::VoidPtrFuncPtr>(
           dynamicStructInfo.isUnion() ? setMutableUnion : setMutableStruct),
       /* .typeExt */ &dynamicStructInfo.getStructInfo(),
   };
@@ -1673,24 +1680,47 @@ const detail::StringFieldType ioBufFieldType =
 const detail::TypeInfo stringTypeInfo{
     /* .type */ protocol::TType::T_STRING,
     /* .get */ getString,
-    /* .set */ reinterpret_cast<detail::VoidFuncPtr>(setString),
+    /* .set */ reinterpret_cast<detail::VoidPtrFuncPtr>(setString),
     /* .typeExt */ &stringFieldType,
 };
 
 const detail::TypeInfo binaryTypeInfo{
     /* .type */ protocol::TType::T_STRING,
     /* .get */ getString,
-    /* .set */ reinterpret_cast<detail::VoidFuncPtr>(setString),
+    /* .set */ reinterpret_cast<detail::VoidPtrFuncPtr>(setString),
     /* .typeExt */ &binaryFieldType,
 };
 
 const detail::TypeInfo iobufTypeInfo{
     /* .type */ protocol::TType::T_STRING,
     /* .get */ getIOBuf,
-    /* .set */ reinterpret_cast<detail::VoidFuncPtr>(setIOBuf),
+    /* .set */ reinterpret_cast<detail::VoidPtrFuncPtr>(setIOBuf),
     /* .typeExt */ &ioBufFieldType,
 };
 
+PyObject* getStandardImmutableDefaultValuePtrForType(
+    const detail::TypeInfo& typeInfo) {
+  auto ret = getStandardImmutableDefaultValueForType(typeInfo);
+  return std::get<0>(ret).release();
+}
+PyObject* getStandardMutableDefaultValuePtrForType(
+    const detail::TypeInfo& typeInfo) {
+  auto ret = getStandardMutableDefaultValueForType(typeInfo);
+  return std::get<0>(ret).release();
+}
+
+void tag_object_as_sequence(PyTypeObject* type_object) {
+  DCHECK(PyType_Check(type_object));
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 10
+  type_object->tp_flags |= Py_TPFLAGS_SEQUENCE;
+#endif
+}
+void tag_object_as_mapping(PyTypeObject* type_object) {
+  DCHECK(PyType_Check(type_object));
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 10
+  type_object->tp_flags |= Py_TPFLAGS_MAPPING;
+#endif
+}
 } // namespace apache::thrift::python
 
 namespace apache::thrift::python::capi {
