@@ -14,7 +14,10 @@
  * limitations under the License.
  */
 
+use std::io;
 use std::io::Cursor;
+use std::num::ParseFloatError;
+use std::str::FromStr;
 
 use anyhow::anyhow;
 use anyhow::bail;
@@ -153,6 +156,25 @@ impl<B: BufMutExt> SimpleJsonProtocolSerializer<B> {
             _ => false,
         }
     }
+    #[inline]
+    fn write_floating_point_number<F, W>(&mut self, value: F, finite_writer: W)
+    where
+        F: num_traits::Float,
+        W: FnOnce(&mut CompactFormatter, &mut Writer<B>, F) -> io::Result<()>,
+    {
+        if value.is_infinite() {
+            if value.is_sign_positive() {
+                self.write_string("Infinity");
+            } else {
+                self.write_string("-Infinity");
+            }
+        } else if value.is_nan() {
+            self.write_string("NaN");
+        } else {
+            finite_writer(&mut CompactFormatter, &mut self.buffer, value)
+                .expect("Somehow failed to do \"io\" on a buffer");
+        }
+    }
 }
 
 #[derive(Debug)]
@@ -335,9 +357,7 @@ impl<B: BufMutExt> ProtocolWriter for SimpleJsonProtocolSerializer<B> {
             self.write_string(&value.to_string());
             return;
         }
-        CompactFormatter
-            .write_f64(&mut self.buffer, value)
-            .expect("Somehow failed to do \"io\" on a buffer");
+        self.write_floating_point_number(value, CompactFormatter::write_f64::<Writer<B>>);
     }
     #[inline]
     fn write_float(&mut self, value: f32) {
@@ -345,9 +365,7 @@ impl<B: BufMutExt> ProtocolWriter for SimpleJsonProtocolSerializer<B> {
             self.write_string(&value.to_string());
             return;
         }
-        CompactFormatter
-            .write_f32(&mut self.buffer, value)
-            .expect("Somehow failed to do \"io\" on a buffer");
+        self.write_floating_point_number(value, CompactFormatter::write_f32::<Writer<B>>);
     }
     #[inline]
     fn write_string(&mut self, value: &str) {
@@ -380,6 +398,75 @@ enum CommaState {
     End,
 }
 
+struct JsonNumberString {
+    string: Vec<u8>,
+    quoted: bool,
+}
+
+trait From32Infinity {
+    fn from_f32_inf() -> Self;
+}
+
+impl From32Infinity for f32 {
+    fn from_f32_inf() -> Self {
+        f32::INFINITY
+    }
+}
+
+impl From32Infinity for f64 {
+    fn from_f32_inf() -> Self {
+        3.4028237e38f64
+    }
+}
+
+trait From32NegInfinity {
+    fn from_f32_neg_inf() -> Self;
+}
+
+impl From32NegInfinity for f32 {
+    fn from_f32_neg_inf() -> Self {
+        f32::NEG_INFINITY
+    }
+}
+
+impl From32NegInfinity for f64 {
+    fn from_f32_neg_inf() -> Self {
+        -3.4028237e38f64
+    }
+}
+
+trait FromF64Infinity {
+    fn from_f64_inf() -> Self;
+}
+
+impl FromF64Infinity for f32 {
+    fn from_f64_inf() -> Self {
+        f32::NAN
+    }
+}
+
+impl FromF64Infinity for f64 {
+    fn from_f64_inf() -> Self {
+        f64::INFINITY
+    }
+}
+
+trait FromF64NegInfinity {
+    fn from_f64_neg_inf() -> Self;
+}
+
+impl FromF64NegInfinity for f32 {
+    fn from_f64_neg_inf() -> Self {
+        f32::NAN
+    }
+}
+
+impl FromF64NegInfinity for f64 {
+    fn from_f64_neg_inf() -> Self {
+        f64::NEG_INFINITY
+    }
+}
+
 impl<B: Buf> SimpleJsonProtocolDeserializer<B> {
     #[inline]
     pub fn new(buffer: B) -> Self {
@@ -480,11 +567,10 @@ impl<B: Buf> SimpleJsonProtocolDeserializer<B> {
         }
     }
     #[inline]
-    fn read_json_number(&mut self) -> Result<serde_json::Number> {
-        self.strip_whitespace();
-        let mut ret = Vec::new();
+    fn read_json_number_string(&mut self) -> Result<JsonNumberString> {
+        let mut string = Vec::new();
 
-        let as_string = match self.peek() {
+        let quoted = match self.peek() {
             Some(b'"') => {
                 self.advance(1);
                 true
@@ -494,26 +580,121 @@ impl<B: Buf> SimpleJsonProtocolDeserializer<B> {
 
         while let Some(b) = self.peek() {
             match b {
-                b' ' | b'\t' | b'\n' | b'\r' | b'}' | b']' | b',' | b':' | b'"' => {
-                    if as_string && b == b'"' {
+                b' ' | b'\t' | b'\n' | b'\r' | b'}' | b']' | b',' | b':' => {
+                    if quoted {
+                        bail!("Missing closing quote \" for number")
+                    }
+                    break;
+                }
+                b'"' => {
+                    if quoted {
                         self.advance(1);
                     }
                     break;
                 }
                 _ => {
-                    ret.push(b);
+                    string.push(b);
                     self.advance(1);
                 }
             }
         }
 
-        let v: std::result::Result<serde_json::Value, _> = serde_json::from_slice(&ret);
+        Ok(JsonNumberString { string, quoted })
+    }
+    /// Used for deserializing into a `serde_json::Value`.
+    #[inline]
+    fn read_json_number(&mut self) -> Result<serde_json::Number> {
+        self.strip_whitespace();
+        let JsonNumberString { string, .. } = self.read_json_number_string()?;
+
+        serde_json::from_slice(&string).map_err(Into::into)
+    }
+    #[inline]
+    fn read_json_integer_number(&mut self) -> Result<serde_json::Number> {
+        self.strip_whitespace();
+        let JsonNumberString { string, .. } = self.read_json_number_string()?;
+
+        let v: std::result::Result<serde_json::Value, _> = serde_json::from_slice(&string);
         match v {
             Ok(serde_json::Value::Number(n)) => Ok(n),
             _ => bail!("Invalid number"),
         }
     }
     #[inline]
+    fn read_floating_point_number<F>(&mut self) -> Result<F>
+    where
+        F: num_traits::Float
+            + FromStr<Err = ParseFloatError>
+            + From32Infinity
+            + From32NegInfinity
+            + FromF64Infinity
+            + FromF64NegInfinity,
+    {
+        self.strip_whitespace();
+        let JsonNumberString { string, quoted } = self.read_json_number_string()?;
+
+        let result = match &string[..] {
+            b"NaN" => {
+                if !quoted {
+                    bail!("NaN must be quoted")
+                }
+                F::nan()
+            }
+            b"-NaN" => {
+                if !quoted {
+                    bail!("-NaN must be quoted")
+                }
+                -F::nan()
+            }
+            b"Infinity" => {
+                if !quoted {
+                    bail!("Infinity must be quoted")
+                }
+                F::infinity()
+            }
+            b"-Infinity" => {
+                if !quoted {
+                    bail!("-Infinity must be quoted")
+                }
+                F::neg_infinity()
+            }
+            b"1.797693134862316e308" => F::from_f64_inf(),
+            b"-1.797693134862316e308" => F::from_f64_neg_inf(),
+            b"3.4028237e38" => F::from_f32_inf(),
+            b"-3.4028237e38" => F::from_f32_neg_inf(),
+            other => {
+                let s = std::str::from_utf8(other)
+                    .context("Invalid UTF-8 for floating point number")?;
+                // Curiously, this parses the below incorrectly-serialized nan/infinities as infinite nans,
+                // instead of erroring out like other float parsers.
+                // We currently rely on this behavior (but its unit-tested!).
+                let x = s.parse::<F>().context("Invalid floating point number")?;
+
+                // This if-condition is added here for backward compatibility with the original Rust simplejson_protocol::serialize
+                // and can be removed in the future along with excessive match arms and corresponding tests. For reference, it used
+                // to be that the original Rust serializer produced a string representation of a floating point number by
+                // serde_json::Formatter::write_f64 or serde_json::Formatter::write_32, both of each boil down to ryu::Buffer::format_finite.
+                //
+                // Examples of this include but are not limited to:
+                // - "2.696539702293474e308" for f64::NAN
+                // - "-2.696539702293474e308" for -f64::NAN
+                // - "1.797693134862316e308" for f64::INFINITY
+                // - "-1.797693134862316e308" for f64::NEG_INFINITY
+                // - "5.1042355e38" for f32::NAN
+                // - "-5.1042355e38" for -f32::NAN
+                // - "3.4028237e38" for f32::INFINITY
+                // - "-3.4028237e38" for f32::NEG_INFINITY
+                //
+                // Given that we've checked for string representations of positive and negative infinity above, combined with the fact that
+                // the standard parser successfully parses every grammar correct string representation of NaN values into ±Infinity or ±NaN as
+                // "closest representable floating-point number", we can safely turn infinities into NaNs and all other values into themselves.
+                if x.is_infinite() { F::nan() } else { x }
+            }
+        };
+
+        Ok(result)
+    }
+    #[inline]
     fn read_json_value(&mut self, max_depth: i32) -> Result<serde_json::Value> {
         if max_depth <= 0 {
             bail_err!(ProtocolError::SkipDepthExceeded)
@@ -539,7 +720,7 @@ impl<B: Buf> SimpleJsonProtocolDeserializer<B> {
             }
             ValueKind::Array => {
                 let mut vec = Vec::new();
-                self.read_list_begin()?;
+                self.read_list_begin_unchecked()?;
                 while self.read_list_value_begin()? {
                     let element = self.read_json_value(max_depth - 1)?;
                     vec.push(element);
@@ -611,7 +792,7 @@ impl<B: Buf> SimpleJsonProtocolDeserializer<B> {
                 self.read_struct_end()?;
             }
             TType::List => {
-                let (_, len) = self.read_list_begin()?;
+                let (_, len) = self.read_list_begin_unchecked()?;
                 let mut idx = 0;
                 loop {
                     let more = self.read_list_value_begin()?;
@@ -663,7 +844,7 @@ impl<B: Buf> SimpleJsonProtocolDeserializer<B> {
             Some(b'"') => Ok(ValueKind::String),
             Some(b'n') => Ok(ValueKind::Null),
             Some(b't') | Some(b'f') => Ok(ValueKind::Bool),
-            Some(b'-') => Ok(ValueKind::Number),
+            Some(b'-') | Some(b'I') | Some(b'N') => Ok(ValueKind::Number),
             Some(b) if (b as char).is_ascii_digit() => Ok(ValueKind::Number),
             ch => bail!(
                 "Expected [, {{, or \", or number after {:?}",
@@ -750,7 +931,7 @@ impl<B: Buf> ProtocolReader for SimpleJsonProtocolDeserializer<B> {
         Ok(())
     }
     #[inline]
-    fn read_map_begin(&mut self) -> Result<(TType, TType, Option<usize>)> {
+    fn read_map_begin_unchecked(&mut self) -> Result<(TType, TType, Option<usize>)> {
         self.eat(b"{").context("Expected a start of a list")?;
         // Meaningless type, self.skip_inner and deserialize do not depend on it
         Ok((TType::Stop, TType::Stop, None))
@@ -789,7 +970,7 @@ impl<B: Buf> ProtocolReader for SimpleJsonProtocolDeserializer<B> {
         Ok(())
     }
     #[inline]
-    fn read_list_begin(&mut self) -> Result<(TType, Option<usize>)> {
+    fn read_list_begin_unchecked(&mut self) -> Result<(TType, Option<usize>)> {
         self.eat(b"[").context("Expected a start of a list")?;
         Ok((TType::Stop, None))
     }
@@ -817,8 +998,8 @@ impl<B: Buf> ProtocolReader for SimpleJsonProtocolDeserializer<B> {
         Ok(())
     }
     #[inline]
-    fn read_set_begin(&mut self) -> Result<(TType, Option<usize>)> {
-        self.read_list_begin()
+    fn read_set_begin_unchecked(&mut self) -> Result<(TType, Option<usize>)> {
+        self.read_list_begin_unchecked()
     }
     #[inline]
     fn read_set_value_begin(&mut self) -> Result<bool> {
@@ -857,19 +1038,17 @@ impl<B: Buf> ProtocolReader for SimpleJsonProtocolDeserializer<B> {
     }
     #[inline]
     fn read_i64(&mut self) -> Result<i64> {
-        self.read_json_number()?
+        self.read_json_integer_number()?
             .as_i64()
             .ok_or_else(|| anyhow!("Invalid number"))
     }
     #[inline]
     fn read_double(&mut self) -> Result<f64> {
-        self.read_json_number()?
-            .as_f64()
-            .ok_or_else(|| anyhow!("Invalid number"))
+        self.read_floating_point_number()
     }
     #[inline]
     fn read_float(&mut self) -> Result<f32> {
-        Ok(self.read_double()? as f32)
+        self.read_floating_point_number()
     }
     #[inline]
     fn read_string(&mut self) -> Result<String> {
