@@ -14,10 +14,14 @@
  * limitations under the License.
  */
 
+use std::collections::HashMap;
+use std::collections::HashSet;
 use std::io::Cursor;
 
 use bytes::Buf;
+use bytes::BufMut;
 use bytes::Bytes;
+use bytes::BytesMut;
 
 use super::BOOL_VALUES;
 use super::BYTE_VALUES;
@@ -26,6 +30,10 @@ use super::FLOAT_VALUES;
 use super::INT16_VALUES;
 use super::INT32_VALUES;
 use super::INT64_VALUES;
+use crate::bufext::BufMutExt as _;
+use crate::compact_protocol::CType;
+use crate::deserialize::Deserialize;
+use crate::errors::ProtocolError;
 use crate::thrift_protocol::MessageType;
 use crate::ttype::TType;
 use crate::CompactProtocol;
@@ -53,7 +61,7 @@ fn read_write_bool_list() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (thetype2, thelen2) = deserializer
-            .read_list_begin()
+            .read_list_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(thetype, thetype2);
@@ -92,7 +100,7 @@ fn read_write_string_list() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (thetype2, thelen2) = deserializer
-            .read_list_begin()
+            .read_list_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(thetype, thetype2);
@@ -127,7 +135,7 @@ fn read_write_byte_list() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (thetype2, thelen2) = deserializer
-            .read_list_begin()
+            .read_list_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(thetype, thetype2);
@@ -159,7 +167,7 @@ fn read_write_i16_list() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (thetype2, thelen2) = deserializer
-            .read_list_begin()
+            .read_list_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(thetype, thetype2);
@@ -191,7 +199,7 @@ fn read_write_i32_list() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (thetype2, thelen2) = deserializer
-            .read_list_begin()
+            .read_list_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(thetype, thetype2);
@@ -223,7 +231,7 @@ fn read_write_i64_list() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (thetype2, thelen2) = deserializer
-            .read_list_begin()
+            .read_list_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(thetype, thetype2);
@@ -255,7 +263,7 @@ fn read_write_f32_list() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (thetype2, thelen2) = deserializer
-            .read_list_begin()
+            .read_list_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(thetype, thetype2);
@@ -293,7 +301,7 @@ fn read_write_f64_list() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (thetype2, thelen2) = deserializer
-            .read_list_begin()
+            .read_list_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(thetype, thetype2);
@@ -331,7 +339,7 @@ fn read_write_f64_set() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (thetype2, thelen2) = deserializer
-            .read_set_begin()
+            .read_set_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(thetype, thetype2);
@@ -370,7 +378,7 @@ fn read_write_string_i64_map() {
     let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(buf));
     {
         let (key_type2, value_type2, thelen2) = deserializer
-            .read_map_begin()
+            .read_map_begin_unchecked()
             .expect("failed to read header");
 
         assert_eq!(key_type, key_type2);
@@ -599,3 +607,36 @@ fn skip_stop_in_container() {
         },
     }
 }
+
+#[test]
+fn test_overallocation() {
+    let mut malicious = BytesMut::new();
+    malicious.put_u8(0xf0 | CType::I16 as u8);
+    malicious.put_varint_i64(1_000_000_000_000_000);
+    malicious.put_bytes(0, 10);
+    let malicious = malicious.freeze();
+    let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(malicious.clone()));
+    let err = <Vec<i16> as Deserialize<_>>::read(&mut deserializer).unwrap_err();
+    assert_eq!(
+        err.downcast_ref::<ProtocolError>(),
+        Some(&ProtocolError::EOF),
+    );
+
+    let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(malicious));
+    let err = <HashSet<i16> as Deserialize<_>>::read(&mut deserializer).unwrap_err();
+    assert_eq!(
+        err.downcast_ref::<ProtocolError>(),
+        Some(&ProtocolError::EOF),
+    );
+
+    let mut malicious = BytesMut::new();
+    malicious.put_varint_i64(1_000_000_000_000_000);
+    malicious.put_u8(((CType::Binary as u8) << 4) | (CType::I16 as u8));
+    malicious.put_bytes(0, 10);
+    let mut deserializer = <CompactProtocol>::deserializer(Cursor::new(malicious.freeze()));
+    let err = <HashMap<String, i16> as Deserialize<_>>::read(&mut deserializer).unwrap_err();
+    assert_eq!(
+        err.downcast_ref::<ProtocolError>(),
+        Some(&ProtocolError::EOF),
+    );
+}
