@@ -20,6 +20,7 @@ import types
 import typing
 import unittest
 from datetime import datetime
+from sys import platform
 
 import thrift.python.mutable_serializer as mutable_serializer
 
@@ -47,7 +48,7 @@ from thrift.python.types import (
     StructOrUnion as ImmutableStructOrUnion,
 )
 
-from thrift.test.thrift_python.struct_test.thrift_mutable_types import (  # @manual=//thrift/test/thrift-python:struct_test_thrift-python-types
+from thrift.test.thrift_python.struct_test.thrift_mutable_types import (
     bool_constant,
     byte_constant,
     double_constant,
@@ -191,6 +192,47 @@ class ThriftPython_ImmutableStruct_Test(unittest.TestCase):
         self.assertEqual(w2, TestStructImmutable())
         self.assertIsNone(w2.optional_string)
 
+    def test_intrinsic_default_values_for_unqualified_fields(self) -> None:
+        # GIVEN
+        # Even though this test is primarily about
+        # the intrinsic default values for unqualified fields,
+        # for sanity, set the optional fields to None.
+        expected_primitive = TestStructAllThriftPrimitiveTypesImmutable(
+            unqualified_string="",
+            optional_string=None,
+            unqualified_i32=0,
+            optional_i32=None,
+            unqualified_double=0.0,
+            optional_double=None,
+            unqualified_bool=False,
+            optional_bool=None,
+            unqualified_byte=0,
+            optional_byte=None,
+            unqualified_i16=0,
+            optional_i16=None,
+            unqualified_i64=0,
+            optional_i64=None,
+            unqualified_float=0,
+            optional_float=None,
+        )
+
+        self.assertEqual(
+            expected_primitive, TestStructAllThriftPrimitiveTypesImmutable()
+        )
+
+        expected_container = TestStructAllThriftContainerTypesImmutable(
+            unqualified_list_i32=[],
+            optional_list_i32=None,
+            unqualified_set_string=set(),
+            optional_set_string=None,
+            unqualified_map_string_i32={},
+            optional_map_string_i32=None,
+        )
+
+        self.assertEqual(
+            expected_container, TestStructAllThriftContainerTypesImmutable()
+        )
+
     def test_default_values(self) -> None:
         # Custom default values:
         # Newly created instance has custom default values for non-optional
@@ -267,11 +309,16 @@ class ThriftPython_ImmutableStruct_Test(unittest.TestCase):
         )
 
     def test_subclass(self) -> None:
-        types.new_class(
-            "TestImmutableSubclass",
-            bases=(TestStructImmutable,),
-            exec_body=lambda ns: ns.update(_fbthrift_SPEC=()),
+        err = (
+            r"Inheritance from generated thrift struct \w+ is deprecated."
+            r" Please use composition."
         )
+        with self.assertRaisesRegex(TypeError, err):
+            types.new_class(
+                "TestImmutableSubclass",
+                bases=(TestStructImmutable,),
+                exec_body=lambda ns: ns.update(_fbthrift_SPEC=()),
+            )
 
     def test_base_classes(self) -> None:
         self.assertIsInstance(TestStructImmutable(), ImmutableStruct)
@@ -300,14 +347,17 @@ class ThriftPython_ImmutableStruct_Test(unittest.TestCase):
 
         # Attributes of immutable types cannot be deleted.
         #
-        # Note the interesting (and somewhat inconsistent) current behavior:
+        # Note the interesting (and somewhat inconsistent) current behavior on Linux:
         # Calling `del` prior to accessing an attribute raises an AttributeError
         # (at the cinder level), but doing so after accessing it is a silent
         # no-op.
         with self.assertRaisesRegex(AttributeError, "unqualified_string"):
             del w.unqualified_string
         self.assertEqual(w.unqualified_string, "hello, world!")
-        del w.unqualified_string  # silent no-op
+
+        if platform == "linux":
+            del w.unqualified_string  # silent no-op
+
         self.assertEqual(w.unqualified_string, "hello, world!")
 
         # However, a new instance of the object can be created with a specific
@@ -436,6 +486,90 @@ class ThriftPython_ImmutableStruct_Test(unittest.TestCase):
         self.assertNotEqual(w_immutable_containers, w_mutable_containers)
         self.assertEqual(list(w_immutable_containers), list(w_mutable_containers))
 
+    def test_match(self) -> None:
+        # Canonical case: match fields
+        match TestStructImmutable(unqualified_string="Hello, world!"):
+            case TestStructImmutable(unqualified_string=x, optional_string=None):
+                self.assertEqual(x, "Hello, world!")
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Any instance will match a Class pattern with no argument:
+        match TestStructImmutable(
+            unqualified_string="Hello, ", optional_string="world!"
+        ):
+            case TestStructImmutable():
+                pass  # Expected
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Capturing value of unset optional field (i.e., None):
+        match TestStructImmutable(unqualified_string="Hello, world!"):
+            case TestStructImmutable(optional_string=x):
+                self.assertIsNone(x)
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Do not match if values differ
+        match TestStructImmutable(unqualified_string="Hello"):
+            case TestStructImmutable(unqualified_string="world!", optional_string=x):
+                self.fail(f"Unexpected match: {x}")
+            case _:
+                pass  # Expected
+
+        # Match default values
+        match TestStructWithDefaultValuesImmutable():
+            case TestStructWithDefaultValuesImmutable(
+                unqualified_integer=x,
+                unqualified_list_i32=y,
+                optional_integer=None,
+            ):
+                self.assertEqual(x, 42)
+                self.assertEqual(y, [1, 2, 3])
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Same, but using sequence capture pattern
+        match TestStructWithDefaultValuesImmutable():
+            case TestStructWithDefaultValuesImmutable(
+                unqualified_integer=x,
+                unqualified_list_i32=[a, b, c],
+                optional_integer=None,
+            ):
+                self.assertEqual(x, 42)
+                self.assertEqual(a, 1)
+                self.assertEqual(b, 2)
+                self.assertEqual(c, 3)
+
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Match nested struct
+        match TestStructWithDefaultValuesImmutable():
+            case TestStructWithDefaultValuesImmutable(unqualified_struct=x):
+                self.assertEqual(x, TestStructImmutable(unqualified_string="hello"))
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Match nested struct, capture nested field
+        match TestStructWithDefaultValuesImmutable():
+            case TestStructWithDefaultValuesImmutable(
+                unqualified_struct=TestStructImmutable(unqualified_string=x)
+            ):
+                self.assertEqual(x, "hello")
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Match adapted types
+        match TestStructAdaptedTypesImmutable(
+            unqualified_adapted_i32_to_datetime=datetime.fromtimestamp(1733556290)
+        ):
+            case TestStructAdaptedTypesImmutable(unqualified_adapted_i32_to_datetime=x):
+                self.assertIsInstance(x, datetime)
+                self.assertEqual(x, datetime.fromtimestamp(1733556290))
+            case _:
+                self.fail("Expected match, got none.")
+
 
 class ThriftPython_MutableStruct_Test(unittest.TestCase):
     def setUp(self) -> None:
@@ -480,7 +614,56 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
         self.assertIsNot(w, w3)
         self.assertEqual(w, w3)
 
+    def test_intrinsic_default_values_for_unqualified_fields(self) -> None:
+        # GIVEN
+        # Even though this test is primarily about
+        # the intrinsic default values for unqualified fields,
+        # for sanity, set the optional fields to None.
+        expected_primitive = TestStructAllThriftPrimitiveTypesMutable(
+            unqualified_string="",
+            optional_string=None,
+            unqualified_i32=0,
+            optional_i32=None,
+            unqualified_double=0.0,
+            optional_double=None,
+            unqualified_bool=False,
+            optional_bool=None,
+            unqualified_byte=0,
+            optional_byte=None,
+            unqualified_i16=0,
+            optional_i16=None,
+            unqualified_i64=0,
+            optional_i64=None,
+            unqualified_float=0,
+            optional_float=None,
+        )
+
+        self.assertEqual(expected_primitive, TestStructAllThriftPrimitiveTypesMutable())
+
+        expected_container = TestStructAllThriftContainerTypesMutable(
+            unqualified_list_i32=to_thrift_list([]),
+            optional_list_i32=None,
+            unqualified_set_string=to_thrift_set(set()),
+            optional_set_string=None,
+            unqualified_map_string_i32=to_thrift_map({}),
+            optional_map_string_i32=None,
+        )
+
+        self.assertEqual(expected_container, TestStructAllThriftContainerTypesMutable())
+
     def test_default_values(self) -> None:
+        # Newly created instance has custom default values for non-optional
+        # fields, but custom default values for optional fields are ignored.
+        self.assertEqual(
+            TestStructWithDefaultValuesMutable(),
+            TestStructWithDefaultValuesMutable(
+                unqualified_integer=42,
+                optional_integer=None,
+                unqualified_struct=TestStructMutable(unqualified_string="hello"),
+                optional_struct=None,
+            ),
+        )
+
         # Intrinsic default values:
         # optional struct field is None
         self.assertIsNone(
@@ -492,6 +675,14 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
             TestStructWithDefaultValuesMutable().unqualified_struct_intrinsic_default,
         )
 
+    def test_reset(self) -> None:
+        w = TestStructWithDefaultValuesMutable(optional_integer=123)
+        self.assertEqual(w.optional_integer, 123)
+        self.assertEqual(w.unqualified_integer, 42)
+        w.fbthrift_reset()
+        self.assertIsNone(w.optional_integer)
+        self.assertEqual(w.unqualified_integer, 42)
+
     def test_equality_and_hashability(self) -> None:
         # Equality
         w_mutable = TestStructMutable(unqualified_string="hello, world!")
@@ -947,8 +1138,6 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
         s2.unqualified_map_string_i32 = to_thrift_map(my_map)
         my_map["c"] = 3
         self.assertEqual(3, len(my_map))
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MutableMap[str, int]`.
         self.assertEqual(2, len(s2.unqualified_map_string_i32))
 
         s3 = TestStructAllThriftContainerTypesMutable(
@@ -1165,8 +1354,6 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
             unqualified_map_string_i32=to_thrift_map({"a": 1, "b": 2})
         )
 
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MutableMap[str, int]`.
         self.assertEqual(2, len(s.unqualified_map_string_i32))
         self.assertEqual({"a": 1, "b": 2}, s.unqualified_map_string_i32)
 
@@ -1222,7 +1409,6 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
         self.assertEqual(None, s.unqualified_map_string_i32.get("Not Exists"))
         self.assertEqual(
             "MyDefaultValue",
-            # pyre-ignore[6]: Intentional for test
             s.unqualified_map_string_i32.get("Not Exists", "MyDefaultValue"),
         )
 
@@ -1262,8 +1448,6 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
         self.assertEqual(0, len(python_set))
 
         # `keys()`
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MapKeysView[str]`.
         self.assertEqual(3, len(s.unqualified_map_string_i32.keys()))
 
         python_set = {"a", "b", "c"}
@@ -1273,21 +1457,15 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
 
         # `keys()` returns a view
         keys = s.unqualified_map_string_i32.keys()
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MapKeysView[str]`.
         self.assertEqual(3, len(keys))
         self.assertEqual({"a", "b", "c"}, set(keys))
 
         s.unqualified_map_string_i32["d"] = 4
 
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MapKeysView[str]`.
         self.assertEqual(4, len(keys))
         self.assertEqual({"a", "b", "c", "d"}, set(keys))
 
         # `values()`
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MapValuesView[int]`.
         self.assertEqual(4, len(s.unqualified_map_string_i32.values()))
 
         python_list = [1, 2, 3, 4]
@@ -1296,21 +1474,15 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
 
         # `values()` returns a view
         values = s.unqualified_map_string_i32.values()
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MapValuesView[int]`.
         self.assertEqual(4, len(values))
         self.assertEqual([1, 2, 3, 4], sorted(values))
 
         s.unqualified_map_string_i32["e"] = 5
 
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MapValuesView[int]`.
         self.assertEqual(5, len(values))
         self.assertEqual([1, 2, 3, 4, 5], sorted(values))
 
         # `items()`
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MapItemsView[str, int]`.
         self.assertEqual(5, len(s.unqualified_map_string_i32.items()))
         self.assertEqual(
             [("a", 1), ("b", 2), ("c", 3), ("d", 4), ("e", 5)],
@@ -1319,8 +1491,6 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
 
         # `items()` returns a view
         items = s.unqualified_map_string_i32.items()
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MapItemsView[str, int]`.
         self.assertEqual(5, len(items))
         self.assertEqual(
             [("a", 1), ("b", 2), ("c", 3), ("d", 4), ("e", 5)],
@@ -1329,8 +1499,6 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
 
         s.unqualified_map_string_i32["f"] = 6
 
-        # pyre-fixme[6]: For 1st argument expected
-        #  `pyre_extensions.PyreReadOnly[Sized]` but got `MapItemsView[str, int]`.
         self.assertEqual(6, len(items))
         self.assertEqual(
             [("a", 1), ("b", 2), ("c", 3), ("d", 4), ("e", 5), ("f", 6)],
@@ -2033,3 +2201,208 @@ class ThriftPython_MutableStruct_Test(unittest.TestCase):
         s3.map_int_to_list_int = to_thrift_map({})
         self.assertEqual({2: [2, 3, 4]}, s2.map_int_to_list_int)
         self.assertEqual({}, s3.map_int_to_list_int)
+
+    def test_match(self) -> None:
+        # Canonical case: match fields
+        match TestStructMutable(unqualified_string="Hello, world!"):
+            case TestStructMutable(unqualified_string=x, optional_string=None):
+                self.assertEqual(x, "Hello, world!")
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Any instance will match a Class pattern with no argument:
+        match TestStructMutable(unqualified_string="Hello, ", optional_string="world!"):
+            case TestStructMutable():
+                pass  # Expected
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Capturing value of unset optional field (i.e., None):
+        match TestStructMutable(unqualified_string="Hello, world!"):
+            case TestStructMutable(optional_string=x):
+                self.assertIsNone(x)
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Do not match if values differ
+        match TestStructMutable(unqualified_string="Hello"):
+            case TestStructMutable(unqualified_string="world!", optional_string=x):
+                self.fail(f"Unexpected match: {x}")
+            case _:
+                pass  # Expected
+
+        # Match default values
+        match TestStructWithDefaultValuesMutable():
+            case TestStructWithDefaultValuesMutable(
+                unqualified_integer=x,
+                unqualified_list_i32=y,
+                optional_integer=None,
+            ):
+                self.assertEqual(x, 42)
+                self.assertEqual(y, [1, 2, 3])
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Same, but using sequence capture pattern
+        match TestStructWithDefaultValuesMutable():
+            case TestStructWithDefaultValuesMutable(
+                unqualified_integer=x,
+                unqualified_list_i32=[a, b, c],
+                optional_integer=None,
+            ):
+                self.assertEqual(x, 42)
+                self.assertEqual(a, 1)
+                self.assertEqual(b, 2)
+                self.assertEqual(c, 3)
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Match nested struct
+        match TestStructWithDefaultValuesMutable():
+            case TestStructWithDefaultValuesMutable(unqualified_struct=x):
+                self.assertEqual(x, TestStructMutable(unqualified_string="hello"))
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Match nested struct, capture nested field
+        match TestStructWithDefaultValuesMutable():
+            case TestStructWithDefaultValuesMutable(
+                unqualified_struct=TestStructMutable(unqualified_string=x)
+            ):
+                self.assertEqual(x, "hello")
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Match adapted types
+        match TestStructAdaptedTypesMutable(
+            unqualified_adapted_i32_to_datetime=datetime.fromtimestamp(1733556290)
+        ):
+            case TestStructAdaptedTypesMutable(unqualified_adapted_i32_to_datetime=x):
+                self.assertIsInstance(x, datetime)
+                self.assertEqual(x, datetime.fromtimestamp(1733556290))
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Cases above are similar to immutable structs. Checking with mutations next...
+
+        # Similar to canonical case above, but field is changed after initialization.
+        w = TestStructWithDefaultValuesMutable(
+            unqualified_integer=1, optional_integer=2
+        )
+        w.unqualified_integer = 3
+        w.optional_integer = None
+        match w:
+            case TestStructWithDefaultValuesMutable(
+                optional_integer=None, unqualified_integer=x
+            ):
+                self.assertEqual(x, 3)
+            case _:
+                self.fail("Expected match, got none.")
+
+        w.optional_integer = 4
+        match w:
+            case TestStructWithDefaultValuesMutable(
+                optional_integer=None, unqualified_integer=x
+            ):
+                self.fail(f"Unexpected match: {x}")
+            case TestStructWithDefaultValuesMutable(
+                optional_integer=y, unqualified_integer=x
+            ):
+                self.assertEqual(y, 4)
+                self.assertEqual(x, 3)
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Identity of captured struct field is the same as the field itself:
+        sub_w = TestStructMutable(optional_string="Lorem ipsum")
+        w.optional_struct = sub_w
+        match w:
+            case TestStructWithDefaultValuesMutable(optional_struct=x):
+                self.assertIs(x, sub_w)
+            case _:
+                self.fail("Expected match, got none.")
+
+        # Match after resetting
+        w.fbthrift_reset()
+        match w:
+            case TestStructWithDefaultValuesMutable(
+                unqualified_integer=x, optional_struct=y
+            ):
+                self.assertEqual(x, 42)
+                self.assertIsNone(y)
+            case _:
+                self.fail("Expected match, got none.")
+
+    def test_fbthrift_copy_from(self) -> None:
+        """
+        `lhs.fbthrift_copy_from(rhs)` copies the content of the `rhs` struct
+        into the lhs struct. It is semantically equivalent to assigning each
+        field of `rhs` to `lhs`."
+        """
+        # Struct with primitive fields
+        s1 = TestStructAllThriftPrimitiveTypesMutable(
+            unqualified_bool=True,
+            optional_byte=0,
+            unqualified_i16=1,
+            optional_i32=2,
+            unqualified_i64=3,
+            optional_float=4.0,
+            unqualified_double=5.0,
+            optional_string="abc",
+        )
+
+        s2 = TestStructAllThriftPrimitiveTypesMutable()
+        self.assertNotEqual(s1, s2)
+        s2.fbthrift_copy_from(s1)
+        self.assertEqual(s1, s2)
+
+        # Struct with container fields
+        s3 = TestStructAllThriftContainerTypesMutable(
+            unqualified_list_i32=to_thrift_list([1, 2, 3]),
+            optional_set_string=to_thrift_set({"a", "b", "c"}),
+            unqualified_map_string_i32=to_thrift_map({"a": 1, "b": 2}),
+        )
+
+        s4 = TestStructAllThriftContainerTypesMutable()
+        self.assertNotEqual(s3, s4)
+        s4.fbthrift_copy_from(s3)
+        self.assertEqual(s3, s4)
+
+        # Container assignment is refernce semantics, after `fbthrift_copy_from()`
+        # s3 and s4 container fields are the "same" containers.
+        self.assertEqual([1, 2, 3], s3.unqualified_list_i32)
+        self.assertEqual([1, 2, 3], s4.unqualified_list_i32)
+
+        s3.unqualified_list_i32.append(4)
+
+        self.assertEqual([1, 2, 3, 4], s3.unqualified_list_i32)
+        self.assertEqual([1, 2, 3, 4], s4.unqualified_list_i32)
+
+        # Struct with struct fields
+        n2 = TestStructNested_2_Mutable(i32_field=2)
+        n1 = TestStructNested_1_Mutable(i32_field=3, nested_2=n2)
+        s5 = TestStructNested_0_Mutable(i32_field=5, nested_1=n1)
+
+        s6 = TestStructNested_0_Mutable()
+        self.assertNotEqual(s5, s6)
+        s6.fbthrift_copy_from(s5)
+        self.assertEqual(s5, s6)
+
+        # Struct assignment is refernce semantics, after `fbthrift_copy_from()`
+        # s5 and s4 struct fields are the "same" structs.
+        self.assertEqual(3, s5.nested_1.i32_field)
+        self.assertEqual(3, s6.nested_1.i32_field)
+
+        s5.nested_1.i32_field = 33
+
+        self.assertEqual(33, s5.nested_1.i32_field)
+        self.assertEqual(33, s6.nested_1.i32_field)
+
+        # `lhs` and `rhs` must be the same type
+        s7 = TestStructAllThriftPrimitiveTypesMutable()
+        with self.assertRaisesRegex(
+            TypeError,
+            "Cannot copy from.*TestStructAllThriftContainerTypes.*to"
+            ".*TestStructAllThriftPrimitiveTypes",
+        ):
+            s7.fbthrift_copy_from(TestStructAllThriftContainerTypesMutable())
